! ----------------------------------------------------------------------------------------
!    RMATRX-PSTGF R-Matrix outer region calculations for electron-impact excitation
!    Copyright (C) 2019  
!    Michael J. Seaton, University College London;
!    Dario M. Mitnik, Universidad de Buenos Aires;
!    Nigel R. Badnell, University of Strathclyde;
!    Connor P. Ballance, Queen's University Belfast;
!    Luis Fernandez-Menchero, Queen's University Belfast.
!    This work used the ARCHER UK National Supercomputing Service (http://www.archer.ac.uk).
!    All rights reserved.
!
!    This program is free software; you can redistribute it and/or
!    modify it under the terms of the GNU Lesser General Public
!    License as published by the Free Software Foundation; either
!    version 2.1 of the License, or (at your option) any later version.
!
!    This program is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied warranty of
!    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
!    Lesser General Public License for more details.
!
!    You should have received a copy of the GNU Lesser General Public
!    License along with this program; if not, write to the Free Software
!    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
!
!    Email contact: l.fernandezmenchero@qub.ac.uk
!    Post:  Dr Luis Fernandez Menchero
!           Queen's University Belfast
!           2017 University Road
!           Belfast BT9 6EW
!           United Kingdom
!
!    https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html
! ----------------------------------------------------------------------------------------
!
!    Version 1.1 (Beta)     October 2019
!
!    Compile, gnu version for archer
!    module swap PrgEnv-cray PrgEnv-gnu
!    module swap gcc gcc/7.2.0
!    module swap cray-libsci cray-libsci/17.12.1
!    ftn -O2 -mcmodel=medium -ftree-vectorize -funroll-loops -o pstgf.x pstgf.f
!
!    Local cluster
!    mpif90 -O3 -mcmodel=medium -o pstgf.x pstgf.f -llapack -lblas
!
!    CUDA version
!    source /opt/intel/switch.sh gpu
!    mpif90 pstgf.f -o pstgf_cuda.x  -Mcuda=cc60 -llapack -lblas 
!    -mcmodel=medium -O3 -ta=tesla:cc60 
!
!    Memory efficiency
!    -----------------
!    The PARAM file has been removed, all the dimension of arrays are dynamically
!    allocatable.
!    The only two remaining old parameters are MZMEG and MZPTS, they are now introduced
!    as input parameters in the Namelist /STGF/ (see below the description of the 
!    input variables).
!    All the arrays are allocated to exactly the required dimension.
!    All common blocks have been removed and replaced by modules
!
!    Input / Output efficiency
!    -------------------------
!    Each processor starts in a different symmetry, what means that the H.DATXXX files
!    are not read simultaneously by all processors.
!    One single OMEGA file is written, not necessary post-processing to merge all the
!    different OMEGAXXXX files from each processor.
!    New subroutines READ2N(iopt1,kase) and FASTREAD2(iopt1)
!    Arrays IWHD(kslp) MSLP(kslp) introduced, IONEONE removed
!    IWHD(kase=1:kslp)
!          = 1 - nhdat : which H.DATXXX file contains the information
!            about the kase partial wave
!          = -1 : partial wave information is stored in one single H.DAT or DSTGH.DAT file
!          = -10 : information about partial wave is in no file. Warning printed.
!    FASTREAD2 determines before starting the main calculations the total number of
!    partial waves to calculate kslp and the array IWHD
!    Several standard output files are written, one per processor,
!    routf by processor 0
!    routfgXXXX by rest of processors
!    There are no overwritting in standard output. If the calculation stops normally,
!    no info.ne.0 messages, routfgXXXX files can be safely removed.
!
!    Balancing processor time
!    ------------------------
!    The calculation has to process "fast energies", when the number of open channels
!    NCHOP is very different to the number of closed channels NCC;
!    and "slow energies", when NCHOP is similar to NCC.
!    With a constant energy mesh, some processors will work all the slow energies, 
!    while other ones will work only fast energies.
!    For the same processor, each partial wave the energy mesh is shifted so it has
!    to work slow energies for some partial waves,
!    and only fast energies for other ones.
!    With this method, all processors work approximately the same amount of slow 
!    energies, so the computing time is equally distributed in all the processors.
!
!    Calculation efficiency
!    ----------------------
!    Calculation subroutines ALPHAN RINIT and PETKMX modified to improve efficiency
!    Introduced subroutine RINIT4, using pointers for large matrix products (NCHF.gt.2000)
!    reactance matrix is split in sectors and the products are at cache level.
!    Logical variable lfail and array LFARR(1:mxe) introduced
!    If lfail=.true. a part of the calculation for energy ie failed, info.ne.0
!    it is variable in the calling sequence of subroutines REACT AIVB PQ MQDTK MQDTS
!    Then the calculation does not crash, it only removes the wrongly calculated energy,
!    prints a warning and continues with the following energy.
!
!    CUDA version for GPU
!    --------------------
!    Remove the comment marks in the module mmul_mod and the marks with CU in subroutine
!    RINIT, also comment in REACT the options for RINIT2 and RINIT4, always use RINIT
!    It splits the calculation of the reactance matrix in the processors of the GPU,
!    it reduces considerably the computing time.
!    Subroutine mmul_kernel presumes that the result, matrix C, is going to be 
!    symmetrical. If it is not the case, comment the if(j.le.i) statement.
!
! ----------------------------------------------------------------------------------------
!
!      Version v0.93 (Beta) 16/3/2016
!                
!      Compile : mpif90 pstgf_proto.f -o pstgf_proto.x -llapack -lblas -O3 -mcmodel=medium 
!
!
!      Updated pstgf , with the majority of COMMON blocks involving 
!      matrices now removed. The focus is upon scaling up , and 
!      with allocatable matrices, we can take advantage of a 'just in time '
!      approach to electron-impact collisions.
!
!       Memory efficiency  
!
!       1. Read H.DAT file
!
!       2. Form RMAT in 3 blocks, then symmetrize , deallocate WMAT(MNP2,NCHF)  
!
!         ONLY HAVE RMAT (NCHF, NCHF) left in memory
!
!       3. NOW ALLOCATE all COULOMB functions and perturbing matrices (NRBRCT)
!
!          SHOULD be perfectly allocated (NCHF,NCHF) for each partial wave,
!          with the side benefit of faster DGEMM calls due to smaller matrices
!          ie (NCHF,NCHF) rather than (MZCHF,MZCHF)  
!
!          THEY EXPAND and CONTRACT with CHANNEL size
!
!
!       4. CALCULATE COLLISION STRENGTH  
!
!       5. DEALLOCATE ALL THE COULOMB FUNCTIONS for next energy and repeat, goto 1. 
!
!        NEVER HAVE THE SURAFCE AMPLITUDES and COULOMB functions in memory at the same time
!
!       I/O efficiency :  Instead of merging H.DATXXX files into a single H.DAT file 
!
!                         KEEP H.DATXXX files in the outer region
!
!              This allows 100's perhaps 10000 processors to open and read multiple 
!                         H.DATXXX files rather than a monolithic single file. IF 
!                         IONEONE is used in pstg3r, it allows the user to skip 
!                         through symmetries not required, or determine how 
!                         many will go through in a 12/24/72 hr batch system.
!
!           LIST (IOPT1=2) symmetries required  + IONEONE in stgf namelist
!
!       Example
!
!   &STGF IONEONE=1 IMESH=1 IQDT=0 PERT='NO' IPRINT=-2 IOPT1=2 LRGLAM=-8 &END
!   &MESH1 MXE=2 E0=0.2 EINCR=0.1 &END
!    0 6 0 
!    0 8 1
!   -1 -1 -1
!
!
!    $ mpirun -np 1 ./pstgf23.x 
!      SKIP            1
!      SKIP            0
!      SKIP          201
!      SKIP          200
!      SKIP          401
!      SKIP          400
!      SKIP          601
!      CALC          600
! 
!
!           Single H.DAT still works!
!
! -----------------------------------------------------------------------------------------    
!
!       PSTGF.F ::   PARALLEL VERSION OF STGF.        JAN 2016 
!
!       UPDATE : THE PROGRAM NOW CALLS RINIT2 INSTEAD OF RINIT AND
!                FORMS THE RMATRIX IN 3 MULTIPLICATIONS
!                SAVING MEMORY AT A SMALL EXPENSE TO SPEED. 
!
!       THIS IS THE STANDARD VERSION OF PSTGF.F.  IT USES THE  LAPACK
!       SUBROUTINE DGEMM TO FORM THE R-MATRIX IN SUBROUTINE RINIT2.
!
!       THE PARALLEL VERSIONS OF THESE CODES WERE DEVELOPED WITH THE
!       SUPPORT OF A SCIDAC GRANT FROM DOE AND A SUBCONTRACT FROM LANL
!       BY A CONSORTIUM AT ROLLINS COLLEGE, STRATHCLYDE UNIVERSITY,
!       AND AUBURN UNIVERSITY.
!
!       PRINCIPAL INVESTIGATORS:
!       C. P. BALLANCE, D. M. MITNIK, N. R. BADNELL, AND D. C. GRIFFIN 
!                     PROGRAM STGF
!
!  CALCULATIONS FOR FREE ELECTRONS.
!
!  RECENT MODIFICATIONS
!        NEW NAMELIST I/O
!        PERTURBING POTENTIALS REWORKED
!        QUADRUPOLE TOP-UP
!        DIPOLE TOP-UP CAN USE CBE
!        RADIATIVE DECAYS IN B.P. MODE
!        INFINITE ENERGY OMEGA'S (DIPOLE)
!        OCTUPOLE ETC. TOP-UP
!        OCTUPOLE ETC. PERTURBING POTENTIALS
!        DETAILED SQDT, WITH TYPE-I DAMPING, FOR DR/RE
!        DETAILED MQDT, WITH TYPE-I DAMPING, FOR DR/RE
!        NEUTRAL CASE ADDED
!        DARC HD.DAT (AND H.DAT VIA DARC INTERFACE)
!        PARTITIONED H.DAT
!
! -----------------------------------------------------------------------------------------    
!  
        Module comm_interface
!        Use MPI
        Implicit none
        Include "mpif.h"
        Integer iam,nproc
        Integer istop
        Integer STATUS(MPI_STATUS_SIZE)
!        Real(8), Allocatable :: BUFFER1(:)       ! Dim=(mzmsh)
        End Module comm_interface
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!       module mmul_mod
!       use cudafor
!       integer, parameter :: igpusplt=32
!       contains
!
!! mmul_kernel computes A*B into C where A is NxM, B is MxL, C is then NxL
!
!       attributes(global) subroutine mmul_kernel( A, B, C, N, M, L )
!       real*8,device :: A(N,M), B(M,L), C(N,L)
!       integer, value :: N, M, L
!       integer :: i, j, kb, k, tx, ty
!
!! submatrices are declared to be in CUDA shared memory
!
!       real*8, shared :: Asub(igpusplt,igpusplt), 
!     &                   Bsub(igpusplt,igpusplt)
!
!! the value of C(i,j) being computed, a temporary scalar
!
!       real*8 :: Cij
!
!! Start execution, first get my thread indices
!
!       tx = threadidx%x
!       ty = threadidx%y
!
!! This thread computes C(i,j) = sum(A(i,:) * B(:,j))
!
!       i = (blockidx%x-1) * igpusplt + tx
!       j = (blockidx%y-1) * igpusplt + ty
!
!       Cij = 0.0d0
!
!
!! Do the k loop in chunks of igpusplt (16), the block size
!
!       do kb = 1, M, igpusplt 
!
!! Fill the submatrices; each of igpusplt x igpusplt threads in the thread block
!! loads one element of Asub and Bsub
!
!          Asub(tx,ty) = A(i,kb+ty-1)
!          Bsub(tx,ty) = B(kb+tx-1,j)
!
!! Wait until all elements are filled
!
!          call syncthreads()
!
!! Multiply the two submatrices; ! Each of the igpusplt x igpusplt threads accumulates the
!! dot product for its element of C(i,j)
!
!!          if(j.le.i)then
!          do k = 1,igpusplt
!             Cij = Cij + Asub(tx,k) * Bsub(k,ty)
!          enddo
!!          endif
!
!! Synchronize to make sure all threads are done reading the submatrices before 
!! overwriting them in the next iteration of the kb loop
!
!          call syncthreads()
!
!       enddo
!
!! Each of the igpusplt x igpusplt threads stores its element to the global C array
!
!        C(i,j) = Cij
!
!       end subroutine mmul_kernel
!
!! The host routine to drive the matrix multiplication
!
!       subroutine mmul( A, B, C )
!
!! assumed shape input arrays
!
!       real*8, allocatable :: A(:,:), B(:,:), C(:,:)
!
!! Array dimensions
!
!       integer :: N, M, L
!
!! allocatable device arrays
!
!       real*8, device, allocatable, dimension(:,:) :: Adev,Bdev,Cdev
!
!! dim3 variables to define the grid and block shapes
!
!       type(dim3) :: dimGrid, dimBlock
!       integer :: r
!
!! Get the array sizes
!
!!       real*8 ctimeall, ctimekernel, flops, mflopskernel, mflopsall
!!       integer c1, c2, c3, c4
!
!! Begin execution, first determine the sizes of the input arrays
!
!       N = size( A, 1 )
!       M = size( A, 2 )
!       L = size( B, 2 )
!
!
!! Start data xfer-inclusive timer and allocate the device arrays using 
!! F90 ALLOCATE
!
!!       call system_clock( count=c1 )
!       allocate( Adev(N,M), Bdev(M,L), Cdev(N,L) )
!
!! Copy A and B to the device using F90 array assignments
!
!      istat = cudaMemcpy2D(Adev(1,1),N, A(1,1), N,N,M)
!      if(istat.ne.0)write(0,*)'Failure to cpy A to Adev on device'
!      istat = cudaMemcpy2D(Bdev(1,1),M, B(1,1), M,M,L)
!      if(istat.ne.0)write(0,*)'Failure to cpy B to Bdev on device'
!
!! Create the grid and block dimensions
!
!       dimGrid = dim3( N/igpusplt, L/igpusplt, 1 )
!       dimBlock = dim3( igpusplt, igpusplt, 1 )
!
!! Start data xfer-exclusive timer, launch the GPU kernel, wait for completion
!
!      call mmul_kernel<<<dimGrid,dimBlock>>>( Adev,Bdev,Cdev,N,M,L)
!
!      deallocate(Adev,Bdev) 
!
!      istat = cudaMemcpy2D(C(1,1),N,Cdev(1,1),N,N,L)   
!      if(istat.ne.0)write(0,*)'Failure to cpy Cdev to C on host'
!
!! Create the grid and block dimensions
!      
!       r = cudathreadsynchronize()
!
!! Stop data xfer-exlusive timer, copy the results back, stop data xfer-
!! inclusive timer
!
!!       call system_clock( count=c3 )
!!       C(1:N,1:L) = Cdev
!!       call system_clock( count=c4 )
!
!! Calculate inclusive/exclusive execution times, and report MFLOPS
!
!!       flops = float(N) * float(M) * float(L)
!!       ctimekernel = c3 - c2
!!       mflopskernel = flops / ctimekernel
!!       ctimeall = c4 - c1
!!       mflopsall = flops / ctimeall
!
!!  Print out results
!
!!       print *, 'Kernel time excluding data xfer:', ctimekernel,
!!     X ' microseconds'
!!       print *, 'Megaflops excluding data xfer:  ', mflopskernel
!!       print *, 'Total time including data xfer: ', ctimeall,
!!     X  ' microseconds' 
!!       print *, 'Megaflops including data xfer:  ', mflopsall
!
!! Deallocate device arrays and exit
!
!       deallocate( Cdev )
!
!       end subroutine mmul
!       end module mmul_mod
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
        MODULE WMATALLOC
        REAL*8,ALLOCATABLE :: VALUE(:),CF(:,:,:),TTT(:,:,:)
        REAL*8,ALLOCATABLE,TARGET :: WMAT(:,:)
        REAL*8,POINTER :: WM1(:,:),WM2(:,:)
        INTEGER :: MXEONE=0,MAXMUL=2   ! PROBABLY RESET TO 2 BY INNER
        END MODULE WMATALLOC
!
        MODULE TERMSYM
        INTEGER,ALLOCATABLE :: MSLP(:),IWHD(:)   ! Dim=(kslp) 
        INTEGER :: KSLP,NHDAT
!       INTEGER :: IONEONE
        END MODULE TERMSYM
!
        MODULE NRBRCT
        REAL*8,ALLOCATABLE :: S(:),SP(:),C(:),CP(:),CS(:,:)
        REAL*8,ALLOCATABLE :: SPP(:),A(:,:),B(:,:),RK(:,:)
        REAL*8,ALLOCATABLE :: CSPP(:,:),PPP(:,:),QQQ(:,:)
!	
        REAL*8,ALLOCATABLE,TARGET :: CPP(:),CSP(:,:),CC(:,:),CCP(:,:)
        REAL*8,ALLOCATABLE,TARGET :: DSP(:,:),RMAT(:,:),DC(:,:),DCP(:,:)
        REAL*8,ALLOCATABLE,TARGET :: CCPP(:,:),DS(:,:),DFPP(:,:)
        
!
        REAL*8,POINTER :: P(:,:),Q(:,:),POLD(:,:),QOLD(:,:)
        REAL*8,POINTER :: P2(:,:),Q2(:,:),Y(:,:),D(:,:)
        REAL*8,POINTER :: TEMP1(:,:),TEMP2(:,:)
        REAL*8,POINTER :: TEMP3(:,:),TEMP4(:,:)
        END MODULE NRBRCT
!
        MODULE NRBWRK
        REAL*8, ALLOCATABLE,TARGET :: WORK(:),WORKM(:,:),WORKM2(:,:)
        REAL*8, POINTER :: TEMP44(:,:)
        COMPLEX*16,ALLOCATABLE :: ZWORK(:)
        END MODULE NRBWRK
!
        MODULE NRBCBE
        REAL*8,ALLOCATABLE,TARGET :: RBE(:,:)
        REAL*8,POINTER :: TEMP33(:,:)
        INTEGER :: LCBE
        END MODULE NRBCBE

        MODULE NRBKHI
        COMPLEX*16,ALLOCATABLE :: ZKHICC(:,:),ZVAL(:)
        COMPLEX*16,ALLOCATABLE,TARGET :: ZKHIOC(:,:)
        COMPLEX*16,POINTER :: ZKHICO(:,:)
        END MODULE NRBKHI
!
        MODULE CALP
        REAL*8,ALLOCATABLE :: ASS(:,:),ASC(:,:),ACS(:,:),ACC(:,:)
        END MODULE CALP
!
        MODULE COULSC
        REAL*8,ALLOCATABLE :: FS(:,:),FSP(:),FC(:,:),FCP(:)
        END MODULE COULSC
!
        MODULE CPOT
        REAL*8,ALLOCATABLE :: BW(:,:)
        INTEGER,ALLOCATABLE :: LAMP(:,:)
        END MODULE CPOT 

        MODULE MEMORY 
        REAL*8,ALLOCATABLE :: OMEM(:)
        INTEGER,ALLOCATABLE :: MPOS(:)
        INTEGER :: ITMAX,JTMAX,MZMEG
        END MODULE MEMORY
!	
        MODULE CTOP
        REAL*8,ALLOCATABLE :: TOPA(:),TOPB(:),FTOPA(:,:),FTOPB(:,:)
        INTEGER,ALLOCATABLE :: LITLAM(:),NTOP(:,:),NTCHAN(:,:)
        INTEGER,ALLOCATABLE :: NTOPA(:,:),NTOPB(:,:),MTOPA(:,:),
     &                         MTOPB(:,:)
        INTEGER,ALLOCATABLE :: KTOPA(:),KTOPB(:)
        INTEGER :: LRGLAM,INDM,LRGLMN
        END MODULE CTOP
!
        MODULE PART
        REAL*8 :: TRACE
        INTEGER :: NRANG2,IPRCENT
        REAL*8,ALLOCATABLE :: EIGENS(:,:),ENDS(:,:),SI(:)
        INTEGER,ALLOCATABLE :: NRANG1(:)
        END MODULE PART
!
        MODULE NRBTOP
        INTEGER :: ITOP
        INTEGER,ALLOCATABLE :: ITST(:),JTST(:),KTST(:,:)
        REAL*8,ALLOCATABLE :: OMST(:)  
        END MODULE NRBTOP	

        MODULE DBUT 
        REAL*8,ALLOCATABLE :: EBUTD(:,:),CBUTD(:,:)
        INTEGER,ALLOCATABLE :: NBUTD(:),K2P(:)
        END MODULE DBUT	

        MODULE CBODE
        REAL*8,ALLOCATABLE :: WBODE(:),TBODE(:,:)
        END MODULE CBODE
!
        MODULE NRBDR
        REAL*8, ALLOCATABLE :: OMEGDR(:,:)
        INTEGER, ALLOCATABLE :: PDR(:)
        INTEGER :: NDRMET
        END MODULE NRBDR

        MODULE COMEGA
        REAL*8,ALLOCATABLE :: OMEGA(:)
        INTEGER :: IE,NOMWRT
        END MODULE COMEGA
!
        MODULE CEN
        INTEGER :: MXE,NWT,NZ
        REAL*8 :: ETOT
        END MODULE CEN
!
        MODULE CTHET
        REAL*8,ALLOCATABLE :: BB(:,:),BG(:,:)
        INTEGER,ALLOCATABLE :: MSUM(:)
        END MODULE CTHET

        MODULE CINPUT
        INTEGER :: LAMAX,LRANG2,LRGL2,MNP2,NAST,NCHAN
        INTEGER :: NELC,NPTY2,NSPN2,NZED,MORE2,KFLAG
        INTEGER,ALLOCATABLE :: ISAT(:),LAT(:),NCONAT(:),KJ(:),L2P(:)   
        END MODULE CINPUT
!
        MODULE CINPTX
        REAL*8 :: BSTO,RA
        REAL*8,ALLOCATABLE :: COEFF(:,:),ENAT(:),ENATK(:)
        END MODULE CINPTX
!
        MODULE CHAN
        INTEGER NCHF,NCHPP,NCHPP1
        INTEGER,ALLOCATABLE :: ITARG(:),LLCH(:)       ! DIM=(MZCHF)=(NCHAN)
        REAL*8,ALLOCATABLE :: ECH(:),EPS(:),FKNU(:),CCT(:),RINF(:)   ! DIM=(MZCHF)=(NCHAN)
        END MODULE CHAN
!
        MODULE CDEGEN
        REAL*8,ALLOCATABLE :: ENATR(:)
        INTEGER,ALLOCATABLE :: NLEV(:),NCNATR(:),IWD(:)
        INTEGER :: NASTD,NASTR,IWT
        END MODULE CDEGEN
!
        MODULE CDEC
        REAL*8,ALLOCATABLE :: ARAD(:),ARDEC(:),SLIN(:)
        INTEGER,ALLOCATABLE :: IPAR(:)
        INTEGER :: IRDEC,IEND,NEWAR
        END MODULE CDEC
!
        MODULE NRBKUT
        INTEGER :: KUTPS,NCHOPT
        END MODULE NRBKUT
!
        MODULE NRBLMX
        INTEGER LMX
        END MODULE NRBLMX
!
        MODULE NRBHYB
        REAL*8 :: FNUHYB
        INTEGER :: NCHCL,NCHHYB
        INTEGER,ALLOCATABLE :: ICHCL(:),ICHHYB(:)
        END MODULE NRBHYB
!
        MODULE NRBQDT
        REAL*8 :: RTWOC
        INTEGER :: KP2C,IQDT,NCUTOFF,IMODE,INTPQ,IJBIN
        INTEGER,ALLOCATABLE :: IEE(:)
        END MODULE NRBQDT
!
        MODULE NRBSKP 
        INTEGER :: ISKP0,LINC
        INTEGER,ALLOCATABLE :: ISKP(:)
        CHARACTER*3 :: ELAS
        END MODULE NRBSKP

        MODULE NRBOMT
        REAL*8 :: FNUMIN
        INTEGER :: IOMSW,IFLAG,IFLEG,ICCINT,ICHAN
        INTEGER,ALLOCATABLE :: IOMIT(:)
        END MODULE NRBOMT
!
        MODULE CACC
        REAL*8 :: AC,ACNUM,ACJWBK,ACZP
        INTEGER :: LACC
        END MODULE CACC
!
        MODULE CPOINT
        REAL*8 :: RZERO,RONE,RTWO,H
        INTEGER :: KP0,KP1,KP2,MZPTS
        END MODULE CPOINT
!
        MODULE CNTRLS
        INTEGER :: ISGPT,ITRMN,ITRMX
        END MODULE CNTRLS
!
        MODULE CNTRL 
        INTEGER :: IPRINT,IPRKM,IWORD,IRAD,IPERT,NPERT,IPRTSW
        END MODULE CNTRL
!
        MODULE CQDT
        REAL*8,ALLOCATABLE :: R2ST(:)
        LOGICAL :: QDT
        INTEGER :: NQ
        END MODULE CQDT
!
        MODULE CMESH
        REAL*8 :: EMAX,EMIN,DEOPEN,DQN,QNMAX
        REAL*8,ALLOCATABLE :: EMESH(:)
        INTEGER :: IMESH
        END MODULE CMESH
!
        MODULE CPBSAVE
        REAL*8,ALLOCATABLE :: P1(:,:),Q1(:,:)
        INTEGER,ALLOCATABLE :: IOMIT1(:),IOMIT2(:)
        END MODULE CPBSAVE
!
        MODULE NRBDD2
        INTEGER :: IFDD2
        REAL*8,ALLOCATABLE :: FSP2(:),FCP2(:)
        END MODULE NRBDD2
!
        MODULE CJWBK
        REAL*8,ALLOCATABLE :: DDDD(:)
        END MODULE CJWBK
!
        MODULE NRBZED
        REAL*8 :: TZED
        INTEGER :: LPRTSW
        END MODULE NRBZED
!
        MODULE CWARN
        LOGICAL :: WARN
        END MODULE CWARN
!
        MODULE CBUT
        REAL*8,ALLOCATABLE :: FKN(:),UKN(:)       ! DIM=(0:NBUT)
        END MODULE CBUT
!
        MODULE CENAT1
        REAL*8 ENAT1
        END MODULE CENAT1
!
        MODULE CLOGB
        LOGICAL NEWBUT
        END MODULE CLOGB
!
        MODULE CPTOLD
        INTEGER KP2O
        REAL*8 RTWOO
        END MODULE CPTOLD
!
        MODULE CBLK
        REAL*8 :: XLAG(1:30) = [ 0.58578644,3.4142136,
     4  0.32254769,1.7457611,4.5366203,9.3950709,
     6  0.22284660,1.1889321,2.9927363,5.7751436,9.8374674,
     6  15.982874,
     8  0.17027963,0.90370178,2.2510866,4.26670017,7.0459054,
     8  10.758516,15.7406786,22.8631317,
     A  0.13779347,0.72945455,1.8083429,3.4014337,5.5524961,
     A  8.3301527,11.8437858,16.279258,21.996586,29.920697 ]
        REAL*8 :: WLAG(1:30) = [ 0.85355339,0.14644661,
     4  0.60315410,0.35741869,0.38887909E-1,0.53929471E-3,
     6  0.45896467,0.41700083,0.11337338,0.10399197E-1,
     6  0.26101720E-3,0.89854791E-6,
     8  0.36918859,0.41878678,0.17579499,3.3343492E-2,2.7945362E-3,
     8  9.0765088E-5,8.4857467E-7,1.0480012E-9,
     A  0.30844112,0.40111993,0.21806829,6.2087456E-2,9.5015170E-3,
     A  7.5300839E-4,2.8259233E-5,4.2493140E-7,1.8395648E-9,
     A  9.9118272E-13 ]
        REAL*8 :: XLEG(1:15) = [ 0.577350269,
     2  0.339981044,0.861136312,
     3  0.238619186,0.661209386,0.932469514,
     4  0.183434642,0.525532410,0.796666477,0.960289856,
     5  0.148874339,0.433395394,0.679409568,0.865063367,0.973906529 ]
        REAL*8 :: WLEG(1:15) = [ 1.,
     2  0.652145159,0.347854845,
     3  0.467913935,0.360761573,0.171324492,
     4  0.362683783,0.313706646,0.222381034,0.101228536,
     5  0.295524225,0.269266719,0.219086363,0.149451349,0.066671344 ]
        END MODULE CBLK
!
!***************************************************************
!
      PROGRAM MAIN
      Use comm_interface
      USE WMATALLOC
      USE TERMSYM
      USE NRBCBE
      USE MEMORY
      USE CTOP
      USE NRBKHI
      USE PART
      USE NRBTOP
      USE DBUT
      USE CBODE
      USE NRBDR
      USE COMEGA
      USE CEN
      USE CTHET
      USE CINPUT
      USE CINPTX
      USE CHAN
      USE CDEGEN
      USE CDEC
      USE NRBKUT 
      USE NRBLMX
      USE NRBHYB
      USE NRBQDT
      USE NRBSKP
      USE NRBOMT
      USE CACC
      USE CPOINT
      USE CNTRLS
      USE CNTRL
      USE CQDT
      USE CALP
      USE NRBRCT, ONLY : A,B,RK,CSPP,RMAT,CSP,DSP,CCPP,DS,DFPP,
     X                   CS,CCP,DCP,CPP,SPP
      USE COULSC
      USE CMESH
      USE NRBZED
      USE CWARN
!
      IMPLICIT REAL*8 (A-H,O-Z)
      PARAMETER (MZKIL=  0)
!
!      PARAMETER (MXTST=(MZTAR*MZTAR+MZTAR)/2)
!      PARAMETER (MXOM=MZMEG*1000000+MZKIL*1000)
!      PARAMETER (MXF=(MZLMX+1)/2)
!
      PARAMETER (ZERO=0.0)
      PARAMETER (ONE=1.0)
      PARAMETER (THREE=3.0)
      PARAMETER (FOUR=4.0)
      PARAMETER (EINF=1.0D6)
!
      LOGICAL WARNE,QJUMP,PQRD,BFORM,EX,EX2,LFAIL
      CHARACTER*1 FILEC,FILED,FILEU,FILEV
      CHARACTER NAME*3,PERT*3,NAMKLS*7,PRINT*6
      LOGICAL,ALLOCATABLE :: LFARR(:),LFTOT(:)
      INTEGER,ALLOCATABLE :: IE0(:)
      INTEGER,ALLOCATABLE :: IAUX1(:),IAUX2(:)
      REAL*8,ALLOCATABLE :: OMEGATOT(:)
      REAL*8,ALLOCATABLE :: OMEGDRTOT(:,:)
      CHARACTER(LEN=1) :: NUM(0:9)=
     &   [ "0","1","2","3","4","5","6","7","8","9" ]
!
      NAMELIST/STGF/IPRINT,IRAD,IPERT,AC,RONE,IMESH,IOPT1,IRDEC,LRGLAM,
     1 IPRKM,PERT,MINLT,MAXLT,ISKP0,LINC,NASTD,LCBE,ELAS,NOMWRT,IBETA,
     2 IBIGE,LMX,KFLAG,IQDT,NDRMET,QETEST,NEWAR,IPRTSW,IOMSW,NCUTOFF,IEQ
     3,IWARN,FNUMIN,ICCINT,INTPQ,ISGPT,ITRMN,ITRMX,FNUHYB,LPRTSW,ITOP
     4,NTYP1,PRINT,IMODE,IDIP,IJBIN,KUTPS,MZMEG,MZPTS
!
      NAMELIST/MESH1/MXE,E0,EINCR,QNMAX,ABVTHR,BELTHR,NCINT,NSEQ,IEQ
      NAMELIST/MESH2/DQN,QNMAX,EMIN,EMAX,DEOPEN
      NAMELIST/MESH3/MXE
!
!
!     **** PARALLEL ****
      CALL MPI_INIT(IERR)
      CALL MPI_COMM_RANK(MPI_COMM_WORLD,IAM,IERR)
      CALL MPI_COMM_SIZE(MPI_COMM_WORLD,NPROC,IERR)
      ISTOP=0
!     **** END PARALLEL ****
      CALL CPU_TIME(TIMEI)
!      WRITE(0,*)'   BEGIN       PROC=:',IAM
!
      WARN=.TRUE.
      WARNE=.TRUE.
      QJUMP=.FALSE.
      IWARN=999
!
!  CALL BLOCK DATA AS SUBROUTINE TO AVOID LINKAGE PROBLEMS
!
!SUB  CALL BLOCK
!
!  INITIALISE INDM
!
      INDM=0
!
!  I/O UNITS
!  *********
!
!          1 FOR OMEGA SCRATCH FILE (USED FOR IRAD.LT.2).
!          2 RESERVED
!          3 RESERVED
!          4 RESERVED
!          5 FOR PROGRAM CONTROL
!          6 FOR PRINTED OUTPUT
!          7 FOR VALUES OF OMEGA (USED FOR IRAD.LT.2). FILE OMEGA.
!          8 FOR DIPOLE DATA INDEPENDENT OF SLPI. FILE F00.
!          9 FOR DIPOLE DATA DEPENDENT ON SLPI. FILES F01 ETC.
!         10 FOR INPUT R-MATRIX DATA. FILE H.DAT.
!         11 RESERVED
!         12 FOR INPUT SYMMETRIES. FILE sizeH.dat OR sizeBP.dat 
!         14 FOR PARTIAL CROSS SECTIONS. FILE SIGPW.DAT
!         17 FOR TERM INFORMATION FOR INPUT TO STGIC. FILE TERM.DAT.
!         18 FOR VALUES OF DR OMEGA (USED FOR NDRMET.GT.0). FILE OMEGDR.
!         19 FOR CHANNEL INFO FOR INPUT TO STGIC. FILE JBINLS.
!         20 FOR REACTANCE MATRICES FOR DIFF XSCN. FILE KMAT.DAT.
!         21 FOR KHI-MATRIX (=1-T) OUTPUT,MQDT OR JJOM FORMAT. FILE JBIN.
!         22 FOR CASE DESCRIPTION AS REQUIRED BY JJOM. FILE JJDAT.
!         23 RESERVED
!         25 RESERVED
!         27 RESERVED
!         28 RESERVED
!         29 RESERVED
!         30 RESERVED
!         31 FOR LINE STRENGTHS FOR INPUT TO STGICF. FILE STRENGTH.DAT.
!         32 FOR UNPHYSICAL K/S-MATRIX FOR STGICF. FILE K/SMTLS.001,002...
!         33 RESERVED
!         34 FOR EIGEN PHASE SUM. FILE ESUM.
!         36 FOR UNPHYSICAL K-MATRIX. FILE KMATLS.
!         37 RESERVED
!         38 RESERVED
!         40 TIME CONTROL PROCESSOR, ONLY DEBUGGING OR PROFILING
!
!
!       **** PARALLEL ****
        I1=IAM/1000
        I2=(IAM-1000*(IAM/1000))/100
        I3=(IAM-(1000*(IAM/1000))-I2*100)/10
        I4=(IAM-(1000*(IAM/1000))-I2*100)-I3*10
        FILEC = NUM(I1)
        FILED = NUM(I2) 
        FILEU = NUM(I3)
        FILEV = NUM(I4)
!
!     **** PARALLEL ****
      IF (IAM.EQ.0) THEN
       OPEN(6,FILE='routf',STATUS='replace')
      ELSE
       OPEN(6,FILE='routfg'//FILEC//FILED//FILEU//FILEV,
     &      STATUS='replace')
      ENDIF
!     **** END PARALLEL ****
      OPEN(5,FILE='dstgf',STATUS='old',ACTION='read')        
!
! Get the nhdat, number of H.DATXXX files to be read
!     **** SINGLE PROC ****
      If (iam.eq.0) Then
       Inquire (File="H.DAT000", Exist=ex2)
       If (ex2) Then     ! Multiple H.DATXXX files
         Open (Unit=10,FILE="H.DAT000",FORM="UNFORMATTED",STATUS="OLD")
         nhdat=1
         Do While (ex2)
           k1=nhdat/100
           k2=(nhdat-100*k1)/10
           k3=nhdat-100*k1-10*k2
           Inquire (File="H.DAT"//NUM(k1)//NUM(k2)//NUM(k3), 
     &              Exist=ex2)
           If(ex2) nhdat=nhdat+1
         End Do
!         Write(0,'(" Found ",I3," H.DATXXX files")') nhdat
         iword=1
       Else
         Inquire (File="H.DAT", Exist=ex)
         If (ex) Then    ! One single H.DAT file
           nhdat=-1
           iword=1
           Open (Unit=10,FILE="H.DAT",FORM="UNFORMATTED",STATUS="OLD")
         Else
           Inquire (File="DSTGH.DAT", Exist=ex)
           If (ex) Then  ! One single DSTGH.DAT file
             nhdat=-1
             iword=2
             Open (Unit=10,FILE="DSTGH.DAT",FORM="UNFORMATTED",
     &             STATUS="OLD")
           Else             ! No suitable file found
             WRITE(6,*) "***ERROR: NO SUITABLE H.DAT FILE FOUND!"
             STOP "***ERROR: NO SUITABLE H.DAT FILE FOUND!"
           End If
         End If
       End If
      End If
!     **** BROADCAST ****
      Call MPI_Bcast(nhdat,1,mpi_int,0,MPI_COMM_WORLD,ierr)
      Call MPI_Bcast(iword,1,mpi_int,0,MPI_COMM_WORLD,ierr)
      If (iam.ne.0) Then
       If (nhdat.ge.1) Then     ! Multiple H.DATXXX files
         Open (Unit=10,FILE="H.DAT000",FORM="UNFORMATTED",STATUS="OLD")
       Else If (nhdat.eq.-1 .and. iword.eq.1) Then  ! One single H.DAT file
         Open (Unit=10,FILE="H.DAT",FORM="UNFORMATTED",STATUS="OLD")
       Else If (nhdat.eq.-1 .and. iword.eq.2) Then  ! One single DSTGH.DAT file
         Open (Unit=10,FILE="DSTGH.DAT",FORM="UNFORMATTED",STATUS="OLD")
       End If
      End If
!     **** END SINGLE PROC ****
!
!  WRITE NEWS
!  **********
!
      WRITE(6,6000)
      WRITE(6,6003)
      WRITE(6,6005)
      WRITE(6,6001)
! LFM 2018 - NO LONGER COMPILING DIMENSIONS
!      WRITE(6,6002)MZCHF,MZTAR,MZLMX,MZLP1,MZPTS,MZMSH,MZMNP,
!     X             MZSLP,MZTET,MZDEG,MZNRG,MZMET,MZREC,MZMEG
!
!
!  READ DATA FROM UNIT 5
!  *********************
!
! MQDT VARIABLES:
!       DEFAULTS: OFF - NON-MQDT OPERATION.
!
!     IMODE --  CONTROLS READ/WRITE OF UNPHYSICAL MATRIX IN MQDT OPERATION.
!           = 0 WRITE THE UNPHYSICAL MATRIX TO FILE (JBIN) - DEFAULT.
!               THIS WOULD NORMALLY BE FOR A COARSE ENERGY MESH.  
!     **** PARALLEL ****
!               NOTE THAT ONLY IMODE<=0 IS IMPLEMENTED IN PARALLEL CODE.
!     **** PARALLEL ****
!           = 1 READ THE UNPHYSICAL MATRIX FROM FILE (JBIN).
!               SOLUTION ON A (NEWLY DEFINED) FINE ENERGY MESH OBTAINED SOLELY
!               BY INTERPOLATION OF THE PREVIOUS COARSE MESH DATA.
!           =-1 SINGLE PASS OPERATION. FULL SOLUTION ON A COARSE MESH.
!               INTERPOLATIVE SOLUTION ON A FINE MESH - SEE IEQ (NO JBIN).
!     IJBIN = 0 DEFAULT, DO NOT WRITE JBIN FOR IMODE=0.
!     IQDT  = 0 NO MQDT EXCEPT GAILITIS VIA N.GT.QNMAX (I.E. QDT=.TRUE.) DEFAULT
!     IQDT  =-1 PARALLEL QDT=.TRUE. OPERATION BUT **NOT** AVERAGED, SO DAMPING
!               STILL RESTRICTED TO RESONANCES CONVERGING TO LOWEST THRESHOLD.
!     IQDT  = 1 FULL MQDT, ALL CHANNELS TREATED AS OPEN, USES UNPHYSICAL K/S-MX. 
!               NOTE IQDT.GT.0 WILL INCLUDE DIPOLE PERTURBING POTENTIALS IF
!               I/PERT IS SET APPROPRIATELY. SEE ALSO IMODE.
!           = 2  WORK WITH UNPHYSICAL K-MX RATHER THAN S-MX.
!     IEQ   --  CONTROLS HOW OFTEN THE UNPHYSICAL MATRIX IS UPDATED. 
!           > 0 THEN UPDATED AT IEQ LINEARLY SPACED ENERGIES ACROSS THE TOTAL
!               ENERGY RANGE DEFINED BY THE INPUT ENERGY MESH. SERIAL (IMODE=-1)
!           < 0 THEN UPDATED AT EVERY |IEQ|'TH POINT OF THE MESH, FINE FOR
!               CONSTANT STEP IN ENERGY, NOT SO GOOD (INEFFICIENT) FOR CONSTANT
!               STEP IN EFFECTIVE QUANTUM NUMBER. PARALLEL TAKES -|IEQ|.
!               DEFAULT = -1, THE K/S-MATRIX IS UPDATED AT EVERY ENERGY SO A
!               COARSE MESH SHOULD BE USED (IMODE=0). THEN RE-RUN WITH A FINE  
!               ENERGY MESH (IMODE=1).
!     QETEST = 1.E-7 SCALED RYDBERGS (DEFAULT). THE K/S-MX IS ONLY RE-INTERPOLATED
!              WHEN ETOT HAS CHANGED BY MORE THAN QETEST SINCE THE LAST TIME.
!              GIVES A SMALL TIME SAVING WHEN USING A VERY FINE MESH E.G. 1.E-8.
!              SHOULD NOT NEED TO BE CHANGED. SHOULD BE SMALLER THAN COARSE STEP
!     FNUMIN= THE EFFECTIVE QUANTUM NUMBER BELOW WHICH THE CLOSED CHANNEL IS
!             OMITTED, USED BY IQDT=1,2 - DEFAULT=0.
!     FNUHYB= THE EFFECTIVE QUANTUM NUMBER BELOW WHICH THE CLOSED CHANNEL IS
!             THETA RATHER THAN S & C, USED BY IQDT=1,2 - DEFAULT=0.
!
! END MQDT SPECIFIC VARIABLES
!
! DAMPING VARIABLES: 
!          DEFAULTS: EXCITATION DAMPING ON, DR/RR OFF.
!
!     NTYP1 = 0 NO TYPE 1 (CORE) RADIATIVE DECAYS (DEFAULT).
!           > 0 INCLUDE TYPE 1 DECAY (AND SWITCHES ON IRDEC).
!     NDRMET= NUMBER OF INITIAL METASTABLE STATES FOR DR, DEFAULT=0.
!     NCUTOFF=MAX PRINCIPAL QUANTUM NUMBER FOR WHICH DR IS CALCULATED,
!             CASE OF IQDT.GT.0 ONLY. DEFAULT ALL (1000000).
!
! END DAMPING SPECIFIC VARIABLES
!
!
!     IPRINT = -2 FOR MINIMUM PRINT
!            = -1 OMEGAS PARTIAL IN BIG L AT EACH ENERGY
!            =  0 OMEGAS PARTIAL IN BIG L, AND TOTAL, AT EACH ENERGY
!            =  1 REACTANCE MX, OMEGAS PARTIAL IN SMALL AND BIG L.."..
!            =  2 DETAILS OF PERTURBING POTENTIALS.
!            =  3 FOR MAXIMUM PRINT (LARGE)
!     IRAD   = 0 FOR NO RADIATIVE DATA ON UNIT 9
!            = 1 FOR RADIATIVE DATA ON UNIT 9
!               (WARNING: NOT IMPLEMENTED IN PARALLEL CODE)
!            = 2 FOR RADIATIVE DATA ON UNIT 9 AND NO COLLISION STRENGTHS
!               (WARNING: NOT IMPLEMENTED IN PARALLEL CODE)
!     IPERT  = 0 FOR OMISSION OF LONG-RANGE MULTIPOLE POTENTIALS
!            = 1,2,3,4 FOR THEIR INCLUSION
!                 1 AND 2 OMIT LONG-RANGE POTENTIALS WHEN CLOSED-
!                       CHANNEL RTWO EXCEEDS MESH.
!                 3 AND 4 DO NOT, THEY JUST NEGLECT CONTRIBUTION FROM
!                       RTWO TO INFINITY INSTEAD.
!                 1 AND 3 PERTURB THE T-MATRIX,
!                 2 AND 4 PERTURB THE K-MATRIX (WHEN IPERT RESET
!                         NEGATIVE INTERNALLY).
!
!     AC     = ACCURACY REQUIRED (1.E-5 DEFAULT)
!     RONE   = DEBUG PARAMETER (NORMALLY TAKE RONE = 1.)
!     IMESH  = 1 FOR FIXED INCREMENT IN ENERGIES
!                FOLLOWED BY
!                MXE, E0, EINCR FOR NUMBER OF ENERGIES,
!                               FIRST ENERGY AND INCREMENT
!                OPTIONALLY SET QNMAX TO APPLY QDT (GAILITIS IN SQDT)
!**** PARALLEL
!  E0=STARTING ENERGY OF GRID
!  EINCR=ENERGY STEP
!  MXE=NUMBER OF ENERGIES
!  NSEQ=NUMBER OF SEQUENTIAL ENERGIES PER PROCESSOR
!  NCINT=NUMBER OF GROUPS OF NSEQ.
!
!  FOR *NON-INTERPOLATION* (DEFAULT) SET ONLY E0,EINCR,MXE.
!  THE NPROCE=MXE/NPROC ENERGIES PER PROCESSOR ARE THEN DISTRIBUTED 
!  LINEARLY ACROSS THE GRID WITH REDEFINED ENERGY STEP EINCR*NPROC.
!
!  FOR *INTERPOLATION* SET ONE OR BOTH OF NSEQ, NCINT POSITIVE, TOGETHER
!  WITH E0,EINCR,MXE, WHERE NPROCE=NSEQ*NCINT. SO, NOW, EACH PROCESSOR
!  HAS NSEQ ENERGY POINTS FOR INTERPOLATION TO TAKE PLACE ON AND THIS
!  IS REPEATED NCINT TIMES ACROSS THE ENTIRE ENERGY RANGE. BOUNDARY
!  ISSUES ARE TAKEN ACCOUNT OF INTERNALLY WITHOUT USER INTERVENTION.
!**** PARALLEL
!
!     IMESH  = 2 FOR FIXED INCREMENT IN EFFECTIVE QUANTUM
!                NUMBERS FOLLOWED BY
!                DQN, QNMAX, EMIN, EMAX, DEOPEN
!                DEFINED IN SUBROUTINE  MESH
!     ********** SETS IPERT=0 FOR NU .GT. QNMAX (I.E. QDT USED)
!
!     IMESH = 3 TO READ ENERGY MESH, FOLLOWED BY
!               MXE
!               (EMESH(M),M=1,MXE)
!     IMESH = -S TO CHOOSE MESH APROPRIATE FOR CASES WITH TOTAL
!                SPIN 2S+1
!                FOLLOWED BY DATA AS FOR IMESH=2
!     IOPT1 = 1 FOR ALL SLPI CASES
!           = 2 FOR SELECTED SLPI CASES FOLLOWED BY
!               IS, IL, IP FOR CASES SELECTED TERMINATING WITH
!               -1 -1 -1
!           = -1 OR -2 AS ABOVE FOR + CASES BUT FIRST FOLLOWED BY
!             NASTD,(NLEV(N),N=1,NASTD)
!           = 10    FOR JAJOM, LS OMEGA WRITTEN
!                   (WARNING NOT IMPLEMENTED IN PARALLEL CODE)
!           = 11    FOR JAJOM, NO OMEGA WRITTEN.
!                   (WARNING NOT IMPLEMENTED IN PARALLEL CODE)
!     NASTD = NUMBER OF GROUPS (CAN BE READ DIRECTLY)
!     IWT     DEFINES ENERGY WEIGHTING USED
!           = 0  MEAN
!           = 1  TERM  (2S+1)*(2L+1)
!           =-1 LEVEL (2J+1)
!     IRDEC = 0 FOR NO RADIATIVE DECAYS (DEFAULT)
!           > 0 RADIATIVE DECAYS FOR EXCITATION (OPTIONAL) AND DR
!               (REQUIRED FOR NON-ZERO DR).
!           = 1 BELL AND SEATON
!           = 2 HICKMAN-ROBICHEAUX (MQDT ONLY, REVERTS TO =1 ELSE).
!     MINLT,MAXLT : IOPT1 IS SUBJECT TO SATSIFYING LRGL2.GE.MINLT AND
!                 LRGL2.LE.MAXLT  (LRGL2 = L OR 2*J). SEE ALSO LRGLAM.
!     LRGLAM=    VALUE OF LRGL2 (TOTAL L, OR 2*J) TO APPLY TOP-UP AT.
!                SET .LT. 0 TO TURN-OFF TOP-UP (DEFAULT).
!                (SO TOP-UP IS NO LONGER CONTROLLED BY IPERT).
!                IN LS COUPLING, THIS IMPLIES MAXLT=LRGLAM SINCE WE HAVE
!                TOP-UP IN SMALL L AND LARGE L (MAXLT IS RESET IF NECESS). 
!                IN B.P. MODE WE TOP-UP ONLY IN SMALL L SO HIGHER LRGL2'S
!                CAN BE PRESENT AND TOP-UP IN K/J IS TAKEN FROM THEM.
!                (IT IS NOT NECESS FOR MAXLT TO BE RESET THEN.)
!                OPTIMALLY, SET LRGLAM=2*MAX(J)-2.
!                IF IOPT1=2, JUST SET NON-NEGATIVE TO TURN ON TOP-UP 
!                (PRECISE VALUE DETERMINED INTERNALLY FROM YOUR PARTIAL 
!                 WAVE LIST IS, IL, IP). THUS, NO J/K TOP-UP.
!                NOTE, IT IS NOW POSSIBLE TO TOP-UP WHILE NEGLECTING
!                THE LONG-RANGE MULTIPOLE POTENTIALS.
!     LCBE  = VALUE OF LRGL2 FOR WHICH CBE OMEGA'S ARE CALCULATED,
!             DIPOLE TOP-UP IS THEN DONE WITH THE CBE OMEGA'S.
!             DEFAULT=999 IS RESET TO LRGLAM, SO SET NEGATIVE TO USE
!             CC OMEGAS IN DIPOLE TOP-UP (NOT RECOMMENDED).
!     ITOP  = CONTROLS HOW NON-DIPOLE TOP-UP GOES OVER TO THE DEGENERATE
!             ENERGY CASE - SEE SR.TOP2 FOR DETAILS.
!             THE DEFAULT, =-1, SHOULD NOT NEED CHANGING.
!     ELAS  ='NO' ELASTIC TRANSITIONS IN OMEGA FILE
!     ELAS  ='YES' ELASTIC TRANSITIONS IN OMEGA FILE
!     IPRKM = 1 TO WRITE K-MATRIX ELEMENTS TO THE SEQUENTIAL FILE KMAT.DAT
!             FOR INPUT TO THE DIFFERENTIAL CROSS SECTION CODE
!             DIFLS (LS) OR DIFJK (BP)
!             (WARNING: IN PARALLEL CODE EACH PROCESSOR WILL WRITE ITS
!             OWN SET OF K-MATRICES FOR ITS SET OF ENERGIES.)
!           = 2 WRITE UNPHYSICAL K-MATRIX DATA TO KMATLS. (TOM) 
!             (WARNING: NOT IMPLEMENTED IN PARALLEL CODE.)
!           = 3 TO WRITE A SEQUENTIAL FILE PWFLST, CONTAINING A LIST OF
!             PARTIAL WAVES AND ENERGIES AND A DIRECT ACCESS FILE RMABS,
!             CONTAINING THE K-MATRIX ELEMENTS. THESE FILES ARE USED FOR
!             INPUT TO THE PROGRAM PCRSX, WHICH GENERATES PARTIAL CROSS
!             SECTIONS OVER A RANGE OF ANGLES, AS WELL AS TOTAL CROSS
!             SECTIONS. ***NO LONGER OPERATIONAL - NRB***
!           = 4 WRITE UNPHYSICAL K/S-MATRIX TO K/SMTLS.DAT SO THAT IT CAN BE
!               READ BY STGICF/DAMP FOR THE CALCULATION OF INTERMEDIATE-COUPLING
!               COLLISION STRENGTHS.
!           = 0 NO WRITES (DEFAULT).
!     IRD0 >= 100, FOR IPRKM=4, WRITE LS K/S-MATRIX FILES BY SYMMETRY VIZ.
!             K/SMTLS.001, K/SMTLS.002 ETC.
!           < 100, FOR IPRKM=4, WRITE A SINGLE LS K/S-MATRIX FILE K/SMTLS.DAT.
!             DEFAULT: 99
!             LFM - REMOVED, ALWAYS MODE .GE.100
!     IDIP  = 1 WRITE DIPOLE LINE STRENGTHS TO STRENGTH.DAT.
!           = 0 NO WRITES.        (DEFAULT: 0 IPRKM=0-3, 1 IPRKM=4)
!     ISKP0 = VALUE OF L/2J ABOVE WHICH OMEGA'S ARE FIRST TESTED TO SEE IF
!             HIGHER PARTIAL WAVES CAN BE NEGLECTED FOR A GIVEN ENERGY.
!             USEFUL FOR NX CODE FOR RANGING L=10-80 OVER E=~0 - 8 RY,
!             SAY, WHERE HIGH-L GIVES NEGLIGIBLE OMEGAS AT LOW-E, BUT
!             CAUSES NUMERICAL PROBLEMS NEVERTHELESS.
!     LINC  = VALUE BY WHICH L/2J IS INCREMENTED; DEFAULT=1/2 AS CASE OF
!             AUTOMATIC GENERATION IN STG2 OR RECUP. USED WITH ISKP0.
!     NOMWRT= NUMBER OF ELEMENTS OF THE OMEGA ARRAY THAT ARE WRITTEN TO
!             AND READ FROM DIRECT ACCESS, AND THEN WRITTEN TO FILE OMEGA
!             IF .EQ. 0 NO OMEGAS ARE WRITTEN.
!             IF .NE. 0, THEN RESET FROM DEFAULT VALUE OF NAST*(NAST+/-1)/2
!                .GT. 0, NOMWRT OMEGAS ARE WRITTEN ROW-WISE, INC ZEROES
!                            -- EDIT OMEG TO DROP ZEROES (DEFAULT, ALL)
!                .LT. 0, -NOMWRT OMEGAS BETWEEN OPEN STATES ARE WRITTEN,
!                                                           COLUMN-WISE.
!     IBETA > 0 THEN WRITE EXTRA STUFF TO STGBF FOR BETA PARAMETER.
!     IBIGE = 1 EVALUATE AN PRINT INFINITE ENERGY (ACTUALLY EINF*ZA**2 RY) 
!               OMEGA. DIPOLE ONLY SO NO LONGER RECOMMENDED.
!           = 0 DO NOT (DEFAULT).
!     LMX   = LARGEST PERTURBING MULTIPOLE INCLUDED WHEN IPERT=1/PERT=YES,
!             DEFAULT=2 I.E. DIPOLE AND QUADRUPOLE. NOTE, ONLY A SINGLE
!             MULTIPOLE POTENTIAL IS TREATED I.E. IF DIPOLE EXISTS THEN
!             THE OCTUPOLE IS NEGLECTED EVEN IF LMX=3.
!     ISGPT, ITRMN, ITRMX .NE. 0 THEN WRITE THE PARTIAL-WAVE CROSS SECTIONS
!            FROM TRANSITION ITRMN TO TRANSITION ITRMX AND ALSO THE SUM OF
!            THE THESE PARTIAL WAVE CROSS SECTIONS FOR EACH LSPI PARTIAL WAVE
!            TO THE FILE SIGPW.DAT
!     ABVTHR - DROP ENERGIES WITHIN ABVTHR (Z-SCALED RY) ABOVE A THRESHOLD
!     BELTHR - DROP ENERGIES WITHIN BELTHR (Z-SCALED RY) BELOW A THRESHOLD
!              APPLIES TO IMESH=1 ONLY. DEFAULT FOR IONS IS -1., -1. I.E.
!              NONE ARE DROPPED. DEFAULT FOR NEUTRALS IS 1.E-3, 1.E-3.
!              NOTE: BOTH MUST BE SET NON-NEGATIVE TO ACTIVATE EITHER ONE,
!              SO IF ONLY ONE IS TO BE RESTRICTIVE SET THE OTHER TO ZERO.
!     PRINT  = 'FORM' THEN THE OMEGA FILE IS FORMATTED (DEFAULT).
!              'UNFORM' THEN AN UNFORMATTED OMEGAU IS CREATED INSTEAD.
!     MZMEG  - MEGA-WORDS OF OMEM ARRAY TO BE STORED IN MEMORY.
!              IT OPENS SCRATCH FILE IF MXOM.GT.MZMEG*1E6
!              DEFAULT = HUGE(1)/1E6 (NEVER USE SCRATCH)
!              WARNING, IF THE NUMBER OF PROCESSORS IS HIGH, THE SCRATCH
!              FILES, ONE PER PROC, CAN EASILY FILL THE HARD DISK
!     MZPTS  - NUMBER OF OUTER-REGION DISCRETIZATION POINTS
!              DEFAULT = 3201
!
!
!****OPTIONS BELOW ARE FOR EXPERIENCED USERS ONLY:
!
!     NEWAR = 1 FORCES RECALCULATION OF ARDEC AT EVERY ENERGY KEEPING ONLY
!               FINAL STATES BELOW THE IONIZATION LIMIT (DEFAULT).
!           = 0 DOES NOT, AND SUMS OVER ALL ARAD - EXTREMELY DANGEROUS FOR DR
!               SINCE RADIATIVE TRANSITIONS TO AUTOIONIZING STATES ARE THEN
!               COUNTED AS RECOMBINED. THIS CAN LEAD TO A VERY LARGE OVER-
!               ESTIMATE OF THE DR CROSS SECTION. ***HOWEVER***, IF YOU ARE
!               INTERESTED IN (DAMPED) EXCITATION, THEN NEWAR=0 MIGHT BE
!               MORE APPROPRIATE.
!     IOMSW = 1 MQDT: OMIT CLOSED CHANNELS WITH N<L+0.1, DEFAULT.
!           = 0 MQDT: KEEP (WITH A=-A, IOMIT(ICHAN)=-1 IN SR.SC WHEN A<0).
!           =-1 HYBRID: USE THETA FOR NU<FNUHYB.AND.NU.LT.L ELSE IOMSW=0.
!     IPRTSW= 1 FORCE IPERT POSITIVE (DEFAULT ONLY FOR IRAD>0 I.E. STGBF I/O)
!           = 0 ALLOW NEGATIVE IPERT, DEFAULT EXCEPT ABOVE CASE.
!           =-1 FORCE NEGATIVE IPERT.
!     LPRTSW=   VALUE OF LRGL2 ABOVE WHICH NEGATIVE IPERT ALLOWED.
!               DEFAULT = -1 FOR IONS, I.E. ANY LRGL2 IN PRINCIPLE BUT THE
!               PRESENCE OF THE COULOMB POTENTIAL MEANS IN PRACTICE THAT
!               IPERT NEVER GOES NEGATIVE FOR THE LOWEST FEW L, EVEN WHEN E=0.
!               DEFAULT=5 FOR NEUTRALS, SINCE RINF SCALES AS 1/SQRT(E).
!     KFLAG = 0 REGENERATE K-QUANTUM NUMBER INTERNALLY, DO NOT ATTEMPT TO
!               READ IT FROM H.DAT (DEFAULT).
!           = 1 READ K-QUANTUM NUMBER FROM H.DAT, NOT WRITTEN BY STANDARD
!               INNER REGION CODES. SHOULD NOT BE NEEDED NOW.
!     ICCIN = 1 INCLUDE CLOSED-CLOSED PERTURBING INTEGRALS, MQDT & NON-MQDT
!               OPERATION - DEFAULT.
!           = 0 OMIT THEM.
!     INTPQ = 0 USE CORINT FOR CLOSED CHANNEL MQDT S,C INTEGRALS (DEFAULT).
!           = 1 USE THETA ETC TO GENERATE Q INTEGRALS.
!
!
! INITIALIZE NAMELIST
!
! MQDT SPECIFIC VARIABLES (DEFAULTS: OFF)
!
      IMODE=0
      IQDT=0
      IEQ=-1           !NEEDED FOR IMODE=-1 ONLY
      QETEST=1.E-7     !SHOULDN'T NEED CHANGING
      FNUMIN=0.0       !NOT REQUIRED FOR PRODUCTION WORK
      FNUHYB=-1.0      !NOT REQUIRED FOR PRODUCTION WORK
! END
!
! DAMPING SPECIFIC VARIABLES (DEFAULTS: OFF)
!
      NTYP1=0
      NDRMET=0
      NCUTOFF=1000000  !SHOULDN'T NEED CHANGING
! END
      IPRINT=-2
      IPRKM=0
      IRAD=0
      IPERT=0
      PERT='NO'
      AC=1.0E-5
      RONE=ONE
      IMESH=2
      IOPT1=1
      IRDEC=0
      LRGLAM=-1
      MINLT=-1
      MAXLT=1000
      ISKP0=-1
      LINC=1
      NASTD=0
      IWT=-1
      LCBE=999
      ELAS='  '
      NOMWRT=HUGE(1)
      IBETA=0
      IBIGE=0
      IDIP=-1
      LMX=-1
      ISGPT=0
      ITRMN=0
      ITRMX=0
      ABVTHR=-1.
      BELTHR=-1.
      ITOP=-1
      PRINT='FORM'
!      IRD0=99                      !DEFAULT FOR PARALLEL
      IJBIN=0
      KUTPS=99999
!      IONEONE=0
! PARAMETERS NOT REMOVED FROM OLD PARAM FILE, NOW STANDARD INPUT
      MZMEG=HUGE(1)/(2**23)  ! MODULE MEMORY
      MZPTS=3201             ! MODULE CPOINT
!
! DO NOT CHANGE NEXT 7 VARIABLES UNLESS YOU REALLY KNOW WHAT YOU ARE DOING.
      NEWAR=1
      IOMSW=1
      IPRTSW=0
      LPRTSW=-1
      KFLAG=0
      ICCINT=1
      INTPQ=0
!
      READ(5,STGF)
!
      IF (HUGE(1)/MZMEG.LT.2**20) THEN
        WRITE(0,'(//10X,"WARNING: MZMEG TOO LARGE, DIMENSION WILL"
     &             /10X,"OVERFLOW, RESET TO MZMEG= ",
     &             I5)') HUGE(1)/2**20
        MZMEG=HUGE(1)/2**20
      END IF
      KP2=4*((MZPTS-1)/4)+1
!     **** PARALLEL ****
      IF(IQDT.NE.0.AND.IMODE.GT.0) THEN
        STOP 'ONLY IMODE<=0 IMPLEMENTED IN MQDT MODE FOR PARALLEL CODE'
      ENDIF
      IF(IOPT1.EQ.10.OR.IOPT1.EQ.11) THEN 
        STOP 'JAJOM INPUT NOT IMPLEMENTED IN PARALLEL CODE'
      ENDIF
      IF (IRAD.GT.0) THEN
        WRITE(6,'(//10X,"RADIATION NOT IMPLEMENTED IN PRESENT CODE",
     &             /10X,"USE PSTGFDAMP INSTEAD")')
        STOP 'RADIATION NOT IMPLEMENTED IN UNDAMPED CODE'
      END IF
!     **** PARALLEL ****
!
!
      BFORM=PRINT.EQ.'FORM'.OR.PRINT.EQ.'form'
      IF(KFLAG.NE.0)KFLAG=1
!
      IF(NTYP1.LT.0)NTYP1=0
!
      IF(IQDT.LE.0)IOMSW=0
      IF(IQDT.EQ.0)INTPQ=0
      IF(IRAD.GT.0.OR.IOMSW.LT.0)INTPQ=0            !FOR NOW
      IF(IWARN.EQ.999)THEN
        IF(IQDT.LE.0)IWARN=0
        IF(IQDT.GT.0)IWARN=1
      ENDIF
      IF(IMODE.GE.0.OR.IQDT.LE.0)IEQ=-1
      IF(IEQ.EQ.0)IEQ=-1
      IF(IEQ.EQ.1)IEQ=2
      IQDT0=IQDT
      IQDT=IQDT/100
      IF(IQDT.EQ.0)IQDT=IQDT0
      IF(IMODE.EQ.0)IQDT0=100*IQDT                  !WRITE JBIN
      IF(IMODE.NE.0)IQDT0=IQDT                      !MAY READ JBIN
      IF(NTYP1.GT.0.AND.IRDEC.EQ.0)THEN
        IF(IQDT.LE.0)THEN
          IRDEC=1
        ELSE
          IRDEC=2
        ENDIF
      ENDIF  
      IF(IRDEC.GT.0.AND.IQDT.EQ.2)THEN              !STGF ONLY
        WRITE(6,*)' ***ERROR: NO RADIATION DAMPING WITH UNPHYSICAL K-MX'
     X, '. USE IQDT=1 OR RUN STGFDAMP INSTEAD'
        STOP 'RESET IQDT=1 FOR IRDEC .GT. 0'
      ENDIF
!
      IF(IPRKM.NE.0.AND.IMODE.LT.0)THEN
        WRITE(6,*)' *** ERROR: SET IPRKM=0 FOR INTERPOLATION, IMODE=-1'
        STOP ' *** ERROR: SET IPRKM=0 FOR INTERPOLATION, IMODE=-1'
      ENDIF
! LFM - OPTION IPRKM.EQ.2 NOT IMPLEMENTED IN PARALLEL CODE,
!       REPLACED BY IPRKM.EQ.4
!      IF(IPRKM.EQ.2.AND.IQDT.NE.2)IPRKM=0
      IF (IPRKM.EQ.2) THEN
        WRITE(6,'(//10X,"WARNING, OPTION IPRKM=2 NOT IMPLEMENTED IN"
     &             /10X,"PARALLEL CODE, DEACTIVATING PRINT OF"
     &             /10X,"UNPHYSICAL K MATRIXES"
     &             /10X,"OR USE IPRKM=4 TO PRINT THEM")')
        IPRKM=0
      END IF
      IF(ISKP0.LE.0)THEN
        IF(IPRKM.EQ.0.AND.IQDT.LE.0)THEN
          ISKP0=10
        ELSE
          ISKP0=15
        ENDIF
      ENDIF
!
      PQRD=IQDT0.GT.0.AND.IQDT0.LT.100.OR.ABS(IEQ).NE.1
      IF(QETEST.LE.ZERO)QETEST=1.E-10
      IF(IPRTSW.EQ.0.AND.IRAD.GT.0)IPRTSW=1
      IF(NDRMET.GT.0.AND.ELAS.NE.'YES')THEN
        WRITE(6,609)
        ELAS='YES'         !ENSURE ELASTIC PRESENT FOR DR
      ENDIF
      IONE=1
      IF(ELAS.EQ.'YES')IONE=0
      IF(LMX.LT.0)LMX=2
      IF(IBIGE.NE.0)IBIGE=1
      IF(MAXLT.LT.0)MAXLT=1000
      INSKP=1000
      IF(MINLT.GT.ISKP0)INSKP=MINLT+1
!OLD  IF(LRGLAM.GE.0.AND.MAXLT.NE.1000)LRGLAM=MAXLT  ! ALLOW BP J/K TOP
      LCBE=MIN(LCBE,LRGLAM-1)
      IF(LRGLAM.GE.0.AND.LCBE.LT.0)WRITE(6,698)
      IF(ITOP.NE.-2)ITOP=-1
!
!      IF(NDRMET.GT.MZMET)THEN
!        WRITE(6,*)' ***DIMENSION ERROR, NDRMET=',NDRMET,' >MZMET',MZMET
!        STOP
!      ENDIF
!
! ADJUST IPERT, NPERT=1 DO NOT SWITCH-OFF PERTURBATION.
!
      IF(PERT.EQ.'YES')THEN
        IF(NDRMET.LE.0)IPERT=4           !ALWAYS USE K-MX PERT
        IF(NDRMET.GT.0)IPERT=4
      ENDIF
      IF(IQDT.LT.0)IPERT=0                   !NO PERT HERE
!OLD      IF(IRAD.GT.0.AND.IPERT.GT.2)IPERT=IPERT-2
!OLD      IF(IQDT*IRDEC*IRAD.NE.0)IPERT=0        !NOT IN STGBF YET
!NEW: DO NOT SWITCH-OFF AS STGBF JUST DROPS THE RTWO TO INFINITY INTEGRAL,
!NEW  FOR NOW, BUT WILL AT SOME POINT DO THE LONG-RANGE INTEGRALS AS IN ALPHAQ.
!
      NPERT=0
      IF(IPERT.GT.2)THEN
        NPERT=1
        IPERT=IPERT-2
      ENDIF
      IPERTR=IPERT
!
! LFM - SOME OF THESE FILES COULD BE NOW MERGED FOR ALL ENERGIES WITH A 
! MPI_REDUCE AND HAVE ONLY ONE OF EACH TYPE
! WARNING, TO USE THESE FILES IN PSTGICF, BE SURE THAT THE ENERGY POINTS
! IN EACH PROCESSOR AND SYMMETRY ARE READ EXACTLY THE SAME WAY
! AS IN PSTGF
!
!  **** PARALLEL ****
      IF(ISGPT.NE.0)OPEN(14,FILE='SIGPW.DAT'//FILEC//FILED//FILEU//FILEV
     +                   ,STATUS='UNKNOWN')
      IF(IPRKM.EQ.1)OPEN(20,FILE='KMAT.DAT'//FILEC//FILED//FILEU//FILEV
     +                   ,STATUS='UNKNOWN')
      IF(IPRKM.EQ.4)THEN
        IF(IDIP.LT.0)IDIP=1
        OPEN(19,FILE='JBINLS'//FILEC//FILED//FILEU//FILEV,
     +       FORM='UNFORMATTED',STATUS='UNKNOWN')
!        IF(IRD0.LT.100)THEN
!          IF(IQDT.EQ.1)THEN
!            OPEN(32,FILE='SMTLS.DAT.'//FILEC//FILED//FILEU//FILEV
!     X          ,FORM='UNFORMATTED',STATUS='UNKNOWN')
!          ELSE
!            OPEN(32,FILE='KMTLS.DAT.'//FILEC//FILED//FILEU//FILEV
!     X          ,FORM='UNFORMATTED',STATUS='UNKNOWN')
!          ENDIF
!        ENDIF
      ENDIF
!  **** END PARALLEL ****
!  **** SINGLE PROC ****
!      IF (IAM.EQ.0) THEN
!        IF(ISGPT.NE.0) OPEN(14,FILE="SIGPW.DAT",STATUS="REPLACE")
!        IF(IPRKM.EQ.1) OPEN(20,FILE="KMAT.DAT",STATUS="REPLACE")
!        IF(IPRKM.EQ.4) THEN
!          IF(IDIP.LT.0) IDIP=1
!          OPEN(19,FILE="JBINLS",FORM="UNFORMATTED",STATUS="REPLACE")
!        END IF
!      END IF
!  **** END SINGLE PROC ****
!
!  SELECT TYPE OF ENERGY MESH
!
      QNMAX=-ONE
      NCINT=1                                        !PARALLEL
      IF(IMODE.LT.0.AND.IMESH.NE.1)THEN
        WRITE(6,*)'*** ERROR, IMODE=-1 ONLY IMPLEMENTED FOR IMESH=1'    
        STOP '*** ERROR, IMODE=-1 ONLY IMPLEMENTED FOR IMESH=1'
      ENDIF
!
      IF (IMESH.EQ.1) THEN
!
!  CASE OF IMESH = 1
!
!  E0=STARTING ENERGY OF GRID
!  EINCR=ENERGY STEP
!  MXE=NUMBER OF ENERGIES
!  NSEQ=NUMBER OF SEQUENTIAL ENERGIES PER PROCESSOR
!  NCINT=NUMBER OF GROUPS OF NSEQ.
!
!  FOR *NON-INTERPOLATION* (DEFAULT) SET ONLY E0,EINCR,MXE.
!  THE NPROCE=MXE/NPROC ENERGIES PER PROCESSOR ARE THEN DISTRIBUTED 
!  LINEARLY ACROSS THE GRID WITH REDEFINED ENERGY STEP EINCR*NPROC.
!
!  FOR *INTERPOLATION* SET ONE OR BOTH OF NSEQ, NCINT POSITIVE, TOGETHER
!  WITH E0,EINCR,MXE, WHERE NPROCE=NSEQ*NCINT. SO, NOW, EACH PROCESSOR
!  HAS NSEQ ENERGY POINTS FOR INTERPOLATION TO TAKE PLACE ON AND THIS
!  IS REPEATED NCINT TIMES ACROSS THE ENTIRE ENERGY RANGE. BOUNDARY
!  ISSUES ARE TAKEN ACCOUNT OF INTERNALLY WITHOUT USER INTERVENTION.
!
        MXE=1
        E0=ONE
        EINCR=ZERO
        NSEQ=-1
        NCINT=-1
!
        READ(5,MESH1)
!
!      ALTHOUGH ALLOCATED ALREADY, BETTER ALLOCATION NOW POSSIBLE
!
!       IF(ALLOCATED(EMESH)) DEALLOCATE(EMESH)
!       ALLOCATE(EMESH(MXE))
!
!       **** PARALLEL **** 
        IEQ=-IABS(IEQ)
        PQRD=PQRD.OR.IEQ.GT.1

        IF (MOD(MXE,NPROC).EQ.0) THEN 
          NPROCE = MXE/NPROC
        ELSE
          NPROCE = (MXE/NPROC)+1
          MXE=NPROCE*NPROC
          WRITE(6,*)
     X   'WARNING: ENERGY MESH EXTENDED BECAUSE MXE/NPROC .NE. INTEGER'
          IF (IAM.EQ.0) WRITE(*,*)
     X   'WARNING: ENERGY MESH EXTENDED BECAUSE MXE/NPROC .NE. INTEGER'
          WRITE(6,'(" NEW VALUE MXE = ",I6)') MXE
          IF (IAM.EQ.0) WRITE(*,'(" NEW VALUE MXE = ",I6)') MXE
        END IF
! CASE INTERPOLATION
        IF (NSEQ.GT.0) THEN
          NCINT=NPORCE/NSEQ
        ELSE IF (NCINT.GT.0) THEN
          IF (MOD(NPROCE,NCINT).EQ.0) THEN
            NSEQ=NPROCE/NCINT
          ELSE
            NSEQ=(NPROCE/NCINT)+1
            NPROCE=NCINT*NSEQ
            WRITE(6,*)
     X      'WARNING: ENERGY MESH EXTENDED BECAUSE NPROCE/NSEQ*NCINT ',
     X      '.NE. INTEGER'
            IF(IAM.EQ.0)WRITE(*,*)
     X      'WARNING: ENERGY MESH EXTENDED BECAUSE NPROCE/NSEQ*NCINT ',
     X      '.NE. INTEGER'
            WRITE(6,'(" NEW VALUE MXE = ",I6)') MXE
            IF (IAM.EQ.0) WRITE(*,'(" NEW VALUE MXE = ",I6)') MXE
          END IF
          MXE=NPROCE*NPROC
        ELSE
          NSEQ=NPROCE
          NCINT=1
        END IF
!
!        IF(NSEQ.GT.0)NCINT=NPROCE/NSEQ
!        IF(NCINT.GT.0)THEN
!          NSEQ=NPROCE/NCINT
!          IF(NCINT*NSEQ.NE.NPROCE)THEN
!            WRITE(6,*)
!     X      'WARNING: ENERGY MESH TRUNCATED BECAUSE NPROCE/NSEQ*NCINT ',
!     X      '.NE. INTEGER'
!            IF(IAM.EQ.0)WRITE(*,*)
!     X      'WARNING: ENERGY MESH TRUNCATED BECAUSE NPROCE/NSEQ*NCINT ',
!     X      '.NE. INTEGER'
!          ENDIF
!        ENDIF
!
!        IF(NCINT.GT.0)THEN
!          E0=E0+IAM*EINCR*NSEQ
!          NSEQ=NSEQ+1
!          MXE=NSEQ*NCINT
!        ELSE
!          E0=E0+IAM*EINCR
!          EINCR=EINCR*NPROC
!          NCINT=1
!          MXE=NPROCE
!          NSEQ=MXE
!        ENDIF
!
! SET THE ARRAY EMESH
!
        ALLOCATE(EMESH(MXE))
        DO NC=1,NCINT
         DO I=1,NSEQ
          DO J=1,NPROC
            K=NPROC*NSEQ*(NC-1)+NPROC*(I-1)+J
            E=E0+EINCR*DBLE(K-1)
            EMESH(K)=E
          END DO
         END DO
        END DO
!       *** END PARALLEL ****
!
!        IF(MXE.GT.MZMSH)THEN
!          WRITE(6,699)MXE
!          STOP
!        ENDIF
!
!        J=0
!        EINT=NPROC*(NSEQ-1)*EINCR
!        DO NC=1,NCINT
!          DO I=1,NSEQ
!            E=E0+(I-1)*EINCR
!            J=J+1
!            EMESH(J)=E
!          ENDDO
!         E0=E0+EINT
!        ENDDO
!        DO I=1,MXE
!          WRITE(*,*)IAM,I,EMESH(I)
!        ENDDO
!
      ELSE IF (IMESH.EQ.2.OR.IMESH.LT.0) THEN
!
!  CASE OF IMESH = 2 OR = -S
!
        DEOPEN=AC
!
        READ(5,MESH2)
!
      ELSE IF(IMESH.EQ.3) THEN
!
!  CASE OF IMESH = 3
!
        READ(5,MESH3)
!
!       **** PARALLEL **** 
        IF (MOD(MXE,NPROC).EQ.0) THEN 
          NPROCE = MXE/NPROC
          ALLOCATE(EMESH(MXE))
        ELSE
          NPROCE = (MXE/NPROC)+1
          ALLOCATE(EMESH(NPROC*NPROCE))
        END IF
!       **** END PARALLEL **** 
!
!  **** SINGLE PROC ****
        If (iam.eq.0) Then
         DO M=1,MXE
           READ(5,*) EMESH(M)
         ENDDO
        End If
        Call MPI_Bcast(EMESH,mxe,mpi_real8,0,MPI_COMM_WORLD,ierr)
!  **** END SINGLE PROC ****
!
        IF (NPROC*NPROCE.GT.MXE) THEN
! EXTRAPOLATION NEEDED
          DE=EMESH(MXE)-EMESH(MXE-1)
          DO M=MXE+1,NPROCE*NPROC
            EMESH(M)=EMESH(MXE)+DFLOAT(M-MXE)*DE
          END DO
          MXE=NPROCE*NPROC
          WRITE(6,'("WARNING: ENERGY MESH EXTRAPOLATED BECAUSE ",
     &              "MXE/NPROC .NE. INTEGER")')
          IF (IAM.EQ.0)
     &      WRITE(*,'("WARNING: ENERGY MESH EXTRAPOLATED BECAUSE ",
     &                "MXE/NPROC .NE. INTEGER")')
          WRITE(6,'(" NEW VALUE MXE = ",I6)') MXE
          IF (IAM.EQ.0) WRITE(*,'(" NEW VALUE MXE = ",I6)') MXE
        END IF
!
!        NPROCE = MXE/NPROC
!        IF(NPROCE*NPROC.NE.MXE)THEN
!          WRITE(6,*)
!     X   'WARNING: ENERGY MESH TRUNCATED BECAUSE MXE/NPROC .NE. INTEGER'
!          IF(IAM.EQ.0)WRITE(*,*)
!     X   'WARNING: ENERGY MESH TRUNCATED BECAUSE MXE/NPROC .NE. INTEGER'
!        ENDIF
!        MXE=NPROCE
!        M0=IAM+1-NPROC
!        DO M=1,MXE
!          EMESH(M)=EMESH(M0+NPROC*M)
!        ENDDO
!       **** END PARALLEL **** 
!
      ENDIF
!
      IF(NPROCE.EQ.1)MXEONE=1    !  CPB  2008
!
!     THIS NOTES IF I CAN DEALLOCATE WMAT AFTER RINIT 
!     IF I AM DOING ONLY 1 ENERGY PT PER PARTIAL WAVE
!
      ISP=0
      IF (IMESH.LT.0) ISP=ABS(IMESH)
!
!   SELECT IOPT1
!
      IF(NASTD.GT.0)IOPT1=-ABS(IOPT1)
!
!  CASE OF IOPT1 NEGATIVE
!
      IF (IOPT1.LT.0) THEN
         IOPT1=-IOPT1
         IF(NASTD.EQ.0)READ(5,*)NASTD
         IF(NASTD.GT.MZTAR)THEN
           WRITE(6,697)NASTD
           STOP
         END IF
         READ(5,*)(NLEV(N),N=1,NASTD)
         WRITE(6,6021)NASTD,(NLEV(N),N=1,NASTD)
      END IF
!
! CASE OF IOPT1 FOR JAJOM
!
      IF (IOPT1.GE.10.OR.IQDT.GT.0) THEN
! OPEN K-/S-MATRIX OUTPUT FILE FOR MQDT.
        IF(IMODE.EQ.0.AND.IJBIN.NE.0)
     X     OPEN(21,FILE='JBIN',STATUS='UNKNOWN',FORM='UNFORMATTED')
! OPEN K-/S-MATRIX INPUT FILE FOR MQDT.
        IF(IMODE.GT.0)
     X     OPEN(21,FILE='JBIN',STATUS='OLD',FORM='UNFORMATTED')
!
        IF(IOPT1.GE.10.AND.IRAD.NE.0) THEN
          WRITE(6,*)' IRAD SET ZERO BECAUSE IOPT1.GE.10'
          IRAD=0
        ENDIF
      ENDIF
!
!  Check and read sizeH.dat or sizeBP.dat file
!
      mzslp=10000   ! Could be higher than that?
      Allocate(IAUX1(mzslp),IAUX2(mzslp))
      nlsh=0
! **** SINGLE PROC ****
      If (iam.eq.0) Then
       ex=.false.
       ex2=.false.
       If (iword.eq.1) Then         ! H.DAT or H.DATXXX files
         Inquire (File="sizeH.dat", Exist=ex)
         If (.not.ex) Then
           Inquire (File="sizeBP.dat", Exist=ex2)
         End If
       Else If (iword.eq.2) Then    ! DSTGH.DAT file
         Inquire (File="sizeBP.dat", Exist=ex2)
       End If
       If (.not.ex .and. .not.ex2) Then
         Write(6,'(" WARNING. NO SUITABLE sizeH.dat FILE FOUND. " / 
     &    " CHECKING NUMBER OF SYMMETRIES DIRECTLY FROM H.DAT FILE ")')
       Else If (ex) Then                   ! sizeH.dat file
         Open (Unit=12,File="sizeH.dat",FORM="FORMATTED",STATUS="OLD")
         nlsh=0
         ncmx=0
! Read from sizeH.dat file
         Do
           Read(12,('(8X,I5,34X,I3,4X,I3,4X,I3)'),iostat=ier) 
     &        nchf,is,il,ip
           If (ier.eq.0) Then        ! good read
             nlsh=nlsh+1
             islp=10000*abs(is)+100*il+ip
             IAUX1(nlsh)=islp        ! MSLP
             IAUX2(nlsh)=nlsh-1      ! IWHD
           Else If (ier.lt.0) Then   ! EOF found
             Exit
           Else
             Stop " BAD READ IN sizeH.dat FILE"
           End If
           ncmx=Max(ncmx,nchf)
         End Do
         Close(12)
       Else If (ex2.and..not.ex) Then    ! sizeBP.dat file
         Open (Unit=12,File="sizeBP.dat",FORM="FORMATTED", 
     &         STATUS="OLD")
         nlsh=0
         ncmx=0
! Read from sizeBP.dat
         Do
           is=0
           Read(12,('(I7,23X,I3,8X,I3)'),iostat=ier) nchf,il,ip
           If (ier.eq.0) Then        ! good read
             nlsh=nlsh+1
             islp=10000*abs(is)+100*il+ip
             IAUX1(nlsh)=islp        ! MSLP
             IAUX2(nlsh)=nlsh-1      ! IWHD
           Else If (ier.lt.0) Then   ! EOF found
             Exit
           Else
             Stop " BAD READ IN sizeBP.dat FILE"
           End If
           ncmx=Max(ncmx,nchf)
         End Do
         Close(12)
       End If
       If (.not.(nhdat.gt.0 .and. nhdat.eq.nlsh)) Then
         Write(6,'(" WARNING, DIFFERENT NUMBER OF SYMMETRIES ",I3,
     &             " AND H.DATXXX FILES ",I3)') nlsh,nhdat
       End If
      End If
! **** BROADCAST ****
      Call MPI_Bcast(nlsh,1,mpi_int,0,MPI_COMM_WORLD,ierr)
      Call MPI_Bcast(IAUX1,mzslp,mpi_int,0,MPI_COMM_WORLD,ierr)
      Call MPI_Bcast(IAUX2,mzslp,mpi_int,0,MPI_COMM_WORLD,ierr)
!      Call MPI_Bcast(ioneone,1,mpi_int,0,MPI_COMM_WORLD,ierr)
! **** END SINGLE PROC ****
!
!  CASE IOPT1 = 1
!  DO ALL THE SIMMETRIES STORED
!
      If (iabs(iopt1).eq.1) Then
        Allocate (MSLP(nlsh),IWHD(nlsh))
        If (nhdat.eq.-1) Then
          MSLP(1:nlsh)=IAUX1(1:nlsh)
          IWHD(1:nlsh)=-1
        Else If (nhdat.eq.nlsh) Then
          MSLP(1:nlsh)=IAUX1(1:nlsh)
          IWHD(1:nlsh)=IAUX2(1:nlsh)
        Else
          Write(6,'(//10X,"WARNING: NUMBER OF PARTIAL WAVES ",
     &    /10X,"STORED IN sizeH.dat OR sizeBP.dat FILE ",I3
     &    /10X,"DOES NOT AGREE WITH NUMBER OF H.DATXXX FILES ",I3
     &    /10X,"READING PARTIAL WAVES DIRECTLY FROM H.DATXXX FILES")')
     &    nlsh,nhdat
        End If
        kslp=nlsh
!
!  CASE IOPT1 = 2
!  READ THE SIMMETRIES FROM STANDARD INPUT
!
      Else If (iopt1.eq.2) Then
        Allocate(MSLP(mzslp),IWHD(mzslp))
        Write (6,'(/5X,"VALUES OF  10000*IS+100*IL+IP"/
     &              5X,"READ FOR IOPT1=2")')
!
!  **** SINGLE PROC ****
        If (iam.eq.0) Then
         kase=0
         Do 
           Read(5,*,iostat=ier) is,il,ip
           If (ier.lt.0) Exit   ! EOF found
           If (is.eq.-1 .or. il.eq.-1) Exit ! Mark of end of data
           kase=kase+1
           islp=10000*abs(is)+100*il+ip
           MSLP(kase)=islp
           Write(6,'(10X,I7)') islp
           If (isp.gt.0 .and. isp.ne.abs(is)) Then
             WRITE(6,'(" SPIN FOR THIS CASE .NE. IMESH")')
             STOP
           End If
!
! Check IWHD for this partial wave
           IWHD(kase)=-10
           If (nhdat.eq.-1) Then
             IWHD(kase)=-1
           Else
             Do k=1,nlsh
               If (MSLP(kase).eq.IAUX1(k)) Then
                 IWHD(kase)=IAUX2(k)
                 Exit
               End If
             End Do
             If (IWHD(kase).eq.-10) Then
               Write(6,'(//10X,"WARNING: SIMMETRY ",I8
     &         /10X,"NOT FOUND IN sizeH.dat OR sizeBP.dat FILE",
     &         /10X,"SIMMETRY WILL BE SKIPPED FROM CALCULATION")')
     &         MSLP(kase)
             End If
           End If
         End Do
         IF(KASE.EQ.0)THEN
           WRITE(6,600)
           STOP
         ENDIF
         IF(KASE.GT.MZSLP)THEN
           WRITE(6,605) KASE
           STOP
         ENDIF
         kslp=kase
        End If
!  **** BROADCAST ****
        Call MPI_Bcast(kslp,1,mpi_int,0,MPI_COMM_WORLD,ierr)
        Call MPI_Bcast(MSLP,mzslp,mpi_int,0,MPI_COMM_WORLD,ierr)
        Call MPI_Bcast(IWHD,mzslp,mpi_int,0,MPI_COMM_WORLD,ierr)
!  **** END SINGLE PROC ****
!
! Redimension arrays MSLP and IWHD
        IAUX1(1:kslp)=MSLP(1:kslp)
        IAUX2(1:kslp)=IWHD(1:kslp)
        Deallocate(MSLP,IWHD)
        Allocate(MSLP(kslp),IWHD(kslp))
        MSLP(1:kslp)=IAUX1(1:kslp)
        IWHD(1:kslp)=IAUX2(1:kslp)
      End If
      Deallocate(IAUX1,IAUX2)
!
 3999 WRITE(6,601)IPRINT,IRAD,IPERT,LMX,AC,IMESH,IOPT1,IRDEC,IMODE,IQDT
     X           ,IEQ,IOMSW,NCUTOFF,FNUHYB,NDRMET,NTYP1,MZPTS
!
! ************************************
!
!  READ R-MATRIX DATA FOR TARGET
!
      CALL READ1
      KMAX=(NAST*(NAST-2*IONE+1))/2
!
      IF(ALLOCATED(ARAD))  DEALLOCATE(ARAD)
      ALLOCATE(ARAD(KMAX))
      IF(ALLOCATED(ARDEC)) DEALLOCATE(ARDEC)
      ALLOCATE(ARDEC(NAST))
      IF(ALLOCATED(SLIN))  DEALLOCATE(SLIN)
      ALLOCATE(SLIN(KMAX))
      IF(ALLOCATED(IPAR))  DEALLOCATE(IPAR)
      ALLOCATE(IPAR(NAST))
!
! ************************************
!
! Time control per processor
!
      Open(unit=40,File="timing."//filec//filed//fileu//filev,
     &  Status="replace",form="formatted",action="write")
      Write(40,'(" Processor ",I4)') iam
      Write(40,'("   ISLP    IE     E        TIME        ")')
!
      WRITE(6,'(" ")')
!
      CALL ACSUB
!
      If ((iabs(iopt1).eq.1 .or. iopt1.eq.2).and.
     &    (nhdat.ne.nlsh)) Then
!
! READ THE MSLP AND IWHD DIRECTLY FROM FILES H.DATXXX
! SAFE BUT SLOW
!
! **** SINGLE PROC ****
        If (iam.eq.0) Call fastread2(iopt1)
! **** BROADCAST ****
        Call MPI_Bcast(kslp,1,mpi_int,0,MPI_COMM_WORLD,ierr)
        If (iam.ne.0) Then
          If (Allocated(MSLP)) Deallocate(MSLP)
          If (Allocated(IWHD)) Deallocate(IWHD)
          Allocate (MSLP(kslp), IWHD(kslp))
          If (nhdat.ge.1) Then  ! Multiple H.DATXXX files
            Rewind(10)
            Close(10,Status="keep")
          End If
        End If
        Call MPI_Bcast(MSLP,kslp,mpi_int,0,MPI_COMM_WORLD,ierr)
        Call MPI_Bcast(IWHD,kslp,mpi_int,0,MPI_COMM_WORLD,ierr)
! **** END SINGLE PROC ****
      Else
        If (nhdat.ge.1) Then  ! Multiple H.DATXXX files
          Rewind(10)
          Close(10,Status="keep")
        End If
      End If
!      Write(0,'(I8," MSLP, IWHD= ",500I8)') kslp,MSLP,IWHD
!      Write(6,'(I8," MSLP, IWHD= ",500I8)') kslp,MSLP,IWHD
!
! CHECK TOP-UP
!
      If (lrglam.ge.0) Then
        lrglam=0
        Do kase=1,kslp
          is=MSLP(kase)/10000
          il=(MSLP(kase)-10000*is)/100
          lrglam=Max(lrglam,il)
        End Do
        If (il.ne.lrglam) Then
          Write(6,'(//10X,"WARNING: LAST PARTIAL WAVE INCLUDED ",
     &      /10X,"IS NOT THE ONE OF HIGHEST L.",
     &      /10X,"STOPPING: TOP-UP CAN NOT BE PERFORMED.")')
          Stop " TOP-UP: WRONG SORTING OF PARTIAL WAVES"
        End If
        Write(6,'(//10X,"TOP-UP ACTIVATED",
     &      /10X,"PARTIAL WAVES CACULATIONS SORTED",
     &      /10X,"ENERGY GRID CONSTANT FOR ALL PARTIAL WAVES"
     &      /10X,"LRGLAM= ",I4)') lrglam
      End If
!
      IF(ELAS.EQ.'  ')THEN
        IF(NZED.EQ.NELC) ELAS='YES'
        IF(NZED.NE.NELC) ELAS='NO'
      ENDIF
      IONE=1
      IF(ELAS.EQ.'YES') IONE=0
!
!  RADIATIVE TRANSITION PROBABILITIES
!  **********************************
!
      IF(IRDEC.GT.0.OR.IBIGE.EQ.1.OR.IRAD.GT.0.OR.IDIP.GT.0)THEN
!
! ALREADY DONE, EVEN IF NOT IBIGE
!       CALL READ1
!        IF(ELAS.EQ.'  ')THEN
!          IF(NZED.EQ.NELC)ELAS='YES'
!          IF(NZED.NE.NELC)ELAS='NO'
!        ENDIF
!        IONE=1
!        IF(ELAS.EQ.'YES')IONE=0
        IPAR(1:NAST)=-1
        ARAD(1:KMAX)=-ONE
!
! TWIDDLE: NRB
        NASTR=NAST
        NASTDO=NASTD
        NASTD=0
!
!    6   CALL READ2(IOPT1)
        KASE=1
    6   CALL READ2N(IOPT1,KASE)
        CALL RAD
        If (nhdat.ge.1) Then  ! Multiple H.DATXXX files
          Rewind(10)
          Close(10,Status="keep")
        End If
!
        IF(IEND.EQ.0)THEN
          IF(MORE2.GT.0 .AND. KASE.LT.KSLP) THEN
            KASE=KASE+1
            GOTO 6
          ELSE
            WRITE(6,606)
            K=0
            DO I=1+IONE,NAST
              DO J=1,I-IONE
              K=K+1
              IF(ARAD(K).LT.ZERO)THEN
                WRITE(6,'(5X,I5,1X,I5,1X,ES12.4)') J,I,ARAD(K)
                ARAD(K)=ZERO
              ENDIF
              ENDDO
            ENDDO
          ENDIF
        ENDIF
!
! RESTORE: NRB
        NASTD=NASTDO
! One single H.DAT file
        If (nhdat.eq.-1) Then
          Rewind(10)
          Call READ1
        End If
!
      ENDIF
!
!  START MAIN CALCULATIONS
!  ***********************
!
!     ALLOCATIONS FOR CDEGEN
!
!*************************************************
!
      IF(ALLOCATED(ENATR))  DEALLOCATE(ENATR)
      IF(ALLOCATED(NLEV))   DEALLOCATE(NLEV)
      IF(ALLOCATED(NCNATR)) DEALLOCATE(NCNATR)
      IF(ALLOCATED(IWD))    DEALLOCATE(IWD)

      ALLOCATE(ENATR(NAST))
      ALLOCATE(NLEV(NAST))
      ALLOCATE(NCNATR(NAST))
      ALLOCATE(IWD(NAST))
!
!     ALLOCATIONS FOR NRBTOP    (ALL N ELEC PARAMS)
!
! *************************************************
!
      MXTST=(NAST*(NAST+1))/2
      IF(ALLOCATED(ITST))  DEALLOCATE(ITST)
      IF(ALLOCATED(JTST))  DEALLOCATE(JTST)
      IF(ALLOCATED(OMST))  DEALLOCATE(OMST)
      IF(ALLOCATED(KTST))  DEALLOCATE(KTST)
      IF(ALLOCATED(OMEGA)) DEALLOCATE(OMEGA)

      ALLOCATE(ITST(MXTST))
      ALLOCATE(JTST(MXTST))
      ALLOCATE(KTST(NAST,NAST))
      ALLOCATE(OMST(MXTST)) 
!
      ALLOCATE(OMEGA(MXTST)) 
!
      CALL SCALE1(IOPT1)
!
! For this routine I have to know MAXMUL, and that is in READ2N
      IF(LRGLAM.GT.0) THEN
        MAXMUL=MAX(2,LAMAX)
        CALL CTOP_ALLOCATE(NAST)
      END IF
!
      IF(NZED.EQ.NELC)THEN
        TZED=ZERO
        IF(LPRTSW.LT.0)LPRTSW=5
        IF(ABVTHR.LT.0.)ABVTHR=1.E-3
        IF(BELTHR.LT.0.)BELTHR=1.E-3
      ELSE
        TZED=ONE
      ENDIF
!
! SET-UP ENERGY MESH
!
      IF(IMESH.EQ.1) CALL PRUNE(ABVTHR,BELTHR)
      IF(IMESH.EQ.2 .OR. IMESH.LT.0) CALL MESH
      If (Mod(mxe,nproc).ne.0) 
     &  Write(6,'(//10X,"WARNING AT EXIT OF PRUNE:"
     &             /10X,"ENERGY MESH POINTS ",I8," IS NOT"
     &    /10X,"EVEN DIVISOR OF NUMBER OF PROCESSORS ",I5)')
     &    mxe,nproc
!
! ALLOCATE ARRAY TO CHECK IF REACTANCE MATRIX CALCULATION FAILED
!
      ALLOCATE(LFARR(MXE))
      LFARR(1:MXE)=.FALSE.
      IF (NPROC.GT.MXE) THEN
        WRITE(6,'(//10X,"ERROR, TOO MANY PROCESSORS"
     &             /10X,"REDUCE THE PROCESSOR NUMBER TO ",I5)') MXE
        STOP " TOO MANY PROCESSORS"
      END IF
!***************************************************
!
      IF(ALLOCATED(IEE)) DEALLOCATE(IEE)
      ALLOCATE(IEE(MXE))
!
      IF(ALLOCATED(ISKP)) DEALLOCATE(ISKP)
      ALLOCATE(ISKP(MXE))
!
!***************************************************
!
!     WRITE PRELIMINARY INFORMATION TO INPUT FILE FOR STGICF
!
      IF(IPRKM.EQ.4)THEN
        WRITE(19) MXE,NZED,NELC
        WRITE(19) (EMESH(IE),IE=1,MXE)
      ENDIF
!
! DECIDE WHEN TO UPDATE UNPHYSICAL K/S-MATRIX
!
      IF(IQDT.GT.0)THEN
        IF(IMESH.EQ.1.OR.IEQ.LT.0)THEN
          IF(IEQ.LT.0)THEN
            IEQ=-IEQ
            IEQ0=IEQ
          ELSE
            IEQ0=MXE/(IEQ-1)
            IF(IEQ0.EQ.0)IEQ0=1
          ENDIF
          IE=0                                                !PARALLEL
          DO NC=1,NCINT
            DO JE=1,NSEQ
             DO J=1,NPROC
!              IE=IE+1
              IE=NPROC*NSEQ*(NC-1)+NPROC*(JE-1)+J
              IF(IE.GT.MXE) CYCLE
              IF (MOD(JE-1,IEQ0).EQ.0) THEN
                IEE(IE)=IE
              ELSE
                IEE(IE)=0
              END IF
             END DO
            ENDDO
            IE=NPROC*NSEQ*NC
            IF(IE.GT.MXE) CYCLE
            IEE(IE)=IE
          ENDDO
        ELSE
          DEQ=(EMAX-EMIN)/(IEQ-1)
          EQ=EMIN
          DO IE=1,MXE
            IF(EMESH(IE).GE.EQ) THEN
              IEE(IE)=IE
              EQ=EQ+DEQ
            ELSE
              IEE(IE)=0
            ENDIF
          ENDDO
        ENDIF
      ENDIF
!
      IF((IRAD.GT.0).OR.(NDRMET.GT.0))THEN
        ALLOCATE(OMEGDR(NDRMET,MXE))
      ENDIF
!
!  INITIAL WRITES TO UNIT 8
!  LFM - THIS IS DONE BY ALL THE PROCESSORS ???
!  TODO - REMOVE
!
      IF (IRAD.GT.0) THEN
        H=ACNUM
        KP2=4*((MZPTS-1)/4)+1
        RTWO=(KP2-1)*H+RZERO
        RTWOC=RTWO
        KP2C=KP2
        IPERTW=IPERT
        IF(IPERT.GT.0) IPERTW=1
!
        CALL BODE(H,KP2,IPERT,RZERO)
!
!  **** SINGLE PROC ****
        if (iam.eq.0) Then
!         OPEN(8,FILE='F00',STATUS='UNKNOWN',FORM='UNFORMATTED')
          OPEN(8,FILE='F00',STATUS='REPLACE',FORM='UNFORMATTED')
!
          REWIND(8)
          WRITE(8)NZED,NELC
          WRITE(8)NAST,(ENAT(I),I=1,NAST),(ISAT(I),LAT(I),I=1,NAST)
     X          ,(ARDEC(I),I=1,NAST)
          WRITE(8)KP2
          WRITE(8)RZERO,H
          WRITE(8)(WBODE(I),I=1,KP2)
          WRITE(8)IPERTW,IQDT
          IF(IQDT.GT.0)THEN
            MXE0=0
            DO IE=1,MXE
              IF(IEE(IE).NE.0)THEN
                MXE0=MXE0+1
                OMEGDR(1,MXE0)=EMESH(IE)
              ENDIF
            ENDDO
            WRITE(8)MXE0
            WRITE(8)(OMEGDR(1,I),I=1,MXE0)
          ELSE
            WRITE(8)MXE
            WRITE(8)(EMESH(I),I=1,MXE)
          ENDIF
          WRITE(8)BSTO
        end if
!  **** END SINGLE PROC ****
        IF(IQDT.GT.0)THEN
          Call MPI_Bcast(MXE0,1,mpi_int,0,MPI_COMM_WORLD,ierr)
          Call MPI_Bcast(OMEGDR(1,1:MXE0),MXE0,mpi_real8,
     &                   0,MPI_COMM_WORLD,ierr)
        ENDIF
      ENDIF
!
!  START CALCULATION OF OMEGAS
!
!  INITIALISE OMEGAS TO ZERO
!  TODO - DO ALWAYS, IRAD=0
!
      IF(IRAD.LT.2)THEN
!
        IF(NOMWRT.EQ.0)THEN
          WRITE(6,*)
          WRITE(6,*)' *** OMEGA FILE IS NOT WRITTEN IN THIS CASE ***'
          NOMT=0
        ELSE
          NOMT0=(NAST*(NAST+1-2*IONE))/2
          NOMT=MIN(NOMT0,ABS(NOMWRT))
          IF(NOMWRT.LT.0)NOMT=-NOMT
          NOMWRT=NOMT
!
!          CALL OMEG(0,EMESH,MXE,ENAT,NAST,OMEGA(1:ABS(NOMWRT)),NOMWRT,IONE)
          CALL OMEG(0)
!
          IF(NDRMET.GT.0)THEN
            OMEGDR(1:NDRMET,1:MXE)=ZERO
          ENDIF
        ENDIF
        ISKP(1:MXE)=INSKP
        INSKP=-INSKP
      ENDIF
!
! SET ARRAY IE0 WITH THE FIRST ENERGY FOR EACH PARTIAL WAVE
!     **** PARALLEL ****
      ALLOCATE(IE0(KSLP))
!      NPROCE=MXE/NPROC
!
      IF (LRGLAM.LT.0) THEN
!     NO TOP-UP
        IF (NPROC.GT.KSLP) THEN
!       USUAL CASE NPROC >> KSLP
          KSTEP=NPROC/KSLP     ! ALWAYS > 1 IN THIS CASE
          DO KASE=1,KSLP
            IE0(KASE)=MOD(IAM+(KASE-1)*KSTEP,NPROC)+1
          END DO
        ELSE
!       LESS USUAL CASE, KSLP > NPROC
!       Actually it is the almost same, KSTEP=0, then set it KSTEP=1
          DO KASE=1,KSLP
            IE0(KASE)=MOD(IAM+KASE-1,NPROC)+1
          END DO
        END IF
      ELSE
!     WITH TOP-UP ALL PROCESSORS MUST DO THE SAME ENERGY-GRID
!     IN ALL THE PARTIAL WAVES
        DO KASE=1,KSLP
          IE0(KASE)=IAM+1
        END DO
      END IF
!      Write(0,'(" proc ",I5," ipw,IE0= ",100(I3,",",I5,4x))') 
!     &  iam,(kase,IE0(kase),kase=1,kslp)
!     **** END PARALLEL ****
!
!  START LOOP ON SLPI CASES
!  ************************
!
!      KASE=1
      DO I=1,NAST
        IPAR(I)=-1
      ENDDO
!
!  READ R-MATRIX DATA FOR NEXT SLPI CASE
!  *************************************
!  
! 1000   CONTINUE
! LFM 2018 - REPLACE THIS LOOP FOR A STANDARD ITERATIVE LOOP
!    DO THAT EACH PROCESSOR WORKS A DIFFERENT SYMMETRY, SO
!    NOT ALL OF THEM OPEN THE SAME H.DATXXX FILE AT A TIME,
!    AT LEAST IN THE FIRST ONE CALCULATED
!
      DO KASF=1,KSLP
        IF (LRGLAM.LT.0) THEN
!       NO TOP-UP
          KASE=MOD(KASF+IAM-1,KSLP)+1
        ELSE
!       WITH TOP-UP THE PARTIAL WAVES MUST BE CALCULATED IN
!       ASCENDING ORDER WITH THE LARGEST L/2J THE LAST ONE
          KASE=KASF
        END IF
!
!  TRY to AVOID DEALLOCATING TARGET/POINTER MATRICES
!
!   ALPHAN, but little more space before READ2
!   appreciated for CF reads. DEALLOCATE_NRBRSCT
!   will clean up any matrices missed before 
!   R-matrix formation
!
        IF(ALLOCATED(ASS)) DEALLOCATE(ASS)
        IF(ALLOCATED(ASC)) DEALLOCATE(ASC)
        IF(ALLOCATED(ACS)) DEALLOCATE(ACS)
        IF(ALLOCATED(ACC)) DEALLOCATE(ACC)

        IF(ALLOCATED(RMAT)) DEALLOCATE(RMAT)
!
!      Likewise for COULFSC
!
        IF(ALLOCATED(FS))  DEALLOCATE(FS)
        IF(ALLOCATED(FSP)) DEALLOCATE(FSP)
        IF(ALLOCATED(FC))  DEALLOCATE(FC)
        IF(ALLOCATED(FCP)) DEALLOCATE(FCP)
!
!     Likewise for NRBRSCT
!
        IF(ALLOCATED(RK))   DEALLOCATE(RK)
        IF(ALLOCATED(A))    DEALLOCATE(A)
        IF(ALLOCATED(B))    DEALLOCATE(B)
        IF(ALLOCATED(SPP))  DEALLOCATE(SPP)
        IF(ALLOCATED(CSPP)) DEALLOCATE(CSPP)
        IF(ALLOCATED(CS))   DEALLOCATE(CS)
!
!
        IF(ALLOCATED(CSP))  DEALLOCATE(CSP)
        IF(ALLOCATED(DSP))  DEALLOCATE(DSP)
        IF(ALLOCATED(CCPP)) DEALLOCATE(CCPP)
        IF(ALLOCATED(DS))   DEALLOCATE(DS)
        IF(ALLOCATED(DFPP)) DEALLOCATE(DFPP)
        IF(ALLOCATED(CPP))  DEALLOCATE(CPP)
        IF(ALLOCATED(CCP))  DEALLOCATE(CCP)
        IF(ALLOCATED(DCP))  DEALLOCATE(DCP)
        IF(ALLOCATED(RBE))  DEALLOCATE(RBE)
!
!       CALL READ2(IOPT1)
        CALL READ2N(IOPT1,KASE)
        If (more2.eq.-1 .and. nhdat.eq.nlsh) Then
          Write(6,'(//10X,"FATAL ERROR WHILE READING SIMMETRY ",I8
     &               /10X,"FROM FILE H.DAT",I3.3
     &               /10X,"CHECK sizeH.dat OR sizeBP.dat FILES")')
     &                MSLP(KASE),IWHD(KASE)
          Stop " BAD sizeH.dat FILE "
        End If
        IF (MORE2.EQ.-1) CYCLE
!
!***************************************************
! Variables for Numerov and numeric subroutines
! For these allocations I need to know NCHF, in READ2N
!
        IF(ALLOCATED(MSUM)) DEALLOCATE(MSUM)
        ALLOCATE(MSUM(NCHAN))
        IF(ALLOCATED(IOMIT)) DEALLOCATE(IOMIT)
        ALLOCATE(IOMIT(NCHAN))
        IF(ALLOCATED(R2ST)) DEALLOCATE(R2ST)
        ALLOCATE(R2ST(NCHAN))
!
!***************************************************
!
        IF (IQDT.EQ.1) THEN     ! MQDT
         IF (ALLOCATED(ZKHIOC)) DEALLOCATE(ZKHIOC)
         ALLOCATE(ZKHIOC(NCHAN,NCHAN))
         IF (ALLOCATED(ZKHICC)) DEALLOCATE(ZKHICC)
         ALLOCATE(ZKHICC(NCHAN,NCHAN))
         IF (ALLOCATED(ZVAL)) DEALLOCATE(ZVAL)
         ALLOCATE(ZVAL(NCHAN))
        ENDIF
!
!  DR
       IF ((IRAD.GT.0).OR.(NDRMET.GT.0)) THEN
         IF(ALLOCATED(PDR)) DEALLOCATE(PDR)
         ALLOCATE(PDR(NCHAN))
       END IF
!
! TERMINATE IF EOF ON H.DAT
! LFM - Not necessary with the new philosophy of loop
!        IF(MORE2.EQ.-777)GO TO 3000
!
        IF (INSKP.LT.0.AND.NSPN2.EQ.0) THEN
          INSKP=-INSKP
          IF (ISKP0.EQ.15.OR.ISKP0.EQ.10) ISKP0=2*ISKP0
          IF (MINLT.GT.ISKP0) INSKP=MINLT+2
          DO IE=1,MXE
            ISKP(IE)=INSKP
          ENDDO
        ENDIF
        INSKP=IABS(INSKP)
!
        ISLP=10000*IABS(NSPN2)+100*LRGL2+NPTY2
!       ISLP=MSLP(KASE)
        IF (NSPN2.LT.0) ISLP=-ISLP
        IF (LRGLAM.GE.0) THEN
          IF(NSPN2.NE.0) MAXLT=MIN(LRGLAM,MAXLT)   ! FOR FTOP
          IF(NSPN2.EQ.0) MAXLT=MIN(LRGLAM+2,MAXLT)
        ENDIF
        IF (LRGL2.LT.MINLT.OR.LRGL2.GT.MAXLT) THEN
!          IF(MORE2.NE.0) GO TO 1000
!          GO TO 3000
          CYCLE
        ENDIF
!
!  FOR IOPT1=2, CHECK WHETHER REQUIRED VALUE OF SLPI HAS BEEN FOUND
!  LFM - DONE IN READ2N
!
!      IF(IOPT1.EQ.2)THEN
!        DO 30 K=1,KSLP
!          IF(ISLP.NE.MSLP(K))GOTO 30
!          KK=K
!          GOTO 40
!   30   CONTINUE
!
!  CASE NOT FOUND
!
!        GOTO 2000
!
!  CASE FOUND
!    RE-ORDER STORED UNIT 5 DATA
!
!   40   IF(KK.NE.KASE)MSLP(KK)=MSLP(KASE)
!
!      ENDIF
!
!
!  SCALE R-MATRIX DATA FOR SLPI CASE
!  *********************************
!
        IPERT=IPERTR
!
        CALL SCALE2
!
        IF(LRGLAM.GE.0) CALL TOP1(LRGLAM)
!
        IFLAG=IWARN                  ! LIMIT WARNING MESSAGES
        IFLEG=IWARN                  ! LIMIT WARNING MESSAGES
!
        IF(IPERT.EQ.0) WRITE(6,641)
        IF(IPRINT.GT.0.AND.IRAD.NE.2) WRITE(6,640)
!
        IF(IRAD.NE.0.AND.IOPT1.LE.9) THEN
! TODO - REMOVE RADIATION PART
!          IF (iam.eq.0) WRITE(8) NSPN2,LRGL2,NPTY2 ! Moved to outside of loop
! This part should be done by one single processor, but for each partial wave
! If this is not removed it should be reprogrammed.
          NAME='F'//NUM(KASE/10)//NUM(KASE-10*(KASE/10))
          OPEN(9,FILE=NAME,STATUS='UNKNOWN',FORM='UNFORMATTED')
          REWIND(9)
          WRITE(9)NSPN2,LRGL2,NPTY2
          WRITE(9)MNP2,NCHF
          WRITE(9)(ECH(I),I=1,NCHF)
          IF(IBETA.EQ.0)THEN
            WRITE(9)(CCT(I),I=1,NCHF)
          ELSE
            WRITE(9)(CCT(I),I=1,NCHF)
     X        ,(L2P(I),I=1,NCHF),(LAT(ITARG(I)),I=1,NCHF)
     Y        ,(ITARG(I),I=1,NCHF),(KJ(I),I=1,NCHF)
          ENDIF
          WRITE(9)(VALUE(I),I=1,MNP2)
          WRITE(9)((WMAT(J,I),J=1,MNP2),I=1,NCHF)
          IF(IPERT.GT.0)WRITE(9)(((CF(I,J,L),I=1,NCHF),J=1,NCHF),L=1,2)
        ENDIF
!
!   INITIALIZE JAJOM DATA FOR UNIT 22 (KAB,JAN94)
!   Warning: JAJOM not implemented in parallel version
!
        IF(IOPT1.GE.10) CALL OUTJJ(0,LRGLAM,MXE)
!
! WRITE UNPHYSICAL K-MATRIX DATA
!
! LFM - NEVER
!        IF (IPRKM.EQ.2.AND..NOT.PQRD) THEN
!          NAMKLS='KMATLS'//NUM(KASE)
!          OPEN(36,FILE=NAMKLS,FORM='UNFORMATTED',STATUS='UNKNOWN')
!          WRITE(36) NSPN2,LRGL2,NPTY2
!          WRITE(36) NAST
!          DO I=1,NAST
!            WRITE(36) I,ISAT(I),LAT(I),ENAT(I)
!          ENDDO
!          WRITE(36) NCHF
!          DO I=1,NCHF
!            WRITE(36) I,ITARG(I),LLCH(I),KJ(I),ECH(I)
!          ENDDO
!          WRITE(36) MXE
!        ENDIF
        IF (IPRKM.EQ.4.AND..NOT.PQRD) THEN
          WRITE(19) NCHF,NSPN2,LRGL2,NPTY2
          DO I=1,NCHF
            WRITE(19) ITARG(I),LLCH(I)
          ENDDO
!          IF(IRD0.GE.100) THEN
            K1=KASE/100
            K2=(KASE-100*K1)/10
            K3=KASE-100*K1-10*K2
            NAME=NUM(K1)//NUM(K2)//NUM(K3)
            IF(IQDT.EQ.1)THEN
          OPEN(32,FILE='SMTLS.'//NAME//'.'//FILEC//FILED//FILEU//FILEV
     X          ,FORM='UNFORMATTED',STATUS='UNKNOWN')
            ELSE
          OPEN(32,FILE='KMTLS.'//NAME//'.'//FILEC//FILED//FILEU//FILEV
     X          ,FORM='UNFORMATTED',STATUS='UNKNOWN')
            ENDIF
! LFM - OR
!  **** SINGLE PROC ****
!            IF (IAM.EQ.0) THEN
!              IF(IQDT.EQ.1)THEN
!                OPEN(32,FILE="SMTLS."//NAME//,FORM="UNFORMATTED",STATUS="REPLACE")
!              ELSE
!                OPEN(32,FILE="KMTLS."//NAME//,FORM="UNFORMATTED",STATUS="REPLACE")
!              ENDIF
!            END IF
!  **** SINGLE PROC ****
!          ENDIF
        ENDIF
!
!
!  START ENERGY LOOP
!  *****************
!
!     WRITE(70+IAM,8889)KASE
! 8889 FORMAT(2X,'SYMMETRY=: ',I4)
!
!        IE=0                                             ! PARALLEL
        DO NC=1,NCINT
!        IE=(NC-1)*NSEQ   ! SERIAL
         IE=((NC-1)*NSEQ-1)*NPROC+IE0(KASE)
         EQSAVE=-999999.
!        ETOT=EMESH(IE+1) ! SERIAL
         IF(IE+NPROC.LE.MXE) ETOT=EMESH(IE+NPROC)
         IF(PQRD) CALL READPQ(-IE,QETEST,ISLP,IOPT1,QJUMP,PQRD)
!
         DO JE=1,NSEQ                                 ! PARALLEL
!           IE=IE+1  ! SERIAL
!           IE=(NC-1)*NSEQ+JE      1:NCINT*NSEQ   ! SERIAL
           IE=((NC-1)*NSEQ+JE-1)*NPROC+IE0(KASE)
           IF(IE.GT.MXE) CYCLE
           IF(LFARR(IE)) CYCLE
!
!          IF (KASE.EQ.1) WRITE(0,8888)IE,EMESH(IE),IAM
!8888      FORMAT(2X,'IE=: ',I5,'  ENERGY=: ',1PG16.4,'  PROC=: ',I4)
!
           IF(IRAD.LT.2.AND.LRGL2.GT.ISKP(IE).AND.IPRKM.LE.0) CYCLE
           IPERT=IPERTR
           ETOT=EMESH(IE)
           IF (WARNE) THEN
             IF (IPRCENT.EQ.100) THEN
               IF (ETOT.GT..5*VALUE(1)) THEN
                 WRITE(6,690) ETOT,.5*VALUE(1)
                 WARNE=.FALSE.
               ENDIF
             ELSE
               IF (ETOT.GT.0.5*TRACE) THEN                         ! EZERO
                 WRITE(6,691) ETOT,0.5*TRACE
                 WARNE=.FALSE.
               ENDIF
             ENDIF
           ENDIF
           IF(IPRINT.GT.0) WRITE(6,680)ETOT
!
! IN IQDT MODE SEE IF WE NEED TO UPDATE
!
           IF (IQDT.GT.0) THEN
             IF (PQRD) THEN
               IF (ETOT-EQSAVE.GT.QETEST) THEN
                 CALL READPQ(IE,QETEST,ISLP,IOPT1,QJUMP,PQRD)
                 EQSAVE=ETOT
               ENDIF
             ELSE
               QJUMP=IEE(IE).EQ.0
             ENDIF
           ENDIF
!
           CALL POINTS(IOPT1,QJUMP)
!
! ALLOCATE BB BG ARRAYS
!
           IF(ALLOCATED(BB)) DEALLOCATE(BB,BG)
           MMAX=12
           DO ICH=1,NCHAN
! LFM - TO INITIALISE MSUM IS JUST SAFETY,
!    MSUM SHOULD BE INITIALISED FOR ALL CASES IN SUBROUTINE THETA
             MSUM(ICH)=NINT(FKNU(ICH))+LLCH(ICH)+12
             MMAX=MAX(MMAX,NINT(FKNU(ICH))+LLCH(ICH)+12)
           END DO
           ALLOCATE(BB(NCHAN,MMAX))
           ALLOCATE(BG(NCHAN,MMAX))
!
!  CALCULATE OMEGAS
           CALL REACT(IOPT1,QJUMP,PQRD,LFAIL)
           IF (LFAIL) THEN
             WRITE(6,'(//10X,"REACTANCE MATRIX CALCULATION FAILED"
     &                  /10X,"FOR SIMMETRY ",I8," AND ENERGY ",
     &            ES10.4/10X,"SKIPING ENERGY")') MSLP(KASE),EMESH(IE)
             LFARR(IE)=.TRUE.
             CYCLE
           END IF
!
! SUPRESS ANY MORE WARNING MESSAGES FOR CURRENT SYMMETRY
           IFLAG=ABS(IFLAG)
           IFLEG=ABS(IFLEG)
!
! Check control time for this energy
!
        Call CPU_TIME(timef)
        time=timef-timei
        write(40,775) islp,ie,EMESH(ie),time
 775    Format(1X,I6,1X,I5,1X,F12.6,2X,F10.2)
!
         END DO      ! JE
        END DO      ! NC
!
        IF(IRAD.NE.0) CLOSE(9,STATUS='KEEP')
        IF (.NOT.PQRD) THEN
!          IF(IPRKM.EQ.4.AND.IRD0.GE.100) CLOSE(32,STATUS='KEEP')
          IF(IPRKM.EQ.4) CLOSE(32,STATUS='KEEP')
!          IF(IPRKM.EQ.2) CLOSE(36,STATUS='KEEP')
        ENDIF
!
! LFM - THIS PART IS TO CHECK IF WE HAVE TO CONTINUE THE LOOP OR
!       EXIT, IN TERMS OF VARIABLE MORE2.
!       NOT NECESSARY NOW WITH NEW LOOP PHILOSOPHY.
!        IF(IOPT1.EQ.2)THEN
!          IF(KASE.EQ.KSLP) GOTO 3000
!          KASE=KASE+1
!        ENDIF
!!
!!  END OF ENERGY LOOP, GO TO NEXT SLPI CASE
!!  ******************
!!
! 2000   IF(IOPT1.EQ.1)THEN
!          IF(MORE2.NE.0)THEN
!            KASE=KASE+1
!            GOTO 1000
!          ENDIF
!          GOTO 3000
!        ENDIF
!
!  CASE OF IOPT1=2
!
!        IF(MORE2.NE.0) GOTO 1000
!        IF(IOPT1.EQ.2)THEN
!
!  CASES NOT FOUND ON UNIT 10
!  LFM - ALREADY DONE IN READ2N
!
!          WRITE(6,660)
!          DO K=KASE,KSLP
!            WRITE(6,661)MSLP(K)
!          ENDDO
!          WRITE(6,662)
!        ENDIF
      END DO    ! KSLP
!
!  END OF SLPI LOOP
!  ****************
!
!
!  PRINT TOTAL OMEGAS
!  ******************
!
!  NO RADIATION, IF(NEVER)
 3000 IF(IRAD.EQ.2) GOTO 3200
!
! COMPOUND THE ARRAY OF THE ENERGY WHICH FAILED
!     **** PARALLEL ****
      IF(IAM.EQ.0) THEN
        ALLOCATE(OMEGATOT(MXTST))
        ALLOCATE(LFTOT(mxe))
      END IF  
      Call MPI_Reduce(LFARR,LFTOT,mxe,mpi_logical,mpi_lor,
     &                0,MPI_COMM_WORLD,ierr)
      If(iam.eq.0) LFARR(1:mxe)=LFTOT(1:mxe)
      Call MPI_Bcast(LFARR,mxe,mpi_logical,0,MPI_COMM_WORLD,ierr)
      If(iam.eq.0) DEALLOCATE(LFTOT)
!     **** END PARALLEL ****
! SUBSTRACT ENERGIES THAT CALCULATION FAILED
      NEDROP=0
      DO IE=1,MXE
        IF(LFARR(IE)) NEDROP=NEDROP+1
      END DO
!
! NO JAJOM IN PARALLEL VERSION
      IF(IOPT1.GE.10)THEN
        CALL OUTJJ(IOPT1,LRGLAM,MXE)
        CLOSE(11,STATUS='KEEP')
      ENDIF
!
      IF(IOPT1.NE.11)THEN
!
        IF(NSPN2.EQ.0)THEN
!
          IF(NASTD.GT.0.AND.IWT.GE.0)WRITE(6,763)
!
          DO I=1,NAST
            IF(NASTD.GT.0)LAT(I)=IWD(I)
            LAT(I)=LAT(I)+1
            LAT(I)=LAT(I)*(-1)**ISAT(I)
            ISAT(I)=0
          ENDDO
        ELSE
!
! CHECK COMPLETENESS OF PARITY
          DO I=1,NAST
            IF(IPAR(I).LT.0)WRITE(6,675)I
            ISAT(I)=ISAT(I)*(-1)**IPAR(I)
          ENDDO
          IF(NASTD.GT.0.AND.IWT.LE.0)WRITE(6,763)
        ENDIF
!
! WRITE TERM INFORMATION FOR USE BY STGICF
!
        IF(IPRKM.EQ.4)THEN
          IF (IAM.EQ.0) THEN      
        OPEN(17,FILE='TERM.DAT',FORM='FORMATTED',STATUS='UNKNOWN')
            WRITE(17,'(I5)') NAST
            AZ=MAX(NZED-NELC,1)
            AZSQ=AZ*AZ
            DO ITT=1,NAST
              ERYD=AZSQ*ENAT(ITT)
              WRITE(17,'(3I5,F16.7)') ABS(ISAT(ITT)),LAT(ITT),
     X                                IPAR(ITT),ERYD
            ENDDO
            CLOSE(17,STATUS='KEEP')
          ENDIF
        ENDIF
!
! WRITE COLLISION STRENGTH OMEGA
! LFM - ONLY ONE OMEGA(U) FILE WRITTEN
!
        IF(NOMWRT.NE.0)THEN
!
          IF(BFORM)THEN
!!           **** PARALLEL ****
!            OPEN(7,FILE="OMEGA"//FILEC//FILED//FILEU//FILEV,
!     +           STATUS="UNKNOWN")
!!           **** END PARALLEL ****
!            REWIND(7)
!           **** SINGLE PROC ****
            IF (IAM.EQ.0) THEN
             OPEN(7,FILE="OMEGA",STATUS="REPLACE")
             WRITE(7,'(2I8)') NZED,NELC
             IF(IPRINT.GT.-1) WRITE(6,650)
             WRITE(7,'(3I8)') NAST,MXE+IBIGE-NEDROP,NOMWRT
             WRITE(7,'(10I8)') (ISAT(I),LAT(I),I=1,NAST)
             WRITE(7,710) (ENAT(I),I=1,NAST)
             IF(NOMWRT.LT.0)THEN               ! GET ROUNDED ENERGIES
               NREC=1+(NAST-1)/5
               DO N=1,NREC
                 BACKSPACE(7)
               ENDDO
               READ(7,710) (ENATR(I),I=1,NAST)
             ENDIF
            END IF
!           **** END SINGLE PROC ****
          ELSE
!!           **** PARALLEL ****
!            OPEN(7,FILE="OMEGAU"//FILEC//FILED//FILEU//FILEV,
!     +           STATUS="UNKNOWN",FORM="UNFORMATTED")
!            **** END PARALLEL ****
!            REWIND(7)
!           **** SINGLE PROC ****
            IF (IAM.EQ.0) THEN
             OPEN(7,FILE="OMEGAU",STATUS="REPLACE",
     &            FORM="UNFORMATTED")
             WRITE(7) NZED,NELC
             IF(IPRINT.GT.-1) WRITE(6,650)
             WRITE(7) NAST,MXE+IBIGE-NEDROP,NOMWRT
             WRITE(7) (ISAT(I),LAT(I),I=1,NAST)
             WRITE(7) (ENAT(I),I=1,NAST)
            END IF
!           **** END SINGLE PROC ****
          ENDIF
!
          IF(NOMWRT.LT.0)THEN
            NTAROP=1
            NTAROR=1
          ENDIF
          Call MPI_Barrier(MPI_COMM_WORLD,IERR)
!
          DO IE=1,MXE
            IF (LFARR(IE)) CYCLE
!
            IF(NOMWRT.GT.0)THEN
              NOMT=NOMWRT
            ELSE
              DO IT=NTAROP,NAST
                IF(EMESH(IE).LT.ENAT(IT)) GO TO 31
              ENDDO
              IT=NAST+1
  31          NTAROP=IT-1
              NOMT=(NTAROP*(NTAROP-2*IONE+1))/2
              NOMT=MIN(NOMT,-NOMWRT)
              NOMT=-NOMT
            ENDIF
!
!            CALL OMEG(IE,EMESH,MXE,ENAT,NAST,OMEGA,NOMT,IONE)
            CALL OMEG(IE)
!           **** PARALLEL ****
            Call MPI_Reduce(OMEGA,OMEGATOT,MXTST,
     &        mpi_real8,mpi_sum,0,MPI_COMM_WORLD,ierr)
!           **** END PARALLEL ****
!
            NOMT=ABS(NOMT)
!
! TO DO - PARALLEL OUTPUT
!           **** SINGLE PROC ****
            IF (IAM.EQ.0) THEN
             IF(IPRINT.GT.-1) WRITE(6,701) EMESH(IE),
     &         (OMEGATOT(N),N=1,NOMT)
!            IF(.NOT.BFORM)WRITE(7)EMESH(IE),(OMEGA(N),N=1,NOMT)
             IF (BFORM) THEN
               IF(NOMWRT.LT.0)THEN  ! GET NOMT CONSISTENT WITH ENATR
                 WRITE(7,700) EMESH(IE)
                 BACKSPACE(7)
                 READ(7,700) E
                 BACKSPACE(7)
                 DO IT=NTAROR,NAST
                   IF(E.LT.ENATR(IT)) GO TO 32
                 ENDDO
                 IT=NAST+1
  32             NTAROR=IT-1
                 NOMTR=(NTAROR*(NTAROR-2*IONE+1))/2
                 NOMTR=MIN(NOMTR,-NOMWRT)
                 IF(NOMTR.GT.NOMT)THEN
                   DO N=NOMT+1,NOMTR
!                     OMEGA(N)=0.0
                     OMEGATOT(N)=0.D0
                   ENDDO
                 ENDIF
                 NOMT=NOMTR
               ENDIF
               WRITE(7,700) EMESH(IE),(OMEGATOT(N),N=1,NOMT)
             ELSE
               WRITE(7) EMESH(IE),(OMEGATOT(N),N=1,NOMT)
             ENDIF
            END IF
!           **** END SINGLE PROC ****
          ENDDO
          IF(IAM.EQ.0) DEALLOCATE(OMEGATOT)
!
! WRITE INFINITE ENERGY OMEGA (WELL EINF Z-SCALED RYDBERGS).
! CURRENTLY, DIPOLE ONLY, BUT LS AND IC.
!
!         **** SINGLE PROC ****
          IF (IAM.EQ.0) THEN
           NOMT=ABS(NOMWRT)
           IF (IBIGE.EQ.1) THEN
             IF (BFORM) THEN
               WRITE(7,700) EINF,(-ABS(SLIN(N)),N=1,NOMT)
             ELSE
               WRITE(7) EINF,(-ABS(SLIN(N)),N=1,NOMT)
             END IF
           ENDIF
!
           CLOSE(7,STATUS="KEEP")
           IF(BFORM) WRITE(6,695)
           IF(.NOT.BFORM) WRITE(6,694)
           IF(ELAS.EQ."YES") WRITE(6,609)
          END IF
!         **** END SINGLE PROC ****
!
        ENDIF
!
! WRITE THE LINE STRENGTHS FOR INPUT TO STGICF - NOTE THE SIGN OF
! SLIN IS THE SIGN OF THE DIPOLE TRANSITION MATRIX ELEMENT
!
        IF(IDIP.GT.0)THEN
!         **** SINGLE PROC ****
          IF (IAM.EQ.0) THEN
            OPEN(31,FILE='STRENGTH.DAT',FORM='FORMATTED'
     X          ,STATUS='UNKNOWN')
            WRITE(31,'(2(I5))') NAST,SIGN(NOMT0,NOMWRT)
            AZ=MAX(NZED-NELC,1)
            CONS=THREE/(LOG(EINF*AZ**2)*FOUR)
            WRITE(31,'(6(1PE12.5))') (CONS*SLIN(N),N=1,NOMT0)
            CLOSE(31,STATUS='KEEP')
          ENDIF
!         **** END SINGLE PROC ****
        ENDIF
!
! TERMINATOR
!
        IF(IPRKM.EQ.4)THEN
          WRITE(19) -1,-1,-1,-1
          CLOSE(19,STATUS='KEEP')
!          IF(IRD0.LT.100) CLOSE(32,STATUS='KEEP')
        END IF 
!
! WRITE OMEGDR
! LFM - TODO REMOVE, USE PSTGFDAMP
!
        IF(NDRMET.GT.0)THEN
!         **** PARALLEL ****
!          OPEN(18,FILE='OMEGDR'//FILEC//FILED//FILEU//FILEV,
!     +         STATUS='UNKNOWN')
          IF(IAM.EQ.0) ALLOCATE(OMEGDRTOT(NDRMET,MXE))
          Call MPI_Reduce(OMEGDR,OMEGDRTOT,ndrmet*mxe,
     &      mpi_real8,mpi_sum,0,MPI_COMM_WORLD,ierr)
!         **** END PARALLEL ****
!         **** SINGLE PROC ****
          IF (IAM.EQ.0) THEN
           OPEN(18,FILE="OMEGDR",STATUS="REPLACE")
!           REWIND(18)
           WRITE(18,'(2I8)') NZED,NELC
           WRITE(18,'(3I8)') NAST,MXE-NEDROP,NDRMET
           WRITE(18,'(10I8)') (ISAT(I),LAT(I),I=1,NAST)
           WRITE(18,710) (ENAT(I),I=1,NAST)
           DO IE=1,MXE
             IF(LFARR(IE)) CYCLE
             WRITE(18,700) EMESH(IE),(OMEGDRTOT(J,IE),J=1,NDRMET)
           ENDDO
!
           CLOSE(18,STATUS='KEEP')
           DEALLOCATE(OMEGDRTOT)
           WRITE(6,696)
          END IF
!         **** END SINGLE PROC ****
!
        ENDIF
!
! WRITE INFO ON ENERGIES AT WHICH S-MATRIX WAS UPDATED IN IQDT MODE.
!
!       IF(IQDT.GT.0.AND.IEQ.NE.1)THEN
!         OPEN(33,FILE='INFQDT',STATUS='UNKNOWN')
!         WRITE(33,*)(IEE(I),I=1,MXE)
!       ENDIF
!
      ENDIF
!
 3200 IF(IRAD.NE.0) THEN
! TODO - REMOVE, NO RADIATION
! ****  SINGLE PROC ****
        If (iam.eq.0) Then
          DO KASE=1,KSLP
            ISLP=MSLP(KASE)
            NSPN2=ISLP/10000
            LRGL2=(ISLP-10000*NSPN2)/100
            NPTY2=ISLP-10000*NSPN2-100*LRGL2
            WRITE(8) NSPN2,LRGL2,NPTY2
          END DO
          WRITE(8) -1,-1,-1
          CLOSE(8,STATUS='KEEP')
          WRITE(6,663)
        End If
! ****  END SINGLE PROC ****
      ENDIF
!
! FINISH
!
! Time control per processor
      Close(40,status="keep")
!
      CALL MPI_BARRIER(MPI_COMM_WORLD, IERR)
      CALL CPU_TIME(TIMEF)
!
! Write license
      WRITE(6,6006)
!
      TIME=TIMEF-TIMEI
      WRITE(6,999) TIME/60.,NPROC
999   FORMAT(//1X,'CPU TIME=',F9.3,' MIN  -- PROCESSORS=:',I4)
!
      CALL MPI_FINALIZE(IERR)
      STOP
!
!  FORMATS
!  *******
!
  600 FORMAT( //10X,'READS IOPT1 = 2 FOLLOWED BY TERMINATOR'//)
  601 FORMAT(//5X,'DATA READ FROM UNIT 5'/
     1  10X,'IPRINT = ',I2/
     2  10X,'IRAD   = ',I2/
     3  10X,'IPERT  = ',I2/
     *  10X,'LMX    = ',I2/
     4  10X,'AC     = ',1PE9.2/
     5  10X,'IMESH  = ',I2/
     6  10X,'IOPT1  = ',I2/
     *  10X,'IRDEC  = ',I2/
     *  10X,'IMODE  = ',I2/
     *  10X,'IQDT   =',I3/
     *  10X,'IEQ    =',I3/
     *  10X,'IOMSW  = ',I2/
     *  10X,'NCUTOFF= ',I7/
     *  10X,'FNUHYB = ',0PF4.1/
     *  10X,'NDRMET = ',I2/
     7  10X,'NTYP1  = ',I2/
     *  10X,'MZPTS  = ',I6//)
  602 FORMAT(/5X,'VALUES OF  10000*IS+100*IL+IP  READ FOR IOPT1=2')
 6021 FORMAT(5X,'PARAMETERS READ FOR IOPT1 INITIALLY NEGATIVE',/
     +       10X,'NASTD  =',I3/10X,'NLEV   =',20I3)
  605 FORMAT(//10X,30('*')//10X,
     + 'NUMBER OF SLPI CASES = ',I3,
     + ' EXCEEDS MAXIMUM OF MZSLP'//10X,30('*')//)
  606 FORMAT(///5X,'***** DATA ON UNIT 10 INSUFFICIENT '
     +   /5X,'***** FOR CALCULATION OF ALL RADIATIVE PROBABILTIES'
     +   /5X,'TRANSITIONS NOT CALCULATED:'
     +   /5X,'    I     J         A    ')
  609   FORMAT(/'*** ATTENTION: OMEGA FILE CONTAINS ELASTIC '
     X           ,'TRANSITIONS BECAUSE DR IS SWITCHED-ON')
  640 FORMAT(//'   ETOT',3X,'QDT',2X,'IPERT',2X,
     1 'INITIAL AND FINAL TARGET LEVELS, AND COLLISION STRENGTHS'/)
  641 FORMAT(/10X,10('+'),' RUN WITH IPERT = 0 ',10('+')/)
  650 FORMAT(///80('*')/80('*')//20X,'ENERGIES AND TOTAL OMEGAS'/
     1  20X,25('*')/)
  660 FORMAT(//10X,30('*')/
     1 10X,'NO DATA ON R-MATRIX FILE FOR'/
     2 10X,'10000*IS+100*IL+IP = '/)
  661 FORMAT(10X,I10)
  662 FORMAT(/10X,30('*')//)
  663 FORMAT(//10X,'RADIATIVE FILE WRITTEN'//)
  675 FORMAT(' WARNING: PARITY OF TARGET STATE',I4
     X  ,' HAS NOT BEEN DETERMINED')
  680 FORMAT(///5X,'ETOT = ',1PE14.6/5X,21('=')/)
  690 FORMAT(//5X,60('*')/5X,'* ETOT = ',E11.3,
     + ' LARGER THAN .5*VALUE(1) = ',E11.3,' *',5X,
     + /5X,'* RESULTS MAY BE INACCURATE.',
     + ' NO MORE SIMILAR WARNINGS.',5X,'*'/5X,60('*')//)
  691 FORMAT(//5X,57('*')/5X,'* ETOT = ',E11.3,
     + ' LARGER THAN .5*EZERO = ',E11.3,' *',5X,
     + /5X,'* RESULTS MAY BE INACCURATE.',
     + ' NO MORE SIMILAR WARNINGS.',2X,'*'/5X,57('*')//)
  694 FORMAT(//10X,46('*')/
     + 10X,'* COLLISION STRENGTHS WRITTEN TO FILE OMEGAU *'/
     + 10X,46('*')//)
  695 FORMAT(//10X,45('*')/
     + 10X,'* COLLISION STRENGTHS WRITTEN TO FILE OMEGA *'/
     + 10X,45('*')//)
  696 FORMAT(//8X,49('*')/
     + 8X,'* DR COLLISION STRENGTHS WRITTEN TO FILE OMEGDR *'/
     + 8X,49('*')//)
  697 FORMAT(10X,30('*')/10X,'NASTD = ',I5,
     + ' IS LARGER THAN MZTAR'/10X,30('*')//)
  699 FORMAT(10X,33('*')/10X,'MXE = ',I6,
     + ' IS LARGER THAN MZMSH'/10X,33('*')//)
  698 FORMAT(/10X,62('*')//10X,'STRONG WARNING: CC OMEGAS ARE BEING'
     X,' USED FOR THE DIPOLE TOP-UP'/25X,
     X ' RECOMMEND SETTING LCBE=LRGLAM'//10X,62('*'))
 6000 FORMAT(///1X,70('+')//9X,'PSTGF UOS V2.6'/
     + 9X,14('*')//5X,'HISTORY OF MODIFICATIONS'//
     + 10X,'(1) BETTER ACCURACY FOR LARGE L '/
     + 10X,'(2) SUM TO INFINITY (TOP-UP) FOR DIPOLE TRANSITIONS'/
     + 10X,'(3) USE OF QDT TO GIVE',
     + ' GAILITIS AVERAGE '/
     + 10X,'(4) USE OF IRDEC TO GIVE RADIATIVE DECAYS '
     + /10X,'(5) HANDLES NEW BUTTLE FIT'
     + /10X,'(6) OPENS FILES B00, B01, ....'
     + /10X,'(7) DATA ON',
     + ' B FILES REQUIRES NUMEROV INTEGRATIONS IN STGBB, STGBF')
 6003 FORMAT(10X,'(8) CORRECTION OF ROUNDING-ERRORS AT THRESHOLDS'
     + /10X,'(9) PRINTS DIMENSIONS '
     + /9X,'(10) COMBINES NEARLY DEGENERATE TARGET LEVELS '
     + /9X,'(11) NAMELIST INPUT'
     + /9X,'(12) REACTANCE MATRIX OUTPUT '
     + /9X,'(13) WORD ADDRESSABLE OUTPUT'/9X
     + ,'(14) PRINTS MAJOR WARNINGS IF IPERT RESET=0 '/9X
     +,'(15) MANY MINOR AND 1 MAJOR DEVELOPMENT SO IPERT NEVER RESET=0'
     + /9X,'(16) TOP-UP CONTROL SEPARATED FROM IPERT '
     + /9X,'(17) QUADRUPOLE TOP-UP INCLUDED'
     + /9X,'(18) DIPOLE TOP-UP CAN BE EVALUATED WITH CBE OMEGAS'
     + /9X,'(19) RADIATIVE DECAYS IN B.P. MODE ADDED'
     +)
 6005 FORMAT(9X,'(20) INFINITE ENERGY OMEGA ADDED (DIPOLE ONLY)'
     + /9X,'(21) DIMENSIONS FROM INCLUDE, NO PREPROCESSING'
     + /9X,'(22) OCTUPOLE ETC. TOP-UP (FOR PSEUDO-STATE IONIZATION)'
     + /9X,'(23) OCTUPOLE ETC. PERTURBING POTENTIALS (FOR DITTO)'
     + /9X,'(24) DETAILED SQDT, WITH TYPE-I DAMPING, FOR DR/RE'
     + /9X,'(25) DETAILED MQDT, WITH TYPE-I DAMPING, FOR DR/RE'
     + /9X,'(26) NEUTRAL CASE ADDED'
     + /9X,'(27) DARC HD.DAT (AND H.DAT VIA DARC INTERFACE)'
     + /9X,'(28) PARTITIONED R-MATRIX'
     +)
 6001 FORMAT(//
     + 5X,'IQDT= 0 (DEFAULT) FOR NON-QDT OPERATION, EXCEPTING GAILITIS'/
     + 22X,'AVERAGE WHEN NU.GT.QNMAX - ANY MESH.'//
     + 5X,'IQDT=-1 FOR QDT APPLIED TO SINGLE CLOSED TARGET-STATE:'/
     + 13X,'DETAILED & AVERAGED, DAMPED & UNDAMPED, EXCITATION & DR.'//
     + 5X,'IQDT=+1 FOR S-MX QDT APPLIED TO ALL CLOSED TARGET-STATES:'/
     + 13X,'DETAILED, DAMPED & UNDAMPED, EXCITATION & DR.'//
     + 5X,'IQDT=+2 DITTO BUT FOR FOR K-MX.'//
     + 5X,'CASE IQDT .GT. 0 THEN:'//
     + 5X,'IMODE= 0 (DEFAULT) WRITE UNPHYSICAL K/S-MATRIX TO FILE.'//
     + 5X,'IMODE=+1 READ (AND INTERPOLATE) K/S-MATRIX FROM FILE.'//
     + 5X,'IMODE=-1 SINGLE PASS: COMPUTE AND INTERPOLATE (NO FILES).'//
     + 5X,'IN THIS CASE SET IEQ EQUAL TO THE NUMBER OF COARSE MESH'
     +    ,' ENERGIES.'//
     +//1X,70('+')/)
! 6002 FORMAT(//10X,'COMPILED FOR DIMENSIONS -'//
!     + 15X,'CHANNELS                   MZCHF =',I6/
!     + 15X,'TARGET STATES              MZTAR =',I6/
!     + 15X,'MULTIPOLES                 MZLMX =',I6/
!     + 15X,'SMALL L VALUES             MZLP1 =',I6/
!     + 15X,'OUTER-REGION RADIAL POINTS MZPTS =',I6/
!     + 15X,'ENERGY-MESH POINTS (PHYS)  MZMSH =',I6/
!     + 15X,'R-MATRIX POLES             MZMNP =',I6/
!     + 15X,'S, L, PI CASES             MZSLP =',I6/
!     + 15X,'COEFFICIENTS FOR THETA     MZTET =',I6/
!     + 15X,'DEGENERATE CHANNELS        MZDEG =',I6/
!     + 15X,'TERMS IN BUTTLE FIT        MZNRG =',I6/
!     + 15X,'METASTABLES FOR DR         MZMET =',I6/
!     + 15X,'RECORD LENGTH FOR ONE WORD MZREC =',I6/
!     + 15X,'M-WORDS OF INTERNAL MEM.   MZMEG =',I6//)
 6006 FORMAT(//" RMATRX-PSTGF  Copyright (C) 2019"
     &   /" Michael J. Seaton, University College London;"
     &   /" Dario M. Mitnik, Universidad de Buenos Aires;"
     &   /" Nigel R. Badnell, University of Strathclyde;"
     &   /" Connor P. Ballance, Queen's University Belfast;"
     &   /" Luis Fernandez-Menchero, Queen's University Belfast."
     &   /" This work used the ARCHER UK National Supercomputing"
     &   ," Service (http://www.archer.ac.uk)."
     &  //" This program is free software;"
     &   ," you can redistribute it and/or"
     &   /" modify it under the terms of the GNU"
     &   ," Lesser General Public"
     &   /" License as published by the Free Software Foundation;"
     &   ," either"
     &   /" version 2.1 of the License,"
     &   ," or (at your option) any later version."
     &  //" This program is distributed in the hope"
     &   ," that it will be useful,"
     &   /" but WITHOUT ANY WARRANTY;"
     &   ," without even the implied warranty of"
     &   /" MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE."
     &   ," See the GNU"
     &   /" Lesser General Public License for more details."
     &  //" You should have received a copy of the "
     &   ," GNU Lesser General Public"
     &   /" License along with this program;"
     &   ," if not, write to the Free Software"
     &   /" Foundation, Inc., 51 Franklin Street, Fifth Floor,"
     &   ," Boston, MA  02110-1301  USA"
     &   /" https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html")
  700 FORMAT(1PE14.8,6(1PE11.3)/(14X,6(E11.3)))
  701 FORMAT(1PE12.5,6(1PE11.3)/(12X,6(E11.3)))
  710 FORMAT(1P5E16.6)
  763 FORMAT(//' *********WARNING, STAT. WEIGHTS IN OMEGA FILE ARE '
     X,'MEANINGLESS FOR GROUPED TERMS/LEVELS !!'//)
!
      END PROGRAM MAIN
!
!**********************************************************
!
      SUBROUTINE ABG(E0,L,AC,A,BG)
!
!  COMPUTES FUNCTION G(X,L) TO ACCURACY AC.
!
      IMPLICIT REAL*8 (A-H,O-Z)
!
      PARAMETER (TZERO=0.0)
      PARAMETER (ONE=1.0)
      PARAMETER (TWO=2.0)
      PARAMETER (P0=ONE/252.)
      PARAMETER (P1=1.05)
      PARAMETER (P2=2.1)
!
      PI=ACOS(-ONE)
      E=E0
      X=ONE/SQRT(-E)
!
!  CALCULATION OF A AND EAC=E*A*C
!
      IF(L.GT.0)GOTO 2
      A=ONE
      EAC=TZERO
      GOTO 20
    2 IF(L.GT.1)GOTO 4
      A=ONE+E
      EAC=E
      GOTO 20
    4 IF(X.LT.DBLE(L+1)) GOTO 12
      C=TZERO
      A=ONE
      A1=1
      A2=-E
      A3=TWO*E
      DO 10 I=1,L
      A2=A2+A3
      A1=A1+A2
      A=A*A1
      C=C+DBLE(I)/A1
   10 CONTINUE
      EAC=E*A*C
      GOTO 20
!  CASE OF X.LT.(L+1)
   12 A=ONE
      A1=TZERO
      DO 16  I=1,L
      A=A*(ONE+DBLE(I*I)*E)
      A2=DBLE(I)
      DO 14 J=1,L
      IF(J.EQ.I) GOTO 14
      A2=A2*(ONE+DBLE(J*J)*E)
   14 CONTINUE
      A1=A1+A2
   16 CONTINUE
      EAC=E*A1
!
!  COMPUTE A1=PI*BG/A-E*C=1/(2*X)+PSI(X)-LN(X)
!
   20 A1=TZERO
!
!  TEST CONVERGENCE OF ASYMPTOTIC EXPANSION
!
      XN=(754.*AC)**(-.125)
      IF(X.GT.XN)GOTO 40
!
!  USE RECURRENCE FORMULAE
!
      N=XN-X+1
      XN=X+N
      E=-ONE/(XN*XN)
      A1=A1-(ONE/X+ONE/XN)/TWO+LOG(XN/X)
      IF(N.LT.2)GOTO 40
      N=N-1
      DO 30 I=1,N
      A1=A1-ONE/(X+DBLE(I))
   30 CONTINUE
!
!  USE ASYMPTOTIC EXPANSION
!
   40 A1=A1+(((P1*E+ONE)*E+P2)*E+21)*E*P0
!
!  COMPLETE CALCULATION
!
      BG=(A*A1+EAC)/PI
!
      RETURN
      END
!
!**********************************************************
!
      SUBROUTINE ACSUB
      USE CACC
!
      IMPLICIT REAL*8 (A-H,O-Z)
!
      ACNUM=(24.*AC)**.166666667
      ACJWBK=(6.*AC)**.2
      ACZP=16.*AC
      LACC=0
      IF(AC.LT.1.E-3)LACC=2
      IF(AC.LT.1.E-4)LACC=4
!
      RETURN
      END
!***************************************************************
!
      SUBROUTINE AINVB(LFAIL)
      USE CHAN
      USE NRBRCT
      USE NRBWRK
      USE NRBCBE
      USE NRBQDT
      USE NRBOMT
      USE CNTRL
!
!NRB:
!  COMPUTE K=-A**(-1)*B
!  USE KAB'S PARTITIONING, BUT SOLVE A*K=-B INSTEAD.
!
      IMPLICIT REAL*8 (A-H,O-Y)
      IMPLICIT COMPLEX*16 (Z)
      LOGICAL,INTENT(OUT) :: LFAIL
!      PARAMETER (LWORK=MZCHF*MZCHF)
!      PARAMETER (MWORK=MZDEG*MZDEG)
!
      PARAMETER (TZERO=0.0)
      PARAMETER (ONE=1.0)
      PARAMETER (TWO=2.0)
!
!
       INTEGER,ALLOCATABLE :: IPIV3(:),IPIVA(:)
!
      LFAIL=.FALSE.
      ALLOCATE(IPIV3(NCHF))
      ALLOCATE(IPIVA(NCHF))
!
      IF(ALLOCATED(WORKM))DEALLOCATE(WORKM)
      ALLOCATE(WORKM(NCHF,NCHF))

      TEMP1=>RMAT
      TEMP2=>DFPP
      TEMP44=>WORKM     
      TEMP33=>RBE
!
      NHOLD=NCHPP
!TEST IF(IQDT.GT.0) NCHPP=NCHF/2            !MAKE USE OF SYMMETRY OF K
      NCC=NCHF-NCHPP
!
      DO I=1,NCHF
        IF(IOMIT(I).GT.0)A(I,I)=ONE
      ENDDO
!
      IF(NCC.GT.0) THEN
!
!  KOO=-[AOO-AOC*ACC**(-1)*ACO]**(-1)*[BOO-AOC*ACC**(-1)*BCO]
!
        DO J=1,NCC
          JJ=NCHPP+J
          DO I=1,NCHPP
            TEMP44(J,I)=A(I,JJ)                     !TRANSPOSE
          ENDDO
          DO I=1,NCC
            TEMP33(J,I)=A(JJ,NCHPP+I)
          ENDDO
        ENDDO
!
!STRTNBL
!NBL    CALL LU(TEMP33,NCHF,NCC,IERR)
!NBL    IF (IERR.NE.0) THEN
!NBL      WRITE(6,600)IERR
!NBL      LFAIL=.TRUE.
!NBL      RETURN
!!NBL     STOP ' ERROR IN LU'
!NBL    END IF
!NBL    CALL LUBT(TEMP33,TEMP44,NCHF,NCHPP,IERR)    !TRANSPOSE
!NBL    IF (IERR.NE.0) THEN
!NBL      WRITE(6,601)IERR
!NBL      LFAIL=.TRUE.
!NBL      RETURN
!!NBL     STOP ' ERROR IN LUBT'
!NBL    END IF
!ENDNBL
!
!STRTBL
        CALL DGETRF(NCC,NCC,TEMP33,NCHF,IPIV3,INFO)
        IF(INFO.NE.0)THEN
          WRITE(6,602)INFO
          LFAIL=.TRUE.
          RETURN
!          STOP ' ERROR IN DGETRF'
        ENDIF
        CALL DGETRS('T',NCC,NCHPP,TEMP33,NCHF,IPIV3,TEMP44,NCHF,INFO)
        IF(INFO.NE.0)THEN
          WRITE(6,603)INFO
          LFAIL=.TRUE.
          RETURN
!          STOP ' ERROR IN DGETRS'
        ENDIF
!ENDBL
!
! TRANSPOSE BACK
!
        DO I=1,NCHPP
          DO J=1,NCC
            TEMP2(I,J)=TEMP44(J,I)
          ENDDO
        ENDDO
        DO I=1,NCHPP
          DO J=1,NCC
            TEMP44(I,J)=TEMP2(I,J)
          ENDDO
        ENDDO
!
        DO J=1,NCHPP
          DO I=1,NCC
            TEMP2(I,J)=A(NCHPP+I,J)                     !ACO
          ENDDO
        ENDDO
!
!STRTNBL
!NBL    DO J=1,NCHPP
!NBL      DO I=1,NCHPP
!NBL        TEMP1(I,J)=TZERO
!NBL      ENDDO
!NBL      DO K=1,NCC
!NBL        DO I=1,NCHPP
!NBL          TEMP1(I,J)=TEMP1(I,J)+TEMP44(I,K)*TEMP2(K,J)
!NBL        ENDDO
!NBL      ENDDO
!NBL    ENDDO
!ENDNBL
!
!STRTBL
        CALL DGEMM('N','N',NCHPP,NCHPP,NCC,ONE,
     X             TEMP44,NCHF,TEMP2,NCHF,TZERO,TEMP1,NCHF)
!ENDBL
!
        DO J=1,NCHPP
          DO I=1,NCHPP
            A(I,J)=A(I,J)-TEMP1(I,J)
          ENDDO
        ENDDO
        DO J=1,NCHPP
          DO I=1,NCC
            TEMP1(I,J)=B(NCHPP+I,J)
          ENDDO
        ENDDO
!
!STRTNBL
!NBL    DO J=1,NCHPP
!NBL      DO I=1,NCHPP
!NBL        TEMP2(I,J)=TZERO
!NBL      ENDDO
!NBL      DO K=1,NCC
!NBL        DO I=1,NCHPP
!NBL          TEMP2(I,J)=TEMP2(I,J)+TEMP44(I,K)*TEMP1(K,J)
!NBL        ENDDO
!NBL      ENDDO
!NBL    ENDDO
!ENDNBL
!
!STRTBL
        CALL DGEMM('N','N',NCHPP,NCHPP,NCC,ONE,
     X             TEMP44,NCHF,TEMP1,NCHF,TZERO,TEMP2,NCHF)
!ENDBL
!
        DO J=1,NCHPP
          DO I=1,NCHPP
            B(I,J)=B(I,J)-TEMP2(I,J)
          ENDDO
        ENDDO
        DO J=1,NCHPP
          DO I=1,NCC
            TEMP44(NCHPP+I,J)=A(NCHPP+I,J)             !HOLD ACO
          ENDDO
        ENDDO
!
      ENDIF
!
! ENTRY POINT FOR ALL CHANNELS OPEN
!
      DO J=1,NCHPP
        DO I=1,NCHPP
          RK(I,J)=-B(I,J)
        ENDDO
      ENDDO
!
!STRTNBL
!NBL  CALL LU(A,NCHF,NCHPP,IERR)                    !A DESTROYED
!NBL  IF (IERR.NE.0) THEN
!NBL    WRITE(6,600)IERR
!NBL      LFAIL=.TRUE.
!NBL      RETURN
!!NBL     STOP ' ERROR IN LU'
!NBL  END IF
!NBL  CALL LUB(A,RK,NCHF,NCHPP,IERR)
!NBL  IF (IERR.NE.0) THEN
!NBL    WRITE(6,601)IERR
!NBL      LFAIL=.TRUE.
!NBL      RETURN
!!NBL     STOP ' ERROR IN LUB'
!NBL  END IF
!ENDNBL
!
!STRTBL
      CALL DGETRF(NCHPP,NCHPP,A,NCHF,IPIVA,INFO)
      IF(INFO.NE.0)THEN
        WRITE(6,602)INFO
        LFAIL=.TRUE.
        RETURN
!        STOP ' ERROR IN DGETRF'
      ENDIF
      CALL DGETRS('N',NCHPP,NCHPP,A,NCHF,IPIVA,RK,NCHF,INFO)
      IF(INFO.NE.0)THEN
        WRITE(6,603)INFO
        LFAIL=.TRUE.
        RETURN
!        STOP ' ERROR IN DGETRS'
      ENDIF
!ENDBL
!
!  SYMMETRISE OPEN-OPEN PART OF REACTANCE MATRIX
!
      DO I=1,NCHPP-1
         DO J=I+1,NCHPP
            RK(I,J)=(RK(I,J)+RK(J,I))/TWO
            RK(J,I)=RK(I,J)
         ENDDO
      ENDDO
!
      IF(NCC.EQ.0)THEN
      NULLIFY(TEMP1,TEMP2,TEMP44)
      DEALLOCATE(WORKM,IPIV3,IPIVA)
      RETURN
      ENDIF
!
!
      IF(IRAD.GT.0.OR.IPERT.LT.0.OR.IQDT.GT.0)THEN
!
!  KCO=-ACC**(-1)*[BCO+ACO*KOO]
!
        DO J=1,NCHPP
          DO I=1,NCC
            TEMP1(I,J)=TEMP44(NCHPP+I,J)               !ACO
            TEMP2(I,J)=B(NCHPP+I,J)
          ENDDO
        ENDDO
!
!STRTNBL
!NBL    DO J=1,NCHPP
!NBL      DO K=1,NCHPP
!NBL        DO I=1,NCC
!NBL          TEMP2(I,J)=TEMP2(I,J)+TEMP1(I,K)*RK(K,J)
!NBL        ENDDO
!NBL      ENDDO
!NBL    ENDDO
!
!NBL    CALL LUB(TEMP33,TEMP2,MZCHF,NCHPP,IERR)
!NBL    IF (IERR.NE.0) THEN
!NBL      WRITE(6,601)IERR
!NBL      LFAIL=.TRUE.
!NBL      RETURN
!!NBL     STOP ' ERROR IN LUB'
!NBL    END IF
!ENDNBL
!
!STRTBL
        CALL DGEMM('N','N',NCC,NCHPP,NCHPP,ONE,
     X             TEMP1,NCHF,RK,NCHF,ONE,TEMP2,NCHF)
        CALL DGETRS('N',NCC,NCHPP,TEMP33,NCHF,IPIV3,TEMP2,NCHF,INFO)
        IF(INFO.NE.0)THEN
          WRITE(6,603)INFO
          LFAIL=.TRUE.
          RETURN
!          STOP ' ERROR IN DGETRS'
        ENDIF
!ENDBL
!
        DO J=1,NCHPP
          DO I=1,NCC
            RK(NCHPP+I,J)=-TEMP2(I,J)
          ENDDO
        ENDDO
!
!  COPY CLOSED-OPEN TO OPEN-CLOSED
!
        DO J=NCHPP+1,NCHF
          DO I=1,NCHPP
            RK(I,J)=RK(J,I)
          ENDDO
        ENDDO
!
      ENDIF
!
      IF(IQDT.GT.0.AND.IPERT.GT.0)THEN                  !TEST ONLY
!
!  KOC=-[AOO-AOC*ACC**(-1)*ACO]**(-1)*[BOC-AOC*ACC**(-1)*BCC]
!
        DO J=1,NCC
          DO I=1,NCC
            TEMP1(I,J)=B(NCHPP+I,NCHPP+J)
          ENDDO
        ENDDO
        DO J=1,NCC
          DO I=1,NCHPP
            TEMP2(I,J)=B(I,NCHPP+J)
          ENDDO
        ENDDO
!
!STRTNBL
        DO J=1,NCC
          DO K=1,NCC
            DO I=1,NCHPP
              TEMP2(I,J)=TEMP2(I,J)-TEMP44(I,K)*TEMP1(K,J)
            ENDDO
          ENDDO
        ENDDO
        CALL LUB(A,TEMP2,NCHF,NCC,IERR)
        IF (IERR.NE.0) THEN
          WRITE(6,601)IERR
          LFAIL=.TRUE.
          RETURN
!          STOP ' ERROR IN LUB'
        END IF
!ENDNBL
!
!STRTBL
!BL     CALL DGEMM('N','N',NCHPP,NCC,NCC,-ONE,
!BL  +             TEMP44,NCHF,TEMP1,NCHF,ONE,TEMP2,NCHF)
!BL     CALL DGETRS('N',NCHPP,NCC,A,NCHF,IPIVA,TEMP2,NCHF,INFO)
!BL     IF(INFO.NE.0)THEN
!BL       WRITE(6,603)INFO
!BL       LFAIL=.TRUE.
!BL       RETURN
!!BL      STOP ' ERROR IN DGETRS'
!BL     ENDIF
!ENDBL
!
        DO J=1,NCC
          DO I=1,NCHPP
            RK(I,NCHPP+J)=-TEMP2(I,J)
          ENDDO
        ENDDO
!
!  SYMMETRIZE "CLOSED"-OPEN AND OPEN-"CLOSED" PART OF REACTANCE MATRIX
!
        DO J=NCHPP+1,NCHF
          DO I=1,NCHPP
            RK(I,J)=(RK(I,J)+RK(J,I))/TWO
            RK(J,I)=RK(I,J)
          ENDDO
        ENDDO
!
      ENDIF
!
      IF(IQDT.GT.0)THEN                               !TEST
!
!  KCC=-ACC**(-1)*[BCC+ACO*KOC]
!
        DO J=1,NCHPP
          DO I=1,NCC
            TEMP1(I,J)=TEMP44(NCHPP+I,J)               !ACO
          ENDDO
        ENDDO
        DO J=1,NCC
          DO I=1,NCHPP
            TEMP2(I,J)=RK(I,NCHPP+J)
          ENDDO
        ENDDO
        DO J=1,NCC
          DO I=1,NCC
            TEMP44(I,J)=B(NCHPP+I,NCHPP+J)
          ENDDO
        ENDDO
!
!STRTNBL
        DO J=1,NCC
          DO K=1,NCHPP
            DO I=1,NCC
              TEMP44(I,J)=TEMP44(I,J)+TEMP1(I,K)*TEMP2(K,J)
            ENDDO
          ENDDO
        ENDDO
        CALL LUB(TEMP33,TEMP44,NCHF,NCC,IERR)
        IF (IERR.NE.0) THEN
          WRITE(6,601)IERR
          LFAIL=.TRUE.
          RETURN
!          STOP ' ERROR IN LUB'
        END IF
!ENDNBL
!
!STRTBL
!BL     CALL DGEMM('N','N',NCC,NCC,NCHPP,ONE,
!BL  X             TEMP1,NCHF,TEMP2,NCHF,ONE,TEMP44,NCHF)
!BL     CALL DGETRS('N',NCC,NCC,TEMP33,NCHF,IPIV3,TEMP44,NCHF,INFO)
!BL     IF(INFO.NE.0)THEN
!BL       WRITE(6,603)INFO
!BL       LFAIL=.TRUE.
!BL       RETURN
!!BL      STOP ' ERROR IN DGETRS'
!BL     ENDIF
!ENDBL
!
        DO J=1,NCC
          DO I=1,NCC
            RK(NCHPP+I,NCHPP+J)=-TEMP44(I,J)
          ENDDO
        ENDDO
!
!  SYMMETRISE "CLOSED"-"CLOSED" PART OF REACTANCE MATRIX
!
        DO I=NCHPP+1,NCHF
           DO J=I+1,NCHF
              RK(I,J)=(RK(I,J)+RK(J,I))/TWO
              RK(J,I)=RK(I,J)
           ENDDO
        ENDDO
!
      ENDIF
!
      NCHPP=NHOLD
      NULLIFY(TEMP1,TEMP2,TEMP44,TEMP33)
      DEALLOCATE(WORKM,IPIVA,IPIV3)
      RETURN
!
  600 FORMAT(' SR.AINVB: LU RETURNED WITH INFO = ',I4)
  601 FORMAT(' SR.AINVB: LUB RETURNED WITH INFO = ',I4)
  602 FORMAT(//10X,10('*'),' SR. AINVB: DGETRF RETURNED WITH INFO = ',
     &       I4)
  603 FORMAT(//10X,10('*'),' SR. AINVB: DGETRS RETURNED WITH INFO = ',
     &       I4)
      END
!***************************************************************
!
      SUBROUTINE ALPHA(NCHOPO)
      USE CALP
      USE CHAN
      USE NRBQDT
      USE NRBOMT
!
! NRB;
! EVALUATE LONG-RANGE PERTURBING INTEGRALS
!
      IMPLICIT REAL*8 (A-H,O-Y)
      PARAMETER(TZERO=0.0)
!
!     FORTUNATELY APLHA HAS ONLY ONE ENTRY POINT IN THE CODE
!
      IF(ALLOCATED(ASS)) DEALLOCATE(ASS)
      IF(ALLOCATED(ASC)) DEALLOCATE(ASC)
      IF(ALLOCATED(ACS)) DEALLOCATE(ACS)
      IF(ALLOCATED(ACC)) DEALLOCATE(ACC)
!
      ALLOCATE(ASS(NCHF,NCHF))
      ALLOCATE(ASC(NCHF,NCHF))
      ALLOCATE(ACS(NCHF,NCHF))
      ALLOCATE(ACC(NCHF,NCHF))
!
!
! EVALUATE ALPHA INTEGRALS IN IQDT AND NON-IQDT CASES
!
      IF(IQDT.EQ.0.OR.INTPQ.NE.0)THEN
        CALL ALPHAN(NCHOPO)
      ELSE
        CALL ALPHAQ(NCHOPO)
      ENDIF
!
! ZERO-OUT INTEGRALS FOR OMITTED CHANNELS (IF NOT ALREADY DONE)
!
      DO J=1,NCHF
        DO I=1,NCHF
          IF(IOMIT(I).GT.0.OR.IOMIT(J).GT.0)THEN
            ASS(I,J)=TZERO
            ASC(I,J)=TZERO
            ACS(I,J)=TZERO
            ACC(I,J)=TZERO
          ENDIF
        ENDDO
      ENDDO
!
      RETURN
      END
!***************************************************************
!
      SUBROUTINE ALPHAN(NCHOP)
      USE NRBRCT
      USE CALP
      USE COULSC
      USE CPOT
      USE CBODE
      USE CHAN
      USE NRBLMX
      USE NRBQDT
      USE NRBOMT
      USE CACC
      USE CPOINT
      USE CNTRL
      USE CQDT
!
! NRB: WE'S ALPHA WITH INDICES OF FS(K,I),FC(K,J) INTERCHANGED
!      (THROUGHOUT THE CODE) FOR GREATER SPEED. ALSO USE OF TEMP
!      FOR A LITTLE SOMETHING EXTRA.      25/07/97
!
!
!  CALCULATES ALPHA INTEGRALS
!
      IMPLICIT REAL*8 (A-H,O-Y)
      IMPLICIT COMPLEX*16 (Z)
      PARAMETER (TZERO=0.0)
      PARAMETER (ONE=1.0)
      PARAMETER (TWO=2.0)
      INTEGER,ALLOCATABLE :: IOMTT(:)
      REAL*8,ALLOCATABLE :: CNORM(:),SNORM(:)
!
      ALLOCATE(IOMTT(NCHF))
      ALLOCATE(CNORM(NCHF))
      ALLOCATE(SNORM(NCHF))
!
      PI=ACOS(-ONE)
      NCHOP1=NCHOP+1
      IOCINT=IABS(ICCINT)
      JMAX=NCHF
      IF(IOCINT.NE.1)JMAX=NCHOP
!
!  INITIALISE ALPHA TO ZERO
!
      DO I=1,NCHF
        DO J=I,NCHF
          ASS(J,I)=TZERO
          ASC(J,I)=TZERO
          ACS(J,I)=TZERO
          ACC(J,I)=TZERO
        ENDDO
      ENDDO
!
      DO I=1,NCHF
        IOMTT(I)=IOMIT(I)
        IF(RINF(I).LT.TZERO)IOMTT(I)=1
        IF(RINF(I).GT.RZERO.AND.R2ST(I).GT.RTWO)IOMTT(I)=1
        IF(RINF(I).GT.RZERO.AND.IPERT.GT.0)IOMTT(I)=1
      ENDDO
!
!
!  CONTRIBUTION FROM RZERO TO RTWO
!  START LOOP ON RADIAL POINTS
!
! NRB: THIS RZERO TO RTWO PART IS VERY TIME CONSUMING,
!      EVEN FOR JUST A FEW HUNDRED POINTS.
!
      IF(KP2.GT.1)THEN
!
!  OPEN-OPEN AND OPEN-CLOSED PARTS
!
        IF(NCHOP.NE.0) THEN
          DO I=1,NCHOP
            IF(IOMTT(I).EQ.0)THEN
            DO J=I,JMAX
              LIJ=LAMP(J,I)
              IF(LIJ.NE.1.AND.LIJ.LE.LMX+1.AND.IOMTT(J).EQ.0) THEN
                A1=BW(J,I)
                A2=ASS(J,I)
                A3=ACS(J,I)
                A4=ASC(J,I)
                A5=ACC(J,I)
                DO K=1,KP2
                  WIJ=TBODE(K,LIJ)*A1
                  T1=WIJ*FS(K,J)
                  T2=WIJ*FC(K,J)
                  A2=A2+FS(K,I)*T1
                  A3=A3+FS(K,I)*T2
                  A4=A4+FC(K,I)*T1
                  A5=A5+FC(K,I)*T2
                ENDDO
                ASS(J,I)=A2
                ACS(J,I)=A3
                ASC(J,I)=A4
                ACC(J,I)=A5
              ENDIF
            ENDDO
            ENDIF
          ENDDO
        ENDIF
!
!  CLOSED-CLOSED PART
!
        IF(ICCINT.EQ.1.AND.NCHOP.NE.NCHF) THEN
          DO I=NCHOP1,NCHF
            IF(IOMTT(I).EQ.0)THEN
            DO J=I,NCHF
              LIJ=LAMP(J,I)
              IF(LIJ.NE.1.AND.LIJ.LE.LMX+1.AND.IOMTT(J).EQ.0) THEN
                A1=BW(J,I)
                A2=ASS(J,I)
                A3=ASC(J,I)
                A4=ACS(J,I)
                DO K=1,KP2
                  WIJ=TBODE(K,LIJ)*A1
                  T1=WIJ*FS(K,J)
                  A2=A2+FS(K,I)*T1
                  A3=A3+FC(K,I)*T1
                  A4=A4+FS(K,I)*WIJ*FC(K,J)
                ENDDO
                ASS(J,I)=A2
                ASC(J,I)=A3
                ACS(J,I)=A4
              ENDIF
            ENDDO
            ENDIF
          ENDDO
        ENDIF
      ENDIF
!
!
!  ASYMPTOTIC INTEGRALS FOR R=RTWO TO INFINITY
!
! NRB: THIS IS FAST COMPARED TO RZERO TO RTWO, EXCEPT CASE
!      WHEN RZERO APPROXS RTWO.
!
      DO I=1,NCHF
        IF(R2ST(I).GT.RTWO)IOMTT(I)=1
      ENDDO
!
      IF(ICCINT.EQ.1.AND.NCHOP.EQ.0) THEN
        CALL CCINT(IOMTT,NCHOP)
      ELSE
        CALL OOINT(IOMTT,NCHOP)
        IF(NCHOP.NE.NCHF) THEN
          IF(IOCINT.EQ.1)CALL OCINT(IOMTT,NCHOP)
          IF(ICCINT.EQ.1)CALL CCINT(IOMTT,NCHOP)
        ENDIF
      ENDIF
!
! CONVERT TO (P) Q INTEGRALS
!
      IF(IQDT.GT.0.AND.INTPQ.NE.0)THEN
!
! OPEN-CLOSED
!
        DO J=NCHOP1,NCHF
          TNORM=SIN(PI*FKNU(J))*C(J)-COS(PI*FKNU(J))*S(J)
          TNORM=TNORM/FS(1,J)
          CNORM(J)=TNORM*COS(PI*FKNU(J))
          SNORM(J)=TNORM*SIN(PI*FKNU(J))
        ENDDO
        DO I=1,NCHOP
          DO J=NCHOP1,NCHF
            ACS(J,I)=ASS(J,I)
            ACC(J,I)=ASC(J,I)
            ASS(J,I)=-CNORM(J)*ASS(J,I)
            ASC(J,I)=-CNORM(J)*ASC(J,I)
            ACS(J,I)=SNORM(J)*ACS(J,I)
            ACC(J,I)=SNORM(J)*ACC(J,I)
          ENDDO
        ENDDO
!
! CLOSED-CLOSED
!
        DO I=NCHOP1,NCHF
          DO J=I,NCHF
            ASC(J,I)=ASS(J,I)
            ACS(J,I)=ASS(J,I)
            ACC(J,I)=ASS(J,I)
            ASS(J,I)=CNORM(J)*CNORM(I)*ASS(J,I)
            ASC(J,I)=-CNORM(J)*SNORM(I)*ASC(J,I)
            ACS(J,I)=-SNORM(J)*CNORM(I)*ACS(J,I)
            ACC(J,I)=SNORM(J)*SNORM(I)*ACC(J,I)
          ENDDO
        ENDDO
!
      ENDIF
!
!  SYMMETRISE ALPHA
!
      DO I=1,NCHF
        AA=(ASC(I,I)+ACS(I,I))/TWO
        ASC(I,I)=AA
        ACS(I,I)=AA
      ENDDO
!
      IF(NCHF.NE.1) THEN
        DO I=2,NCHF
          K=I-1
          DO J=1,K
            ASS(J,I)=ASS(I,J)
            ACC(J,I)=ACC(I,J)
            ASC(J,I)=ACS(I,J)
            ACS(J,I)=ASC(I,J)
          ENDDO
        ENDDO
      ENDIF
!
      IF(IPRINT.GT.1)THEN
        WRITE(6,750)
        DO I=1,NCHOP
          DO J=I,NCHOP
            LIJ=LAMP(J,I)
            IF(LIJ.NE.1.AND.LIJ.LE.LMX+1) THEN
              WRITE(6,760)J,I,ASS(J,I),ASC(J,I),ACS(J,I),ACC(J,I)
            ENDIF
          ENDDO
        ENDDO
        IF(IOCINT.EQ.1)THEN
        DO I=1,NCHOP
          DO J=NCHOP1,NCHF
            LIJ=LAMP(J,I)
            IF(LIJ.NE.1.AND.LIJ.LE.LMX+1) THEN
              WRITE(6,760)J,I,ASS(J,I),ASC(J,I)
            ENDIF
          ENDDO
        ENDDO
        ENDIF
        IF(ICCINT.EQ.1)THEN
        DO I=NCHOP1,NCHF
          DO J=NCHOP1,NCHF
            LIJ=LAMP(J,I)
            IF(LIJ.NE.1.AND.LIJ.LE.LMX+1) THEN
              WRITE(6,760)J,I,ASS(J,I),ASC(J,I),ACS(J,I)
            ENDIF
          ENDDO
        ENDDO
        ENDIF
      ENDIF
!
      DEALLOCATE(IOMTT)
      DEALLOCATE(CNORM,SNORM)
!
  750 FORMAT(/' J,I AND ASS(J,I), ASC(J,I), ACS(J,I), ACC(J,I)'/)
  760 FORMAT(2I5,4E14.6)
!
      RETURN
      END
!**********************************************************
!
      SUBROUTINE ALPHAQ(NCHOP)
      USE CALP 
      USE COULSC 
      USE CPOT
      USE CBODE
      USE CHAN
      USE NRBLMX
      USE NRBQDT
      USE NRBOMT
      USE CACC
      USE CPOINT
      USE CNTRL
      USE CQDT
      USE NRBDD2
      USE NRBZED
!
! NRB:
!  CALCULATES ALPHA INTEGRALS FOR MQDT CASE
!  INTERFACES WITH FR'S CORINT ROUTINE.
!
      IMPLICIT REAL*8 (A-H,O-Z)
      PARAMETER (TZERO=0.0)
      PARAMETER (ONE=1.0)
      PARAMETER (TWO=2.0)
      INTEGER, ALLOCATABLE :: IOMTT(:)
!
      ALLOCATE(IOMTT(NCHF))
!
!  INITIALISE ALPHA TO ZERO
!
      DO J=1,NCHF
        DO I=J,NCHF
          ASS(I,J)=TZERO
          ASC(I,J)=TZERO
          ACS(I,J)=TZERO
          ACC(I,J)=TZERO
        ENDDO
      ENDDO
!
      IF(ICCINT.NE.1.AND.NCHOP.EQ.0) RETURN
!
      IF(IFDD2.NE.0)STOP'ALPHAQ: NEED DERIVATIVES AT RTWO'
!
      DO I=1,NCHF
        IOMTT(I)=IOMIT(I)*2
        IF(IOMIT(I).LE.0)THEN
          IF(I.GT.NCHOP.AND.R2ST(I).LT.RTWOC)IOMTT(I)=1
          IF(RINF(I).LE.TZERO)IOMTT(I)=2
          IF(RINF(I).GT.RZERO.AND.IPERT.GT.0)IOMTT(I)=2
        ENDIF
      ENDDO
!
      NCHOP1=NCHOP+1
      IOCINT=IABS(ICCINT)
      JMAX=NCHF
      IF(IOCINT.NE.1)JMAX=NCHOP
!
!
!  CONTRIBUTION FROM RZERO TO RTWO
!  START LOOP ON RADIAL POINTS
!
! NRB: THIS RZERO TO RTWO/C PART IS VERY TIME CONSUMING,
!      EVEN FOR JUST A FEW HUNDRED POINTS.
!
      IF(IRAD.EQ.0)THEN
        R0=RTWOC
        KP2T=KP2C
      ELSE             !RTWO ARTIFICIALLY LARGE (BUT LOW-L SO....)
        R0=RZERO
        KP2T=1
        IF(KP0.NE.KP2T)STOP'KP0 .NE. 1 ??'
      ENDIF
!
      IF(KP2T.GT.1)THEN
!
!  OPEN-OPEN AND OPEN-CLOSED PARTS
!
        IF(NCHOP.GT.0) THEN
          DO I=1,NCHOP
            DO J=I,JMAX
              LIJ=LAMP(J,I)
              IF(LIJ.NE.1.AND.LIJ.LE.LMX+1.AND.IOMTT(J).LE.1) THEN
                DO K=1,KP2T
                  WIJ=TBODE(K,LIJ)*BW(J,I)
                  T1=WIJ*FS(K,J)
                  T2=WIJ*FC(K,J)
                  ASS(J,I)=ASS(J,I)+FS(K,I)*T1
                  ACS(J,I)=ACS(J,I)+FS(K,I)*T2
                  ASC(J,I)=ASC(J,I)+FC(K,I)*T1
                  ACC(J,I)=ACC(J,I)+FC(K,I)*T2
                ENDDO
              ENDIF
            ENDDO
          ENDDO
        ENDIF
!
!  CLOSED-CLOSED PART
!
        IF(ICCINT.EQ.1.AND.NCHOP.NE.NCHF) THEN
          DO I=NCHOP1,NCHF
            IF(IOMTT(I).LE.0)THEN
              DO J=I,NCHF
                LIJ=LAMP(J,I)
                IF(LIJ.NE.1.AND.LIJ.LE.LMX+1.AND.IOMTT(J).LE.0) THEN
                  DO K=1,KP2T
                    WIJ=TBODE(K,LIJ)*BW(J,I)
                    T1=WIJ*FS(K,J)
                    T2=WIJ*FC(K,J)
                    ASS(J,I)=ASS(J,I)+FS(K,I)*T1
                    ACS(J,I)=ACS(J,I)+FS(K,I)*T2
                    ASC(J,I)=ASC(J,I)+FC(K,I)*T1
                    ACC(J,I)=ACC(J,I)+FC(K,I)*T2
                  ENDDO
                ENDIF
              ENDDO
            ENDIF
          ENDDO
        ENDIF
      ENDIF
!
!
!  ASYMPTOTIC INTEGRALS FOR R=RTWO/C TO INFINITY
!
      DO I=1,NCHF
        IF(RINF(I).GT.RTWO)IOMTT(I)=2
      ENDDO
!
      NCH=NCHOP
      IF(ICCINT.EQ.1)NCH=NCHF
      DO 55 I=1,NCH
        IF(IOMTT(I).GT.1)GO TO 55
        DO 50 J=I,JMAX
          IF(IOMTT(J).GT.1)GO TO 50
          IF(I.GT.NCHOP.AND.IOMTT(I)+IOMTT(J).GT.0)GO TO 50
          LIJ=LAMP(J,I)
          IF(LIJ.EQ.1.OR.LIJ.GT.LMX+1)GOTO 50
          L1=LLCH(I)
          L2=LLCH(J)
          E1=EPS(I)/TWO
          E2=EPS(J)/TWO
          KTRUE=LIJ
          IF(KP2T.GT.1)THEN
            DF1=FSP2(I)
            DG1=-FCP2(I)
            DF2=FSP2(J)
            DG2=-FCP2(J)
          ELSE
            DF1=FSP(I)
            DG1=-FCP(I)
            DF2=FSP(J)
            DG2=-FCP(J)
          ENDIF
          F1=FS(KP2T,I)
          G1=-FC(KP2T,I)
          F2=FS(KP2T,J)
          G2=-FC(KP2T,J)
!
          CALL CORINT(L1,L2,E1,E2,KTRUE,R0,TZED,F1,DF1,G1,DG1,F2,DF2
     X            ,G2,DG2,F1F2,G1F2,F1G2,G1G2)
!
          BWIJ=BW(I,J)
          ASS(J,I)=ASS(J,I)+BWIJ*F1F2
          ASC(J,I)=ASC(J,I)-BWIJ*G1F2
          ACS(J,I)=ACS(J,I)-BWIJ*F1G2
          ACC(J,I)=ACC(J,I)+BWIJ*G1G2
   50   CONTINUE
   55 CONTINUE
!
!  SYMMETRISE ALPHA
!
      DO I=1,NCHF
        A=(ASC(I,I)+ACS(I,I))/TWO
        ASC(I,I)=A
        ACS(I,I)=A
      ENDDO
!
      IF(NCHF.NE.1) THEN
        DO I=2,NCHF
          K=I-1
          DO J=1,K
            ASS(J,I)=ASS(I,J)
            ACC(J,I)=ACC(I,J)
            ASC(J,I)=ACS(I,J)
            ACS(J,I)=ASC(I,J)
          ENDDO
        ENDDO
      ENDIF
!
      IF(IPRINT.GT.1)THEN
        WRITE(6,750)
        WRITE(6,*)'OPEN-OPEN'
        DO I=1,NCHOP
          DO J=I,NCHOP
            LIJ=LAMP(J,I)
            IF(LIJ.NE.1.AND.LIJ.LE.LMX+1) THEN
              WRITE(6,760)J,I,ASS(J,I),ASC(J,I),ACS(J,I),ACC(J,I)
            ENDIF
          ENDDO
        ENDDO
        IF(IOCINT.EQ.1)THEN
        WRITE(6,*)'CLOSED-OPEN'
        DO I=1,NCHOP
          DO J=NCHOP1,NCHF
            LIJ=LAMP(J,I)
            IF(LIJ.NE.1.AND.LIJ.LE.LMX+1) THEN
              WRITE(6,760)J,I,ASS(J,I),ASC(J,I),ACS(J,I),ACC(J,I)
            ENDIF
          ENDDO
        ENDDO
        ENDIF
        IF(ICCINT.EQ.1)THEN
        WRITE(6,*)'CLOSED-CLOSED'
        DO I=NCHOP1,NCHF
          DO J=NCHOP1,NCHF
            LIJ=LAMP(J,I)
            IF(LIJ.NE.1.AND.LIJ.LE.LMX+1) THEN
              WRITE(6,760)J,I,ASS(J,I),ASC(J,I),ACS(J,I),ACC(J,I)
            ENDIF
          ENDDO
        ENDDO
        ENDIF
      ENDIF
      DEALLOCATE(IOMTT)
!
 750  FORMAT(/' J,I AND ASS(J,I), ASC(J,I), ACS(J,I), ACC(J,I)'/)
 760  FORMAT(2I5,4E14.6)
!
      RETURN
      END
!***********************************************************************
       REAL*8 FUNCTION ARGAM(L,A)
!
       IMPLICIT REAL*8 (A-H,O-Z)
!
! CALCULATES ARGGAMMA(L+1+I*A)
! WHERE L IS AN INTEGER NOT LESS THAN ZERO
!
       B=ABS(A)
       B=250.0D0*B**0.25D0-A*A
       J0=L+1
       C=J0
       D=C*C
       Z=0.0D0
       IF(D -B)1,6,6
    1  B=SQRT (B)
       J1=B
       DO 5 J=J0,J1
       D=J
       D=A/D
       G1=ABS(D)
       IF(G1-0.1D0)2,3,3
    2  G1=D*D
       G2=-35.0D0*G1+45.0D0
       G2=-G1*G2+63.0D0
       G2=-G1*G2+105.0D0
       G1=D -D*G1*G2/315.0D0
       GO TO 4
    3  G1=ATAN (D)
    4  Z=Z+G1
    5  CONTINUE
       J0=J1+1
    6  D=J0
       G0=D*D
       U=A*A
       G1=1.0D0/(G0+U)
       G2=G1*G1
       G3=10.0D0*G0*G0-20.0D0*G0*U+2.0D0*U*U
       G3=G3*G2-21.0D0*G0+7.0D0*U
       G3=G3*G2+210.0D0
       G1=A*G3*G1/2520.0D0
        ARGAM=-Z+0.5D0*A*LOG(G0+U)+(D -0.5D0)*ATAN(A/D)-A-G1
       RETURN
      END
!***************************************************************
!
      REAL*8 FUNCTION ARGC(E,L,AC)
      USE NRBZED
!
!  CALCULATES ARG(GAMMA(L+1-I/K)) -1/K -(1/K)*LN(K) - L*PI/2
!  NRB: NOT ACTUALLY CALLED BY INJWBK CASE TZED=0.
!
      IMPLICIT REAL*8 (A-H,O-Z)
!
      IF(TZED.EQ.0.)THEN
        ARGC=-DBLE(L)*1.570796327
        RETURN
      ENDIF
!
      IF(E.GT.0)GOTO 10
      ARGC=-(DBLE(L)+.25)*3.141592654
      RETURN
!
   10 FK=SQRT(E)
      ET=1./FK
      IP=L+1
      P=IP
      PP=IP*IP
!
      IF(AC.LT.1.E-4)GOTO 100
      A1=10.*SQRT(ET)-ET*ET
      IF(A1.GT.PP)GOTO 20
      X=PP*E
      XP1=X+1.
      XH=P*FK
      A=-1.570796327*(P+DBLE(L)-.5)
      GOTO 200
   20 L1=IP
      IP=1.+SQRT(A1)
      P=IP
      PP=IP*IP
      X=PP*E
      XP1=X+1.
      XH=P*FK
      A=-1.570796327*(P+DBLE(L)-.5)
      L2=IP-1
      DO 30 I=L1,L2
   30 A=A+ATAN(ET/DBLE(I))
      GOTO 200
!
  100 A1=35.*ET**.25-ET*ET
      IF(A1.GT.PP)GOTO 120
      X=PP*E
      XP1=X+1.
      XH=P*FK
      A=-1.570796327*(P+DBLE(L)-.5)
      GOTO 140
  120 L1=IP
      IP=1.+SQRT(A1)
      P=IP
      PP=IP*IP
      X=PP*E
      XP1=X+1.
      XH=P*FK
      A=-1.570796327*(P+DBLE(L)-.5)
      L2=IP-1
      DO 130 I=L1,L2
  130 A=A+ATAN(ET/DBLE(I))
  140 A=A+.000396825540*FK*E*(7.*(1.-3.*X)*XP1*XP1+
     C 2.*E*(1.-10.*X+5.*X*X))*XP1**(-5)
!
  200 A1=FK*X*X*.1667*PP
      IF(A1.GT.AC)GOTO 210
      A=A-FK*(2.-X)*.25*PP
      GOTO 220
  210 A=A-.5*ET*LOG(XP1)
  220 A2=(P-.5)*XH
      A1=A2*X*X
      IF(A1.GT.AC)GOTO 230
      A=A+A2*(1.-X*.33333333)
      GOTO 240
  230 A=A+(P-.5)*ATAN(XH)
  240 ARGC=A+FK/(12.*(1.+X))
!
      RETURN
      END
!***************************************************************
!
      SUBROUTINE BODE(H,KP2,IPERT,RZERO)
      USE CBODE
      USE NRBLMX
!
      IMPLICIT REAL*8 (A-H,O-Z)
      PARAMETER (ONE=1.0)
      PARAMETER (T1=14.0)
      PARAMETER (T2=64.0)
      PARAMETER (T3=24.0)
      PARAMETER (T4=28.0)
!
      IF(ALLOCATED(WBODE))DEALLOCATE(WBODE)
      IF(ALLOCATED(TBODE))DEALLOCATE(TBODE)
      ALLOCATE(WBODE(KP2))
      ALLOCATE(TBODE(KP2,LMX+1))
!
      W=T1*H/45
      WBODE(1)=W
      WBODE(KP2)=W
      W=T2*H/45
      M=KP2-1
      DO K=2,M,2
        WBODE(K)=W
      ENDDO
      W=T3*H/45
      M=KP2-2
      DO K=3,M,4
        WBODE(K)=W
      ENDDO
      W=T4*H/45
      M=KP2-4
      DO K=5,M,4
        WBODE(K)=W
      ENDDO
!
      IF(IPERT.EQ.0.OR.LMX.LT.1.OR.KP2.LE.1)RETURN
!
      R=RZERO-H
      DO K=1,KP2
        R=R+H
        TBODE(K,1)=ONE/R
      ENDDO
      DO I=1,LMX
        IP=I+1
        DO K=1,KP2
          TBODE(K,IP)=TBODE(K,I)*TBODE(K,1)
        ENDDO
      ENDDO
      DO I=1,LMX
        IP=I+1
        DO K=1,KP2
          TBODE(K,IP)=TBODE(K,IP)*WBODE(K)
        ENDDO
      ENDDO
!
      RETURN
      END
!
!***********************************************************************
!
      REAL*8 FUNCTION BUT0(NBUT,U)
      USE CBUT
!
      IMPLICIT REAL*8 (A-H,O-Z)
      LOGICAL POLE
!
      BUT0=0.
!
!  CASE OF U.GT.0.04
      IF(U.GT..04)THEN
        FK=SQRT(U)
        POLE=.FALSE.
        DO 10 N=0,NBUT
        IF(ABS(FK-FKN(N)).GT..3)THEN
          BUT0=BUT0+1./(U-UKN(N))
        ELSE
          POLE=.TRUE.
          D1=FK-FKN(N)
        ENDIF
   10   CONTINUE
        IF(POLE)THEN
          D2=D1**2
          D=.33333333*D1*(1.+.066666667*D2*(1.+.0952381*D2))
          BUT0=2.*BUT0+(D+1./(2.*FK-D1))/FK
        ELSE
          BUT0=2.*BUT0+TAN(FK)/FK
        ENDIF
!
!  SUM FOR U.LT..04
      ELSE
        DO 20 N=0,NBUT
   20   BUT0=BUT0+1./(U-UKN(N))
!
!  CASE OF U.LT..04 AND U.GT.-.04
        IF(U.GT.-.04)THEN
          BUT0=2.*BUT0+1.+.33333333*U*(1.+.4*U)
!
!  CASE OF U.LT.-.04
        ELSE
          FK=SQRT(-U)
          BUT0=2.*BUT0+TANH(FK)/FK
        ENDIF
!
      ENDIF
!
      RETURN
      END
!***************************************************************
!
      SUBROUTINE CCINT(IOMTT,NCHOP)
      USE CALP
      USE CHAN
      USE CPOT
      USE CTHET
      USE NRBLMX
      USE CACC
      USE CPOINT
      USE CNTRL
      USE CQDT
      USE CBLK
!
!
! CLOSED-CLOSED:
!  CALCULATES T INTEGRALS USING LAGUERRE QUADRATURE
!
      IMPLICIT REAL*8 (A-H,O-Y)
      IMPLICIT COMPLEX*16 (Z)
!
      DIMENSION TI(30),TDI(30),TPI(30),TJ(30),TDJ(30),TPJ(30)
!     X,IOMTT(NCHF)
      INTEGER, INTENT(IN) :: IOMTT(*)
!
      NCHOP1=NCHOP+1
!
      DO I=NCHOP1,NCHF
        IF(IOMTT(I).EQ.0)THEN
        DO J=I,NCHF
          IF(IOMTT(J).EQ.0)THEN
          LIJ=LAMP(J,I)
          IF(LIJ.NE.1.AND.LIJ.LE.LMX+1) THEN
            NLAG=2*LIJ+LACC
            NLAG=MIN(NLAG,10)
            BIJ=BW(J,I)
!
!  CAP K
            FNUI=BB(I,1)
            FNUJ=BB(J,1)
            FK=1./FNUI+1./FNUJ
!  INITIALISE FOR QUADRATURE
            NS=NLAG/2
            M=NS*(NS-1)
            N1=M+1
            N2=M+NLAG
            CALL TANDTDN(FK,N1,N2,I,TI,TDI,TPI)
            CALL TANDTDN(FK,N1,N2,J,TJ,TDJ,TPJ)
            T1=0.
            T2=0.
            T3=0.
!  START QUADRATURE
            DO N=N1,N2
              U=XLAG(N)
              R=RTWO+U/FK
!  CALCULATE THETA FUNCTIONS
!  ADD TO SUM
!++ VAX MOD
!     A1=(R**(-LIJ))*WLAG(N)*EXP(U+TPI+TPJ)
              A1=(R**(-LIJ))*WLAG(N)
              U2=.5*U
              AI=EXP(U2+TPI(N))
              AJ=EXP(U2+TPJ(N))
              TI(N)=TI(N)*AI
              TDI(N)=TDI(N)*AI
              TJ(N)=TJ(N)*AJ
              TDJ(N)=TDJ(N)*AJ
!++ END MOD
              T1=T1+TI(N)*A1*TJ(N)
              T2=T2+TDI(N)*A1*TJ(N)
              T3=T3+TI(N)*A1*TDJ(N)
            ENDDO
            F1=1./FK
            T1=T1*F1
            T2=T2*F1
            T3=T3*F1
            ASS(J,I)=ASS(J,I)+T1*BIJ
            ASC(J,I)=ASC(J,I)+T2*BIJ
            ACS(J,I)=ACS(J,I)+T3*BIJ
          ENDIF
          ENDIF
        ENDDO
        ENDIF
      ENDDO
      RETURN
      END
!************************************************************************
      SUBROUTINE CORINT(L1,L2,E1,E2,KTRUE,R0,ZCH
     1     ,F1,DF1,G1,DG1,F2,DF2,G2,DG2,F1F2,G1F2,F1G2,G1G2)
!
      USE CPOINT, only : MZPTS
      IMPLICIT REAL*8 (A-H,O-Z)
!      PARAMETER (NPTS0=MZPTS+250)
!      COMPLEX(8) Z,ZZ,AK1,F12,DH,FUN2(NPTS0),FUN3(NPTS0),
!     1  K21,K22,FUN1(NPTS0),COEF(4),K23,YP,Y0,YM,WP,W0,WM
      COMPLEX(8) Z,ZZ,AK1,F12,DH,K21,K22,K23,YP,Y0,YM,WP,W0,WM
      COMPLEX(8) COEF(4)
      COMPLEX(8), ALLOCATABLE :: FUN1(:),FUN2(:),FUN3(:)   ! DIM=(NPTS)
!
! F. ROBICHEAUX
! THIS SUBROUTINE CALCULATES THE 4 INTEGRALS
! INT _R_0^INFTY (F_1,G_1)(F_2,G_2)/R^KTRUE BY DIRECT SOLUTION OF THE
! SCHR. EQ IN THE COMPLEX R-PLANE.
! IT IS ASSUMED THE F_I,G_I ARE SOLUTIONS OF THE DIFFERENTIAL EQUATION
! Y''_I+2(EI+ZCH/R-LI(LI+1)/R^2)Y_I=0
!
! THE INPUTS ARE:
! ZCH -- THE CHARGE
! R0 -- RADIUS TO BEGIN THE INTEGRATION
! KTRUE -- THE MULTIPOLE OF THE POTENTIAL (SEE ABOVE)
! LI -- INTEGER, THE ANGULAR MOMENTUM OF THE PARTIAL WAVE
! EI -- THE ENERGY OF THE PARTIAL WAVE (NOTE: E2<E1)
! FI -- THE VALUE OF F_I(R_0)
! GI -- THE VALUE OF G_I(R_0)
! DFI -- THE VALUE OF DF/DR|_I(R_0)    - I=1 NOT USED - NRB
! DGI -- THE VALUE OF DG/DR|_I(R_0)
! F1F2, F1G2, G1F2, G1G2 -- THE OUTPUT INTEGRALS
!
! MODIFIED BY NRB TO HANDLE INTEGRAL FROM RTWOC RATHER THAN RZERO.
!
      NPTS0=MZPTS+250
      AC=1.E-4
!
      COEF(1)=1.7D0/4.8D0
      COEF(2)=5.9D0/4.8D0
      COEF(3)=4.3D0/4.8D0
      COEF(4)=4.9D0/4.8D0
      ITIM=0
      IF(E2 .GT. E1) STOP 'REVERSE 1 AND 2 CORINT'
!
      AL1=DBLE(L1*L1+L1)/2.D0
      AL2=DBLE(L2*L2+L2)/2.D0
      TES1=2.D0*ABS(E1+ZCH/R0-AL1/(R0*R0))
      IF(TES1 .LT. (2.D0*ABS(E1)))TES1=2.D0*ABS(E1)
      TES2=2.D0*ABS(E2+ZCH/R0-AL2/(R0*R0))
      IF(TES2 .LT. (2.D0*ABS(E2)))TES2=2.D0*ABS(E2)
      IF(TES1 .GT. TES2) THEN
        H=.2D0/SQRT(TES1)
      ELSE
        H=.2D0/SQRT(TES2)
      END IF
!      IF(AC.GT.5.E-5)H=2.*H
      IF(E2 .GT. 0.D0) THEN
        YF=1.6D1/(SQRT(E1)-SQRT(E2)+1.D-6)
      ELSE
        IF(E1 .GT. 0.D0) THEN
          YF=1.6D1/SQRT(E1)
        ELSE
          YF=10000*H
        END IF
      END IF
      IGO=0
      NPTS=NINT(YF/H)
      IF(NPTS.LT.8)THEN
        F1F2=0.D0
        G1F2=0.D0
        F1G2=0.D0
        G1G2=0.D0
        RETURN                      !QUICK RETURN
      ENDIF
!
! LFM - SAFETY AGAINST TOO LARGE CASES
      IF(NPTS .GT. NPTS0) THEN
        IGO=1
        TL=L2*L1
        T=SQRT(TL)
        T=10.*T
        IF(AC.LT.5.E-5)T=T*2.
        TL=MIN(T,TL)
        NPTS=250+TL
        NPTS=MIN(NPTS,NPTS0)
      END IF
      ALLOCATE(FUN1(NPTS),FUN2(NPTS),FUN3(NPTS))
!
      H2=H*H/6.D0
      DH=(0.D0,1.D0)*H
      R=H
      R0SQ=R0*R0
      DO J = 1,NPTS
        R=R-H
        T=1.D0/(R0SQ+R*R)
        F12=(R0*T)+(0.D0,1.D0)*(R*T)
        FUN1(J)=F12
        FUN2(J)=F12*F12
        FUN3(J)=F12**KTRUE
      ENDDO
      DO J = 1,4
        FUN3(J)=FUN3(J)*COEF(J)
        FUN3(NPTS+1-J)=FUN3(NPTS+1-J)*COEF(J)
      ENDDO
!
      J=NPTS
!
      AL2H2=AL1*H2
      E2H2=1.D0-E1*H2
      ZCHH2=-ZCH*H2
      K21=2.D0*(E1+ZCH*FUN1(J)-AL1*FUN2(J))
      K22=2.D0*(E1+ZCH*FUN1(J-1)-AL1*FUN2(J-1))
      AK1=SQRT(K21)+SQRT(K22)
      Y0=(1.D0,1.D0)/1.D10
      YP=Y0*EXP(.5D0*H*AK1)
      K23=1.D0-K22*H2*.5D0
      K22=1.D0-K21*H2*.5D0
      FUN3(J)=FUN3(J)*Y0
      FUN3(J-1)=FUN3(J-1)*YP
!
      K=NPTS-2
      DO J = K,1,-1
        K21=K22
        K22=K23
        K23=E2H2+ZCHH2*FUN1(J)+AL2H2*FUN2(J)
        YM=Y0
        Y0=YP
        YP=((12.D0-10.D0*K22)*Y0-K21*YM)/K23
        FUN3(J)=FUN3(J)*YP
      ENDDO
!
      AL2H2=AL2*H2
      E2H2=1.D0-E2*H2
      K22=2.D0*(E2+ZCH*FUN1(1)-AL2*FUN2(1))
      K21=-2.D0*(ZCH-2.D0*AL2*FUN1(1))*FUN2(1)
      K23=E2H2+ZCHH2*FUN1(2)+AL2H2*FUN2(2)
      Y0=F2
      YP=Y0+DH*DF2+H2*K22*F2*3.D0+DH*H2*(K22*DF2+K21*F2)
      W0=G2
      WP=W0+DH*DG2+H2*K22*G2*3.D0+DH*H2*(K22*DG2+K21*G2)
      K22=1.D0-K22*H2*.5D0
      AK1=Y0*FUN3(1)+YP*FUN3(2)
      F12=W0*FUN3(1)+WP*FUN3(2)
!
      DO J = 3,NPTS
        K21=K22
        K22=K23
        K23=E2H2+ZCHH2*FUN1(J)+AL2H2*FUN2(J)
        YM=Y0
        Y0=YP
        Z=1.D0/K23
        ZZ=12.D0-10.D0*K22
        YP=(ZZ*Y0-K21*YM)*Z
        WM=W0
        W0=WP
        WP=(ZZ*W0-K21*WM)*Z
        AK1=AK1+YP*FUN3(J)
        F12=F12+WP*FUN3(J)
      ENDDO
!
      AK1=AK1*DH
      F12=F12*DH
      IF (IGO .EQ. 1)THEN
        IF(E2 .GT. 0.D0) THEN
          K21=(0.D0,1.D0)*ZCH*(1.D0/SQRT(2.D0*E2)
     X        -1.D0/SQRT(2.D0*E1))-KTRUE
          K22=(0.D0,1.D0)*(SQRT(2.D0*E1)-SQRT(2.D0*E2))
          Z=R0+DH*(NPTS-1)
          YM=FUN3(NPTS)/COEF(1)
          TES=ABS(Z)*ABS(SQRT(ABS(E1))-SQRT(ABS(E2)))
          IF(TES .LT. 1.D-6) THEN
            WM=YM*Z/(KTRUE-1)
          ELSE
            WM=(0.D0,0.D0)
            K23=K22+K21/Z
            DO J = 1,100
              Y0=-.05D0/K23
              Z=Z+Y0*2.D0
              WM=WM+YM*Y0
              K23=K22+K21/Z
              W0=.95D0/(1.D0-Y0*K23)
              YM=YM*W0
              WM=WM+YM*Y0
            ENDDO
          END IF
          AK1=AK1+WM*YP
          F12=F12+WM*WP
        END IF
      END IF
      Z=(F1+(0.D0,1.D0)*G1)*COEF(1)/(FUN3(1)*(R0**KTRUE))
      AK1=AK1*Z
      F12=F12*Z
!
      DEALLOCATE(FUN1,FUN2,FUN3)
      F1F2=DBLE(AK1)
      G1F2=DIMAG(AK1)
      F1G2=DBLE(F12)
      G1G2=DIMAG(F12)
!
!       WRITE(6,900)F1F2,G1F2,F1G2,G1G2
! 900   FORMAT(4F18.10)
      RETURN
      END
!**********************************************************
!
      SUBROUTINE COUL(INOUT)
      USE NRBRCT
      USE COULSC
      USE CEN
      USE CINPUT
      USE CHAN
      USE NRBHYB
      USE NRBQDT
      USE NRBOMT
      USE CACC
      USE CPOINT
      USE CPTOLD
      USE CNTRL
      USE CQDT
      USE NRBDD2
      USE NRBZED
!
! NRB:
!  CALCULATION OF COULOMB FUNCTIONS
!  GENERALIZED FOR MQDT CASE AND DERIVATIVES AT RTWOC.
!
      IMPLICIT REAL*8 (A-H,O-Z)
      PARAMETER (TZERO=0.0)
      PARAMETER (ONE=1.0)
!
      LOGICAL BDD2
!
      INTEGER, INTENT(OUT) :: INOUT(*)   ! DIM=(NCHF)
      REAL(8), ALLOCATABLE :: FST(:)     ! DIM=(MZPTS)=(KP2)
!
!      ALLOCATE(FST(MZPTS))
      ALLOCATE(FST(KP2))
      FST(1:KP2)=0.D0
      BDD2=IQDT.GT.0.AND.IPERT.NE.0.AND.IRAD.EQ.0.AND.INTPQ.EQ.0
!
! NEW CODE (NPERT=1, SET VIA PERT='YES'- DEFAULT) DOES NOT SWITCH-OFF
! PERTURBATIONS. IF R2.GT.RTWO ALL SOLUTIONS ARE EVALUATED OUTWARDS
! FROM RZERO AND SUBSEQUENT CALL TO ALPHA NEGLECTS THE CONTRIBUTION
! FROM R.GT.RTWO. NRB-94
!
      TOL0=1.D-150
      IFLUG=0
      IFDD2=0
      RTEST=RTWO
      IF(NPERT.GT.0) RTEST=RTWOO
      PI=ACOS(-ONE)
!
!  CALCULATE OPEN-CHANNEL SOLUTIONS AT RTWO
!  AND PERFORM INWARDS INTERGATIONS TO RZERO
!  (FOR CASE OF RINF.GT.RZERO, THE REGULAR SOLUTION IS CALCULATED
!  AT RZERO USING SERIES AND INTEGRATED OUTWARDS).
!  IF R2.GT.RTWO REGULAR AND IRREGULAR SOLUTIONS ARE EVALUATED FROM
!  SERIES AT RZERO AND INTEGRATED OUTWARDS.
!
      IF(NCHPP.GT.0)THEN
        J=0
!
        DO I=1,NCHPP
          IF(IOMIT(I).EQ.0)THEN
          ICHAN=I
!
!...    CASE OF R2ST(I).LE.RTEST, PERTURBATION MAY BE USED
!
          IF(R2ST(I).LE.RTEST)THEN
!
            CALL INJWBK(EPS(I),LLCH(I),J)
!
            IF(R2ST(I).LE.RTWO)THEN
!OLD             IPERT.EQ.0.AND.
              IF(               KP2.GT.1)THEN    ! KP2=1 WHEN H=0.0
                RTWOT=MAX(R2ST(I),RZERO)
                KP2T=(RTWOT-RZERO)/H +0.1*H
!                KP2T=MIN(KP2T,MZPTS-1)
                KP2T=MIN(KP2T,KP2-1)
                IF(IRAD.GT.0)KP2T=MAX(5,KP2T)    ! NEED 2 POINTS
                KP2T=4*((KP2T-1)/4)
                RTWOT=DBLE(KP2T)*H+RZERO
                KP2T=KP2T+1
              ELSE
                RTWOT=RTWO
                KP2T=KP2
              ENDIF
!
              CALL JWBK(RTWOT,J,S(I),SP(I),C(I),CP(I))
!
              FS(KP2T,I)=S(I)
              FSP(I)=SP(I)
              FC(KP2T,I)=C(I)
              FCP(I)=CP(I)
              FSP2(I)=SP(I)
              FCP2(I)=CP(I)
            ELSE
              KP2T=(R2ST(I)-RZERO)/H+0.1*H
              KP2T=4*((KP2T-1)/4)
              RTWOT=DBLE(KP2T)*H+RZERO
              KP2T=KP2T+1
!
              CALL JWBK(RTWOT,J,S(I),SP(I),C(I),CP(I))  !RTWOO->RTWOT
!
!              FS(MZPTS,I)=S(I)                          !HOLD, NOT USED?
!              FC(MZPTS,I)=C(I)                          !HOLD, NOT USED?
              FS(KP2,I)=S(I)                            !HOLD, NOT USED?
              FC(KP2,I)=C(I)                            !HOLD, NOT USED?
!              RTWOT=RTWOO                              !RTWOO NOT USED FURTHER?
!              KP2T=KP2O                                !KP2O NOT USED FURTHER?
            ENDIF
!
!  INWARD INTEGRATION FROM R2(I) TO RZERO 
!         (EXCEPT S WHEN RINF.GT.RZERO - OUTWARD)
!
            IF(KP2T.GT.1)THEN
              IF(RINF(I).LE.RZERO)THEN    ! 1.2*RINF(I) SLIGHTLY MORE ACCURATE
                W2=C(I)*SP(I)-CP(I)*S(I)
                IF(BDD2.AND.KP2T.GT.KP2C)THEN               !KP2T=KP2
                  CALL NUMSC(EPS(I),CCT(I),RTWOT,H,KP2T,KP2C,SP(I)
     X                      ,CP(I),I)
                  FSP2(I)=SP(I)
                  FCP2(I)=CP(I)
                  CALL NUMSC(EPS(I),CCT(I),RTWOC,H,KP2C,1,SP(I),CP(I),I)
                ELSE
                  CALL NUMSC(EPS(I),CCT(I),RTWOT,H,KP2T,1,SP(I),CP(I),I)
                ENDIF
                  S(I)=FS(1,I)
                  C(I)=FC(1,I)
                  FSP(I)=SP(I)
                  FCP(I)=CP(I)
                  INOUT(I)=0
                  W0=C(I)*SP(I)-CP(I)*S(I)
                  IF(IPRINT.GT.1.OR.ABS(W0-ONE).GT.10*AC)
     X                           WRITE(6,600)I,RINF(I),W0,W2
              ELSE
                IF(IPERT.GT.0)THEN
!                  WRITE(0,*)'INSIDE COUL',LRGL2,LPRTSW,IPERT
                  IF(LRGL2.GT.LPRTSW)IPERT=-IPERT
                  IF(IPRINT.GT.0)WRITE(6,720)
                ENDIF
! EVALUATE S                                       BY SERIES AT MIN(0.8*RINF,RZERO)
                KINF=(RZERO-0.8*RINF(I))/H
                IF(KINF.LT.0)KINF=0
                RINF0=RZERO-KINF*H
                CALL COULS(LLCH(I),EPS(I),RINF0,S(I),SP(I))
                                    INOUT(I)=1
                KINF=KINF+1
                IF(KINF.GT.1)THEN
! LFM - PROBABLY NEVER WITH ALLOCATIONS
!                  IF(KINF.GT.MZPTS) THEN
                  IF(KINF.GT.KP2) THEN
                    WRITE(6,610)KINF
                    STOP 'SR.COUL: INCREASE MZPTS'
                  ENDIF
             CALL NUMS(EPS(I),CCT(I),RINF0,H,1,KINF,S(I),SP(I),FST)
                ENDIF
                FSP(I)=SP(I)
                SS=S(I)
                SSP=SP(I)
                KPT=MIN(KP2C,KP2T)
                IF(IPERT.EQ.0)KPT=1
                IF(IPRINT.GT.1)KPT=KP2T
                IF(IRAD.GT.0)KPT=MAX(2,KPT)           !FIX FOR F-FILES
                IF(BDD2.AND.KPT.GT.KP2C)THEN
                  CALL NUMS(EPS(I),CCT(I),RZERO,H,1,KP2C,SS,SSP,FST)
                  FSP2(I)=SSP
                  CALL NUMS(EPS(I),CCT(I),RTWOC,H,KP2C,KPT,SS,SSP,FST)
                ELSE
                  CALL NUMS(EPS(I),CCT(I),RZERO,H,1,KPT,SS,SSP,FST)
                  FSP2(I)=SSP
                ENDIF
                W2=C(I)*SSP-CP(I)*SS
                DO K=1,KPT
                  FS(K,I)=FST(K)
                ENDDO
! EVALUATE C
                IF(BDD2.AND.KP2T.GT.KP2C)THEN
                  CALL NUMS(EPS(I),CCT(I),RTWOT,H,KP2T,KP2C,C(I),CP(I)
     X                     ,FST)
                  FCP2(I)=CP(I)
                  CALL NUMS(EPS(I),CCT(I),RTWOC,H,KP2C,1,C(I),CP(I),FST)
                ELSE
                  FCP2(I)=CP(I)
                  CALL NUMS(EPS(I),CCT(I),RTWOT,H,KP2T,1,C(I),CP(I),FST)
                ENDIF
                W0=C(I)*FSP(I)-CP(I)*FS(1,I)
                IF(IPRINT.GT.1.OR.ABS(W0-ONE).GT.50*AC)
     X                         WRITE(6,600)-I,RINF(I),W0,W2
                FCP(I)=CP(I)
                KPT=KP2T
                IF(IPERT.EQ.0.AND.IRAD.GT.0.AND.KP2C.GT.KP2T)THEN
                  KPT=KP2C
                  CX=FST(KP2T)
                  CXP=FCP2(I)
                  CALL NUMS(EPS(I),CCT(I),RTWOT,H,KP2T,KP2C,CX,CXP,FST)
                ENDIF
!                KPT=MIN(KPT,MZPTS)                 !CASE R2ST(I).GT.RTWO
                KPT=MIN(KPT,KP2)                   !CASE R2ST(I).GT.RTWO
                DO K=1,KPT
                  FC(K,I)=FST(K)
                ENDDO
              ENDIF
            ENDIF
!
!   OUTWARD INTEGRATION FROM R2(I) TO RTWO.
!
            IF(IPERT.NE.0.AND.KP2C.GT.KP2T)THEN     !KP2C=KP2 FOR IQDT=0
            	CALL NUMSC(EPS(I),CCT(I),RTWOT,H,KP2T,KP2C,FSP2(I)
     X      						  ,FCP2(I),I)
              W2C=FC(KP2C,I)*FSP2(I)-FCP2(I)*FS(KP2C,I)
              IF(IPRINT.GT.1.OR.ABS(W2C-ONE).GT.50*AC)
     X      		     WRITE(6,601)I,R2ST(I),W2,W2C
            ENDIF
!
!...CASE OF R2ST(I).GT.RTEST, PERTURBATION CANNOT BE USED
!   NO LONGER ENTER HERE IF DEFAULT NPERT=1/PERT='YES' IS USED.
!
          ELSE
            IPERT=0
            IF(IRAD.EQ.0) THEN
              CALL SC(EPS(I),LLCH(I),RZERO,AC,
     X                FS(1,I),FSP(I),FC(1,I),FCP(I))
              S(I)=FS(1,I)
              SP(I)=FSP(I)
              C(I)=FC(1,I)
              CP(I)=FCP(I)
            ELSE
              IF(RINF(I).LT.RZERO) THEN
                CALL SC(EPS(I),LLCH(I),RZERO,AC,FS(1,I),FSP(I)
     X                 ,FC(1,I),FCP(I))
                S(I)=FS(1,I)
                C(I)=FC(1,I)
                SP(I)=FSP(I)
                CP(I)=FCP(I)
                FSP2(I)=FSP(I)
                FCP2(I)=FCP(I)
                CALL NUMSC(EPS(I),CCT(I),RZERO,H,1,KP2,FSP2(I)
     X                    ,FCP2(I),I)
                INOUT(I)=0
              ELSE
                WRITE(6,620)ETOT,I,R2ST(I),RTWO,I,RINF(I),RZERO,KP2
                STOP 'SR.COUL: NEED C-FUNCTION INWARD FROM RINF'
              ENDIF
            ENDIF
          ENDIF
          ENDIF
          J=J+15
        ENDDO
      ENDIF
!
!  CALCULATE MQDT SOLUTIONS AT RZERO
!
      NCHP1=NCHPP+1
      IF(QDT.OR.IQDT.NE.0)THEN
        NQQ=NQ
        IF(IQDT.GT.0)NQQ=NCHF
        IF(IOMSW.LT.0)THEN
          NCHP1=1
          NQQ=NCHCL
        ENDIF
!
        DO 1000 I0=NCHP1,NQQ
          I=I0
          IF(IOMSW.LT.0)I=ICHCL(I0)
          IF(IOMIT(I).NE.0)GO TO 1000
!
          ICHAN=I
!
          KSTART=1
          RSTART=RZERO
!
!          WRITE(78,*)I,LLCH(I),EPS(I),RINF(I)
!          RSTART=MIN(RINF(I),RZERO)
!          IF(TZED.EQ.TZERO)RSTART=5.            
!          RSTART=MIN(RZERO,RSTART)
!          KSTART=(RZERO-RSTART)/H
!          IF(KSTART.LT.0)KSTART=0
!          RSTART=RZERO-KSTART*H
!          KSTART=KSTART+1
!
          CALL SC(EPS(I),LLCH(I),RZERO,AC,S(I),SP(I),C(I),CP(I))
!
          IF(IOMIT(I).GT.0)GO TO 1000
          IF(KSTART.GT.1)THEN
            CALL NUMS(EPS(I),CCT(I),RSTART,H,1,KSTART,S(I),SP(I),FST)
            CALL NUMS(EPS(I),CCT(I),RSTART,H,1,KSTART,C(I),CP(I),FST)
            W=C(I)*SP(I)-S(I)*CP(I)
            IF(ABS(W-ONE).GT.10.*AC)THEN
              IF(IPRINT.GT.1)WRITE(6,631)IE,I,LLCH(I),FKNU(I),RZERO,W
              IOMIT(I)=1
! ALLOW EXECUTION TO PROCEED AND OMIT LATER
              S(I)=0.5
              SP(I)=-0.05
              C(I)=100.
              CP(I)=-10.
            ELSE
              IF(IPRINT.GT.1)WRITE(6,631)IE,I,LLCH(I),FKNU(I),RZERO,W
            ENDIF
          ENDIF
          FS(1,I)=S(I)
          FC(1,I)=C(I)
          FSP(I)=SP(I)
          FCP(I)=CP(I)
          FSP2(I)=SP(I)
          FCP2(I)=CP(I)
          IF(IOMIT(I).GT.0)GO TO 1000
!
          INOUT(I)=0
          IF(KP2.GT.1)THEN
            KINF=(0.6*RINF(I)-RZERO)/H
            IF(KINF.GE.KP2)RINF(I)=-RINF(I)
            IF(IOMIT(I).LE.0.AND.RINF(I).GT.TZERO)THEN
              IF(KINF.LT.0)KINF=0
              RINF0=RZERO+KINF*H
              KINF=KINF+1
              IF(KINF.GT.1)THEN
                CALL NUMS(EPS(I),CCT(I),RZERO,H,1,KINF,S(I),SP(I),FST)
                DO K=1,KINF
                  FS(K,I)=FST(K)
                ENDDO
                CALL SC(EPS(I),LLCH(I),RINF0,AC,S(I),SP(I),C(I),CP(I))
                CP0=CP(I)
                CALL NUMS(EPS(I),CCT(I),RINF0,H,KINF,1,C(I),CP(I),FST)
                FCP(I)=CP(I)
                CP(I)=CP0
                DO K=1,KINF
                  FC(K,I)=FST(K)
                ENDDO
              ENDIF
              IF(INTPQ.EQ.0)THEN 
                IFLGW=0
                IF(BDD2.AND.KP2.GT.KP2C)THEN
                  IFLGW=1
                  CALL NUMSC(EPS(I),CCT(I),RINF0,H,KINF,KP2C,SP(I)
     X                      ,CP(I),I)
                    FSP2(I)=SP(I)
                    FCP2(I)=CP(I)
                  CALL NUMSC(EPS(I),CCT(I),RTWOC,H,KP2C,KP2,SP(I)
     X                      ,CP(I),I)
                ENDIF
                IF(IRAD.GT.0.OR.BDD2.AND.KP2.EQ.KP2C)THEN
                  IFLGW=1
                  CALL NUMSC(EPS(I),CCT(I),RINF0,H,KINF,KP2,SP(I)
     X                      ,CP(I),I)
                  IF(KP2C.GT.1)THEN
                    FSP2(I)=SP(I)
                    FCP2(I)=CP(I)
                  ENDIF
                ENDIF
                W0=FC(1,I)*FSP(I)-FS(1,I)*FCP(I)
                W2=W0
                IF(IFLGW.GT.0)W2=FC(KP2C,I)*FSP2(I)-FS(KP2C,I)*FCP2(I)
                IF(IPRINT.GT.1)WRITE(6,600)I,RINF(I),W0,W2
                IF(ABS(W0-W2).GT.100.*AC)RINF(I)=-RINF(I)
              ENDIF
            ENDIF
            S(I)=FS(1,I)
            SP(I)=FSP(I)
            C(I)=FC(1,I)
            CP(I)=FCP(I)
          ENDIF
!          WRITE(0,*)'INSIDE COUL',LRGL2,LPRTSW,IPERT,' 2'
          IF(RINF(I).GT.RZERO.AND.LRGL2.GT.LPRTSW)IPERT=-IABS(IPERT)
 1000   ENDDO
      ENDIF
!
!  CALCULATE CLOSED CHANNEL SOLUTIONS AT RTWO
!  AND PERFORM INWARDS INTEGRATIONS TO RZERO.
!  CASE R2.GT.RTWO EVALUATE THETA, THETADOT AT RZERO AND
!  INTEGRATE OUTWARDS.
!
      NCHFF=NCHF
      IF(INTPQ.EQ.0)NCHP1=NCHPP1
      IF(IOMSW.LT.0)THEN
        NCHP1=1
        NCHFF=NCHHYB
      ENDIF
!
      DO I0=NCHP1,NCHFF
        I=I0
        IF(IOMSW.LT.0)I=ICHHYB(I0)
        IF(IOMIT(I).EQ.0)THEN
        ICHAN=I
!
!...  CASE OF R2ST(I).LE.RTEST, PERTURBATION MAY BE USED
!
        IF(R2ST(I).LE.RTEST)THEN
          IF(R2ST(I).LE.RTWO)THEN
            KP2T=KP2
            RTWOT=RTWO
  205       CALL THETA(RTWOT,I,T,TP,TD,TDP,ICONV)
!
            FS(KP2T,I)=T
            FC(KP2T,I)=TD
!
! CHECK FUNCTION NOT TOO SMALL FOR NUMT
!
            IF(ABS(T).LT.TOL0.OR.ABS(TD).LT.TOL0)THEN
              IFLUG=1
              KP2T=KP2T-4
              IF(KP2T.LT.1)KP2T=1
              RTWOT=(KP2T-1)*H+RZERO
!             IF(KP2T.LT.KP0)STOP 'SR.COUL: KP2T .LT. KP0'
              DO K=KP2T,KP2T+4
                FS(K,I)=TZERO
                FC(K,I)=TZERO
              ENDDO
              IF(KP2T.EQ.1)GO TO 207
              GO TO 205
            ENDIF
!
  207       IF(IPRINT.GT.1.AND.KP2T.LT.KP2)WRITE(6,735)I,RTWOT,RTWO
            FSP(I)=TP
            FCP(I)=TDP
            KRA=KP2T
            KRB=KP0          !=1
            TKRA=RTWOT
            KINF=1
            RINF0=RZERO
          ELSE
            KINF=0
            IF(IPERT.NE.0)THEN
              KINF=(0.6*RINF(I)-RZERO)/H
              IF(KINF.GE.KP2)THEN
                RINF(I)=-ABS(RINF(I))
                KINF=0
              ENDIF
              IF(KINF.LT.0)KINF=0
            ENDIF
            RINF0=RZERO+KINF*H
            KINF=KINF+1
          ENDIF
!
          IF(R2ST(I).GT.RTWO.OR.RINF(I).GT.RZERO.AND.IPRINT.GT.0)THEN
!
! ASSUME SD=0=CD. (W(C,S)=1=-W(S,C) IN SR.SC)
!
            CALL SC(EPS(I),LLCH(I),RINF0,AC,FSA,FSPA,FCA,FCPA)
!
            IF(KINF.GT.1.AND.IOMIT(I).GT.0)THEN
              IOMIT(I)=0
              CALL SC(EPS(I),LLCH(I),RZERO,AC,FSA,FSPA,FCA,FCPA)
              KINF=1
              RINF0=RZERO
              RINF(I)=-RINF(I)
            ENDIF
!
            SINF=SIN(PI*FKNU(I))
            COSF=COS(PI*FKNU(I))
            T=FCA*SINF-FSA*COSF
            TP=FCPA*SINF-FSPA*COSF
            SSP0=T/TP
!
! RE-NORMALISED ENERGY DENSITY
!
!             W(T,TD)=1
            TD=SINF*FSA+COSF*FCA
            TDP=SINF*FSPA+COSF*FCPA
!
!             W(T,TD)=2/PI*NU**3
!           TD=TD*1.570796327*FKNU(I)**3
!           TDP=TDP*1.570796327*FKNU(I)**3
!
            IF(R2ST(I).GT.RTWO)THEN
              KRA=KINF
              KRB=KP2
              TKRA=RINF0
              ICONV=0
              FS(KRA,I)=T
              FSP(I)=TP
              FC(KRA,I)=TD
              FCP(I)=TDP
            ENDIF
!NRB
          ENDIF
          IF(ICONV.EQ.0)THEN
            IF(RINF(I).GE.TZERO.AND.IOMIT(I).LE.0)THEN
              IF(KRA.GT.KRB.OR.IPERT.NE.0.OR.IRAD.GT.0)
     X        CALL NUMT(EPS(I),CCT(I),TKRA,H,KRA,KRB,I)
                          IF(KRA.GT.KRB)INOUT(I)=0
                          IF(KRA.LE.KRB)INOUT(I)=1
              IF(KINF.GT.1)CALL NUMT(EPS(I),CCT(I),TKRA,H,KINF,-1,I)
              IF(IPRINT.GT.0.AND.RINF(I).GT.RZERO.AND.R2ST(I).LE.RTWO)
     X                                                           THEN
                SSP2=FS(1,I)/FSP(I)
                IF(ABS((SSP0-SSP2)/SSP2).GT.100*AC)THEN
                  WRITE(6,640)I,RINF(I),SSP0,SSP2
                ENDIF
              ENDIF
            ENDIF
          ELSE
            IPERT=0
            IF(IRAD.GT.0) THEN
              WRITE(6,630)ETOT,I,LLCH(I),FKNU(I)
              STOP 630
            ENDIF
          ENDIF
!
!...  CASE OF R2ST(I).GT.RTEST, PERTUBATION CANNOT BE USED
!   NO LONGER ENTER HERE IF DEFAULT NPERT=1/PERT='YES' IS USED.
!
        ELSE
          IPERT=0
          CALL SC(EPS(I),LLCH(I),RZERO,AC,FSA,FSPA,FCA,FCPA)
          SINF=SIN(PI*FKNU(I))
          COSF=COS(PI*FKNU(I))
          FS(1,I)=FCA*SINF-FSA*COSF
          FSP(I)=FCPA*SINF-FSPA*COSF
          IF(IRAD.GT.0) THEN
            SS=FS(1,I)
            SSP=FSP(I)
            CALL NUMS(EPS(I),CCT(I),RZERO,H,1,KP2,SS,SSP,FST)
            INOUT(I)=1
            DO K=1,KP2
              FS(K,I)=FST(K)
            ENDDO
          ENDIF
        ENDIF
!          WRITE(0,*)'INSIDE COUL',LRGL2,LPRTSW,IPERT,' 3'
        IF(RINF(I).GT.RZERO.AND.LRGL2.GT.LPRTSW)IPERT=-IABS(IPERT)
        ENDIF
      ENDDO
!
      IF(IFLUG.GT.0.AND.IPRINT.GT.1)WRITE(6,730)
!
      IF(IPRINT.GT.1)THEN
        WRITE(6,701)ETOT
        WRITE(6,705)RTWO,KP2,H
        WRITE(6,700)
        IMAX=1
!        IMAX=KP2
        DO J=1,NCHF
          DO I=1,IMAX
            WRITE(6,710)J,I,FS(I,J),FSP(J),FC(I,J),FCP(J)
          ENDDO
        ENDDO
      ENDIF
!
!
      IF(IPRTSW.GT.0)IPERT=IABS(IPERT)
      IF(IPRTSW.LT.0)IPERT=-IABS(IPERT)
!      WRITE(0,*)'IPRTSW',IPRTSW
      DEALLOCATE(FST)
!
      RETURN
!
  600 FORMAT(23X,' I =',I4,', RINF =',F7.2,', W0 =',F9.6,
     +  ', W2 =',F9.6/)
  601 FORMAT(23X,' I =',I4,', R2(I)=',F7.2,', W2 =',F9.6,
     +  ', W2C=',F9.6/)
  610 FORMAT('SR.COUL: INCREASE MZPTS TO ',I6)
  620 FORMAT(///10X,30('*')//10X,'FOR ETOT = ',E14.6/
     + 10X,'(R2ST(',I2,')=',F8.2,').GT.(RTWO = ',F8.2,')  AND'/
     + 10X,'(RINF(',I2,') = ',F8.2,').GT.(RZERO=',F8.2,')'/
     + 10X,'KP2 = ',I4,/
     + 10X,'CANNOT CALCULATE RADIATIVE DATA FOR THIS CASE'/
     + 10X,'TRY LARGER VALUE OF MZPTS'///)
  630 FORMAT(///10X,30('*')//10X,'FOR ETOT = ',E14.6/
     + 10X,'CHANNEL ',I2,' HAS'/
     + 10X,'CHANNEL ANGULAR MOMENTUM QUANTUM NUMBER = ',I2/
     + 10X,'CHANNEL EFFECTIVE QUANTUM NUMBER = ',F8.2/
     + 10X,'CANNOT CALCULATE RADIATIVE DATA FOR THIS CASE.'
     + /10X,'TRY SMALLER VALUE OF QNMAX OR LARGER'
     +, ' VALUE OF MZTET'///)
  631 FORMAT(/5X,'SUBROUTINE COUL, IE=',I5,'  ICHAN=',I3,'  LL = ',I2,
     +   '  FNU = ', F10.2,'  RHO = ',1PE10.3,'  W = ',0PF10.6)
  640 FORMAT(' ***WARNING: POSSIBLE INACCURACY IN SR.THETA, RINF'
     X,' EXCEEDS RZERO FOR CHANNEL'/
     X 3X,' I =',I3,', RINF =',F7.2,', AT R0 S/SP(SC) =',1PE13.6,
     +  ', S/SP(THETA) =',1PE13.6/)
  700 FORMAT(//10X,'COULOMB FUNCTIONS S,SP,C AND CP'/)
  701 FORMAT(//10X,' E = ',F10.6/11X,14('-'))
  705 FORMAT(//'   RTWO = ',1PE12.4,',  KP2 = ',I4,',  H = ',0PF10.6)
  710 FORMAT(2I5,4E15.6)
  720 FORMAT(5X,5('*'),' REGULAR COULOMB FUNCTION FROM SERIES ',5('*')/)
  730 FORMAT(3X,'I',3X,'RTWOT',8X,'RTWO')
  735 FORMAT(I5,2F10.4)
!
      END
!
!**********************************************************
!
      SUBROUTINE COULFG(LL,EPS,RHO,ACC,F,FP,G,GP,K,IERR,ACTACC)
!     + ABSF,ABSG)
!
!  CALCULATES COULOMB FUNCTIONS F AND G AND THEIR DERIVATIVES
!
!  ORIGINAL VERSION PUBLISHED IN COMP. PHYS. COMM.25, 87, 1982.
!  PRESENT VERSION MODIFIED TO AVOID UNDERFLOW AND OVERFLOW
!  CONDITIONS IN THE SUMMATIONS OVER N OF
!         U(N)=A(N)*RHO**(N+L+1)
!     AND V(N)=D(N)*RHO**(N+L+1)
!  U(N) AND V(N) ARE CALCULATED RECURSIVELY.
!
!
!  INPUT -
!        LL=ANGULAR MOMENTUM QUANTUM NUMBER
!        EPS=Z-SCALED ENERGY IN RYDBERGS
!        RHO=Z-SCALED RADIAL VARIABLE IN ATOMIC UNITS
!        ACC=ACCURACY REQUIRED
!
!  OUTPUT -
!        F=REGULAR FUNCTION
!        FP=DERIVATIVE OF F
!        G=IRREGULAR FUNCTION
!        GP=DERIVATIVE OF G
!        K=NUMBER OF TERMS NEEDED IN EXPANSION
!        IERR=ERROR CODE
!        ACTACC=ACCURACY ACTUALLY ACHIEVED
!
!  CONVERGENCE CRITERION -
!        VALUE OF WRONSKIAN CONVERGED TO ACCURACY OF 0.5*ACC
!
!  ERROR CODES -
!        IERR=0, CONVERGED WITH ACTACC.LT.ACC
!        IERR=1, CONVERGED WITH ACTACC.GT.ACC
!        IERR=2, NOT CONVERGED WITH 301 TERMS IN MAIN SUMMATION
!        IERR=3, NOT CONVERGED - DANGER OF OVERFLOW
!
      IMPLICIT REAL*8 (A-H,O-Z)
!
      DOUBLE PRECISION DUBU0,DUBU1,DUBU2,DUBV0,DUBV1,DUBV2,
     +  DUBQ1,DUBF,DUBFP,DUBS,DUBSP,DA4,DP2,DREPS,DFNPLP,DRHO,DP1,DCLP
     + ,W1D,W2D,DDW1,DDW2,DONE,DTWO
!    + ,DABSF,DABSG
!
!NRB:
!     DO NOT USE ANYTHING LESS THAN *8 ANYWHERE. IF POSSIBLE, USE
!     EXPLICIT *16 FOR LARGE CASES, ESPECIALLY ON A CRAY. 
!
      PARAMETER (TZERO=0.0)
      PARAMETER (ONE=1.0)
      PARAMETER (TWO=2.0)
      PARAMETER (FOUR=4.0)
!
!  INITIALIZATION
!
      PI=ACOS(-ONE)
      EULER=-0.577215664901532860606512      !INELEGANT
      R2PI=ONE/(TWO*PI)
      PS0=ONE+TWO*EULER
!
      IERR=0
      LP1=LL+1
      L2=2*LL
      L2P1=L2+1
      FL=LL
      FLP1=LP1
      FL2P1=L2P1
      E2=EPS/TWO
      R2=TWO*RHO
      ACC2=TWO*ACC
!
!     INITIALIZE FA=FACTORIAL(2*LL+1)
!     AND PS=PSI(2*LL+2)+PSI(1)
!
      FA=ONE
      PS=PS0
!
!
!  CALCULATE ALPHA(N) AND BETA(N) AND INITIALIZE S AND SP
!  CONTINUE CALCULATION OF FA AND PS
!
!     S AND SP FOR N=0
      X3=-L2
      X2=L2P1
      X1=-TWO*R2**(-LP1)
      SP=X3*X1
      X1=R2*X1
      S=X1
!                ABSG=ABS(X1)
!
!     INITIALIZE FOR COEFFICIENTS IN RECURSION FORMULAE
      P1=FL*E2
      P2=P1
      Q1=-E2
!
!     INITIALIZE ALPHA AND BETA
      ALP1=ONE
      ALP2=ONE+P2
      BET1=TZERO
      BET2=Q1
!
      IF(LL.EQ.0)GOTO 20
!
!     S AND SP FOR N=1
      X3=X3+TWO
      X2=X2-ONE
      X1=X1/X2
      SP=SP+X3*X1
      X1=R2*X1
      S=S+X1
!               ABSG=ABSG+ABS(X1)
!
!     LOOP FOR N=2 TO 2*LL
      DO 10 N=2,L2
!
!     CONTINUE CALCULATION OF FA AND PSI
      FN=N
      FA=FN*FA
      PS=PS+ONE/FN
!
!     CONTINUE CALCULATION OF S AND SP
      X3=X3+TWO
      X2=X2-ONE
      X1=X1/(X2*FN)
      SP=SP+X3*X1*ALP2
      X1=R2*X1
      S=S+X1*ALP2
!                ABSG=ABSG+ABS(X1*ALP2)
!
!     COMPUTE COEFFICIENTS IN RECURSION FORMULAE
      P1=P1-E2
      P2=P2+P1
      Q1=Q1-E2
!     NOW HAVE P2=-N*(N-2*LL-1)*EPS/4
!     AND Q1=-N*EPS/2
!
!     NEW ALPHA AND BETA
      ALP0=ALP1
      ALP1=ALP2
      ALP2=ALP1+P2*ALP0
      BET0=BET1
      BET1=BET2
      BET2=BET1+P2*BET0+Q1*ALP0
  10  CONTINUE
!
!     NORMALIZE S AND SP, COMPLETE CALCULATION OF FA AND PS
      S=S*FA
      SP=SP*FA
!                ABSG=ABSG*ABS(FA)
      FA=FL2P1*FA
      PS=PS+ONE/FL2P1
!
!     COMPLETE CALCULATION OF ALPHA AND BETA
      P1=P1-E2
      P2=P2+P1
      Q1=Q1-E2
      ALP0=ALP1
      ALP1=ALP2
      BET0=BET1
      BET1=BET2
      BET2=BET1+P2*BET0+Q1*ALP0
!
   20 CONTINUE
!     NOW HAVE ALP1=ALPHA(2*LL+1)
!     AND BET1=BETA(2*LL+1), BET2=BETA(2*LL+2)
!
!     VALUE OF A=A(EPS,LL)
      A=ALP1
      A4=FOUR*A
      CL=TWO*A*LOG(ABS(R2))
      CLP=TWO*A/RHO
!
!  CALCULATE F AND FP AND CONTINUE CALCULATION OF S AND SP
!
!     CALCULATE A0,A1,D0,D1
      A0=(TWO**LP1)/FA
      A1=-A0/FLP1
      PS=TWO*PS*A
      D0=(BET1-PS)*A0
      D1=(BET2-PS-(TWO+ONE/FLP1)*A)*A1
!
!     INITIALIZE F,FP, CONTINUE CALCULATION OF S,SP
!          -VALUES FOR N=0
!           U0 AND V0
      FNPLP1=FLP1
      C1=RHO**LL
      U0=A0*C1
      V0=D0*C1
      FP=FNPLP1*U0
      SP=SP+FNPLP1*V0
      U0=U0*RHO
      V0=V0*RHO
      F=U0
!                ABSF=ABS(F)
      S=S+V0
!                ABSG=ABS(V0)      +ABSG
      W1D=F*(CLP*F+SP)-FP*S
      NNN=0
!
!          - VALUES FOR N=1
!            U1 AND V1
      FNPLP1=FNPLP1+ONE
      C1=C1*RHO
      U1=A1*C1
      V1=D1*C1
      FP=FP+FNPLP1*U1
      SP=SP+FNPLP1*V1
      U1=U1*RHO
      V1=V1*RHO
      F=F+U1
!                ABSF=ABSF+ABS(U1)
      S=S+V1
!                ABSG=ABSG+ABS(V1)
      W2D=F*(CLP*F+SP)-FP*S
      DDW2=ABS(W2D-W1D)
!
!     INITIALIZE FOR COEFFICIENTS IN RECURSION FORMULAE
      P1=-TWO*FLP1
      P2=P1
      Q1=A4+TWO*A*FL2P1
      REPS=RHO*EPS
!
!  CONVERT TO DOUBLE
                            DONE=1
                            DTWO=2
                          DUBU0=U0
                          DUBU1=U1
                          DUBV0=V0
                          DUBV1=V1
                          DUBQ1=Q1
                          DUBS=S
                          DUBSP=SP
                          DUBF=F
                          DUBFP=FP
!           DABSF=ABSF
!           DABSG=ABSG
                     DA4=A4
                     DP2=P2
                     DREPS=REPS
                     DFNPLP=FNPLP1
                     DRHO=RHO
                     DP1=P1
                     DCLP=CLP
!                  W1D=W1
!                  W2D=W2
!                  DDW1=DW1
!                  DDW2=DW2
!     LOOP FOR N=2 TO 300
      DO 40 N=2,300
!
!     COMPUTE COEFFICIENTS IN RECURSION FORMULAE
      DP1=DP1-DTWO
      DP2=DP2+DP1
      DUBQ1=DUBQ1+DA4
!     NOW HAVE P2=-N*(N+2*LL+1)
!     AND DUBQ1=2*A*(2*N+2*LL+1)
!
!      COMPUTE DUBU2=U(N) AND DUBV2=V(N)
      DUBU2=(DTWO*DUBU1+DREPS*DUBU0)/DP2
      DUBV2=(DTWO*DUBV1+DREPS*DUBV0+DUBQ1*DUBU2)/DP2
!
!     INCREMENT DUBFP AND DUBSP
      DFNPLP=DFNPLP+DONE
      DUBFP=DUBFP+DFNPLP*DUBU2
      DUBSP=DUBSP+DFNPLP*DUBV2
!
!     INCREMENT DUBF AND DUBS
      DUBU2=DUBU2*DRHO
      DUBV2=DUBV2*DRHO
      DUBF=DUBF+DUBU2
!                DABSF=DABSF+ABS(DUBU2)
      DUBS=DUBS+DUBV2
!                DABSG=DABSG+ABS(DUBV2)
!
!     CALCULATE WRONSKIAN
      W1D=W2D
      DDW1=DDW2
      W2D=DUBF*(DCLP*DUBF+DUBSP)-DUBFP*DUBS
      DDW2=ABS(W2D-W1D)
!
!     CONVERGENCE TEST
      K=N+1
      IF(DDW1.GT.ACC2)GOTO 30
      IF(DDW2.GT.ACC2)GOTO 30
      IF(ABS(W2D).LT.1.E10)GOTO 50
!
! TEST FOR OVERFLOW
   30 IF(ABS(DUBS)+ABS(DUBSP)+ABS(DUBF)+ABS(DUBFP).GT.1.D100)THEN
      IERR=3
      RETURN
      ENDIF
!  NEW DUBU0,DUBU1,DUBV0,DUBV1
      DUBU0=DUBU1
      DUBU1=DUBU2
      DUBV0=DUBV1
      DUBV1=DUBV2
!
   40 CONTINUE
!
!  NOT CONVERGED
!
      IERR=2
      ACTACC=ABS(0.25*W2D-1.)
      GOTO 60
!
!  CONVERGED
!
   50 ACTACC=ABS(0.25*W2D-1.)
      IF(ACTACC.GT.ACC)IERR=1
!
!  COMPLETE CALCULATION OF G AND GP
!
   60                    S=DUBS
                         SP=DUBSP
                         F=DUBF
                         FP=DUBFP
!                         ABSF=DABSF
!                         ABSG=DABSG
!                         ABSG=ABSG*R2PI
      G=(S+CL*F)*R2PI
      GP=(SP+CL*FP+CLP*F)*R2PI
!
      RETURN
!
      END
!**********************************************************
!
      SUBROUTINE COULS(LL,EPS,RHO,S,SP)
      USE CACC
      USE CNTRL
      USE NRBZED
!
!  CALCULATES COULOMB FUNCTION S AND ITS DERIVATIVE SP
!  FROM POWER-SERIES EXPANSION.
! NRB
!  NEUTRAL CASE ADDED
!
      IMPLICIT REAL*8 (A-H,O-Z)
!
      DOUBLE PRECISION C1,D0,D1,D2,DM,DEPS,DRHO,F,FLP1,FNPLP1,FP,
     +  P1,P2,REPS,U0,U1,U2,UM, A,B,C, T0,DZ
!
!NRB:
!     DO NOT USE ANYTHING LESS THAN *8 ANYWHERE. IF POSSIBLE, USE
!     EXPLICIT *16 FOR LARGE CASES, ESPECIALLY ON A CRAY. 
!
      LOGICAL T(0:2),TP(0:2)
!
      PARAMETER (TZERO=0.0)
      PARAMETER (ONE=1.0)
      PARAMETER (TWO=2.0)
!
!  INITIALISATIONS
!
      TPI=TWO*ACOS(-ONE)
      DRHO=RHO
      DEPS=EPS
      DZ=TZED
      FLP1=LL+1
      ACC10=.1*AC
      DO I=0,1
      T(I)=.FALSE.
      TP(I)=.FALSE.
      ENDDO
      T0=ONE
      NSUM=200
      IF(TZED.EQ.0)THEN
        IF(DEPS.LT.0)THEN
          WRITE(6,*)
     X 'ERROR: SR.COULS IS NOT CODED FOR NEGATIVE ENERGY NEUTRALS'
          STOP'SR.COULS IS NOT CODED FOR NEGATIVE ENERGY NEUTRALS'
        ENDIF
        T0=SQRT(SQRT(DEPS))
        NSUM=400
      ENDIF
!
!  POWER-SERIES EXPANSION
!  **********************
!
!   VALUES FOR N=0
      FNPLP1=FLP1
      C1=T0*DRHO**LL
      U0=C1
      D0=FNPLP1*U0
      DM=ABS(D0)
      FP=D0
      U0=U0*DRHO
      UM=ABS(U0)
      F=U0
!
!   VALUES FOR N=1
      FNPLP1=FNPLP1+ONE
      C1=C1*DRHO
      U1=-C1*DZ/FLP1
      D1=FNPLP1*U1
      DM=MAX(ABS(D1),DM)
      FP=FP+D1
      U1=U1*DRHO
      UM=MAX(ABS(U1),UM)
      F=F+U1
!
!   INITIALIZE FOR COEFFICIENTS IN RECURSION FORMULAE
      P1=-TWO*FLP1
      P2=P1
      REPS=DRHO*DEPS
!
!   LOOP FOR N=2 TO 200
      DO 40 N=2,NSUM
!     COMPUTE COEFFICIENTS IN RECURSION FORMULAE
      P1=P1-TWO
      P2=P2+P1
!     NOW HAVE P2=-N*(N+2*LL+1)
!     COMPUTE U2 AND INCREMENT FP
      FNPLP1=FNPLP1+ONE
      U2=(TWO*DZ*U1+REPS*U0)/P2
      D2=FNPLP1*U2
      DM=MAX(ABS(D2),DM)
      FP=FP+D2
!     MODIFY U2 AND INCREMENT F
      U2=U2*DRHO
      UM=MAX(ABS(U2),UM)
      F=F+U2
!        WRITE(67,*)N,F,U2,FP,D2
!     TEST CONVERGENCE
      IF(ABS(U2).LT.ABS(F)*ACC10)THEN
        T(2)=.TRUE.
      ELSE
        T(2)=.FALSE.
      ENDIF
      IF(ABS(D2).LT.ABS(FP)*ACC10)THEN
        TP(2)=.TRUE.
      ELSE
        TP(2)=.FALSE.
      ENDIF
      DO I=0,2
      IF(.NOT.T(I))GOTO 21
      ENDDO
      IF(IPRINT.GT.1)THEN
        UM=UM/ABS(F)
        WRITE(6,610)LL,EPS,UM
      ENDIF
      GOTO 50
   21 DO I=0,2
      IF(.NOT.TP(I))GOTO 23
      ENDDO
      IF(IPRINT.GT.1)THEN
        DM=DM/ABS(FP)
        WRITE(6,620)LL,EPS,DM
      ENDIF
      GOTO 50
!     NEW U0,U1,T AND TP
   23 U0=U1
      U1=U2
      DO I=0,1
      T(I)=T(I+1)
      TP(I)=TP(I+1)
      ENDDO
   40 CONTINUE
!
!  SERIES NOT CONVERGED
      WRITE(6,600)LL,EPS,RHO
      STOP 'COULS: SERIES NOT CONVERGED'
!
!  NORMALISATION
!  *************
!
!   NORMALISE FOR FUNCTIONS .5*F AND .5*FP
!
   50 S=F
      SP=FP
      DO K=1,LL
      C=ONE/DBLE(K*(2*K+1))
      S=S*C
      SP=SP*C
      ENDDO
!
!   CALCULATE CAP B AND FUNCTIONS S AND SP
      IF(EPS.GT.0)THEN
        A=ONE
        IF(LL.GT.0)THEN
          A1=DZ
          A2=-EPS
          A3=EPS+EPS
          DO I=1,LL
          A2=A2+A3
          A1=A1+A2
          A=A*A1
          ENDDO
        ENDIF
        IF(EPS.LT.0.01.OR.TZED.EQ.0)THEN
          B=A
        ELSE
          B=A/(ONE-EXP(-TPI/SQRT(EPS)))
        ENDIF
      C=SQRT(B)
      ELSE
      C=ONE
      ENDIF
      IF(TZED.GT.0)C=C*SQRT(TPI)
      S=S*C
      SP=SP*C
!
      RETURN
!
  600 FORMAT(//10X,60('*')//10X,'SERIES IN COULS NOT CONVERGED'
     + /10X,' LL =',I3,',  EPS =',1PE15.5,',  RHO =',
     +  E15.5//10X,60('*')//)
  610 FORMAT(/5X,'SUBROUTINE COULS, LL = ',I2,',  EPS = ',
     +  1PE12.5,',  UM = ',E10.2)
  620 FORMAT(/5X,'SUBROUTINE COULS, LL = ',I2,',  EPS = ',
     +  1PE12.5,',  DM = ',E10.2)
      END
      SUBROUTINE DIAG(N,IUP,Z,D,E,MXMAT)
!
      IMPLICIT REAL*8 (A-H,O-Z)
!
! BADNELL & BURGESS  D.A.M.T.P. CAMBRIDGE
!
! DIAGONALIZATION OF REAL SYMMETRIC N-BY-N MATRIX Z.
!
! METHOD: HOUSEHOLDER REDUCTION TO TRI-DIAGONAL FORM AND SHIFTED
!         QL ALGORITHM TO DETERMINE THE E-VALUES AND E-VECTORS.
!
! BASED ON MARTIN, REINSCH & WILKINSON: NUM. MATH. 11, 181-95 (1968).
!
! INPUT REQUIRED. N, IUP AND Z. ONLY LOWER TRIANGLE OF Z NEED BE SUPPLIED.
!                          MATRIX Z OVERWRITTEN BY EIGENVECTORS OF Z.
!                 IUP=1/-1 ASC/DESCENDING SORT, 0 NO SORT.
!                 MXMAT, IS THE ROW DIMENSION OF Z IN THE CALLING ROUTINE.
!
! OUTPUT.         Z AND D, WHERE Z CONSISTS OF COLUMN EIGENVECTORS
!                          AND D CONSISTS OF CORRESPONDING EIGENVALUES.
!
! NOTE: E IS A WORKING ARRAY.
!
      PARAMETER (TOL = 1.0D-75)
      PARAMETER (EPS = 1.0D-15)
      PARAMETER (ZERO = 0.0D0)
      PARAMETER (ONE = 1.0D0)
      PARAMETER (JMAX = 30)
!
      DIMENSION D(N),E(N),Z(MXMAT,N)
!
!
      DO 1 I = 1,N
      D(I) = Z(N,I)
   1  CONTINUE
      IF (N.LE.1) GO TO 20
!
! HOUSEHOLDER REDUCTION TO TRI-DIAGONAL FORM
!
      DO 19 I = N,2,-1
      L = I - 1
      F = D(I-1)
      G = ZERO
      DO 5 K = 1,I-2
      G = G + D(K)*D(K)
  5   CONTINUE
      H = G + F*F
      IF (G.GT.TOL) GO TO 8
      E(I) = F
      H = ZERO
      DO 7 J = 1,L
      D(J) = Z(L,J)
      Z(I,J) = ZERO
      Z(J,I) = ZERO
  7   CONTINUE
      GO TO 18
  8   G = SQRT(H)
      IF (F.GE.ZERO) G = -G
      E(I) = G
      H = H - F*G
      D(L) = F - G
      DO 14 J = 1,L
      E(J) = ZERO
  14  CONTINUE
      DO 15 J = 1,L
      Z(J,I) = D(J)
      G = E(J) + Z(J,J)*D(J)
      DO 13 K = J+1,L
      G = G + Z(K,J)*D(K)
      E(K) = E(K) + Z(K,J)*D(J)
  13  CONTINUE
      E(J) = G
  15  CONTINUE
      F = ZERO
      DO 12 J = 1,L
      E(J) = E(J)/H
      F = F + E(J)*D(J)
  12  CONTINUE
      HH = F/(H+H)
      DO 11 J = 1,L
      E(J) = E(J) - HH*D(J)
  11  CONTINUE
      DO 17 J = 1,L
      F = D(J)
      G = E(J) 
      DO 16 K = J,L
      Z(K,J) = Z(K,J) - F*E(K) - G*D(K)
  16  CONTINUE
      D(J) = Z(L,J)
      Z(I,J) = ZERO
  17  CONTINUE
  18  D(I) = H
  19  CONTINUE
!
!
! ACCUMULATE TRANSFORMATION MATRICES
!
      DO 28 I = 2,N
      L = I - 1
      Z(N,L) = Z(L,L)
      Z(L,L) = ONE
      H = D(I)
      IF (H.EQ.ZERO) GO TO 25
      DO 21 K = 1,L
      D(K) = Z(K,I)/H
  21  CONTINUE
      DO 24 J = 1,L
      G = ZERO
      DO 22 K = 1,L
      G = G + Z(K,I)*Z(K,J)
  22  CONTINUE
      DO 23 K = 1,L
      Z(K,J) = Z(K,J) - G*D(K)
  23  CONTINUE
  24  CONTINUE
  25  DO 27 J = 1,L
      Z(J,I) = ZERO
  27  CONTINUE
  28  CONTINUE
      DO 29 I = 1,N
      D(I) = Z(N,I)
      Z(N,I) = ZERO
  29  CONTINUE
  20  E(1) = ZERO
      Z(N,N) = ONE
!
!
! SHIFTED QL ALGORITHM TO DETERMINE E-VALUES & E-VECTORS
!
      DO 32 I = 2,N
      E(I-1) = E(I)
  32  CONTINUE
      E(N) = ZERO
      B = ZERO
      F = ZERO
      DO 54 L = 1,N
      J = 0
      H = EPS*(ABS(D(L))+ABS(E(L)))
      IF (B.LT.H) B = H
      DO 36 M = L,N
      IF (ABS(E(M)).LE.B) GO TO 37
  36  CONTINUE
  37  IF (M.EQ.L) GO TO 53
  38  IF (J.EQ.JMAX) GO TO 62
      J = J+ 1
      P = E(L) + E(L)
      G = D(L)
      H = D(L+1) - G
      IF (ABS(H).GE.ABS(E(L))) GO TO 43
      P = H/P
      R = SQRT(P*P+ONE)
      H = P + R
      IF (P.LT.ZERO) H = P - R
      D(L) = E(L)/H
      GO TO 44
  43  P = P/H
      R = SQRT(P*P+ONE)
      D(L) = E(L)*P/(R+ONE)
  44  H = G - D(L)
      DO 46 I = L+1,N
      D(I) = D(I) - H
  46  CONTINUE
      F = F + H
      P = D(M)
      C = ONE
      S = ZERO
      DO 52 I = M-1,L,-1
      G = C*E(I)
      H = C*P
      IF (ABS(P).LT.ABS(E(I))) GO TO 49
      C = E(I)/P
      R = SQRT(C*C+ONE)
      E(I+1) = S*P*R
      S = C/R
      C = ONE/R
      GO TO 50
  49  C = P/E(I)
      R = SQRT(C*C+ONE)
      E(I+1) = S*E(I)*R
      S = ONE/R
      C = C/R
  50  P = C*D(I) - S*G
      D(I+1) = H + S*(C*G+S*D(I))
      DO 51 K = 1,N
      H = Z(K,I+1)
      Z(K,I+1) = S*Z(K,I) + C*H
      Z(K,I)   = C*Z(K,I) - S*H
  51  CONTINUE
  52  CONTINUE
      E(L) = S*P
      D(L) = C*P
      IF (ABS(E(L)).GT.B) GO TO 38
  53  D(L) = D(L) + F
  54  CONTINUE
!
      IF(IUP.EQ.0)RETURN
!
! BEGIN SORTING INTO ASCENDING E-VALUES
!
      DO 61 I = 1,N
      K = I
      P = D(I)
      DO 57 J = I+1,N
      IF (IUP.GT.0.AND.D(J).GT.P) GO TO 57
      IF (IUP.LT.0.AND.D(J).LT.P) GO TO 57
      K = J
      P = D(J)
  57  CONTINUE
      IF (K.EQ.I) GO TO 61
      D(K) = D(I)
      D(I) = P
      DO 60 J = 1,N
      P = Z(J,I)
      Z(J,I) = Z(J,K)
      Z(J,K) = P
  60  CONTINUE
  61  CONTINUE
!
      RETURN
!
!
  62  WRITE(6,100)
 100  FORMAT(' FAILED IN DIAG, TOO MANY ITERATIONS')
      RETURN
!
      END
!***************************************************************
!
      SUBROUTINE EPHASE
      USE NRBRCT
      USE COMEGA
      USE CEN
      USE CINPUT
      USE CHAN
!
! EVALUATE EIGENPHASE SUM
!
      IMPLICIT REAL*8 (A-H,O-Y)
      IMPLICIT COMPLEX*16 (Z)
!     PARAMETER (MZKIL=  0)
!     PARAMETER (MXTST=(MZTAR*MZTAR+MZTAR)/2)
!
      PARAMETER (ONE=1.0)
      PARAMETER (TZERO=0.0)
      PARAMETER (TWO=2.0)
      PARAMETER (FOUR=4.0)
!
      REAL*8,ALLOCATABLE :: RK2(:,:),DDD(:),EEE(:)
!
      DATA ICASE/0/
!
      ALLOCATE(RK2(NCHF,NCHF))
      ALLOCATE(DDD(NCHF))
      ALLOCATE(EEE(NCHF))
!
!
      IF(ICASE.EQ.0)THEN
        OPEN(UNIT=34,FILE='ESUM',STATUS='UNKNOWN')
        ICASE=1
      ENDIF
!
      DO J=1,NCHPP
        DO I=1,NCHPP
          RK2(I,J)=RK(I,J)
        ENDDO
      ENDDO
!
      IUP=0
      CALL DIAG(NCHPP,IUP,RK2,DDD,EEE,NCHF)
!
      SUM=TZERO
      DO I=1,NCHPP
        SUM=SUM+ATAN(DDD(I))
      ENDDO
      PI=FOUR*ATAN(ONE)
      EMIN=TZERO
      IF(LRGL2.GT.1)EMIN=-0.05
   99 CONTINUE
      IF(SUM.LT.EMIN)THEN
        SUM=SUM+PI
        GO TO 99
      ENDIF
      IF(SUM.GT.PI)THEN
        SUM=SUM-PI
        GO TO 99
      ENDIF
      IF(IE.EQ.1)WRITE(34,1000)NSPN2,LRGL2,NPTY2
      WRITE(34,1001)ETOT,SUM
!
 1000 FORMAT('#',I3,I4,I2)
 1001 FORMAT(1P2E16.8)
!
!
      DEALLOCATE(RK2,DDD,EEE)
!
      RETURN
      END
!***********************************************************************
       REAL*8 FUNCTION F21(A,B,C,D,EPS,IFAIL)
       IMPLICIT REAL*8 (A-H,O-Z)
!
       IPRINT=IFAIL
       IFAIL=0
       T=(A*B*D)/C
       DD=1.0D0/(1.0D0-D)
       SUM=1.0D0+T
       TN1=0.0D0
       I=1
    3  AI=I
       T=T*(A+AI)*(B+AI)*D/((C+AI)*(1.0D0+AI))
       TN2=T*DD
       F21=SUM+TN2
       SUM=SUM+T
       AT=ABS(T+TN2-TN1)
       AS=ABS(F21)*EPS
       IF(AS-AT)1,2,2
    1  TN1=TN2
       I=I+1
       IF(I-300)3,3,4
    4  IF(IPRINT.GT.0)WRITE(6,100)
       IFAIL=3
  100  FORMAT(' FAILED TO CONVERGE IN F21')
    2  RETURN
      END
!
!***************************************************************
      Subroutine fastread2(iopt2)
!
! Subroutine to do a fast read of the H.DATXXX files and
! relate which one contains the information about the partial
! wave islp. 
! This information is stored in the array IWHD(1:kslp)
! See subroutine READ2 for more details about what is stored
! in each register
!
      USE TERMSYM
      USE PART
      USE CINPUT
      USE CNTRL
!
      Implicit None
      Integer, Intent(in) :: iopt2
      Logical found
      Integer iam,nproc
      Integer kslp2,k,l,iupper,ilow,ndiv,icolour,ic,id,iu,ier,ncfgp
      Character(len=1) :: 
     &    NUM(0:9)=["0","1","2","3","4","5","6","7","8","9"]
      Integer, Allocatable :: IAUX1(:),IAUX2(:)
!
      Allocate(IAUX1(1000),IAUX2(1000))
      If (nhdat.eq.-1) Then
! Single file ...
        kslp2=0
        more2=1
        Do While (more2.ne.0)
          If (iword.eq.1) Then
! H.DAT
            READ(10,Iostat=ier) LRGL2,NSPN2,NPTY2,NCHAN,MNP2,MORE2
            If (ier.lt.0) Exit     ! EOF found
          Else If (iword.eq.2) Then
! DSTGH.DAT
            READ(10,Iostat=ier) LRGL2,NPTY2,NCHAN,MNP2,NCFGP,LAMAX
            If (ier.lt.0) Exit     ! EOF found
            NSPN2=0
            LRGL2=ABS(LRGL2)-1
            IF (NPTY2.EQ.-1) THEN
              NPTY2=1
            ELSE
              NPTY2=0
            ENDIF
          End If
          kslp2=kslp2+1
! Store ISLP and IWHD
          IAUX1(kslp2)=10000*IABS(NSPN2)+100*LRGL2+NPTY2
          IAUX2(kslp2)=-1
!
          READ(10) ! (NCONAT(I),I=1,NASTR)    
          Read(10) ! (L2P(I),I=1,NCHAN) or (L2P(I),I=1,NCHAN), (KJ(I),I=1,NCHAN)
          If (LAMAX.GT.0) Then
            READ(10) ! (((CF(I,N,M),I=1,NCHAN),N=1,NCHAN),M=1,LAMAX)
          End If
          If (iprcent.ne.100) Then
            Read(10) ! trace
          End If
          If (MNP2.ge.0) then                 ! begin
            READ(10) ! (VALUE(I),I=1,MNP2)
            READ(10) ! ((WMATA(I,K),K=1,NCHAN),I=1,MNP2)
          Else
            READ(10) ! (VALUE(I), I=1,MNP2)
            ilow=999
            Do While (ilow.ne.1)
              READ(10) ILOW,IUPPER,NDIV
              READ(10) ! ((WMATA(K,I-ILOW+1),K=1,NCHAN),I=ILOW,IUPPER)
            End Do
          End If
        End Do
        Rewind(10)
!
      Else  
! Multiple H.DATXXX files, nhdat ones
! File H.DAT000 should be open and in position after the read of the
! SLPI independent information
        kslp2=0
        Do icolour=0,nhdat-1
          If (icolour.ge.1) Then
            ic = icolour/100
            id = (icolour - 100*ic)/10
            iu = (icolour - 100*ic - 10*id)
            OPEN(UNIT=10,FILE="H.DAT"//NUM(ic)//NUM(id)//NUM(iu), 
     &           STATUS="old",FORM="unformatted")
          End If
          Do
            READ(10,Iostat=ier) LRGL2,NSPN2,NPTY2,NCHAN,MNP2,MORE2
            If (ier.lt.0) Then      ! EOF found
              Exit
            End If
            kslp2=kslp2+1
! Store ISLP and IWHD
            IAUX1(kslp2)=10000*IABS(NSPN2)+100*LRGL2+NPTY2
            IAUX2(kslp2)=icolour
!
            READ(10) ! (NCONAT(I),I=1,NASTR)    
            Read(10) ! (L2P(I),I=1,NCHAN) or (L2P(I),I=1,NCHAN), (KJ(I),I=1,NCHAN)
            If (LAMAX.GT.0) Then
              READ(10) ! (((CF(I,N,M),I=1,NCHAN),N=1,NCHAN),M=1,LAMAX)
            End If
            If (iprcent.ne.100) Then
              Read(10) ! trace
            End If
            If (MNP2.ge.0) then                 ! begin
              READ(10) ! (VALUE(I),I=1,MNP2)
              READ(10) ! ((WMATA(I,K),K=1,NCHAN),I=1,MNP2)
            Else
              READ(10) ! (VALUE(I), I=1,-MNP2)
              ilow=999
              Do While (ilow.ne.1)
                READ(10) ILOW,IUPPER,NDIV
                READ(10) ! ((WMATA(K,I-ILOW+1),K=1,NCHAN),I=ILOW,IUPPER)
              End Do
            End If
          End Do
          Rewind(10)
          Close(10,status="keep")
        End Do
      End If
!
      If (iopt2.eq.1) Then
! Take all the symmetries stored in the files
        If (Allocated(MSLP)) Deallocate(MSLP)
        If (Allocated(IWHD)) Deallocate(IWHD)
        kslp=kslp2
        Allocate (MSLP(kslp), IWHD(kslp))
        MSLP(1:kslp)=IAUX1(1:kslp)
        IWHD(1:kslp)=IAUX2(1:kslp)
      Else If (iopt2.eq.2) Then
! Take just the symmetries already stored in MSLP
        If (Allocated(IWHD)) Deallocate(IWHD)
        Allocate (IWHD(kslp))
        Do k=1,kslp
! Check if each symmetry has been read in H.DAT files
          found=.false.
          Do l=1,kslp2
            If (MSLP(k).eq.IAUX1(l)) Then
              found=.true.
              IWHD(k)=IAUX2(l)
              Exit
            End If
          End Do
          If (.not.found) Then
            Write(6,660) MSLP(k)
            IWHD(k)=-10
          End If
        End Do
!
! Now reallocate MSLP to its new dimension
        IAUX1(1:kslp)=MSLP(1:kslp)
        Deallocate(MSLP)
        Allocate(MSLP(kslp))
        MSLP(1:kslp)=IAUX1(1:kslp)
      End If
  660 FORMAT(//10X,"WARNING NO DATA ON R-MATRIX FILE FOR"
     &        /10X,"10000*IS+100*IL+IP = ",I10)
      Deallocate(IAUX1,IAUX2)

      Return
      End Subroutine fastread2
!
!**********************************************************
!
      REAL*8 FUNCTION FDIP(EK1,L1,EK2,L2,IFAIL)
      USE NRBZED
!
      IMPLICIT REAL*8 (A-H,O-Z)
!
! ALAN BURGESS DEPT. OF APPLIED MATHS. AND THEORETICAL PHYSICS,CAMBRIDGE
! CALCULATES THE FUNCTION I(KAPPA1,L1,KAPPA2,L2,1) DEFINED IN PHIL.
! TRANS. ROY. SOC. A226,255,1970, WHERE EK1=KAPPA1**2 AND EK2=KAPPA2**2.
! IT IS SUITABLE FOR USE IN EQUATIONS (8),(9),(10) OR (11) OF
! J. PHYS. B. 7,L364,1974.
! NRB - IFAIL
      DATA EPS/1.D-4/
!
      IF(TZED.EQ.0)THEN
        FDIP=FDIP0(EK1,L1,EK2,L2,EPS,IFAIL)
        RETURN
      ENDIF
!
      IPRINT=IFAIL
      IF(EK1+EK2-1.0D-40) 11,11,12
   11   FDIP=0.0D0
        IFAIL=1
        IF(IPRINT.GT.0)WRITE(6,100)IFAIL
        RETURN
   12 IF(EK1-EK2) 1,1,2
    1 EMIN=EK1
      EMAX=EK2
      GO TO 3
    2 EMIN=EK2
      EMAX=EK1
    3 T=EMIN/EMAX
      IF(T-0.02944D0) 4,4,5
    4 FDIP=FDIP1(EK1,L1,EK2,L2)
      GO TO 9
    5 IF(T-0.16667D0) 7,6,6
    6 FDIP=FDIP2(EK1,L1,EK2,L2)
      GO TO 9
    7 FDIP=FDIP1(EK1,L1,EK2,L2)
      IF(FDIP*FDIP-1.0D-40) 6,6,8
    8   IF(FDIP.LT.0.0.OR.FDIP.GT.1.)THEN
          IFAIL=3
          IF(IPRINT.GT.0)WRITE(6,100)IFAIL
          FDIP=0.0
          RETURN
        ENDIF
        FA=FDIPA(EK1,L1,EK2,L2)
        IFAIL=0
        IF(FA.EQ.0.0)THEN
          FA=FDIP0(EK1,L1,EK2,L2,EPS,IFAIL)
          IFAIL=-IFAIL
          IF(FA.EQ.0.0)RETURN
        ENDIF
        RAT=FDIP/FA
        IF(RAT.GT.10.)THEN
          IFAIL=4
          IF(IPRINT.GT.0)WRITE(6,100)IFAIL
          FDIP=0.0
        ENDIF
        RETURN
    9 IF(FDIP*FDIP-1.0D-40) 10,10,8
   10   IFAIL=2
        IF(IPRINT.GT.0)WRITE(6,100)IFAIL
        RETURN
  100 FORMAT('***FDIP FAILURE: IFAIL=',I2)
      END
!***********************************************************************
      REAL*8 FUNCTION FDIP0(EK1,L1,EK2,L2,EPS,IFAIL)
!
      IMPLICIT REAL*8 (A-H,O-Z)
!
! ALAN BURGESS,DEPT OF APPLIED MATHS. AND THEORETICAL PHYSICS,CAMBRIDGE
! CALCULATES THE FUNCTION I0(K1,L1,K2,L2,1) DEFINED IN PHIL. TRANS.
! ROY. SOC. A266,255,1970, WHERE EK1=K1*K1, EK2=K2*K2, AND THE RELATIVE
! ACCURACY IS APPROXIMATELY EPS.
! IT IS SUITABLE FOR USE IN EQUATIONS (13) ETC. OF J.PHYS.B. 7,L364,1974
! NRB - IFAIL
!
      IPRINT=IFAIL
      IFAIL=0
      IF(L1-L2)1,2,4
1     L=L1
      GO TO 5
2     IF(IPRINT.GT.0)WRITE(6,100)L1
      IFAIL=1
      FDIP0=0.0D0
3     RETURN
4     L=L2
5     EL=L
      FDIP0=0.5D0/(EL+1.0D0)
      IF(EK1-EK2)6,3,7
6     E=EK1/EK2
      P=L1-L
      GO TO 8
7     E=EK2/EK1
      P=L2-L
8     FDIP0=FDIP0*E**((EL+P+0.5D0)*0.5D0)
! TO OBTAIN THE FUNCTION EK1 OF M.J. SEATON, PROC. PHYS. SOC. A68,457,
! 1955, REMOVE THE 'C' ON THE NEXT LINE.
!     FDIP0=1.0D0
      IF(E -0.5D0)21,20,20
20    P1=P-0.5D0
      T=P1*(EL+1.0D0)*(E -1.0D0)
      I0=L+1
      H0=0.0D0
      DO 9 I=1,I0
      TI=I
      H0=H0+1.0D0/TI
9     CONTINUE
      X=1.0D0-E
      H=1.0D0-(P+P+H0+LOG(0.25D0*X))
      S=1.0D0+T*H
      A=EL+1.0D0
      B=P1
      C=1.0D0
      D=0.0D0
10    A=A+1.0D0
      B=B+1.0D0
      C=C+1.0D0
      D=D+1.0D0
      T=T*A*B*X/(C*D)
      H=H+P1/(D*B)+EL/(C*A)
      T1=T*H
      S=S+T1
      IF(ABS(T1)-EPS*ABS(S))13,11,11
11    IF(C-300.0D0)10,12,12
12    IF(IPRINT.GT.0)WRITE(6,101)
      IFAIL=2
13    FDIP0=FDIP0*S
      RETURN
21    A=EL+1.0D0
      B=P-0.5D0
      C=EL+P+1.5D0
      F=F21(A,B,C,E,EPS,IFAIL)
      L=L+1
      EL=L
      IF(P-0.5D0)23,23,24
23    C1=EL+EL+1.0D0
      GO TO 25
24    C1=1.0D0
25    DO 22 I=1,L
      AI=I
      AII=AI+AI
      C1=C1*AI*AI*4.0D0/(AII*(AII+1.0D0))
22    CONTINUE
      FDIP0=FDIP0*F*C1
      RETURN
100   FORMAT(' FAILED IN FDIP0, L1=L2=',I5)
101   FORMAT(' FAILED TO CONVERGE IN FDIP0')
      END
!***********************************************************************
       REAL*8 FUNCTION FDIP1(EK1,L1,EK2,L2)
!
       IMPLICIT REAL*8 (A-H,O-Z)
!
       IF(L1-L2)1,2,3
    1  L=L1
       A1=EK1
       A2=EK2
       GO TO 4
    2  FDIP1=0.0D0
       RETURN
    3  L=L2
       A1=EK2
       A2=EK1
    4  LP=L+1
       ELP=LP
       B1=SQRT(1.0D0+ELP*ELP*A2)*FMON1(EK1,EK2,L)
       B2=SQRT(1.0D0+ELP*ELP*A1)*FMON1(EK1,EK2,LP)
       IF(B1*B2-1.0D-40)5,5,6
    5  FDIP1=0.0D0
       RETURN
    6  FDIP1=(B1-B2)/ELP
       RETURN
      END
!***********************************************************************
       REAL*8 FUNCTION FDIP2(EK1,L1,EK2,L2)
!
       IMPLICIT REAL*8 (A-H,O-Z)
!
       WMAX=200.0D0
       ETA1=1.0D0/SQRT(EK1)
       ETA2=1.0D0/SQRT(EK2)
       W1=ETA2-ETA1
       PI=3.141592653589793D0
       A=ABS(W1)
       B=PI*A
       IF(B-0.01D0)1,1,2
    1  C=3.0D0/(3.0D0-B*(3.0D0-B*(2.0D0-B)))
       C=SQRT(C)
       GO TO 5
    2  IF(B-14.0D0)4,3,3
    3  C=SQRT(B+B)
       GO TO 5
    4  B=B+B
       C1=1.0D0-EXP(-B)
       C=SQRT(B/C1)
    5  C=0.5D0*C/SQRT(ETA1*ETA2)
       C2=ETA1+ETA2
       C1=4.0D0*ETA1*ETA2/(C2*C2)
       L=L1
       IF(L2-L1)6,6,7
    6  L=L2
       T1=ETA1
       ETA1=ETA2
       ETA2=T1
       W1=-W1
    7  C=C*C1**(L+1)
       U0=L+1
       U1=ETA1
       V0=U0
       V1=-ETA2
       W0=1.0D0
       X0=W1/(C2*C2)
       Y2=-ETA2-ETA2
       Y0=-U0*W1+Y2
       Y1=ETA2*W1
       T1=X0/(1.0D0+W1*W1)
       Z0=U0*T1
       Z1=U1*T1
       T=Z0-Z1*W1
       Z1=Z0*W1+Z1
       Z0=T
       Q0=-1.0D0+Z0*Y0-Z1*Y1
       Q1=Z0*Y1+Z1*Y0
       X=W1*X0
    8  U0=U0+1.0D0
       V0=V0+1.0D0
       W0=W0+1.0D0
       IF(W0-WMAX)21,21,20
   20  FDIP2=0.0D0
       RETURN
   21  CONTINUE
       Y0=Y0+Y2
       T=Z0*U0-Z1*U1
       Z1=Z0*U1+Z1*U0
       Z0=T
       T=Z0*V0-Z1*V1
       Z1=Z0*V1+Z1*V0
       Z0=T
       T=Z0*W0-Z1*W1
       Z1=Z0*W1+Z1*W0
       Z0=T
       X0=X/(W0*(W0*W0+W1*W1))
       Z0=Z0*X0
       Z1=Z1*X0
       T0=Z0*Y0-Z1*Y1
       T1=Z0*Y1+Z1*Y0
       Q0=Q0+T0
       Q1=Q1+T1
       T1=T0*T0+T1*T1
       T0=Q0*Q0+Q1*Q1
       IF(T0-1.0D+24*T1)8,8,9
    9  J1=0
       J2=L+1
       P=ARGAM(J1,W1)+ARGAM(L,ETA1)-ARGAM(J2,ETA2)
       IW0=W0
       IF(A-1.0D-40)11,11,10
   10  P=P+W1*LOG(C2/A)
   11  P0=COS(P)
       P1=SIN(P)
       T=P0*Q0-P1*Q1
       Q1=P0*Q1+P1*Q0
       Q0=T
       FDIP2=C*Q1
       RETURN
      END
!
!***********************************************************************
!
      REAL*8 FUNCTION FDIPA(EK1,L1,EK2,L2)
      IMPLICIT REAL*8 (A-H,O-Z)
!
! N.R. BADNELL
! ASYMPTOTIC EXPRESSION FOR I(KAPPA1,L1,KAPPA2,L2,1) BASED ON A40,1 OF BHT
!
      IF(EK1*EK2.GT.1.0D-50)THEN
      X1=1.0D0/SQRT(EK1)
      X2=1.0D0/SQRT(EK2)
      XP=ABS(X1-X2)
      IF(XP.GT.1.D2)GO TO 9
      PI=ACOS(-1.0D0)
      XP=EXP(0.5D0*PI*XP)
      IF(EK1-EK2)1,1,2
   1  E=EK1/EK2
      IF(L1-L2)3,3,4
   3  L=L1
      GO TO 7
   4  L=L2
      GO TO 8
   2  E=EK2/EK1
      IF(L1-L2)5,5,6
   5  L=L1
      GO TO 8
   6  L=L2
      GO TO 7
! A40
   7  TL=L
      T0=1.0D0-E
      IF(TL*T0.LT.E)GO TO 9
      T=PI*TL
      EE=SQRT(E)
      F0=SQRT(T*T0*EE)*EE**L
      TL=L+L+1
      FDIPA=F0*XP/TL
      RETURN
! A41
   8  T0=1.0D0-E
      TL=L
      IF(TL*T0.LT.E)GO TO 9
      T0=1.0D0/T0
      T=TL*PI
      EE=SQRT(E)
      F0=SQRT(T*T0*EE)*EE**(L+1)
      TL=L+L+1
      TL2=L+L+3
      FDIPA=F0*XP/(TL*TL2)
      RETURN
      ENDIF
   9  FDIPA=0.0D0
      RETURN
      END
!
!***************************************************************
!
      REAL*8 FUNCTION FKHI(E,L,AC)
!
!  CALCULATES REAL*4 PART OF PSI(L+1+I*GAM) - LN(GAM)
!  WHERE E = 1/(GAM**2).
!  THIS IS REQUIRED FOR CALCULATION OF SCRIPT G.
!
      IMPLICIT REAL*8 (A-H,O-Y)
      IMPLICIT COMPLEX*16 (Z)
!
      PARAMETER (TZERO=0.0)
      PARAMETER (ONE=1.0)
      PARAMETER (TWO=2.0)
      PARAMETER (P0=ONE/252.)
      PARAMETER (P1=1.05)
      PARAMETER (P2=2.1)
!
      FKHI=TZERO
      IF(E.EQ.0)RETURN
!
      AC1=(20.*AC)**.333
!
      IF(E.GT.AC1)GOTO 100
!
      C=TZERO
      IF(L.EQ.0)GOTO 20
      A1=ONE
      A2=-E
      A3=E+E
      DO 10 I=1,L
      A2=A2+A3
      A1=A1+A2
      C=C+DBLE(I)/A1
   10 CONTINUE
   20 FKHI=E*((((P1*E+ONE)*E+P2)*E+21)*P0+C)
      RETURN
!
  100 AC1=ONE/SQRT(AC1)
      FL=DBLE(L+1)
      IF(FL.GT.AC1)GOTO 300
!
      N=AC1
      FL=N+1
      L1=L+1
      DO 210 I=L1,N
      FI=I
      FKHI=FKHI+FI/(ONE+E*FI*FI)
  210 CONTINUE
      FKHI=-FKHI*E
!
  300 X1=FL*E
      X=ONE+X1*FL
      ZE=DCMPLX(FL,ONE/SQRT(E))
      ZE=-ONE/(ZE*ZE)
      FKHI=FKHI+(LOG(X)-(X1/X))/TWO+DBLE((((P1*ZE+ONE)*ZE
     C +P2)*ZE+21)*ZE)*P0
!
      RETURN
      END
!***********************************************************************
       REAL*8 FUNCTION FMON1(EK1,EK2,L)
!
       IMPLICIT REAL*8 (A-H,O-Z)
!
       IF(EK1+EK2-1.0D-40)28,28,29
   28  FMON1=1.0D+50
       RETURN
   29  CONTINUE
       VMAX=200.0D0
       X1=SQRT(EK1)
       X2=SQRT(EK2)
       X3=X1+X2
       X4=X3*X3
       X5=X1*X2
       X6=X2-X1
       X7=4.0D0/X4
       PI=3.141592653589793D0
       IF(EK1-EK2)1,1,2
    1  ETA=1.0D0/X2
       GO TO 3
    2  ETA=1.0D0/X1
    3  G=0.5D0*PI*EXP(-PI*ETA)
       IF(G.EQ.0.0D0)GO TO 20           !NRB
       A1=1.0D0
       A2=1.0D0
       MG=0
       MA1=0
       MA2=0
       M=-1
    4  M=M+1
       EM=M
       T=EM+EM+1.0D0
       G=G*X7/(T*(T+1.0D0))
       EMM=EM*EM
       A1=A1*(1.0D0+EMM*EK1)
       A2=A2*(1.0D0+EMM*EK2)
   30  IF(G-0.015625D0) 31,32,32
   31  G=64.0D0*G
       MG=MG-1
       GO TO 30
   32  IF(G-64.0D0) 34,34,33
   33  G=0.015625D0*G
       MG=MG+1
       GO TO 32
   34  IF(A1-64.0D0) 36,36,35
   35  A1=0.015625D0*A1
       MA1=MA1+1
       GO TO 34
   36  IF(A2-64.0D0) 38,38,37
   37  A2=0.015625D0*A2
       MA2=MA2+1
       GO TO 36
   38  CONTINUE
       IF(M-L)4,5,5
    5  G=G*(T+1.0D0)
       IF(X1-300.0D0)7,6,6
    6  B=PI/X1
       A1=1.5D0*A1/(B*(3.0D0-B*(3.0D0-B*(2.0D0-B))))
       GO TO 9
    7  IF(X1-0.2D0)9,9,8
    8  B=-PI/X1
       A1=A1/(1.0D0-EXP(B+B))
    9  IF(X2-300.0D0)11,10,10
   10  B=PI/X2
       A2=1.5D0*A2/(B*(3.0D0-B*(3.0D0-B*(2.0D0-B))))
       GO TO 13
   11  IF(X2-0.2D0)13,13,12
   12  B=-PI/X2
       A2=A2/(1.0D0-EXP(B+B))
   13  G=G*SQRT(A1*A2)*(8.0D0)**(MG+MG+MA1+MA2)
       S0=1.0D0
       S1=0.0D0
       U=L
       V=0.0D0
       W=U+U+1.0D0
       T0=1.0D0
       T1=0.0D0
   14  U=U+1.0D0
       V=V+1.0D0
       W=W+1.0D0
       IF(V-VMAX)21,21,20
   20  FMON1=0.0D0
       RETURN
   21  CONTINUE
       U0=U*U*X5+1.0D0
       U1=U*X6
       T=T0*U0-T1*U1
       T1=T0*U1+T1*U0
       T0=T
       T=X7/(V*W)
       T0=T*T0
       T1=T*T1
       S0=S0+T0
       S1=S1+T1
       S=S0*S0+S1*S1
       T=T0*T0+T1*T1
       SM=1.0D0/S
       TM=1.0D0/T
       IF(SM*TM.EQ.0.0D0)GO TO 20      !NRB
       IF(S-1.0D+24*T)14,15,15
   15  FMON1=G*SQRT(S)
       IV=V
       RETURN
      END
!***************************************************************
!
      SUBROUTINE INJWBK(E,L,J)
      USE CACC
      USE CJWBK
      USE NRBZED
!
!  COMPUTES ARRAY D WHICH IS HELD IN MODULE CJWBK  AND
!  USED FOR CALCULATION OF JWBK FUNCTIONS.
!NRB
!  NEUTRAL CASE ADDED
!
      IMPLICIT REAL*8 (A-H,O-Z)
!      PARAMETER (MX15N=15*MZCHF)
!
      DDDD(J+1)=E
!
      IF(TZED.EQ.0)THEN
        FK=SQRT(E)
        DDDD(J+2)=FK
        IF(L.GT.0)THEN
          DDDD(J+3)=1./FK
          C=DBLE(L*(L+1))
          DDDD(J+4)=C
          SC=SQRT(C)
          DDDD(J+5)=SC
          DDDD(J+6)=(C+.125)/SC
          DDDD(J+7)=E*C
          DDDD(J+12)=6.*E*C
          DDDD(J+14)=-C*C
          DDDD(J+15)=-L*1.5707963
        ELSE
          DDDD(J+4)=0.0
          DDDD(J+15)=0.0
        ENDIF
        RETURN
      ENDIF
!
      IF(L.GT.0)GOTO 10
!
!  CASE OF L.EQ.0
      DDDD(J+4)=0.
      IF(E.EQ.0)GOTO 30
!  CASE OF L.EQ.0 AND E.GT.0
      FK=SQRT(E)
      DDDD(J+2)=FK
      DDDD(J+3)=1./FK
      GOTO 30
!
   10 IF(E.GT.0)GOTO 20
!
!  CASE OF L.GT.0 AND E.EQ.0
      C=DBLE(L*(L+1))
      DDDD(J+4)=C
      SC=SQRT(C)
      DDDD(J+5)=SC
      DDDD(J+6)=(C+.125)/SC
      DDDD(J+13)=6.*C
      DDDD(J+14)=-C*C
      GOTO 30
!
!  CASE OF L.GT.0 AND E.GT.0
   20 FK=SQRT(E)
      DDDD(J+2)=FK
      DDDD(J+3)=1./FK
      C=DBLE(L*(L+1))
      DDDD(J+4)=C
      SC=SQRT(C)
      DDDD(J+5)=SC
      DDDD(J+6)=(C+.125)/SC
      A=1.+E*C
      DDDD(J+7)=A
      A=3.*A
      DDDD(J+8)=A-1.
      DDDD(J+9)=A+1.
      DDDD(J+10)=FK*C
      DDDD(J+11)=-4.*E
      DDDD(J+12)=-9.+2.*A
      DDDD(J+13)=6.*C
      DDDD(J+14)=-C*C
!
!  TERM IN ARG GAMMA ETC
   30 DDDD(J+15)=ARGC(E,L,AC)
!
      RETURN
      END
!***************************************************************
!
      SUBROUTINE INTBUT(KK,XX,BUTTLE)
      USE DBUT
!
! DETRMINE BUTTLE CORRECTION FROM DARC
!
      IMPLICIT REAL*8(A-H,O-Z)
!
      FX(X)=((X-X2)/(X1-X2))*((X-X3)/(X1-X3))*Y1+((X-X1)/(X2-X1))*
     X      ((X-X3)/(X2-X3))*Y2+((X-X1)/(X3-X1))*((X-X2)/(X3-X2))*Y3
!
      MM=NBUTD(KK)
!
      IF (MM.EQ.2) THEN
        X1=EBUTD(1,KK)
        X2=EBUTD(2,KK)
        Y1=CBUTD(1,KK)
        Y2=CBUTD(2,KK)
        BUTTLE=(XX-X2)/(X1-X2)*Y1+(XX-X1)/(X2-X1)*Y2
        RETURN
      ENDIF
!
      DO M=1,MM
        IF(XX.LT.EBUTD(M,KK))GO TO 10
      ENDDO
      II=MM
      GO TO 20
!
   10 CONTINUE
      II=M
   20 CONTINUE
!
      IF (II.EQ.1) THEN
        X1=EBUTD(1,KK)
        X2=EBUTD(2,KK)
        X3=EBUTD(3,KK)
        Y1=CBUTD(1,KK)
        Y2=CBUTD(2,KK)
        Y3=CBUTD(3,KK)
        BUTTLE=FX(XX)
        RETURN
      ENDIF
!
      IF (II.EQ.MM) THEN
        X1=EBUTD(MM-2,KK)
        X2=EBUTD(MM-1,KK)
        X3=EBUTD(MM  ,KK) 
        Y1=CBUTD(MM-2,KK)
        Y2=CBUTD(MM-1,KK)
        Y3=CBUTD(MM  ,KK)
        BUTTLE=FX(XX)
        RETURN
      ENDIF
!
      X1=EBUTD(II-1,KK)
      X2=EBUTD(II  ,KK)
      X3=EBUTD(II+1,KK)
      Y1=CBUTD(II-1,KK)
      Y2=CBUTD(II  ,KK)
      Y3=CBUTD(II+1,KK)
      BUTTLE=FX(XX)
      RETURN
!
      END
!
!***************************************************************
!
      SUBROUTINE JWBK(R,J,S,SP,C,CP)
      USE CACC
      USE CJWBK
      USE NRBZED
!
!  COMPUTES FUNCTIONS S AND C AND THEIR DERIVATIVES SP AND
!  CP USING IJWBK METHOD.
!  USES DATA IN ARRAY D WHICH IS HELD IN MODULE CJWBK
!  AND SHOULD HAVE BEEN COMPUTED IN SUBROUTINE INJWBK.
!NRB
!  NEUTRAL CASE ADDED
!
      IMPLICIT REAL*8 (A-H,O-Z)
!      PARAMETER (MX15N=15*MZCHF)
!
      E=DDDD(J+1)
      C=DDDD(J+4)
      X=1./R
!
      IF(TZED.EQ.0)THEN
        IF(C.EQ.0)THEN
          FK=DDDD(J+2)
          WH=FK
          ET=1.0/SQRT(WH)
          ETP=0.
          P=FK*R
          ZET=FK
          GO TO 110
        ELSE
          XSQ=X*X
          W=E-C*XSQ
          A1=0.0625*(X/W)**3
          CC=A1*(DDDD(J+14)*XSQ+DDDD(J+12))*X
          BB=A1*(6.*DDDD(J+14)*XSQ+4.*DDDD(J+12))*X
          WH=SQRT(W)
          Z=R*WH
          ZSQ=Z*Z
          P=(ZSQ-0.125-.2083333*C/ZSQ)/Z+DDDD(J+15)
          S=DDDD(J+5)
          G=Z
          P=P+DDDD(J+6)*ATAN2(S,G)
          GO TO 100
        ENDIF
      ENDIF
!
      IF(C.EQ.0)GOTO 30
      IF(E.EQ.0)GOTO 70
!
!  CASE OF C.GT.0 AND E.GT.0
      W=E+X*(2.-C*X)
      WH=SQRT(W)
      Z=R*WH
      FK=DDDD(J+2)
      RK=R*FK
      RMC=R-C
      ALP=Z+RK
      CK=DDDD(J+10)
!  COMPUTE PHASE
      P=Z+DDDD(J+15)
!  LOG TERM
      B=FK*ALP
      IF(B.GT.ACJWBK)GOTO 10
      B=-B
      P=P+ALP*((((.2*B+.25)*B+.33333333)*B+.5)*B+1.)
      GOTO 20
   10 P=P+DDDD(J+3)*LOG(1.+B)
!  ARCTAN TERM
   20 S=DDDD(J+5)*(Z-FK*RMC)
      G=CK*Z+RMC
      P=P+DDDD(J+6)*ATAN2(S,G)
!  CAP. PHI TERM
      P=P+((5.*RMC/(Z*Z))-(Z*DDDD(J+9)+RK*DDDD(J+8)+CK)/
     C (ALP*DDDD(J+7)))/(24.*Z)
!  COMPUTE AMPLITUDE
      A1=.0625*(X/W)**3
      CC=A1*(((DDDD(J+14)*X+DDDD(J+13))*X+DDDD(J+12))*X+DDDD(J+11))
      BB=A1*(((6.*DDDD(J+14)*X+5.*DDDD(J+13))*X+4.*DDDD(J+12))
     1 *X+3.*DDDD(J+11))
      GOTO 100
!
   30 IF(E.EQ.0)GOTO 60
!
!  CASE OF C.EQ.0 AND E.GT.0
      W=2.*X+E
      WH=SQRT(W)
      Z=R*WH
      FK=DDDD(J+2)
      RK=R*FK
      ALP=Z+RK
!  COMPUTE PHASE
      P=Z+DDDD(J+15)
      B=FK*ALP
      IF(B.GT.ACJWBK)GOTO 40
      B=-B
      P=P+ALP*((((.2*B+.25)*B+.33333333)*B+.5)*B+1.)
      GOTO 50
   40 P=P+DDDD(J+3)*LOG(1.+B)
   50 P=P+1/(4.*ALP)+(5.*R/(Z*Z)-2.*(Z+ALP)/ALP)/(24.*Z)
!  COMPUTE AMPLITUDE
      A1=.0625*(X/W)**3
      CC=A1*(-4.*E-3.*X)
      BB=-12.*A1*(E+X)
      GOTO 100
!
!  CASE OF C.EQ.0 AND E.EQ.0
   60 W=2.*X
      WH=SQRT(W)
      Z=R*WH
      P=2.*Z*(1.+.046875*X)+DDDD(J+15)
      WMQ=1./SQRT(WH)
      ET=(1.+.0234375*X)*WMQ
      ZET=(1.-.046875*X)*WH
      ETP=.25*(1.-.0703125*X)*X*WMQ
      GOTO 110
!
!  CASE OF E.EQ.0 AND C.GT.0
   70 W=X*(2.-C*X)
      WH=SQRT(W)
      Z=R*WH
      RMC=R-C
!  COMPUTE PHASE
      P=2.*Z+DDDD(J+15)
      S=DDDD(J+5)*Z
      P=P+DDDD(J+6)*ATAN2(S,RMC)
      P=P-(3.*R+C)/(24.*(RMC+R)*Z)
!  COMPUTE AMPLITUDE
      A1=.0625*(X/W)**3
      CC=((DDDD(J+14)*X+DDDD(J+13))*X-3.)*X*A1
      BB=((6.*DDDD(J+14)*X+5*DDDD(J+13))*X-12.)*X*A1
!
!  COMPLETE CALCULATION OF S,SP,C AND CP
 100  WMQ=1./SQRT(WH)
      ET=(1.-CC)*WMQ
      ETP=(.5*(X*X/W)*(TZED-C*X)*(1.-13.*CC)+X*BB)*WMQ
      ZET=(1.+2.*CC)*WH
  110 SI=SIN(P)
      CO=COS(P)
      S=ET*SI
      C=ET*CO
      SP=ETP*SI+C*ZET
      CP=ETP*CO-S*ZET
!
      RETURN
      END
!******************************************************************
!
      SUBROUTINE MESH
      USE WMATALLOC
      USE CEN
      USE CINPUT
      USE CINPTX
      USE CDEGEN
      USE CNTRL
      USE CMESH
!
!  CALCULATES ENERGY MESH FOR CASE OF IMESH=2
!  OR IMESH=-S WHERE S=2*TOTAL SPIN +1
!
!
      IMPLICIT REAL*8 (A-H,O-Z)
      REAL*8,ALLOCATABLE :: EMESH0(:)
!
!  PARAMETERS READ FOR IMESH=2
!  ***************************
!
!  DQN     INTERVAL FOR EFFECTIVE QUANTUM NUMBER
!
!  QNMAX   LARGEST ALLOWED VALUE OF EFFECTIVE QUANTUM NUMBER
!
!  EMIN    LOWEST VALUE OF ETOT
!
!  EMAX    HIGHEST VALUE OF ETOT
!
!  DEOPEN  INTERVAL IN ETOT FOR ALL CHANNELS OPEN
!
!  ALL ENERGIES ARE Z-SCALED AND RELATIVE TO GROUND ENERGY
!
!  IRDEC   RADIATIVE DECAYS INCLUDED FOR IRDEC=1
!
      WRITE(6,600)
      WRITE(6,605)DQN,QNMAX,EMIN,EMAX,DEOPEN
!
!  CASE OF IMESH =  -TOTAL SPIN  (ADDED 9.06.88)
!
!  NOTE CHANGES TO /CINPUT/
!       NAST REPLACED BY KAST
!       ENAT   ..     .. ENATK
!  FORM NEW VALUE NAST FOR USE ONLY BY SUBROUTINE MESH WHERE
!       NAST=NO. OF STATES WHICH CAN FORM CHANNELS FOR TOTAL
!            SPIN GIVEN BY ABS(IMESH)
!       ENAT=CORRESPONDING ARRAY OF TARGET ENERGIES
!  NOTE SPECIAL TREATMENT IF DEGENERATE LEVELS HAVE BEEN COMBINED
!  IN SUBROUTINE SCALE1
!  IF IMESH IS .GT. 0 NAST, ENAT COPIED FROM KAST, ENATK
!
!  LFM - Array EMESH0 is used to reallocate EMESH if the dimension
!        MZE, flexible, is going to be exceeded.
!
      mze=10000
      ALLOCATE(EMESH(mze),EMESH0(mze))
!
      IF (IMESH .LT. 0)THEN
         ISP=ABS(IMESH)
         ISP1=ISP-1
         ISP2=ISP+1
         IMESH=2
         NAST=0
         IF (NASTD .EQ. 0)THEN
            DO 1   I=1,KAST
            ISATI=ISAT(I)
            IF (ISATI .GE. ISP1 .AND. ISATI .LE. ISP2) THEN
               NAST=NAST+1
               ENAT(NAST)=ENATK(I)
            END IF
    1       CONTINUE
         ELSE
            N2=0
            DO 2   ID=1,KAST
            N1=N2+1
            N2=NLEV(ID)+N1-1
            DO 3   IN=N1,N2
            ISATN=ISAT(IN)
            IF (ISATN .GE. ISP1 .AND. ISATN .LE. ISP2) THEN
               NAST=NAST+1
               ENAT(NAST)=ENATK(ID)
               GO TO 2
            END IF
    3       CONTINUE
    2       CONTINUE
         END IF
      ELSE
         NAST=KAST
         DO 4   I=1,KAST
         ENAT(I)=ENATK(I)
    4    CONTINUE
      END IF
!     WRITE(6,'('' NAST='',I4)') NAST
!     WRITE(6,'(1X,5F12.6)')(ENAT(I),I=1,NAST)
!
!  CASE OF EMIN.GE.ENAT(NAST)  (ADDED 9.12.87)
      IF(EMIN.GE.ENAT(NAST))THEN
            F=EMAX-EMIN
            IF(F.LT.0)THEN
                  IE=0
                  GOTO 400
            ENDIF
            J=1+F/DEOPEN
            F=F/DBLE(J)
            E=EMIN
            IE=1
            EMESH(IE)=E
!            IF(IE+J.GT.MZMSH)GOTO 500
            If (ie+j.gt.mze) Then  
! Redimension EMESH
              Deallocate(EMESH0)
              mze=2*(ie+j)
              Allocate(EMESH0(mze))
              EMESH0(1:ie)=EMESH(1:ie)
              Deallocate(EMESH)
              Allocate(EMESH(mze))
              EMESH(1:ie)=EMESH0(1:ie)
            End If
            DO 5 K=1,J
            IE=IE+1
            E=E+F
    5       EMESH(IE)=E
            GOTO 400
      ENDIF
!
!  INITIALISATIONS
      QNINT=1./DQN
      SE=QNMAX**(-2)
      E=MAX(EMIN,-SE)
!      E=MAX(EMIN,ENAT(1)-SE)            !ENAT(1) USUALLY ZERO....
      IE=0
      IF(EMAX.LT.E)GOTO 400
!
!  FIND NC1, LOWEST LEVEL ABOVE EMIN
      DO 10 N=1,NAST
      IF(EMIN.GE.ENAT(N))GOTO 10
      NC1=N
      GOTO 20
   10 CONTINUE
      IE=IE+1
      EMESH(IE)=EMIN
      WRITE(6,650)
      WRITE(6,640)IE,EMESH(IE)
      GOTO 300
   20 WRITE(6,610)NC1
!
!  FIND NC2, HIGHEST LEVEL BELOW EMAX
      DO 30 N=NAST,1,-1
      IF(ENAT(N).GT.EMAX)GOTO 30
      NC2=N
      GOTO 40
   30 CONTINUE
      GOTO 50
   40 WRITE(6,620)NC2
!
!     RPS-BRAKE - WE'93MAY2:
!     IF(IOPT1.LE.9) GO TO 50
      IF(ENAT(NC1)-SE.GE.EMIN) GO TO 50
      WRITE(6,605)
      EMIN=ENAT(NC1)+.000001
!
!   ENERGIES IN RANGE EMIN TO ENAT(NC2)
!  ************************************
!
   50 IF(IPRINT.GT.0)WRITE(6,625)
      EN=-1
      DO 160 N=NC1,NC2
      ENM=EN
      EN=ENAT(N)
      IF(EN.EQ.ENM)GOTO 160
!
!   RANGE UP TO (EN-SE)
      IF((EN-SE).GT.E)THEN
        F=1./SQRT(EN-E)
        D=.999*(QNMAX-F)
        J=2+D*QNINT
        D=D/DBLE(J-1)
        IF(IPRINT.GT.0)WRITE(6,660)
!        IF((IE+J).GT.MZMSH)GOTO 500
        If (ie+j.gt.mze) Then  
! Redimension EMESH
          Deallocate(EMESH0)
          mze=2*(ie+j)
          Allocate(EMESH0(mze))
          EMESH0(1:ie)=EMESH(1:ie)
          Deallocate(EMESH)
          Allocate(EMESH(mze))
          EMESH(1:ie)=EMESH0(1:ie)
        End If
        DO 110 K=1,J
        IE=IE+1
        EMESH(IE)=EN-F**(-2)
        IF(IPRINT.GT.0)WRITE(6,630)IE,EMESH(IE),N,N,F
  110   F=F+D
        E=EN-QNMAX**(-2)
      ENDIF
!
!    RANGE UP TO EN
!...FIND NEXT HIGHER THRESHOLD
      DO 120 M=N+1,NAST
      IF(ENAT(M).NE.EN)THEN
!----
        MM=M
!----
        EM=ENAT(M)
        GOTO 140
      ENDIF
  120 CONTINUE
!... NO NEXT THRESHOLD, USE DEOPEN
  125 F=.999*(ENAT(N)-E)
      J=2+F/DEOPEN
      F=F/DBLE(J-1)
      IF(IPRINT.GT.0)WRITE(6,690)
!      IF((IE+J).GT.MZMSH)GOTO 500
      If (ie+j.gt.mze) Then  
! Redimension EMESH
        Deallocate(EMESH0)
        mze=2*(ie+j)
        Allocate(EMESH0(mze))
        EMESH0(1:ie)=EMESH(1:ie)
        Deallocate(EMESH)
        Allocate(EMESH(mze))
        EMESH(1:ie)=EMESH0(1:ie)
      End If
      DO 130 K=1,J
      IE=IE+1
      EMESH(IE)=E
      IF(IPRINT.GT.0)WRITE(6,680)IE,E,N
  130 E=E+F
      E=ENAT(NAST)
      GOTO 300
!... USING NEXT HIGHER THRESHOLD
  140 F1=1./SQRT(EM-E)
      F2=1./SQRT(EM-EN)
      D=.999*(F2-F1)
!----
!  MODIFICATION FOR NEAR-DEGENERATE CHANNELS (KAB,JAN94)
      IF(F2.GT.QNMAX) THEN
        D=QNMAX-F1-DQN
        IF(D.LT.0.0) GO TO 155
      ENDIF
!----
      J=2+D*QNINT
      D=D/DBLE(J-1)
      F=F1
      IF(IPRINT.GT.0)WRITE(6,660)
!      IF((IE+J).GT.MZMSH)GOTO 500
      If (ie+j.gt.mze) Then  
! Redimension EMESH
        Deallocate(EMESH0)
        mze=2*(ie+j)
        Allocate(EMESH0(mze))
        EMESH0(1:ie)=EMESH(1:ie)
        Deallocate(EMESH)
        Allocate(EMESH(mze))
        EMESH(1:ie)=EMESH0(1:ie)
      End If
      DO 150 K=1,J
      IE=IE+1
      EMESH(IE)=EM-F**(-2)
      IF(IPRINT.GT.0)WRITE(6,630)IE,EMESH(IE),N,M,F
  150 F=F+D
      IF(IPRINT.GT.0)WRITE(6,660)
      E=EN
!----
      IF(F2.LE.QNMAX) GO TO 160
      E=EM-QNMAX**(-2)
  155 MM=MM+1
      IF(MM.GT.NAST) GO TO 125
      EM=ENAT(MM)
      GO TO 140
!----
  160 CONTINUE
!
!
!  ENERGIES IN RANGE ENAT(NC2) TO EMAX
!  ***********************************
!
!  FIND NEXT HIGHER THRESHOLD
      DO 210 M=NC2+1,NAST
      IF(ENAT(M).NE.EN)THEN
!----
        MM=M
!----
        EM=ENAT(M)
        GOTO 220
      ENDIF
  210 CONTINUE
      GOTO 280
!
!  RANGE UP TO MIN(EMAX,(EM-SE))
  220 IF((EM-SE).GT.E)THEN
        IF((EM-SE).GT.EMAX)THEN
!...  CASE OF EMAX.LT.(EM-SE)
          F1=1./SQRT(EM-E)
          F2=1./SQRT(EM-EMAX)
          D=F2-F1
          J=2+D*QNINT
          D=D/DBLE(J-1)
          F=F1-D
          IF(IPRINT.GT.0)WRITE(6,660)
!          IF((IE+J).GT.MZMSH)GOTO 500
          If (ie+j.gt.mz) Then  
! Redimension EMESH
            Deallocate(EMESH0)
            mze=2*(ie+j)
            Allocate(EMESH0(mze))
            EMESH0(1:ie)=EMESH(1:ie)
            Deallocate(EMESH)
            Allocate(EMESH(mze))
            EMESH(1:ie)=EMESH0(1:ie)
          End If
          DO 230 K=1,J
          F=F+D
          IE=IE+1
          EMESH(IE)=EM-F**(-2)
  230     IF(IPRINT.GT.0)WRITE(6,630)IE,EMESH(IE),N,M,F
          IF(IPRINT.GT.0)WRITE(6,660)
          GOTO 400
        ELSE
!... CASE OF EMAX.GT.(EM-SE)
          F=1./SQRT(EM-E)
          D=.999*(QNMAX-F)
          J=2+D*QNINT
          D=D/DBLE(J-1)
          F=F-D
          IF(IPRINT.GT.0)WRITE(6,660)
!          IF((IE+J).GT.MZMSH)GOTO 500
          If (ie+j.gt.mze) Then  
! Redimension EMESH
            Deallocate(EMESH0)
            mze=2*(ie+j)
            Allocate(EMESH0(mze))
            EMESH0(1:ie)=EMESH(1:ie)
            Deallocate(EMESH)
            Allocate(EMESH(mze))
            EMESH(1:ie)=EMESH0(1:ie)
          End If
          DO 240 K=1,J
          IE=IE+1
          F=F+D
          EMESH(IE)=EM-F**(-2)
  240     IF(IPRINT.GT.0)WRITE(6,630)IE,EMESH(IE),N,M,F
          E=EM-QNMAX**(-2)
        ENDIF
      ENDIF
!
!  FIND NEXT HIGHER THRESHOLD
!
!----
      N=MM
!----
!      N=M
      EN=EM
      DO 250 M=N+1,NAST
      IF(ENAT(M).NE.EN)THEN
        EM=ENAT(M)
        MM=M
        GOTO 260
      ENDIF
  250 CONTINUE
      GOTO 280
  260 M=MM
      F1=1./SQRT(EM-E)
      F2=1./SQRT(EM-EMAX)
      D=F2-F1
!----
      IF(F2.GT.QNMAX) THEN
        D=QNMAX-F1-DQN
        IF(D.LT.0.0) GO TO 275
      ENDIF
!----
      J=2+D*QNINT
      D=D/DBLE(J-1)
      F=F1-D
      IF(IPRINT.GT.0)WRITE(6,660)
!      IF((IE+J).GT.MZMSH)GOTO 500
      If (ie+j.gt.mze) Then  
! Redimension EMESH
        Deallocate(EMESH0)
        mze=2*(ie+j)
        Allocate(EMESH0(mze))
        EMESH0(1:ie)=EMESH(1:ie)
        Deallocate(EMESH)
        Allocate(EMESH(mze))
        EMESH(1:ie)=EMESH0(1:ie)
      End If
      DO 270 K=1,J
      F=F+D
      IE=IE+1
      EMESH(IE)=EM-F**(-2)
  270 IF(IPRINT.GT.0)WRITE(6,630)IE,EMESH(IE),N,M,F
      IF(IPRINT.GT.0)WRITE(6,660)
!      GOTO 400
!----
      IF(F2.LE.QNMAX) GO TO 400
      E=EM-QNMAX**(-2)
  275 MM=MM+1
      IF(MM.GT.NAST) GO TO 280
      EM=ENAT(MM)
      GO TO 260
!----
!
!
!  UP TO EMAX USING DEOPEN
  280 F=EMAX-E
      J=2+F/DEOPEN
      F=F/DBLE(J-1)
      E=E-F
      IF(IPRINT.GT.0)WRITE(6,690)
!      IF((IE+J).GT.MZMSH)GOTO 500
      If (ie+j.gt.mze) Then  
! Redimension EMESH
        Deallocate(EMESH0)
        mze=2*(ie+j)
        Allocate(EMESH0(mze))
        EMESH0(1:ie)=EMESH(1:ie)
        Deallocate(EMESH)
        Allocate(EMESH(mze))
        EMESH(1:ie)=EMESH0(1:ie)
      End If
      DO 290 K=1,J
      E=E+F
      IE=IE+1
      EMESH(IE)=E
  290 IF(IPRINT.GT.0)WRITE(6,680)IE,E,N
      GOTO 400
!
!
!
!  ALL CHANNELS OPEN
!  *****************
!
  300 IF(IPRINT.GT.0)WRITE(6,650)
      J=1
!      IF(IE+J.GT.MZMSH)GOTO 500
      If (ie+j.gt.mze) Then  
! Redimension EMESH
        Deallocate(EMESH0)
        mze=2*(ie+j)
        Allocate(EMESH0(mze))
        EMESH0(1:ie)=EMESH(1:ie)
        Deallocate(EMESH)
        Allocate(EMESH(mze))
        EMESH(1:ie)=EMESH0(1:ie)
      End If
      IE=IE+1
      E=ENAT(NAST)
      EMESH(IE)=E
      IF(IPRINT.GT.0)WRITE(6,640)IE,E
      F=EMAX-E
      J=1+F/DEOPEN
      F=F/DBLE(J)
!      IF((IE+J).GT.MZMSH)GOTO 500
      If (ie+j.gt.mze) Then  
! Redimension EMESH
        Deallocate(EMESH0)
        mze=2*(ie+j)
        Allocate(EMESH0(mze))
        EMESH0(1:ie)=EMESH(1:ie)
        Deallocate(EMESH)
        Allocate(EMESH(mze))
        EMESH(1:ie)=EMESH0(1:ie)
      End If
      DO 310 K=1,J
      E=E+F
      IE=IE+1
      EMESH(IE)=E
  310 IF(IPRINT.GT.0)WRITE(6,640)IE,E
!
!  TASK COMPLETED
!  **************
!  Add more extrapolated energies if mxe is not an
!  even divisor of the number of processors nproc.
!  Redimension EMESH to mxe
!
  400 MXE=IE
      Deallocate(EMESH0)
      WRITE(6,670) MXE
!!     **** PARALLEL **** 
!      If (Mod(mxe,nproc).ne.0) Then
!! No extrapolation needed
!        nproce=mxe/nproc
!        Allocate(EMESH0(mxe))
!        EMESH0(1:mxe)=EMESH(1:mxe)
!      Else
!! Extrapolation of energies
!        nproce=(mxe/nproc)+1
!        Allocate(EMESH0(nproce*nproc))
!        de=EMESH(mxe)-EMESH(mxe-1)
!        EMESH0(1:mxe)=EMESH(1:mxe)
!        Do ie=mxe+1,nproce*nproc
!          EMESH0(ie)=EMESH(mxe)+Dfloat(ie-mxe)*de
!        End Do
!        mxe=nproce*nproc
!        WRITE(6,'("WARNING: ENERGY MESH EXTRAPOLATED BECAUSE ",
!     &              "MXE/NPROC .NE. INTEGER")')
!        WRITE(6,'(" NEW VALUE MXE = ",I6)') MXE
!      End If
      Allocate(EMESH0(mxe))
      EMESH0(1:mxe)=EMESH(1:mxe)
      Deallocate(EMESH)
      Allocate(EMESH(mxe))
      EMESH(1:mxe)=EMESH0(1:mxe)
      Deallocate(EMESH0)
      RETURN
!
!  500 WRITE(6,675)
!      MXE=IE+J
!      WRITE(6,670)MXE
!      STOP
!
!  FORMATS
  600 FORMAT(//1X,70('*')//20X,'ENERGY MESH'/20X,11('*')/)
  605 FORMAT(//' DQN    = ',F10.6/' QNMAX  = ',F10.6/
     +  ' EMIN   = ',F10.6/' EMAX   = ',F10.6/' DEOPEN = ',F10.6//)
  610 FORMAT(' LOWEST LEVEL ABOVE EMIN,  NC1 = ',I3)
  620 FORMAT(' HIGHEST LEVEL BELOW EMAX, NC2 = ',I3)
  625 FORMAT(/' VALUES OF  -  IE, E = EMESH(IE), N = LOWEST'
     + ,' LEVEL ABOVE E,'/15X,'M = LEVEL USED FOR EFFECTIVE',
     +' QUANTUM NUMBER'/15X,'AND FNU = EFFECTIVE QUANTUM NUMBER'/
     + /3X,'IE',4X,'EMESH',17X,'N',4X,'M',4X,'FNU'/)
  630 FORMAT(I5,F12.6,10X,2I5,F12.6)
  640 FORMAT(I5,F12.6)
  650 FORMAT(/31X,'ALL OPEN'/)
  660 FORMAT()
  670 FORMAT(/' NUMBER OF ENERGIES, MXE = ',I5/)
! 675 FORMAT(///10X,64('*')/10X,
!    +'NUMBER OF ENERGIES EXCEEDS MAXIMUM OF MZMSH ALLOWED BY'
!    +,' DIMENSIONS' /10X,64('*')//)
  680 FORMAT(I5,F12.6,10X,I5)
  690 FORMAT(/36X,'USING DEOPEN')
!
      END
!***************************************************************
!
      SUBROUTINE MQDTK(LFAIL)
      USE WMATALLOC
      USE NRBRCT
      USE NRBWRK
      USE NRBDR
      USE COMEGA
      USE CEN
      USE CINPUT
      USE CINPTX
      USE CHAN
      USE CDEC
      USE NRBHYB
      USE NRBQDT
      USE NRBOMT
      USE CACC
      USE CQDT
!
! NRB:
!  CALCULATION OF K-PHYS FROM K-UNPHYS IN QDT, ALL CHANNELS OPEN.
!  STGF VERSION HAS NO DAMPING SINCE THIS WOULD MAKE K COMPLEX.
!  EITHER USE STGFDAMP OR GO VIA THE S/KHI-MATRIX ROUTE (SR.MQDTS).
!
      IMPLICIT REAL*8 (A-H,O-Y)
      IMPLICIT COMPLEX*16 (Z)
      LOGICAL,INTENT(OUT) :: LFAIL
!
!      PARAMETER (MXTST=(MZTAR*MZTAR+MZTAR)/2)
!      PARAMETER (LWORK=MZCHF*MZCHF)
!      PARAMETER (MWORK=MZDEG*MZDEG)
      PARAMETER (TZERO=0.0)
      PARAMETER (ONE=1.0)
!
       INTEGER,ALLOCATABLE :: IPIV(:)
!
       LFAIL=.FALSE.
       ALLOCATE(IPIV(NCHF))
!
       P=>CSP
       Q=>CC
       POLD=>DSP
       QOLD=>DC
!
      PI=ACOS(-ONE)
!
      IF(IOMSW.LT.0)THEN
        NCC=NCHCL
      ELSE
        NCC=NCHF-NCHPP
        DO N=1,NCC
          ICHCL(N)=NCHPP+N
        ENDDO
      ENDIF
!
!      IF(NCC.GT.MZDEG)THEN
!        WRITE(6,610)NCC
!        STOP 'INCREASE MZDEG'
!      ENDIF
!
! INITIALIZE KCC
!
      DO N2=1,NCC
        DO N1=1,NCC
          A(N1,N2)=P(ICHCL(N1),ICHCL(N2))
        ENDDO
      ENDDO
!
! A-FDEC         (FDEC=-TAN(PI*NU))
!
      DO N=1,NCC
        PINU=FKNU(ICHCL(N))*PI
        FDEC=-TAN(PINU)
        IF(IOMIT(ICHCL(N)).LT.0)FDEC=-FDEC
        A(N,N)=A(N,N)-FDEC
      ENDDO
!
      DO N=1,NCC
        IF(IOMIT(ICHCL(N)).GT.0)A(N,N)=ONE
      ENDDO
!
! INITIALIZE KCO
!
      DO N2=1,NCHPP
        DO N1=1,NCC
          IF(FKNU(ICHCL(N1)).LE.NCUTOFF)THEN
            B(N1,N2)=P(ICHCL(N1),N2)
          ELSE
            B(N1,N2)=TZERO
          ENDIF
        ENDDO
      ENDDO
!
!STRTNBL
!NBL  CALL LUS(A,NCHF,NCC,WORK,IERR)
!NBL  IF (IERR.NE.0) THEN
!NBL    WRITE(6,600)
!NBL    LFAIL=.TRUE.
!NBL    RETURN
!!NBL   STOP 'ERROR IN LUS'
!NBL  END IF
!NBL  CALL LUBS(A,B,NCHF,NCHPP,IERR)
!NBL  IF (IERR.NE.0) THEN
!NBL    WRITE(6,601)
!NBL    LFAIL=.TRUE.
!NBL    RETURN
!!NBL   STOP 'ERROR IN LUBS'
!NBL  END IF
!ENDNBL
!
!STRTBL
      LWORK=NCHF*NCHF
      ALLOCATE(WORK(NCHF*NCHF))
!      ALLOCATE(ZWORK(MZDEG*MZDEG))
      CALL DSYTRF('L',NCC,A,NCHF,IPIV,WORK,LWORK,INFO)
      IF (INFO.NE.0) THEN
         WRITE(6,602) INFO
         LFAIL=.TRUE.
         RETURN
!         STOP 'FAILURE IN BLAS ROUTINE DSYTRF'
      ENDIF
      CALL DSYTRS('L',NCC,NCHPP,A,NCHF,IPIV,B,NCHF,INFO)
      IF (INFO.NE.0) THEN
         WRITE(6,603) INFO
         LFAIL=.TRUE.
         RETURN
!         STOP 'FAILURE IN BLAS ROUTINE DSYTRS'
      ENDIF
!ENDBL
!
!
! CLOSE-OFF (SOME) OPEN CHANNELS TO FORM PHYSICAL K-MATRIX
! (THERE MAY BE NO SUBSEQUENT CALL TO SQDT)
!
      DO J=1,NCHPP
        DO I=1,J
          RK(I,J)=P(I,J)
        ENDDO
        DO K=1,NCC
          IF(FKNU(ICHCL(K)).LE.NCUTOFF)THEN
            DO I=1,J
              RK(I,J)=RK(I,J)-P(I,ICHCL(K))*B(K,J)
            ENDDO
          ENDIF
        ENDDO
      ENDDO
!
! SYMMETRIZE
!
      DO J=1,NCHPP
        DO I=1,J
          RK(J,I)=RK(I,J)
        ENDDO
      ENDDO
!
! STORE ORIGINAL P,Q
!
      DO J=1,NCHF
        DO I=1,NCHF
          POLD(I,J)=P(I,J)
          QOLD(I,J)=Q(I,J)
        ENDDO
      ENDDO
!
      NULLIFY(P,Q)
      DEALLOCATE(WORK,IPIV)
!
!
      RETURN
!N600 FORMAT(' SR.MQDTK: LUS RETURNED WITH INFO =',I6)
!N601 FORMAT(' SR.MQDTK: LUBS RETURNED WITH INFO =',I6)
  602 FORMAT(//10X,10('*'),' SR.MQDTK: DSYTRF RETURNED WITH INFO = ',I6)
  603 FORMAT(//10X,10('*'),' SR.MQDTK: DSYTRS RETURNED WITH INFO = ',I6)
!  610 FORMAT(//10X,10('*'),' SR.MQDTK: NUMBER OF MQDT CLOSED',
!     X ' CHANNELS, NCC = ',I4/20X,' LARGER THAN DIMENSION',
!     X ' VALUE OF DEG = MZDEG'//)
      END
!***************************************************************
!
      SUBROUTINE MQDTS(LFAIL)
      USE WMATALLOC
      USE NRBRCT
      USE NRBWRK
      USE NRBKHI
      USE NRBDR
      USE COMEGA
      USE CEN
      USE CINPUT
      USE CINPTX
      USE CHAN
      USE CDEC
      USE NRBHYB
      USE NRBQDT
      USE NRBSKP
      USE NRBOMT
      USE CACC
      USE CQDT
!
! NRB:
!  CALCULATION OF OMEGA FROM KHI-MX IN QDT, ALL CHANNELS OPEN.
!  OPTIONAL (TYPE-I) RADIATION DAMPING - BELL & SEATON OR HICKMAN-ROBICHEAUX.
!  DR N CUT-OFF AT NCUTOFF; EXCITATION IS THEN **UNDAMPED** ABOVE NCUTOFF.
!
      IMPLICIT REAL*8 (A-H,O-Y)
      IMPLICIT COMPLEX*16 (Z)
      LOGICAL,INTENT(OUT) :: LFAIL
!
!      PARAMETER (MXTST=(MZTAR*MZTAR+MZTAR)/2)
!      PARAMETER (LWORK=MZCHF*MZCHF)
!      PARAMETER (MWORK=MZDEG*MZDEG)
      PARAMETER (ZERO=(0.0,0.0))
      PARAMETER (ZONE=(1.0,0.0))
      PARAMETER (TZERO=0.0)
      PARAMETER (ONE=1.0)
      PARAMETER (TWO=2.0)
      PARAMETER (QUART=0.25)
      PARAMETER (BIG=150.0)
!
       COMPLEX*16,ALLOCATABLE :: ZKHI(:)
       INTEGER,ALLOCATABLE :: IPIV(:)
!
      LFAIL=.FALSE.
      IF(ALLOCATED(ZKHICC))DEALLOCATE(ZKHICC)
      IF(ALLOCATED(ZVAL))DEALLOCATE(ZVAL)

      ALLOCATE(ZKHI(NCHF))
      ALLOCATE(IPIV(NCHF))
      ALLOCATE(ZKHICC(NCHF,NCHF))
      ALLOCATE(ZVAL(NCHF))
!
!
      ZKHICO=>ZKHIOC
      P=>CSP
      Q=>CC
      POLD=>DSP
      QOLD=>DC      
!
      IF(IOMSW.LT.0)THEN
        NCC=NCHCL
      ELSE
        NCC=NCHF-NCHPP
        DO N=1,NCC
          ICHCL(N)=NCHPP+N
        ENDDO
      ENDIF
!
!      IF(NCC.GT.MZDEG)THEN
!        WRITE(6,610)NCC
!        STOP 'INCREASE MZDEG'
!      ENDIF
!
      PI=ACOS(-ONE)
      TPI=TWO*PI
      CONST=TPI/DBLE((NZED-NELC)**2)
      IONE=1
      IF(ELAS.EQ.'YES')IONE=0
!
! INITIALIZE KHICC
!
      DO N2=1,NCC
        DO N1=1,NCC
          ZKHICC(N1,N2)=DCMPLX(P(ICHCL(N1),ICHCL(N2))
     X                        ,Q(ICHCL(N1),ICHCL(N2)))
        ENDDO
      ENDDO
!
! ZKHICC-ZFDEC
!
      ITARGN=0
      DO N=1,NCC
        NNN=ICHCL(N)
!
! RADIATIVE DECAYS; RECALCULATE ARDEC (DEPENDS ON ETOT) IF NECESS.
!
        IF(IRDEC*NEWAR.GT.0.AND..NOT.QDT
     X                     .AND.ITARG(NNN).NE.ITARGN)THEN
          ITARGN=ITARG(NNN)
          KVEC=((ITARG(NNN)-IONE)*(ITARG(NNN)-1-IONE))/2
          ARDEC(ITARG(NNN))=TZERO
          DO JLOOP=1,ITARG(NNN)-1
            KVEC=KVEC+1
            IF(ETOT-(ENAT(ITARG(NNN))-ENAT(JLOOP)).LE.ENAT(1))THEN
              ARDEC(ITARG(NNN))=ARDEC(ITARG(NNN))+ARAD(KVEC)
            ENDIF
          ENDDO
          ARDEC(ITARG(NNN))=ARDEC(ITARG(NNN))*CONST
        ENDIF
!
        TPINU=FKNU(NNN)*TPI
        IF(IOMIT(NNN).LT.0)TPINU=-TPINU
        IF(IRDEC.EQ.0.OR.FKNU(NNN).GT.NCUTOFF)THEN
! NONE
          ZFDEC=EXP(DCMPLX(TZERO,-TPINU))
        ELSEIF(IRDEC.EQ.1)THEN
! BELL & SEATON
          T=ARDEC(ITARG(NNN))*(FKNU(NNN)**3)/TWO
          T=MIN(T,BIG)
          FDEC=EXP(T)
          ZFDEC=FDEC*EXP(DCMPLX(TZERO,-TPINU))
        ELSEIF(IRDEC.EQ.2)THEN
! HICKMAN-ROBICHEAUX
          TR=ONE/FKNU(NNN)**2
          TI=-ARDEC(ITARG(NNN))/TPI
          ZFKNU=DCMPLX(ONE,TZERO)/SQRT(DCMPLX(TR,TI))
!         IF(IQDT.EQ.2)THEN
!K          Z=DCMPLX(PI,TZERO)
!K          ZFDEC=-TAN(Z*ZFKNU)
!         ELSE
            Z=DCMPLX(TZERO,-TPI)*ZFKNU
            TR=DBLE(Z)
            TR=MIN(TR,BIG)
            TI=DIMAG(Z)
            Z=DCMPLX(TR,TI)
            IF(IOMIT(NNN).LT.0)Z=-Z       !??
            ZFDEC=EXP(Z)
!         ENDIF
        ENDIF
        ZKHICC(N,N)=ZKHICC(N,N)-ZFDEC
      ENDDO
!
      DO N=1,NCC
        IF(IOMIT(ICHCL(N)).GT.0)ZKHICC(N,N)=ZONE
      ENDDO
!
! INITIALIZE KHICO
!
      DO N2=1,NCHPP
        DO N1=1,NCC
!K          IF(FKNU(ICHCL(N1)).LE.NCUTOFF)THEN
          ZKHICO(N1,N2)=DCMPLX(P(ICHCL(N1),N2),Q(ICHCL(N1),N2))
!K          ELSE
!K            ZKHICO(N1,N2)=TZERO
!K          ENDIF
        ENDDO
      ENDDO
!
!STRTNBL
!NBL  CALL ZLUS(ZKHICC,NCHF,NCC,WORK,IERR)
!NBL  IF (IERR.NE.0) THEN
!NBL    WRITE(6,600)
!NBL    LFAIL=.TRUE.
!NBL    RETURN
!!NBL   STOP 'ERROR IN ZLUS'
!NBL  END IF
!NBL  CALL ZLUBS(ZKHICC,ZKHICO,NCHF,NCHPP,IERR)
!NBL  IF (IERR.NE.0) THEN
!NBL    WRITE(6,601)
!NBL    LFAIL=.TRUE.
!NBL    RETURN
!!NBL   STOP 'ERROR IN ZLUBS'
!NBL  END IF
!ENDNBL
!
!STRTBL
      MWORK=NCHF*NCHF
      ALLOCATE(ZWORK(MWORK))
      CALL ZSYTRF('L',NCC,ZKHICC,NCHF,IPIV,ZWORK,MWORK,INFO)
      IF (INFO.NE.0) THEN
         WRITE(6,602) INFO
         LFAIL=.TRUE.
         RETURN
!        STOP
      ENDIF
      CALL ZSYTRS('L',NCC,NCHPP,ZKHICC,NCHF,IPIV,ZKHICO,NCHF,INFO)
      IF (INFO.NE.0) THEN
         WRITE(6,603) INFO
         LFAIL=.TRUE.
         RETURN
!        STOP
      ENDIF
!ENDBL
!
!
! CLOSE-OFF ALL OPEN CHANNELS IF THERE IS TO BE NO SUBSEQUENT CALL
! TO SQDT
!
      IF(.NOT.QDT)THEN
!
! INITIALIZE DR PROBABILITY
!
!      IF(ALLOCATED(PDR))THEN
!       CONTINUE
!      ELSE
!       ALLOCATE(PDR(NCHF))
!      ENDIF
!      DO I=1,NCHPP
!        PDR(I)=ONE
!      ENDDO
      IF(ALLOCATED(PDR)) DEALLOCATE(PDR)
      ALLOCATE(PDR(NCHF))
      PDR(1:NCHPP)=ONE
!
! FORM PHYSICAL S-MATRIX
!
      DO J=1,NCHPP
        DO I=1,J
          ZKHI(I)=DCMPLX(P(I,J),Q(I,J))
        ENDDO
        DO K=1,NCC
!K            IF(FKNU(ICHCL(K)).LE.NCUTOFF)
          DO I=1,J
            ZKHI(I)=ZKHI(I)-
     X              DCMPLX(P(I,ICHCL(K)),Q(I,ICHCL(K)))*ZKHICO(K,J)
          ENDDO
        ENDDO
        DO I=1,J
          PP=ZKHI(I)*CONJG(ZKHI(I))
          PDR(J)=PDR(J)-PP
          PDR(I)=PDR(I)-PP
          IF(I.EQ.J)THEN
            PDR(I)=PDR(I)+PP
            ZKHI(I)=ZKHI(I)-DCMPLX(ONE,TZERO)
          ENDIF
          RK(I,J)=ZKHI(I)*CONJG(ZKHI(I))
        ENDDO
      ENDDO
!
! SYMMETRIZE AND ADJUST WEIGHTING
!
      T=QUART*NWT
      DO J=1,NCHPP
        PDR(J)=PDR(J)*T
        DO I=1,J
          RK(I,J)=T*RK(I,J)
          RK(J,I)=RK(I,J)
        ENDDO
      ENDDO
!
!
      ELSE
!
! STORE ORIGINAL P,Q
!
      DO J=1,NCHF
        DO I=1,NCHF
          POLD(I,J)=P(I,J)
          QOLD(I,J)=Q(I,J)
        ENDDO
      ENDDO
!
! NOW JUST CONTRACT
!
      DO J=1,NCHPP
        DO I=1,J
          ZKHI(I)=DCMPLX(POLD(I,J),QOLD(I,J))
        ENDDO
        DO K=1,NCC
!K            IF(FKNU(NCHPP+K).LE.NCUTOFF)THEN
          DO I=1,J
            ZKHI(I)=ZKHI(I)-
     X           DCMPLX(POLD(I,ICHCL(K)),QOLD(I,ICHCL(K)))*ZKHICO(K,J)
          ENDDO
!K            ENDIF
        ENDDO
        DO I=1,J
          P(I,J)=DBLE(ZKHI(I))
          P(J,I)=P(I,J)
          Q(I,J)=DIMAG(ZKHI(I))
          Q(J,I)=Q(I,J)
        ENDDO
      ENDDO
!
      ENDIF

      NULLIFY(P,Q,POLD,QOLD,ZKHICO)
      DEALLOCATE(ZWORK,ZKHI,IPIV)
!
      RETURN
!N600 FORMAT(' SR.MQDTS: ZLUS RETURNED WITH INFO = ',I6)
!N601 FORMAT(' SR.MQDTS: ZLUBS RETURNED WITH INFO = ',I6)
  602 FORMAT(//10X,10('*'),' SR. MQDTS: ZSYTRF RETURNED WITH INFO = ',
     X       I6)
  603 FORMAT(//10X,10('*'),' SR. MQDTS: ZSYTRS RETURNED WITH INFO = ',
     X       I6)
!  610 FORMAT(//10X,10('*'),' SR. MQDTS: NUMBER OF MQDT CLOSED',
!     X ' CHANNELS, NCC = ',I4/20X,' LARGER THAN DIMENSION',
!     X ' VALUE OF DEG = MZDEG'//)
      END
!***************************************************************
!
      SUBROUTINE NUMS(E,C,R1,HP,N1,N2,F,FP,FST)
      USE NRBZED
      USE CPOINT, ONLY : KP2
!
!  NUMEROV INTEGRATION OF COULOMB FUNCTION F.
!  INTEGRATION FROM POINT N1 TO N2.
!  INTERVAL HP IS POSITIVE.
!  INPUT FUNCTIONS F,FP AT N1 WHERE FP IS R-DERIVATIVE.
!  OUTPUT FUNCTIONS F,FP AT N2  AND
!  INTERMEDIATE POINTS STORED IN FST(I) FOR I.LE. MZPTS
!NRB
!  NEUTRAL CASE ADDED
!
      IMPLICIT REAL*8 (A-H,O-Z)
      PARAMETER (TZERO=0.0)
      PARAMETER (ONE=1.0)
      PARAMETER (TWO=2.0)
      PARAMETER (THREE=3.0)
      PARAMETER (FOUR=4.0)
      PARAMETER (SIX=6.0)
      PARAMETER (SEVEN=7.0)
      PARAMETER (TEN=10.0)
      PARAMETER (TWELVE=12.0)
      PARAMETER (P1=ONE/30.)
      PARAMETER (P2=ONE/40.)
      PARAMETER (P3=SEVEN/15.)
      PARAMETER (P4=TWO/15.)
      PARAMETER (P5=ONE/360.)
      PARAMETER (P6=ONE/20.)
      PARAMETER (P7=ONE/120.)
      PARAMETER (P8=ONE/TWELVE)
!
      REAL(8), INTENT(INOUT) :: F,FP
      REAL(8), INTENT(INOUT) :: FST(*)   ! DIM=(MZPTS)=(KP2)
!
!      MZPTS=SIZE(FST)
      V(X)=EQ+X*(Q2*TZED-X*CQ)
!
      N21=N2-N1
!
      F1=F
      F1P=FP
!      IF(N1.LE.MZPTS) FST(N1)=F1
      IF(N1.LE.KP2) FST(N1)=F1
      IF(N21.EQ.0) RETURN
!
      IP=IABS(N21)
      INC=N21/IP
      H=HP*INC
      K=N1
      IP=IP-1
!
!  FUNCTIONS AT K=(N1+INC)
!
      Q=H*H
      EQ=E*Q
      Q2=TWO*Q
      CQ=C*Q
!
      X1=ONE/R1
      CX=C*X1
      HX=H*X1
      A1=-TWO*HX*HX*H
      U1P=A1*(ONE*TZED-CX)
      A1=-A1*HX
      U1PP=A1*(TWO*TZED-THREE*CX)
      A1=-A1*HX*SIX
      U1PPP=A1*(ONE*TZED-TWO*CX)
      U1=V(X1)
!
      R2=R1+H
      X2=ONE/R2
      U2=V(X2)
!
      A2=ONE+U2*P1
      B1=ONE+U1*(P2*U1-P3)-P4*U1P-P2*U1PP
     1 +P5*(FOUR*U1*U1P-U1PPP)
      C1=H*(ONE+U1*(P5*U1-P4)
     1 -P6*U1P-P7*U1PP)
!
      F2=(B1*F1+C1*F1P)/A2
      K=K+INC
!      IF(K.LE.MZPTS) FST(K)=F2
      IF(K.LE.KP2) FST(K)=F2
!
!
      U3=U2
      U2=U1
      F3=F2
      F2=F1
      X3=X2
      IF(IP.EQ.0)GOTO 20
!
!  CONTINUE INTEGRATION
!
      EQ=EQ*P8
      Q2=Q2*P8
      CQ=CQ*P8
      U2=U2*P8
      U3=U3*P8
      R3=R2
!
      DO M=1,IP
        U1=U2
        U2=U3
        F1=F2
        F2=F3
        R3=R3+H
        X3=ONE/R3
        U3=V(X3)
        D3=ONE/(ONE+U3)
        D2=(TWO-TEN*U2)*D3
        D1=(ONE+U1)*D3
        F3=D2*F2-D1*F1
        K=K+INC
!        IF(K.LE.MZPTS) FST(K)=F3
        IF(K.LE.KP2) FST(K)=F3
      ENDDO
!
      U2=TWELVE*U2
      U3=TWELVE*U3
      EQ=TWELVE*EQ
      Q2=TWELVE*Q2
      CQ=TWELVE*CQ
!
!  CALCULATE FINAL DERIVATIVE
!
   20 H=-H
      CX=C*X3
      HX=H*X3
      A1=-TWO*HX*HX*H
      U3P=A1*(TZED*ONE-CX)
      A1=-A1*HX
      U3PP=A1*(TZED*TWO-THREE*CX)
      A1=-A1*HX*SIX
      U3PPP=A1*(TZED*ONE-TWO*CX)
      A2=ONE+U2*P1
      B3=ONE+U3*(P2*U3-P3)-P4*U3P-P2*U3PP
     1 +P5*(FOUR*U3*U3P-U3PPP)
      C3=H*(ONE+U3*(P5*U3-P4)-P6*U3P
     1 -P7*U3PP)
      F3P=(A2*F2-B3*F3)/C3
!
      F=F3
      FP=F3P
!
      RETURN
      END
!***************************************************************
!
      SUBROUTINE NUMSC(E,C,R1,HP,N1,N2,SP,CP,I)
      USE COULSC
      USE NRBZED
!
! NRB:
!  NUMEROV INTEGRATION OF COULOMB FUNCTIONS S AND C.
!  INTEGRATION FROM POINT N1 TO N2.
!  INTERVAL HP IS POSITIVE.
!  I/O FUNCTIONS IN ARRAYS FS,FC
!  I/O DERIVATIVES IN SP,CP (NOT IN ARRAYS)
!NRB
!  NEUTRAL CASE ADDED
!
      IMPLICIT REAL*8 (A-H,O-Z)
      PARAMETER (ONE=1.0)
      PARAMETER (TWO=2.0)
      PARAMETER (THREE=3.0)
      PARAMETER (FOUR=4.0)
      PARAMETER (SIX=6.0)
      PARAMETER (SEVEN=7.0)
      PARAMETER (TEN=10.0)
      PARAMETER (TWELVE=12.0)
      PARAMETER (P1=ONE/30.)
      PARAMETER (P2=ONE/40.)
      PARAMETER (P3=SEVEN/15.)
      PARAMETER (P4=TWO/15.)
      PARAMETER (P5=ONE/360.)
      PARAMETER (P6=ONE/20.)
      PARAMETER (P7=ONE/120.)
      PARAMETER (P8=ONE/TWELVE)
!
      V(X)=EQ+X*(Q2*TZED-X*CQ)
!
      MZPTS=SIZE(FS,1)
      N21=N2-N1
!
      IF(N21.EQ.0)RETURN
!
      IP=IABS(N21)
      INC=N21/IP
      H=HP*INC
      K=N1
      IP=IP-1
      K0=MIN(K,MZPTS)
!
      F1=FS(K0,I)
      F1P=SP
      G1=FC(K0,I)
      G1P=CP
!
!  FUNCTIONS AT K=(N1+INC)
!
      Q=H*H
      EQ=E*Q
      Q2=TWO*Q
      CQ=C*Q
!
      X1=ONE/R1
      CX=C*X1
      HX=H*X1
      A1=-TWO*HX*HX*H
      U1P=A1*(ONE*TZED-CX)
      A1=-A1*HX
      U1PP=A1*(TWO*TZED-THREE*CX)
      A1=-A1*HX*SIX
      U1PPP=A1*(ONE*TZED-TWO*CX)
      U1=V(X1)
!
      R2=R1+H
      X2=ONE/R2
      U2=V(X2)
!
      A2=ONE+U2*P1
      B1=ONE+U1*(P2*U1-P3)-P4*U1P-P2*U1PP
     1 +P5*(FOUR*U1*U1P-U1PPP)
      C1=H*(ONE+U1*(P5*U1-P4)
     1 -P6*U1P-P7*U1PP)
!
      F2=(B1*F1+C1*F1P)/A2
      G2=(B1*G1+C1*G1P)/A2
      K=K+INC
      IF(K.LE.MZPTS) THEN
        FS(K,I)=F2
        FC(K,I)=G2
      ENDIF
!
      U3=U2
      U2=U1
      F3=F2
      F2=F1
      G3=G2
      G2=G1
      X3=X2
      IF(IP.EQ.0)GOTO 20
!
!  CONTINUE INTEGRATION
!
      EQ=EQ*P8
      Q2=Q2*P8
      CQ=CQ*P8
      U2=U2*P8
      U3=U3*P8
      R3=R2
!
      DO M=1,IP
        U1=U2
        U2=U3
        F1=F2
        F2=F3
        G1=G2
        G2=G3
        R3=R3+H
        X3=ONE/R3
        U3=V(X3)
        D3=ONE/(ONE+U3)
        D2=(TWO-TEN*U2)*D3
        D1=(ONE+U1)*D3
        F3=D2*F2-D1*F1
        G3=D2*G2-D1*G1
        K=K+INC
        IF(K.LE.MZPTS) THEN
          FS(K,I)=F3
          FC(K,I)=G3
        ENDIF
      ENDDO
!
      U2=TWELVE*U2
      U3=TWELVE*U3
      EQ=TWELVE*EQ
      Q2=TWELVE*Q2
      CQ=TWELVE*CQ
!
!  CALCULATE FINAL DERIVATIVE
!
   20 H=-H
      CX=C*X3
      HX=H*X3
      A1=-TWO*HX*HX*H
      U3P=A1*(ONE*TZED-CX)
      A1=-A1*HX
      U3PP=A1*(TWO*TZED-THREE*CX)
      A1=-A1*HX*SIX
      U3PPP=A1*(ONE*TZED-TWO*CX)
      A2=ONE+U2*P1
      B3=ONE+U3*(P2*U3-P3)-P4*U3P-P2*U3PP
     1 +P5*(FOUR*U3*U3P-U3PPP)
      C3=H*(ONE+U3*(P5*U3-P4)-P6*U3P
     1 -P7*U3PP)
      F3P=(A2*F2-B3*F3)/C3
      G3P=(A2*G2-B3*G3)/C3
!
      SP=F3P
      CP=G3P
!
      RETURN
      END
!***************************************************************
!
      SUBROUTINE NUMT(E,C,R1,HP,N1,N2,I)
      USE COULSC
      USE CTHET
      USE NRBDD2
      USE NRBZED
!
!  NUMEROV INTEGRATION OF COULOMB FUNCTIONS.
!  THE INTERVAL HP IS POSITIVE.
!  INTEGRATION FROM TABULAR POINT N1 TO TABULAR POINT N2.
!  FUNCTIONS THETA,THETP STORED IN ARRAYS FS,FSP
!  FUNCTIONS THETAD,THETADP STORED IN ARRAYS FC,FCP
!  STARTS WITH FUNCTIONS AND DERIVATIVES AT N1 STORED IN FS, FSP, FC, FCP
!  CALCULATES FUNCTIONS AT ALL POINTS TO N2 AND THE DERIVATIVE AT THE 
!  POINT N2.
! NRB:
!  MINOR MODS FOR MQDT OPERATION.
!  NEUTRAL CASE ADDED
!
      IMPLICIT REAL*8 (A-H,O-Z)
      PARAMETER (ONE=1.0)
      PARAMETER (TWO=2.0)
      PARAMETER (THREE=3.0)
      PARAMETER (FOUR=4.0)
      PARAMETER (SIX=6.0)
      PARAMETER (SEVEN=7.0)
      PARAMETER (TEN=10.0)
      PARAMETER (TWELVE=12.0)
      PARAMETER (P1=ONE/30.)
      PARAMETER (P2=ONE/40.)
      PARAMETER (P3=SEVEN/15.)
      PARAMETER (P4=TWO/15.)
      PARAMETER (P5=ONE/360.)
      PARAMETER (P6=ONE/20.)
      PARAMETER (P7=ONE/120.)
      PARAMETER (P8=ONE/TWELVE)
!
      V(X)=EQ+X*(Q2*TZED-X*CQ)
!
      MZPTS=SIZE(FS,1)
      N21=ABS(N2)-N1
!
!  RENORMALISE FOR CASE OF N2.EQ.N1
!
      IF(N21.NE.0)GOTO 5
      W=FS(N1,I)*FCP(I)-FSP(I)*FC(N1,I)
      IF(W.EQ.0.0)RETURN
      W1=ONE/SQRT(W)
      FS(N1,I)=FS(N1,I)*W1
      FC(N1,I)=FC(N1,I)*W1
      FSP(I)=FSP(I)*W1
      FCP(I)=FCP(I)*W1
      FSP2(I)=FSP(I)
      FCP2(I)=FCP(I)
      BB(I,2)=BB(I,2)*W1
      NM=MSUM(I)
      DO M=3,NM
        BB(I,M)=BB(I,M)*W1
        BG(I,M)=BG(I,M)*W1
      ENDDO
      RETURN
!
!  INTEGRATIONS FOR N2.NE.N1
!
   5  IP=IABS(N21)
      IS=N21/IP
      H=HP*IS
      K=N1
      IP=IP-1
!
      F1=FS(N1,I)
      F1P=FSP(I)
      G1=FC(N1,I)
      G1P=FCP(I)
!
!  FUNCTIONS AT K=(N1+IS)
!
      Q=H*H
      EQ=E*Q
      Q2=TWO*Q
      CQ=C*Q
!
!      Q=0.0          !FOR XI SOLUTION
!
      X1=ONE/R1
      CX=C*X1
      HX=H*X1
      A1=-TWO*HX*HX*H
      U1P=A1*(ONE*TZED-CX)
      A1=-A1*HX
      U1PP=A1*(TWO*TZED-THREE*CX)
      A1=-A1*HX*SIX
      U1PPP=A1*(ONE*TZED-TWO*CX)
      U1=V(X1)
!
      R2=R1+H
      X2=ONE/R2
      U2=V(X2)
!
      A2=ONE+U2*P1
      B1=ONE+U1*(P2*U1-P3)-P4*U1P-P2*U1PP
     1 +P5*(FOUR*U1*U1P-U1PPP)
      C1=H*(ONE+U1*(P5*U1-P4)
     1 -P6*U1P-P7*U1PP)
      B2=P1*Q
      D1=Q*(-P3+P2*U1+P5*U1P)    !FOR XI DELETE P2*U1
      E1=H*Q*(-P4+P5*U1)
!
      F2=(B1*F1+C1*F1P)/A2
      G2=(B1*G1+C1*G1P+D1*F1+E1*F1P-B2*F2)/A2
      K=K+IS
      FS(K,I)=F2
      FC(K,I)=G2
!
!
      U3=U2
      U2=U1
      F3=F2
      F2=F1
      G3=G2
      G2=G1
      X3=X2
      IF(IP.EQ.0)GOTO 20
!
!  CONTINUE INTEGRATION
!
      EQ=EQ*P8
      Q2=Q2*P8
      CQ=CQ*P8
      U2=U2*P8
      U3=U3*P8
      Q=Q*P8
      R3=R2
!
      DO M=1,IP
        U1=U2
        U2=U3
        F1=F2
        F2=F3
        G1=G2
        G2=G3
        R3=R3+H
        X3=ONE/R3
        U3=V(X3)
        D3=ONE/(ONE+U3)
        D2=(TWO-TEN*U2)*D3
        D1=(ONE+U1)*D3
        F3=D2*F2-D1*F1
        G3=D2*G2-D1*G1-Q*D3*(F3+TEN*F2+F1)
        K=K+IS
        FC(K,I)=G3
        FS(K,I)=F3
      ENDDO
!
      U2=TWELVE*U2
      U3=TWELVE*U3
      EQ=TWELVE*EQ
      Q2=TWELVE*Q2
      CQ=TWELVE*CQ
      Q=TWELVE*Q
!
!  CALCULATE FINAL DERIVATIVE
!
   20 H=-H
      CX=C*X3
      HX=H*X3
      A1=-TWO*HX*HX*H
      U3P=A1*(ONE*TZED-CX)
      A1=-A1*HX
      U3PP=A1*(TWO*TZED-THREE*CX)
      A1=-A1*HX*SIX
      U3PPP=A1*(ONE*TZED-TWO*CX)
      A2=ONE+U2*P1
      B3=ONE+U3*(P2*U3-P3)-P4*U3P-P2*U3PP
     1 +P5*(FOUR*U3*U3P-U3PPP)
      C3=H*(ONE+U3*(P5*U3-P4)-P6*U3P
     1 -P7*U3PP)
      B2=P1*Q
      D3=Q*(-P3+P2*U3+P5*U3P)     !FOR XI DELETE P2*U3
      E3=H*Q*(-P4+P5*U3)
      F3P=(A2*F2-B3*F3)/C3
      G3P=(A2*G2-B3*G3+B2*F2-D3*F3-E3*F3P)/C3
!
!  RE-NORMALISE CLOSED-CHANNEL FUNCTIONS
!
      IF(N1.GT.ABS(N2))THEN
        AMAX=ONE/MAX(ABS(F3),ABS(G3),ABS(F3P),ABS(G3P))
        AF3=F3*AMAX
        AG3=G3*AMAX
        AF3P=F3P*AMAX
        AG3P=G3P*AMAX
        W1=AF3*AG3P-AF3P*AG3
        W1=AMAX/SQRT(W1)
        FSP2(I)=FSP(I)*W1
        FCP2(I)=FCP(I)*W1
        FSP(I)=F3P*W1
        FCP(I)=G3P*W1
      ELSE
        FSP2(I)=F3P
        FCP2(I)=G3P
        RETURN
      ENDIF
      IPMX=IP+2
      IF(N2.LT.0) IPMX=MZPTS
      DO J=1,IPMX
        FS(J,I)=FS(J,I)*W1
        FC(J,I)=FC(J,I)*W1
      ENDDO
!
!  RE-NORMALISE COEFFICIENTS
!
      BB(I,2)=BB(I,2)*W1
      NM=MSUM(I)
      DO M=3,NM
        BB(I,M)=BB(I,M)*W1
        BG(I,M)=BG(I,M)*W1
      ENDDO
!
      RETURN
      END
!***************************************************************
!
      SUBROUTINE OCINT(IOMTT,NCHOP)
      USE CALP
      USE CPOT
      USE NRBLMX
      USE CACC
      USE CHAN
      USE CPOINT
      USE CNTRL
      USE CQDT
      USE CBLK
!
!  OPEN-CLOSED:
!  CALCULATES S INTEGRALS USING LAGUERRE QUADRATURE
!
      IMPLICIT REAL*8 (A-H,O-Y)
      IMPLICIT COMPLEX*16 (Z)
!
      DIMENSION ZAI(30),ZPI(30),ZTAJ(30),ZTDAJ(30),ZTPJ(30),ZR(30)
!     X,IOMTT(MZCHF)
      INTEGER, INTENT(IN) :: IOMTT(*)
!
      NCHOP1=NCHOP+1
!
      X=RTWO
      B=SQRT(2.*X)
      ZB=(0.,1.)/B
      DO I=1,NCHOP
        IF(IOMTT(I).EQ.0)THEN
        DO J=NCHOP1,NCHF
          IF(IOMTT(J).EQ.0)THEN
          LIJ=LAMP(J,I)
          IF(LIJ.NE.1.AND.LIJ.LE.LMX+1) THEN
            BIJ=BW(J,I)
            NLAG=2*LIJ+LACC
            NLAG=MIN(NLAG,10)
            ZFK=DCMPLX(FKNU(I),1./FKNU(J))
            ZG=.5*ZFK*B
!
            ZA1=1./ZG
            ZA2=1.+ZG
            ZA2=ZA2*ZA2
            ZA3=ZB*ZG
!
            NS=NLAG/2
            M=NS*(NS-1)
            N1=M+1
            N2=M+NLAG
            NP = 0
            DO N=N1,N2
              NP = NP + 1
              U=XLAG(N)
              ZET=ZA1*(SQRT(ZA2+ZA3*U)-1.)
              ZMUM=-.5*ZET/(ZB*(1.+ZET*ZG))
              ZET=ZET*ZET
              ZR(NP)=ZET*X
            ENDDO
            CALL ZPHIN(I,ZR,N1,N2,ZAI,ZPI)
            CALL ZTHETAS(J,ZA1,ZA2,ZA3,ZB,ZG,N1,N2,ZTAJ,ZTDAJ,ZTPJ)
!
            ZS3=0.
            ZSD3=0.
            NP = 0
            DO N=N1,N2
              NP = NP + 1
              U=XLAG(N)
              ZET=ZA1*(SQRT(ZA2+ZA3*U)-1.)
              ZMUM=-.5*ZET/(ZB*(1.+ZET*ZG))
              ZB1=(ZR(NP)**(-LIJ))*WLAG(N)*ZMUM*
     X          EXP((0.,1.)*ZPI(NP)+ZTPJ(NP)+U)*ZAI(NP)
              ZS3=ZS3+ZB1*ZTAJ(NP)
              ZSD3=ZSD3+ZB1*ZTDAJ(NP)
            ENDDO
            ZS3=ZS3*BIJ
            ZSD3=ZSD3*BIJ
            ACC(J,I)=ACC(J,I)+DBLE(ZSD3)
            ACS(J,I)=ACS(J,I)+DIMAG(ZSD3)
            ASC(J,I)=ASC(J,I)+DBLE(ZS3)
            ASS(J,I)=ASS(J,I)+DIMAG(ZS3)
          ENDIF
          ENDIF
        ENDDO
        ENDIF
      ENDDO
      RETURN
      END
!***************************************************************
!
      SUBROUTINE OMEG(IE0)
      USE comm_interface, Only : nproc
      USE MEMORY
      USE NRBTOP
      USE CEN      ! MXE
      USE CMESH    ! EMESH
      USE CINPUT   ! NAST 
      USE CINPTX   ! ENAT 
      USE NRBSKP   ! ELAS
      USE COMEGA   ! OMEGA, NOMWRT, IE
!
!   LFM - USE VARIABLES IN MODULES IN STEAD OF CALLING SEQUENCE
!   ALLOCATE ARRAYS TO THE EXACT VALUE, IT SAVES MEMORY
!
!   USE MEMORY FOR SUMMING OMEGA, WITH OVERFLOW ONTO DISK
!   (ADAPTED BY NRB, FROM KAB, FOR ELASTIC TRANSITIONS
!    AND USE OF NOMT=ABS(NOMWRT) TO REDUCE I/O)
!   IE0 = 0 TO INITIALIZE
!   IE0 .GT.0 TO RETRIEVE OMEGA FOR ENERGY IE0
!   IF NOMWRT .LT. 0 THEN RECOVER -NOMWRT NON-ZERO OMEGAS FOR 
!                    FINAL WRITE RUNNING DOWN THE COLUMNS
!   IF NOMWRT .GT. 0 THEN RECOVER NOMWRT OMEGAS (INC ZEROES) FOR
!                    FINAL WRITE RUNNING ALONG THE ROWS
!          (EXC ZEROES CAN BE IMPLEMENTED BY SWITCHING JJJJ BELOW)
!   EMESH(MXE)  = ENERGY MESH
!   ENAT(NAST)  = EXCITATION THRESHOLDS: NEEDED TO MINIMISE STORAGE
!   OMEGA(NOMT)= RETURNS OMEGA MATRIX AS AN ARRAY
!   /MEMORY/ STORAGE IN OMEM(MXOM)
!   MPOS(IE0)   = LAST POSITION OCCUPIED IN OMEM AT ENERGY IE0, OR
!   IF MPOS.LT.0 = -RECORD NUMBER ON DA SCRATCH FILE
!
      IMPLICIT REAL*8 (A-H,O-Z)
      INTEGER, PARAMETER :: MZKIL=0, MZREC=8
      REAL(8), PARAMETER :: TZERO=0.D0
!
      IF (ELAS.EQ.'YES') THEN
        IONE=0
      ELSE
        IONE=1
      END IF
      NOMT=ABS(NOMWRT)
      N=0
      IF(NOMWRT.GT.0)THEN
        DO I=1,NAST
          DO J=I+IONE,NAST
            N=N+1
            OMEGA(N)=TZERO
          ENDDO
          IF(N.GE.NOMT) EXIT
        ENDDO
        I=NAST
      ENDIF
      IF(NOMWRT.LT.0)THEN
        DO J=1+IONE,NAST
          DO I=1,J-IONE
            N=N+1
            OMEGA(N)=TZERO
          ENDDO
          IF(N.GE.NOMT) EXIT
        ENDDO
        J=NAST
      ENDIF
!
!   INITIALISE
!
      MXOM=MZMEG*(2**20)+MZKIL*(2**10)
      IF (IE0.EQ.0) THEN

        ALLOCATE(MPOS(0:MXE))

        IF(NOMWRT.GT.0)THEN
          ITMAX=I
          JTMAX=NAST
        ELSE
          ITMAX=NAST
          JTMAX=J
        ENDIF
        MPOS(0)=0
        NPOS=0
        NREC=0
        NST=1
        DO I=1,MXE
          DO WHILE (NST.LE.NAST .AND. EMESH(I).GE.ENAT(NST))
!    4     IF (NST.LE.NAST)THEN
!            IF (EMESH(I).GE.ENAT(NST)) THEN
              NP=MIN(NST,ITMAX,JTMAX)
              NTRAN=(NP*(NP+1-2*IONE))/2
              IF(NST.GT.ITMAX) NTRAN=NTRAN+(NST-ITMAX)*ITMAX
              NST=NST+1
!              GO TO 4
!            ENDIF
!          ENDIF
          END DO
          NPOS=NPOS+NTRAN
          IF(NPOS.LE.MXOM) THEN
            MPOS(I)=NPOS
          ELSE
            NREC=NREC+1
            MPOS(I)=-NREC
          ENDIF
        ENDDO
!
        MMM=MIN(NPOS,MXOM)
        ALLOCATE(OMEM(MMM))
        OMEM(1:MMM)=TZERO
!
        IF(NREC.GT.0) THEN
          MMEG=(NPOS/1000000)+1
          AMBSC=DFLOAT(NPOS*MZREC*nproc)*1.D-6
          WRITE(6,100) MMEG,AMBSC
          NLEN=MIN(NOMT,(NAST*(NAST+1-2*IONE))/2)
          OPEN(1,STATUS='SCRATCH',ACCESS='DIRECT',RECL=MZREC*NLEN,
     X         FORM='UNFORMATTED')
          DO MREC=1,NREC
            CALL OMWRIT(OMEGA,NLEN,MREC)
          ENDDO
          NPOS=MXOM
        ENDIF
        WRITE(6,101) MXOM,NPOS
!
      ELSE
!
!   FINALLY, RETRIEVE AND RETURN OMEGA
!
        JJJJ=NAST                ! INC ZEROES NOMWRT.GT.0
!       JJJJ=JTMAX               ! EXC ZEROES NOMWRT.GT.0
!
        IF (MPOS(IE0).EQ.0) THEN
!   ALL CHANNEL CLOSED, OMGA=0
!
          DO N=1,NOMT
            OMEGA(N)=TZERO
          ENDDO
!          GO TO 25
!
        ELSE IF (MPOS(IE0).GT.0) THEN
!   EXTRACT FROM SAME REGISTER, ALREADY LOADED OMEM
!
          M=MPOS(IE0-1)
          K=0
          DO JT=1+IONE,JJJJ
            IP=MIN(JT-IONE,ITMAX)
            DO IT=1,IP
              M=M+1
              IF(NOMWRT.GT.0)THEN
                K=JT+1-IONE+(JJJJ+1-IONE)*(IT-1)-(IT*(IT+1))/2
              ELSE
                K=K+1
              ENDIF
              OMEGA(K)=OMEM(M)
            ENDDO
            IF(M.GE.MPOS(IE0)) EXIT
          ENDDO
!
        ELSE IF (MPOS(IE0).LT.0) THEN
!   READ NEW REGISTER
!
          MREC=-MPOS(IE0)
          CALL OMREAD(OMEGA,NOMT,MREC)
          IF(NOMWRT.GT.0)THEN
            DO I=1,NOMT
              OMST(I)=OMEGA(I)
            ENDDO
            M=0
            DO JT=1+IONE,JJJJ
              IP=MIN(JT-IONE,ITMAX)
              DO IT=1,IP
                M=M+1
                K=JT+1-IONE+(JJJJ+1-IONE)*(IT-1)-(IT*(IT+1))/2
                OMEGA(K)=OMST(M)
              ENDDO
              IF(M.GE.NOMT) EXIT
            ENDDO
          ENDIF
        ENDIF
      ENDIF
!
      RETURN
!
  100 FORMAT(//" **** OPENING SCRATCH FILE **** "
     2  /"COULD AVOID BY INCREASING MZMEG TO: ",I6
     3  /"APPROXIMATE USE OF HARD DISK FOR SCRATCH: ",ES10.2," MB")
  101 FORMAT(//" MXOM = ", I10,"  ALLOCATED = ",I10//)
!
      END SUBROUTINE OMEG
!
!************************************************************************
      SUBROUTINE OMREAD(OMEGA,NOMT,IE)
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION OMEGA(NOMT)
!
      READ(1,REC=IE)OMEGA
!
      RETURN
      END
!************************************************************************
      SUBROUTINE OMWRIT(OMEGA,NOMT,IE)
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION OMEGA(NOMT)
!
      WRITE(1,REC=IE)OMEGA
!
      RETURN
      END
!***************************************************************
!
      SUBROUTINE OOINT(IOMTT,NCHOP)
      USE CALP
      USE CPOT
      USE NRBLMX
      USE CACC
      USE CHAN
      USE CPOINT
      USE CNTRL
      USE CQDT
      USE CBLK
!
! OPEN-OPEN:
!  CALCULATES P AND Q INTEGRALS USING LAGUERRE AND/OR LEGENDRE QUADRATURE
!
      IMPLICIT REAL*8 (A-H,O-Y)
      IMPLICIT COMPLEX*16 (Z)
!
      DIMENSION ZAI(30),ZPI(30),ZAJ(30),ZPJ(30),ZR(30)
      DIMENSION ZAI1(30),ZPI1(30),ZAJ1(30),ZPJ1(30),ZR1(30)
      DIMENSION ZAI2(30),ZPI2(30),ZAJ2(30),ZPJ2(30),ZR2(30)
!     DIMENSION IOMTT(NCHF)
      INTEGER, INTENT(IN) :: IOMTT(*)
!
      NCHOP1=NCHOP+1
!
      X=RTWO
      B=SQRT(8.*X)
      ZB=(0.,1.)/B
      DO I=1,NCHOP
        IF(IOMTT(I).EQ.0)THEN
        DO J=I,NCHOP
          IF(IOMTT(J).EQ.0)THEN
          LIJ=LAMP(J,I)
          IF(LIJ.NE.1.AND.LIJ.LE.LMX+1) THEN
            NLAG=2*LIJ+LACC
            NLAG=MIN(NLAG,10)
            BIJ=BW(J,I)
            NS=NLAG/2
            M=NS*(NS-1)
            N1=M+1
            N2=M+NLAG
!
            FK=FKNU(I)+FKNU(J)
            G=FK*.125*B
            IF(FK.GT.0)THEN
              GM=1./G
              G2=1.+G
              G2=G2*G2
            ENDIF
            NP = 0
            DO N=N1,N2
              NP = NP + 1
              U=XLAG(N)
              A1=FK*U
              IF(A1.LE.ACZP)THEN
                ZET=1.+.5*ZB*U
              ELSE
                ZET=(SQRT(G2+ZB*G*U)-1.)*GM
              ENDIF
              ZMU=-8.*ZB*(G+1./ZET)
              ZET=ZET*ZET
              ZR(NP)=ZET*X
            ENDDO
            CALL ZPHIN(I,ZR,N1,N2,ZAI,ZPI)
            CALL ZPHIN(J,ZR,N1,N2,ZAJ,ZPJ)
            NP = 0
            ZP3= 0.
            DO N=N1,N2
              NP = NP + 1
              U=XLAG(N)
              A1=FK*U
              IF(A1.LE.ACZP)THEN
                ZET=1.+.5*ZB*U
              ELSE
                ZET=(SQRT(G2+ZB*G*U)-1.)*GM
              ENDIF
              ZMU=-8.*ZB*(G+1./ZET)
              ZP3=ZP3+ZAI(NP)*ZAJ(NP)*
     X          EXP((0.,1.)*(ZPI(NP)+ZPJ(NP))+U)*
     X          (ZR(NP)**(-LIJ))*WLAG(N)/ZMU
            ENDDO
            ZP3 = ZP3*BIJ
            ALP=RTWO*(FKNU(I)-FKNU(J))
            IF(ALP.GE.2.) THEN
              FK=FKNU(I)-FKNU(J)
              ZMUM=(0.,1.)/FK
!
              NP = 0
!
              DO N=N1,N2
                NP = NP + 1
                U=XLAG(N)
                ZR(NP)=X+U*ZMUM
              ENDDO
              CALL ZPHIN(I,ZR,N1,N2,ZAI,ZPI)
              CALL ZPHIN(J,ZR,N1,N2,ZAJ,ZPJ)
              ZQ3=0.
              NP = 0
              DO N=N1,N2
                NP = NP + 1
                U=XLAG(N)
                ZQ3=ZQ3+ZAI(NP)*ZAJ(NP)*
     X            EXP((0.,1.)*(ZPI(NP)-ZPJ(NP))+U)*
     X            (ZR(NP)**(-LIJ))*WLAG(N)
              ENDDO
              ZQ3=ZQ3*ZMUM*BIJ
            ELSE
              FK=FKNU(I)-FKNU(J)
              IF(FK.EQ.0.)THEN
                ZA=1.
              ELSE
                ZA=(0.,1.)/(1.+FK*X)
              ENDIF
!
              NS=NLAG/2
              J1=(NS*(NS-1))/2
              J2=J1+NS
              J1=J1+1
              JP = 0
              DO JJ=J1,J2
                JP = JP + 1
                V=XLEG(JJ)
                ZR1(JP)=X*(1.+ZA*(1.-V)/(1.+V))
                ZR2(JP)=X*(1.+ZA*(1.+V)/(1.-V))
              ENDDO
              CALL ZPHIN(I,ZR1,J1,J2,ZAI1,ZPI1)
              CALL ZPHIN(J,ZR1,J1,J2,ZAJ1,ZPJ1)
              CALL ZPHIN(I,ZR2,J1,J2,ZAI2,ZPI2)
              CALL ZPHIN(J,ZR2,J1,J2,ZAJ2,ZPJ2)
!
              ZQ3=0.
              JP = 0
              DO JJ=J1,J2
                JP = JP + 1
                V=XLEG(JJ)
                ZF=ZAI1(JP)*ZAJ1(JP)*
     X            EXP((0.,1.)*(ZPI1(JP)-ZPJ1(JP)))*
     X            (ZR1(JP)**(-LIJ))/(1.+V)**2
                ZF=ZF+ZAI2(JP)*ZAJ2(JP)*
     X            EXP((0.,1.)*(ZPI2(JP)-ZPJ2(JP)))*
     X            (ZR2(JP)**(-LIJ))/(1.-V)**2
                ZQ3=ZQ3+ZF*WLEG(JJ)
              ENDDO
              ZQ3=2.*X*ZA*ZQ3*BIJ
            ENDIF
            ASS(J,I)=ASS(J,I)+.5*DBLE(ZQ3-ZP3)
            ACS(J,I)=ACS(J,I)+.5*DIMAG(ZP3+ZQ3)
            ASC(J,I)=ASC(J,I)+.5*DIMAG(ZP3-ZQ3)
            ACC(J,I)=ACC(J,I)+.5*DBLE(ZP3+ZQ3)
          ENDIF
          ENDIF
        ENDDO
        ENDIF
      ENDDO
!
      RETURN
      END
!***************************************************************
!
      SUBROUTINE OUTJJ(IOPT1,LRGLAM,MXE)
      USE WMATALLOC
      USE CINPUT
      USE CINPTX
      USE CDEC
      USE CMESH
!
!    TO SUPPLY NON-K-MATRIX DATA FOR JAJOM ON UNITS 22,23 (FORMATTED).
!
!    IOPT1=0 FOR INITIALISATION, FOR EACH S L PI SYMMETRY
!        =10 FOR JAJOM WITH ALGEBRAIC RECOUPLING WITHOUT,(WITH TOP-UP)
!        =12 FOR JAJOM WITH TERM COUPLING (COUPLING COEFFS READ IN STGF)
!    LRGLAM .GT. 0 FOR TOP-UP
!    MXE = TOTAL NUMBER OF ENERGIES, HELD IN CMESH
!    CINPUT CONTAINS BASIC COLLISION DATA.
!
      IMPLICIT REAL*8 (A-H,O-Z)
!
!      PARAMETER (MXTST=(MZTAR*MZTAR+MZTAR)/2)
!
      PARAMETER(JOUT=22,IOJM=5,NOX=0,IFACT=60) !,NTC=MZTAR*MZTAR)
      PARAMETER(MZSLP=125)
!
      CHARACTER*1 PARITY(0:1),LSPECT(0:6)
!
      DIMENSION ILT(MZSLP),IST(MZSLP)
!      DIMENSION FJ(MZLP1)
!TCC      DIMENSION IDF(NTC),ITF(NTC),FCF(NTC)
      REAL(8), ALLOCATABLE :: FJ(:)
!
      DATA LSPECT/'S','P','D','F','G','H','I'/, PARITY/'E','O'/
      DATA J2F,N2F/2*0/
      DATA ISLM/0/, KCOUNT/0/
!
      SAVE ISLM,KCOUNT,ILT,IST
!
      AZ=MAX(NZED-NELC,1)
      AZAZ=AZ*AZ
!
!  INITIALIZE ...
!  ISLM = COUNTER ON ILT,IST SYMMETRIES
!  KCOUNT = COUNTER ON MATRIX ELEMENTS OUTPUT ON UNIT 11
!  IPAR(NAST) = TARGET PARITIES
!
      IF(IOPT1.EQ.0) THEN
        DO 1 I=1,ISLM
          IF(LRGL2.EQ.ILT(I).AND.NSPN2.EQ.IST(I)) GO TO 2
    1   CONTINUE
        ISLM=ISLM+1
        ILT(ISLM)=LRGL2
        IST(ISLM)=NSPN2
    2   N=1
        NCHOP=0
        D=0.0
        IF(IMESH.EQ.2) D=1./QNMAX**2
        DO 5 I=1,MXE
    4     IF(EMESH(I).GT.ENAT(N)-D.AND.N.LT.NAST) THEN
            N=N+1
            NCHOP=NCHOP+NCONAT(N)
            NUM=NCHOP*(NCHOP+1)
            GO TO 4
          ENDIF
          KCOUNT=KCOUNT+NUM
    5   CONTINUE
        RETURN
      ENDIF
!
!    OPEN UNITS JOUT(=22),23,25
!
      ALLOCATE(FJ(LRANG2))
      IF(MXE.EQ.0.OR.ISLM.EQ.0.OR.KCOUNT.EQ.0) GO TO 42
!
      OPEN(JOUT,FILE='JJDAT',STATUS='UNKNOWN')
      REWIND(JOUT)
!
!SPEC NOT NEEDED BY CURRENT IP JAJOM AND THIS STGF
!SPEC      OPEN(23,FILE='JOMSPECS',STATUS='UNKNOWN')
!SPEC      REWIND(23)
!
!TCC READ DIRECTLY FROM TCC.DAT IN JAJOM
!TCC      OPEN(25,FILE='TCC.DAT',STATUS='UNKNOWN')
!TCC      REWIND(25)
!
!    FIND NAOP = NUMBER OF STATES ENERGETICALLY ALLOWED;
!         MNSPN,MXSPN = MINIMUM,MAXIMUM TARGET STATE 2S+1;
!         JCHAN= NUMBER OF CHANNELS IN INTERMEDIATE COUPLING.
!
      JCHAN=0
      NAOP=0
      MNSPN=999
      MXSPN=0
      DELE=0.
      IF (IMESH.EQ.2) DELE=1/(QNMAX*QNMAX)
      DO 9 I=1,NAST
       IF(EMESH(MXE).LT.ENAT(I)-DELE) GO TO 6
       NAOP=I
       MNSPN=MIN(MNSPN,ISAT(I))
       MXSPN=MAX(MXSPN,ISAT(I))
       K=2*LAT(I)+1
       JHIGH=K+ISAT(I)-1
       JLOW=ABS(K-ISAT(I))+1
    9  JCHAN=JCHAN + ((JHIGH+JLOW)/2) * (1+(JHIGH-JLOW)/2)
!
!    WRITE OUT JAJOM INPUT
!    CARDS A1-A2
!
    6 IF(NAOP.EQ.0) GO TO 42
      IREFL=1
      ITOP=0
      IF(LRGLAM.GT.0) ITOP=1
      MAT=2
      IONQ=NZED-NELC
      IPART=0
      K=NAOP
      IF(IRDEC.GT.0) K=1
      DO 7 I=1,K
    7  ARDEC(I)=0.0
      WRITE(JOUT,'(2X,I2,2I3,10I5)')
     *      NZED,NELC,NAOP,IREFL,8,7,ITOP,MAT,IONQ,IPART,IRDEC,IMESH
      DO 8 I=1,NAOP
       K=ISAT(I)
       IF(IPAR(I).EQ.1) K=-K
    8  WRITE(JOUT,1002) I, K,LAT(I),ENAT(I)*AZAZ,
     *       ISAT(I),LSPECT(LAT(I)),PARITY(IPAR(I)),ARDEC(I)
!
!    CARDS B1-B4
!
      WRITE(JOUT,1004) IMESH,DEOPEN,QNMAX,DQN,'CHI',IOPT1,
     *LRANG2,ISLM,NOX,(I,I=0,LRANG2-1)
      WRITE(JOUT,'(18I4)') (IST(I),ILT(I),I=1,ISLM)
!
!    CARDS C1-C4
!
      WRITE(JOUT,2003) NZED,NELC,RA,LAMAX,KCOUNT,
     *      MXE,MXE
      IF(IMESH.EQ.1)THEN
        DE=EMESH(2)-EMESH(1)
        WRITE(JOUT,2004)1,EMESH(1)*AZAZ,0,DE*AZAZ
      ELSE
        WRITE(JOUT,2004)(K,EMESH(K)*AZAZ,K=1,MXE)
      ENDIF
      WRITE(JOUT,2005) EMESH(MXE)*AZAZ
!
!    FIND ALLOWED TOTAL J VALUES (FJ(JJFSL))
!
      JJFSL=0
      J2=MOD(MNSPN,2)
   22 IZ=MNSPN
      IF(IZ.EQ.2) IZ=0
   23 L2=ABS(J2-IZ)
   24 DO 25 I=1,ISLM
      IF(IZ.EQ.IST(I)-1 .AND.L2.EQ.2*ILT(I)) GO TO 26
   25 CONTINUE
      GO TO 27
   26 L2=L2+2
      IF(L2.LE.J2+IZ) GO TO 24
      IZ=IZ+2
      IF(IZ.LE.MXSPN) GO TO 23
      JJFSL=JJFSL+1
      FJ(JJFSL)=0.5*J2
      J2=J2+2
      GO TO 22
!
!    CARDS E1-E2
!
   27 IFIT=JJFSL+1
      JPUN=NAOP+1
      WRITE(JOUT,'(5H F.S.,2I5,I3)') JJFSL,IFIT,JPUN
      IF(JJFSL.EQ.0) THEN
      WRITE(6,*)' NO J VALUES -- SYMMETRIES SLPI INCOMPLETE'
      GO TO 41
      ENDIF
      WRITE(JOUT,'(7X,13F5.1)') (FJ(J),J=1,JJFSL)
!
!TCC    CARDS F1-F2: TERM COUPLING COEFFICIENTS TO BE READ BY JAJOM
!       ELSE LET JAJOM READ DIRECTLY FROM TCC.DAT FILE.
!
       WRITE(6,*)
!      BUT SKIP TERM LIST
!TCC       READ(25,*,END=32) NTER
!TCC       IF(NTER.GT.0) READ(25,'(I4)') (K,I=1,NTER)
!TCC   31   READ(25,1106)J2F,N2F
!TCC   32   CONTINUE
!
        WRITE(JOUT,1106) J2F,N2F
!
!TCC        IF (N2F.EQ.0) GO TO 33
!TCC        READ(25,1107) (ITF(I),IDF(I),FCF(I),I=1,N2F)
!TCC        WRITE(JOUT,1107) (ITF(I),IDF(I),FCF(I),I=1,N2F)
!TCC        WRITE(6,1108)N2F,J2F
!TCC        GO TO 31
!TCC   33 CONTINUE
!
      WRITE(JOUT,1106) NOX,NOX
   41 WRITE(JOUT,'("ENDDATA")')
!
!     FINALISE OUTPUT TO JAJOM, WRITE PROCESSOR DATA FOR JAJOM ONTO 23
!SPEC ONLY NEEDED BY JAJOM NOT SETUP TO READ CHI-MX FROM IP STGF
!
!SPEC      NEST=MIN(1+(KCOUNT-1)/1000,9999)
!SPEC      WRITE(23,1313) NEST,MXE,JCHAN,NAOP,IOJM,IFACT,NAST
!SPEC      WRITE(6,1313) NEST,MXE,JCHAN,NAOP,IOJM,IFACT,NAST
!     NOTE PARAMETRICALLY FIXED IOJM AND IFACT.
!
   42 WRITE(6,1099) MXE,NAOP,ISLM,KCOUNT,'CHI',JCHAN
      CLOSE(JOUT)
      DEALLOCATE(FJ)
!
!SPEC      CLOSE(23)
!TCC      CLOSE(25)
!
      RETURN
!
 1002 FORMAT(3I4,F20.8,I3,2A1,1P,E16.6)
 1004 FORMAT(' IMESH=',I2,' DE=',F8.6,' QNMAX=',F7.3,' DQN=',F8.6,1X,
     *A3,'-MTRX OPT=',I2/3I5/(20I3))
 1099 FORMAT(/I9,' ENERGIES',I6,' STATES',I6,' SL-SYMMETRIES'/I9,1X,
     * A3,'-MATRIX ELEMENTS (TOTAL)',I8,' PAIR-COUPLING CHANNELS'/)
 1106 FORMAT(7X,I3,I5)
!TCC 1107 FORMAT(5(I3,I2,F9.6))
!TCC 1108 FORMAT(I5,' TERM COUPLING COEFFICIENTS PASSED ON FOR 2J =',I3)
!SPEC 1313 FORMAT('   11   12   13   -1'/6I5,'    R  CDC',I5,' JOMSPECS')
 2003 FORMAT(' Z=',I2,' N=',I2,' RA=',F8.4,' LAMAX=',I2,' COUNT=',I8/
     *  2I5)
 2004 FORMAT(4(I5,F13.7))
 2005 FORMAT(1PE13.6,'EEEE')
      END
!***************************************************************
!
      SUBROUTINE PETFSC
      USE NRBRCT
      USE CALP
      USE CHAN
      USE NRBHYB
!
! NRB;
! PERTURB S AND C WHERE F=S+C*K
!
      IMPLICIT REAL*8 (A-H,O-Y)
      IMPLICIT COMPLEX*16 (Z)
      PARAMETER (RMONE=-1.0)
      PARAMETER (ONE=1.0)
      PARAMETER (TZERO=0.0)
!
!  CALCULATE PERTURBED FUNCTIONS
!
!     OPEN-OPEN
        DO J=1,NCHPP
          DO I=1,NCHPP
            CS(I,J)=S(I)*ACS(I,J)-C(I)*ASS(I,J)
            CSP(I,J)=SP(I)*ACS(I,J)-CP(I)*ASS(I,J)
            CC(I,J)=S(I)*ACC(I,J)-C(I)*ASC(I,J)
            CCP(I,J)=SP(I)*ACC(I,J)-CP(I)*ASC(I,J)
            CSPP(I,J)=SPP(I)*ACS(I,J)-CPP(I)*ASS(I,J)
            CCPP(I,J)=SPP(I)*ACC(I,J)-CPP(I)*ASC(I,J)
            DS(I,J)=CS(I,J)
            DC(I,J)=CC(I,J)
            DSP(I,J)=CSP(I,J)
            DCP(I,J)=CCP(I,J)
          ENDDO
        ENDDO
        DO I=1,NCHPP
          CS(I,I)=CS(I,I)+S(I)
          CSP(I,I)=CSP(I,I)+SP(I)
          CC(I,I)=CC(I,I)+C(I)
          CCP(I,I)=CCP(I,I)+CP(I)
          CSPP(I,I)=CSPP(I,I)+SPP(I)
          CCPP(I,I)=CCPP(I,I)+CPP(I)
        ENDDO
!     CLOSED-OPEN
        IF(NCHPP.EQ.NCHF)GOTO 270
        DO J=1,NCHPP
          DO I=NCHPP1,NCHF
            CS(I,J)=C(I)*ASS(I,J)-S(I)*ACS(I,J)
            CSP(I,J)=CP(I)*ASS(I,J)-SP(I)*ACS(I,J)
            CC(I,J)=C(I)*ASC(I,J)-S(I)*ACC(I,J)
            CCP(I,J)=CP(I)*ASC(I,J)-SP(I)*ACC(I,J)
            CSPP(I,J)=CPP(I)*ASS(I,J)-SPP(I)*ACS(I,J)
            CCPP(I,J)=CPP(I)*ASC(I,J)-SPP(I)*ACC(I,J)
            DS(I,J)=CS(I,J)
            DC(I,J)=CC(I,J)
            DSP(I,J)=CSP(I,J)
            DCP(I,J)=CCP(I,J)
          ENDDO
        ENDDO
!     OPEN-CLOSED
        DO J=NCHPP1,NCHF
          DO I=1,NCHPP
            CC(I,J)=S(I)*ACS(I,J)-C(I)*ASS(I,J)
            CCP(I,J)=SP(I)*ACS(I,J)-CP(I)*ASS(I,J)
            CCPP(I,J)=SPP(I)*ACS(I,J)-CPP(I)*ASS(I,J)
            DC(I,J)=CC(I,J)
            DCP(I,J)=CCP(I,J)
          ENDDO
        ENDDO
!     CLOSED-   CLOSED
        DO J=NCHPP1,NCHF
          DO I=NCHPP1,NCHF
            CC(I,J)=C(I)*ASS(I,J)-S(I)*ACS(I,J)
            CCP(I,J)=CP(I)*ASS(I,J)-SP(I)*ACS(I,J)
            CCPP(I,J)=CPP(I)*ASS(I,J)-SPP(I)*ACS(I,J)
            DC(I,J)=CC(I,J)
            DCP(I,J)=CCP(I,J)
          ENDDO
        ENDDO
        DO I=NCHPP1,NCHF
          CC(I,I)=CC(I,I)+S(I)
          CCP(I,I)=CCP(I,I)+SP(I)
        ENDDO
!
  270   CONTINUE
        IF(NCHHYB.GT.0)THEN
          DO N=1,NCHHYB
            I=ICHHYB(N)
            CC(I,I)=S(I)          !SINCE ALPHA=0
            CCP(I,I)=SP(I)
            CS(I,I)=TZERO
            CSP(I,I)=TZERO
          ENDDO
        ENDIF
!
!  CALCULATE MATRICES A AND B
!
        DO J=1,NCHF
          DO I=1,NCHF
            A(I,J)=CC(I,J)
          ENDDO
!STRTNBL
!NBL    DO K=1,NCHF
!NBL      DO I=1,NCHF
!NBL        A(I,J)=A(I,J)-RMAT(I,K)*CCP(K,J)
!NBL      ENDDO
!NBL    ENDDO
!ENDNBL
        ENDDO
        DO J=1,NCHPP
          DO I=1,NCHF
            B(I,J)=CS(I,J)
          ENDDO
!STRTNBL
!NBL    DO K=1,NCHF
!NBL      DO I=1,NCHF
!NBL        B(I,J)=B(I,J)-RMAT(I,K)*CSP(K,J)
!NBL      ENDDO
!NBL    ENDDO
!ENDNBL
        ENDDO
!STRTBL
        CALL DGEMM('N','N',NCHF,NCHF,NCHF,RMONE,RMAT,NCHF,CCP,NCHF,
     X    ONE,A,NCHF)
        CALL DGEMM('N','N',NCHF,NCHPP,NCHF,RMONE,RMAT,NCHF,CSP,NCHF,
     X    ONE,B,NCHF)
!ENDBL
!
        RETURN
        END
!***************************************************************
!
      SUBROUTINE PETKMX
      USE NRBRCT, only : RMAT,RK,Y,TEMP3,TEMP4,DCP,QQQ,PPP
      USE NRBWRK, only : workm,workm2
      USE CALP
      USE CHAN
      USE CNTRL
      USE NRBZED
!
! NRB:
! EVALUATE PERTURBATION TO K-MATRIX (K->K+Y, Y=-<F|V|F>) SO (UNDAMPED)
! S-MATRIX IS UNITARY, IMPORTANT FOR DR.
! (INCLUDES KB'S REWRITE OF DO LOOPS)
!
      IMPLICIT REAL*8 (A-H,O-Y)
      IMPLICIT COMPLEX*16 (Z)
!      PARAMETER (LWORK=MZCHF*MZCHF)
!      PARAMETER (MWORK=MZDEG*MZDEG)
      PARAMETER (TWO=2.0)
      PARAMETER (ONE=1.0)
      PARAMETER (TZERO=0.0)
!
       INTEGER,ALLOCATABLE :: IPIV(:)
       REAL*8,ALLOCATABLE :: YY(:,:)
!
!--------------------------------------------------------
!

       ALLOCATE(QQQ(1:NCHPP,NCHPP1:NCHF))
       ALLOCATE(PPP(1:NCHPP,NCHPP1:NCHF))

       ALLOCATE(YY(1:NCHPP,1:NCHPP))
       PPP(1:NCHPP,NCHPP1:NCHF)=RK(1:NCHPP,NCHPP1:NCHF)       
       QQQ(1:NCHPP,NCHPP1:NCHF)=RK(1:NCHPP,NCHPP1:NCHF)

!--------------------------------------------------------
      ALLOCATE(WORKM(NCHF,NCHF))
      ALLOCATE(WORKM2(NCHF,NCHF))
      ALLOCATE(IPIV(NCHF))
      Y=>RMAT
      TEMP4=>DCP
!
!  Y=-(F/V/F), F HAS K-MATRIX NORMALISATION,
!    V IS PERTURBATION POTENTIAL
!
!STRTNBL
!NBL  DO J=1,NCHPP
!NBL    DO I=1,NCHPP
!NBL      Y(I,J)=TZERO
!NBL    ENDDO
!NBL    DO K=1,NCHPP
!NBL      DO I=1,NCHPP
!NBL        Y(I,J)=Y(I,J)+ACC(I,K)*RK(K,J)
!NBL      ENDDO
!NBL    ENDDO
!NBL  ENDDO
!ENDNBL
!
!STRTBL
      CALL DGEMM('N','N',NCHPP,NCHPP,NCHPP,ONE,ACC,NCHF,RK,NCHF,
     X  TZERO,Y,NCHF)
!ENDBL
      DO J=1,NCHPP
        DO I=1,NCHPP
          ACC(I,J)=Y(I,J)
          Y(I,J)=ASS(I,J)
        ENDDO
      ENDDO
!STRTNBL
!NBL  DO J=1,NCHPP
!NBL    DO K=1,NCHPP
!NBL      DO I=1,J
!NBL        Y(I,J)=Y(I,J)+RK(I,K)*ACS(K,J)
!NBL      ENDDO
!NBL    ENDDO
!NBL  ENDDO
!NBL  DO J=1,NCHPP
!NBL    DO K=1,NCHPP
!NBL      DO I=1,J
!NBL        Y(I,J)=Y(I,J)+ASC(I,K)*RK(K,J)
!NBL      ENDDO
!NBL    ENDDO
!NBL  ENDDO
!NBL  DO J=1,NCHPP
!NBL    DO K=1,NCHPP
!NBL      DO I=1,J
!NBL        Y(I,J)=Y(I,J)+RK(I,K)*ACC(K,J)
!NBL      ENDDO
!NBL    ENDDO
!NBL  ENDDO
!ENDNBL
!
!STRTBL
      CALL DSYMM('L','U',NCHPP,NCHPP,ONE,RK,NCHF,ACS,NCHF,
     X  ONE,Y,NCHF)
      CALL DSYMM('R','U',NCHPP,NCHPP,ONE,RK,NCHF,ASC,NCHF,
     X  ONE,Y,NCHF)
      CALL DSYMM('L','U',NCHPP,NCHPP,ONE,RK,NCHF,ACC,NCHF,
     X  ONE,Y,NCHF)
!ENDBL
!
! TEST
      IF(IPERT.LE.-22)THEN
        ISIGN=1
        IF(TZED.GT.0)ISIGN=-1
        DO J=1,NCHPP
          DO I=1,J
            Y(J,I)=Y(I,J)
          ENDDO
        ENDDO
        DO J=1,NCHPP
          DO I=1,NCHPP
            WORKM(I,J)=-ISIGN*(ACS(I,J)+ACC(I,J))
          ENDDO
        ENDDO
        DO I=1,NCHPP
          TEMP3(I,I)=ONE+TEMP3(I,I)
        ENDDO
        IF(TZED.EQ.0)CALL VERT(WORKM,NCHF,NCHPP,IPIV,IERR)
        DO J=1,NCHPP
          DO I=1,NCHPP
            TEMP4(I,J)=TZERO
          ENDDO
          DO K=1,NCHPP
            DO I=1,NCHPP
              TEMP4(I,J)=TEMP4(I,J)+Y(I,K)*WORKM(K,J)
            ENDDO
          ENDDO
        ENDDO
        DO J=1,NCHPP
           DO I=1,J
             Y(I,J)=(TEMP4(I,J)+TEMP4(J,I))/TWO
           ENDDO
        ENDDO
      ENDIF
! END TEST
!
      IF(NCHPP.LT.NCHF)THEN

       YY(1:NCHPP,1:NCHPP)=Y(1:NCHPP,1:NCHPP)

        DO J=1,NCHPP
          DO K=NCHPP1,NCHF
            TEMPA=RK(K,J)
            TEMPB=ASS(K,J)
!            DO I=1,J
!              YY(I,J)=YY(I,J)+ASS(I,K)*TEMPA+RK(I,K)*TEMPB
!            ENDDO
            YY(1:J,J)=YY(1:J,J)+ASS(1:J,K)*TEMPA+RK(1:J,K)*TEMPB
          ENDDO
        ENDDO
c        DO J=1,NCHPP
c          DO I=1,NCHPP
c            WORKM(I,J)=TZERO
c          ENDDO
c        ENDDO
        WORKM(1:NCHPP,1:NCHPP)=TZERO
        WORKM2(1:NCHPP,1:NCHPP)=WORKM(1:NCHPP,1:NCHPP)

        DO J=1,NCHPP
          DO K=NCHPP1,NCHF
!            DO I=1,NCHPP
!              WORKM(I,J)=WORKM(I,J)+ACS(I,K)*RK(K,J)
!              WORKM2(I,J)=WORKM2(I,J)+RK(I,K)*ASC(K,J)
!            ENDDO
            TEMPA=RK(K,J)
            TEMPB=ASC(K,J)
        WORKM(1:NCHPP,J)=WORKM(1:NCHPP,J)+ACS(1:NCHPP,K)*TEMPA
        WORKM2(1:NCHPP,J)=WORKM2(1:NCHPP,J)+RK(1:NCHPP,K)*TEMPB
          ENDDO
        ENDDO
        DO J=1,NCHPP
          DO K=1,NCHPP
!            DO I=1,J
!              YY(I,J)=YY(I,J)+RK(I,K)*WORKM(K,J)
!     X               +RK(K,J)*WORKM2(I,K)
!            ENDDO
            TEMPA=RK(K,J)
            TEMPB=WORKM(K,J)
            YY(1:J,J)=YY(1:J,J)+RK(1:J,K)*TEMPB
     X             +WORKM2(1:J,K)*TEMPA
          ENDDO
        ENDDO
c        DO J=1,NCHPP
c          DO I=1,NCHPP
c            WORKM(I,J)=TZERO
c          ENDDO
c        ENDDO
c        DO J=1,NCHPP
c          DO K=NCHPP1,NCHF
c            DO I=1,NCHPP
c              WORKM(I,J)=WORKM(I,J)+RK(I,K)*ASC(K,J)
c            ENDDO
c          ENDDO
c        ENDDO
c        DO J=1,NCHPP
c          DO K=1,NCHPP
c            DO I=1,J
c              YY(I,J)=YY(I,J)+RK(K,J)*WORKM(I,K)
c            ENDDO
c          ENDDO
c        ENDDO
!
!       ORIGINAL VERSION
!
!11        DO L=NCHPP1,NCHF
!11          DO K=NCHPP1,NCHF
!11            TEMPA00=ASS(K,L)
!11            DO J=1,NCHPP
!11              TEMPB00=TEMPA00*RK(J,L)
!11              DO I=1,J
!11                Y(I,J)=Y(I,J)+RK(I,K)*TEMPB00
!11              ENDDO
!11            ENDDO
!11          ENDDO
!11        ENDDO

!
!       UNROLL L LOOP BY 7
!
!71        LSIZ = 7*((NCHF-NCHPP1+1)/7)
!71        DO L=NCHPP1,NCHPP1+LSIZ-1, 7
!71          DO K=NCHPP1,NCHF
!71            TEMPA00=ASS(K,L)
!71            TEMPA01=ASS(K,L+1)
!71            TEMPA02=ASS(K,L+2)
!71            TEMPA03=ASS(K,L+3)
!71            TEMPA04=ASS(K,L+4)
!71            TEMPA05=ASS(K,L+5)
!71            TEMPA06=ASS(K,L+6)
!71            DO J=1,NCHPP
!71              TEMPB00=TEMPA00*RK(J,L)
!71              TEMPB01=TEMPA01*RK(J,L+1)
!71              TEMPB02=TEMPA02*RK(J,L+2)
!71              TEMPB03=TEMPA03*RK(J,L+3)
!71              TEMPB04=TEMPA04*RK(J,L+4)
!71              TEMPB05=TEMPA05*RK(J,L+5)
!71              TEMPB06=TEMPA06*RK(J,L+6)
!71              DO I=1,J
!71                Y(I,J)=Y(I,J)+RK(I,K)*TEMPB00+RK(I,K)*TEMPB01+
!71     *                        RK(I,K)*TEMPB02+RK(I,K)*TEMPB03+
!71     *                        RK(I,K)*TEMPB04+RK(I,K)*TEMPB05+
!71     *                        RK(I,K)*TEMPB06
!71              ENDDO
!71            ENDDO
!71          ENDDO
!71        ENDDO
!71        DO L=NCHPP1+LSIZ,NCHF
!71          DO K=NCHPP1,NCHF
!71            TEMPA00=ASS(K,L)
!71            DO J=1,NCHPP
!71              TEMPB00=TEMPA00*RK(J,L)
!71              DO I=1,J
!71                Y(I,J)=Y(I,J)+RK(I,K)*TEMPB00
!71              ENDDO
!71            ENDDO
!71          ENDDO
!71        ENDDO

!
!       UNROLL L LOOP BY 4 AND K LOOP BY 2
!
        LSIZ = 4*((NCHF-NCHPP1+1)/4)
        KSIZ = 2*((NCHF-NCHPP1+1)/2)
        DO L=NCHPP1,NCHPP1+LSIZ-1, 4
          DO K=NCHPP1,NCHPP1+KSIZ-1, 2
            TEMPA00=ASS(K,L)
            TEMPA10=ASS(K+1,L)
            TEMPA01=ASS(K,L+1)
            TEMPA11=ASS(K+1,L+1)
            TEMPA02=ASS(K,L+2)
            TEMPA12=ASS(K+1,L+2)
            TEMPA03=ASS(K,L+3)
            TEMPA13=ASS(K+1,L+3)
            DO J=1,NCHPP
              TEMPB00=TEMPA00*RK(J,L)
              TEMPB10=TEMPA10*RK(J,L)
              TEMPB01=TEMPA01*RK(J,L+1)
              TEMPB11=TEMPA11*RK(J,L+1)
              TEMPB02=TEMPA02*RK(J,L+2)
              TEMPB12=TEMPA12*RK(J,L+2)
              TEMPB03=TEMPA03*RK(J,L+3)
              TEMPB13=TEMPA13*RK(J,L+3)
!              DO I=1,J
!              YY(I,J)=YY(I,J)+RK(I,K)*(TEMPB00+TEMPB01)+
!     *                        RK(I,K)*(TEMPB02+TEMPB03)+
!     *                        RK(I,K+1)*(TEMPB10+TEMPB11)+
!     *                        RK(I,K+1)*(TEMPB12+TEMPB13)
!              ENDDO
              YY(1:J,J)=YY(1:J,J)+RK(1:J,K)*(TEMPB00+TEMPB01)+
     *                            RK(1:J,K)*(TEMPB02+TEMPB03)+
     *                            RK(1:J,K+1)*(TEMPB10+TEMPB11)+
     *                            RK(1:J,K+1)*(TEMPB12+TEMPB13)
            ENDDO
          ENDDO
          DO K=NCHPP1+KSIZ,NCHF
            TEMPA00=ASS(K,L)
            TEMPA01=ASS(K,L+1)
            TEMPA02=ASS(K,L+2)
            TEMPA03=ASS(K,L+3)
            DO J=1,NCHPP
              TEMPB00=TEMPA00*PPP(J,L)
              TEMPB01=TEMPA01*PPP(J,L+1)
              TEMPB02=TEMPA02*PPP(J,L+2)
              TEMPB03=TEMPA03*PPP(J,L+3)
!              DO I=1,J
!               YY(I,J)=YY(I,J)+QQQ(I,K)*TEMPB00+QQQ(I,K)*TEMPB01+
!     *                         QQQ(I,K)*TEMPB02+QQQ(I,K)*TEMPB03
!              ENDDO
              YY(1:J,J)=YY(1:J,J)+QQQ(1:J,K)*TEMPB00+
     *                            QQQ(1:J,K)*TEMPB01+
     *                            QQQ(1:J,K)*TEMPB02+
     *                            QQQ(1:J,K)*TEMPB03
            ENDDO
          ENDDO
        ENDDO

        DO L=NCHPP1+LSIZ,NCHF
          DO K=NCHPP1,NCHF
            TEMPA00=ASS(K,L)
            DO J=1,NCHPP
              TEMPB00=TEMPA00*PPP(J,L)
!              DO I=1,J
!                YY(I,J)=YY(I,J)+QQQ(I,K)*TEMPB00
!              ENDDO
              YY(1:J,J)=YY(1:J,J)+QQQ(1:J,K)*TEMPB00
            ENDDO
          ENDDO
        ENDDO
       
       Y(1:NCHPP,1:NCHPP)=YY(1:NCHPP,1:NCHPP)
!
      ENDIF
!
      DO J=1,NCHPP
        DO I=1,J
          Y(J,I)=Y(I,J)
        ENDDO
      ENDDO
!
      NULLIFY(TEMP4)       !NOTE 2016 , MIGHT NEED Y OUTSIDE SUB
      DEALLOCATE(WORKM,WORKM2,IPIV,PPP,QQQ,YY)
!
      RETURN
      END
!***************************************************************
!
      SUBROUTINE PETRAD(INOUT)
      USE WMATALLOC
      USE NRBRCT
      USE COULSC
      USE CPOT
      USE CINPTX
      USE CHAN
      USE NRBOMT
      USE CPOINT
      USE CNTRL
      USE CQDT
!
! NRB:
!   FOR CASE OF IRAD.NE.0, EVALUATE PERTURBATIONS TO RADIATIVE
!   DATA  AND WRITE REACTANCE MATRIX AND FUNCTIONS TO UNIT 9
!   PUT FUNCTIONS IN CS,CSP,CSPP - ORIGINAL CONTENTS DESTROYED
!
! LFM:
!   IF RADIATION IS REMOVED, THIS SUBROUTINE IS NEVER CALLED
!
      IMPLICIT REAL*8 (A-H,O-Y)
      IMPLICIT COMPLEX*16 (Z)
!
      PARAMETER (ONE=1.0)
      PARAMETER (TZERO=0.0)
      PARAMETER (TWO=2.0)
!
      INTEGER, INTENT(IN) :: INOUT(*)
      REAL(8), ALLOCATABLE :: TEMP(:)
!
      ALLOCATE(TEMP(NCHF))
      AR=ONE/RZERO
      AR2=AR*AR
      AR3=AR2*AR
      IF(IPERT.GT.0) THEN
!
!  USE A AS WORKSPACE - ORIGINAL CONTENTS DESTROYED
        DO J=1,NCHPP
          DO I=1,NCHPP
            A(I,J)=C(I)*RK(I,J)
          ENDDO
        ENDDO
        DO I=1,NCHPP
          A(I,I)=A(I,I)+S(I)
        ENDDO
        DO J=1,NCHPP
          DO I=NCHPP1,NCHF
            A(I,J)=S(I)*RK(I,J)
          ENDDO
        ENDDO
!
!  INITIALISE DFPP TO V * F
        DO J=1,NCHPP
          DO I=1,NCHF
            DFPP(I,J)=TZERO
          ENDDO
        ENDDO
        DO J=1,NCHPP
          DO K=1,NCHF
            DO I=1,NCHF
                TEMP(I)=TZERO
              IF(LAMP(I,K).EQ.2) THEN
                TEMP(I)=AR2
              ELSE IF(LAMP(I,K).EQ.3) THEN
                TEMP(I)=AR3
              ENDIF
            ENDDO
            DO I=1,NCHF
              DFPP(I,J)=DFPP(I,J)-BW(I,K)*TEMP(I)*A(K,J)
            ENDDO
          ENDDO
        ENDDO
!
        DO J=1,NCHPP
          DO K=1,NCHF
            RKK=RK(K,J)
            DO I=1,NCHF
              CS(I,J)=CS(I,J)+CC(I,K)*RKK
              CSP(I,J)=CSP(I,J)+CCP(I,K)*RKK
              CSPP(I,J)=CSPP(I,J)+CCPP(I,K)*RKK
              DS(I,J)=DS(I,J)+DC(I,K)*RKK
              DSP(I,J)=DSP(I,J)+DCP(I,K)*RKK
            ENDDO
          ENDDO
          DO I=1,NCHF
            DSP(I,J)=DSP(I,J)+BSTO*DS(I,J)
            DFPP(I,J)=DFPP(I,J)+(AR*(AR*CCT(I)-TWO)-EPS(I))*DS(I,J)
            CSP(I,J)=CSP(I,J)+BSTO*CS(I,J)
          ENDDO
        ENDDO
      ELSE
        DO J=1,NCHPP
          DO I=1,NCHPP
            CS(I,J)=C(I)*RK(I,J)
            CSP(I,J)=(CP(I)+BSTO*C(I))*RK(I,J)
            CSPP(I,J)=CPP(I)*RK(I,J)
          ENDDO
        ENDDO
        DO I=1,NCHPP
          CS(I,I)=CS(I,I)+S(I)
          CSP(I,I)=CSP(I,I)+SP(I)+BSTO*S(I)
          CSPP(I,I)=CSPP(I,I)+SPP(I)
        ENDDO
        DO J=1,NCHPP
          DO I=NCHPP1,NCHF
            CS(I,J)=S(I)*RK(I,J)
            CSP(I,J)=(SP(I)+BSTO*S(I))*RK(I,J)
            CSPP(I,J)=SPP(I)*RK(I,J)
          ENDDO
        ENDDO
      ENDIF
!
      WRITE(9) QDT
      WRITE(9) NCHPP,(IOMIT(I),I=1,NCHPP)
      WRITE(9)((RK(I,J),J=1,NCHPP),I=1,NCHF)
      WRITE(9)((CS(I,J),J=1,NCHPP),I=1,NCHF)
      WRITE(9)((CSP(I,J),J=1,NCHPP),I=1,NCHF)
      WRITE(9)((CSPP(I,J),J=1,NCHPP),I=1,NCHF)
!
      DO I=1,NCHPP
        IF(INOUT(I).EQ.0)THEN
          WRITE(9)INOUT(I),FS(1,I),FS(2,I),FC(1,I),FC(2,I)
        ELSE
          WRITE(9)INOUT(I),FS(1,I),FS(2,I),FC(KP2,I),FC(KP2-1,I)
        ENDIF
      ENDDO
      DO I=NCHPP1,NCHF
        IF(INOUT(I).EQ.0)THEN
          WRITE(9)INOUT(I),FS(KP2,I),FS(KP2-1,I)
        ELSE
          WRITE(9)INOUT(I),FS(1,I),FS(2,I)
        ENDIF
      ENDDO
!
      IF(IPERT.GT.0) THEN
        WRITE(9)((DS(I,J),J=1,NCHPP),I=1,NCHF)
        WRITE(9)((DSP(I,J),J=1,NCHPP),I=1,NCHF)
        WRITE(9)((DFPP(I,J),J=1,NCHPP),I=1,NCHF)
      ENDIF
!
      RETURN
      END
!***************************************************************
!
      SUBROUTINE PETTMX
      USE NRBRCT
      USE NRBWRK
      USE CALP
      USE CHAN
      USE CNTRL
!
! NRB:
! HAVING EVALUATED K-MATRIX PERTURBATION (Y) VIA CALL TO PETKMX,
! NOW NEGLECT Y**2 AND K**N*Y**M (N+M>2) TERMS, WHICH REDUCES TO A
! PERTURBATION OF THE T-MATRIX. THE S-MATRIX IS NOT UNITARY SO DO
! **NOT** USE FOR DR. I DON'T SEE WHY ANYONE WOULD WANT TO USE THIS......
! (INCLUDES KB'S REWRITE OF DO LOOPS)
!
      IMPLICIT REAL*8 (A-H,O-Y)
      IMPLICIT COMPLEX*16 (Z)
!      PARAMETER (LWORK=MZCHF*MZCHF)
!      PARAMETER (MWORK=MZDEG*MZDEG)
!BL   PARAMETER (ONE=1.0)
      PARAMETER (TZERO=0.0)
!
       ALLOCATE(WORKM(NCHF,NCHF))
!
       Y=>RMAT
       P=>CSP
       Q=>CC
       D=>CCP
!
! PERTURB (IN EFFECT) T-MATRIX.
!
!  B=Y-RK*Y*RK
!STRTNBL
        DO J=1,NCHPP
          DO I=1,NCHPP
            D(I,J)=TZERO
          ENDDO
          DO K=1,NCHPP
            DO I=1,NCHPP
              D(I,J)=D(I,J)+Y(I,K)*RK(K,J)
            ENDDO
          ENDDO
        ENDDO
!ENDNBL
!
!STRTBL
!BL     CALL DGEMM('N','N',NCHPP,NCHPP,NCHPP,ONE,Y,NCHF,RK,NCHF,
!BL  X    TZERO,D,MZCHF)
!ENDBL
!
        DO J=1,NCHPP
          DO I=1,J
            B(I,J)=Y(I,J)
          ENDDO
        ENDDO
!
!STRTNBL
        DO J=1,NCHPP
          DO K=1,NCHPP
            DO I=1,J
              B(I,J)=B(I,J)-RK(I,K)*D(K,J)
            ENDDO
          ENDDO
        ENDDO
        DO J=1,NCHPP
          DO I=1,J
            B(J,I)=B(I,J)
          ENDDO
        ENDDO
!ENDNBL
!
!STRTBL
!BL     CALL DSYMM('L','U',NCHPP,NCHPP,RMONE,RK,NCHF,D,MZCHF,
!BL  X    ONE,B,NCHF)
!ENDBL
!
!  D=Y*RK+RK*Y
!STRTNBL
        DO J=1,NCHPP
          DO K=1,NCHPP
            DO I=1,J
              D(I,J)=D(I,J)+RK(I,K)*Y(K,J)
            ENDDO
          ENDDO
        ENDDO
        DO J=1,NCHPP
          DO I=1,J
            D(J,I)=D(I,J)
          ENDDO
        ENDDO
!ENDNBL
!
!STRTBL
!BL     CALL DSYMM('L','U',NCHPP,NCHPP,ONE,RK,NCHF,Y,NCHF,
!BL  X    ONE,D,MZCHF)
!ENDBL
!
!  P=P+A*B*A, Q=Q+A*D*A
!STRTNBL
        DO J=1,NCHPP
          DO I=1,NCHPP
            WORKM(I,J)=TZERO
          ENDDO
          DO K=1,NCHPP
            DO I=1,NCHPP
              WORKM(I,J)=WORKM(I,J)+B(I,K)*A(K,J)
            ENDDO
          ENDDO
        ENDDO
        DO J=1,NCHPP
          DO K=1,NCHPP
            DO I=1,J
              P(I,J)=P(I,J)+A(I,K)*WORKM(K,J)
            ENDDO
          ENDDO
        ENDDO
        DO J=1,NCHPP
          DO I=1,J
            P(J,I)=P(I,J)
          ENDDO
        ENDDO
!
        DO J=1,NCHPP
          DO I=1,NCHPP
            WORKM(I,J)=TZERO
          ENDDO
          DO K=1,NCHPP
            DO I=1,NCHPP
              WORKM(I,J)=WORKM(I,J)+D(I,K)*A(K,J)
            ENDDO
          ENDDO
        ENDDO
        DO J=1,NCHPP
          DO K=1,NCHPP
            DO I=1,J
              Q(I,J)=Q(I,J)+A(I,K)*WORKM(K,J)
            ENDDO
          ENDDO
        ENDDO
        DO J=1,NCHPP
          DO I=1,J
            Q(J,I)=Q(I,J)
          ENDDO
        ENDDO
!ENDNBL
!
!STRTBL
!BL     CALL DGEMM('N','N',NCHPP,NCHPP,NCHPP,ONE,B,NCHF,A,NCHF,
!BL  X    TZERO,WORKM,NCHF)
!BL     CALL DGEMM('N','N',NCHPP,NCHPP,NCHPP,ONE,A,NCHF,WORKM,NCHF,
!BL  X    ONE,P,NCHF)
!BL     CALL DGEMM('N','N',NCHPP,NCHPP,NCHPP,ONE,D,MZCHF,A,NCHF,
!BL  X    TZERO,WORKM,NCHF)
!BL     CALL DGEMM('N','N',NCHPP,NCHPP,NCHPP,ONE,A,NCHF,WORKM,NCHF,
!BL  X    ONE,Q,NCHF)
!ENDBL
!
!
      NULLIFY(Y,P,Q,D)
      DEALLOCATE(WORKM)
!
      RETURN
      END
!***************************************************************
!
      SUBROUTINE POINTS(IOPT1,QJUMP)
      USE WMATALLOC
      USE CEN
      USE CINPUT
      USE CINPTX
      USE CHAN
      USE NRBKUT
      USE NRBHYB
      USE NRBQDT
      USE NRBOMT
      USE CACC
      USE CPOINT
      USE CPTOLD
      USE CNTRL
      USE CQDT
      USE CMESH
      USE NRBZED
      USE CWARN
!
!  CALCULATES CHANNELS ENERGIES, NUMBER OF OPEN CHANNELS
!  AND TABULAR POINTS.
! NRB:
! SUBSTANTIALLY REVISED FOR MQDT
!
      IMPLICIT REAL*8 (A-H,O-Z)
      PARAMETER(TINY=-1.D-6)
      PARAMETER(XEPS=0.1D0)
      PARAMETER(TZERO=0.0D0)
!
      LOGICAL QJUMP
      REAL*8,ALLOCATABLE :: TEMPH(:)
!
       ALLOCATE(TEMPH(NCHF))
!      TZED=1.0D0                             !FOR DAMPED
!
      KP0=1
      QDT=.FALSE.
      IQ=0
!
!  FOR IRAD.GT.0, HOLD OLD KP2, RTWO AND H
!
      IF(IRAD.GT.0)THEN
        KP2OLD=KP2
        RTWOLD=RTWO
        HOLD=H
      ENDIF
!
! INITIALIZE IOMIT
!
      IF(.NOT.QJUMP)THEN
        DO I=1,NCHOPT
	   IOMIT(I)=0
        ENDDO
        DO I=NCHOPT+1,NCHF
          IOMIT(I)=1
        ENDDO
      ENDIF
!
!  SET-UP FOR CHANNEL ENERGIES EPS AND NUMBER OF OPEN CHANNELS NCHPP
!
      NCHPP=0
      NCHCL=0
      NCHHYB=0
      DO I=1,NCHF
!OLD        IF(.NOT.QJUMP)IOMIT(I)=0
        E=ETOT-ECH(I)
        IF(E.GT.TINY.AND.E.LT.0.)E=TZERO
        IF(TZED.EQ.TZERO)THEN
          IF(E.LE.-TINY.AND.E.GE.0)E=TZERO
          IF(E.EQ.TZERO)IOMIT(I)=1
        ENDIF
        IF(E.GE.TZERO)THEN
          IF(IOPT1.GE.10)THEN
            QDT=IMESH.EQ.2
            NQ=NCHPP+1
          ENDIF
          NCHPP=NCHPP+1
          FKNU(I)=SQRT(E)
        ELSE
          FKNU(I)=1.D0/SQRT(-E)
          IF(IOMSW.LT.0)THEN
            IF(FKNU(I).LT.FNUHYB.AND.FKNU(I).LT.LLCH(I)+XEPS)THEN
              NCHHYB=NCHHYB+1
              ICHHYB(NCHHYB)=I
            ELSE
              NCHCL=NCHCL+1
              ICHCL(NCHCL)=I
            ENDIF
          ENDIF
        ENDIF
        EPS(I)=E
      ENDDO
!
      NCHPP1=NCHPP+1
!
!  SET QDT FOR CASE OF QNMAX.GT.0
!
      NQ=NCHPP
      IF((QNMAX.GT.TZERO.OR.IQDT.LT.0).AND.NCHPP.LT.NCHF)THEN
        DO I=NCHPP1,NCHF
          IF(ABS(ECH(I)-ECH(NCHPP1)).GT.ABS(TINY))GO TO 121
          IF(IQDT.LT.0)NQ=I
          FL5=DBLE(LLCH(I))+.5D0
          IF(QNMAX.GT.TZERO.AND.FKNU(I)+0.00002D0.GT.MAX(QNMAX,FL5))THEN
            QDT=.TRUE.
            NQ=I
          ELSE
            IF(IQDT.GE.0)GOTO 121
          ENDIF
        ENDDO
  121   IF((QDT.AND.IQDT.EQ.0).OR.IQDT.LT.0)THEN
          IF(NQ.GT.NCHPP)THEN
            NCHPP1=NQ+1
            IPERT=0
            IF(IPRINT.GT.0)WRITE(6,630)NCHPP,NQ
          ENDIF
        ENDIF
      ENDIF
      IF(IQDT.GT.0)NCHPP1=NCHF+1
!
! IF NOT UPDATING S-MATRIX THEN
!
      IF(QJUMP)RETURN
!
!
!  CALCULATION OF RTWO
!  INCLUDES CALCULATION OF INNER POINTS OF INFLECTION RINF
!
      RTWO=RZERO
      RIMX=TZERO
!
!  (1) OPEN CHANNELS
!  FOR OPEN CHANNELS RTWO IS DEFINED BY CONVERGENCE CRITERION
!  FOR THE JWBK METHOD.
!
!      WRITE(77,*)ETOT
      DO I=1,NCHPP
        RINF(I)=RZERO
        R2=RZERO
        IF(IOMIT(I).GT.0)GO TO 138
        E=EPS(I)
        C=CCT(I)
        L=LLCH(I)
        EC=E*C
!NRB
        RINF(I)=C/(TZED+SQRT(TZED+EC)+(1.D0-TZED)*1.D-4)+1.D-4       !KEEP NON-ZERO WHEN L=0
        IF(RINF(I).GT.RIMX)RIMX=RINF(I)
!
        IF(L-1)131,132,133
  131   IF(AC.GE.1.D-3)THEN
          CONST=12.D0
        ELSE
         CONST=56.D0
        ENDIF
        CE=CONST*E
!NRB
        R2=CONST/(TZED+SQRT(TZED+CE)+(1.D0-TZED)*1.D-4)
!
        GOTO 138
  132   IF(AC.GE.1.D-3)THEN
          R2=3.9D0*RINF(I)
        ELSE
          R2=16.D0*RINF(I)
        ENDIF
        GOTO 138
  133   IF(AC.GE.1.D-3)THEN
          R2=RINF(I)*(1.2D0+5.7D0/DBLE(L))
        ELSE
          R2=RINF(I)*(1.4D0+9.8D0/DBLE(L))
        ENDIF
  138   CONTINUE
!        IF(R2.LT.RONE)R2=RONE
        R2ST(I)=R2
        IF(RTWO.LT.R2)RTWO=R2
!        WRITE(77,*)I,LLCH(I),EPS(I),RINF(I),R2
      ENDDO
!
!  (2) CLOSED CHANNELS (STRONGLY CLOSED FOR QDT.EQ..TRUE.)
!  FOR CLOSED CHANNELS RTWO IS EQUAL TO THE OUTER POINT OF
!  INFLECTION, EXCEPT FOR THE CASE OF FNU.LT.(LL+1)
!
      ROMN=RTWO
      DO I=NCHPP+1,NCHF
        IF(TZED.EQ.TZERO.OR.IOMIT(I).GT.0)THEN
          RINF(I)=TZERO
          R2ST(I)=RZERO
        ELSE
          FNU=FKNU(I)
          FLP1=LLCH(I)
          FLP1=FLP1+0.5D0
          IF(FNU.LT.FLP1)THEN
            RINF(I)=TZERO
            R2=RZERO
!            R2=R2+FLP1     ! +3*FLP1 TO CONVERGE HIGH-L
          ELSE
            A1=SQRT(FNU*FNU-CCT(I))
            R2=FNU*(FNU+A1)
            RINF(I)=FNU*(FNU-A1)+1.D-4       !KEEP NON-ZERO WHEN L=0
            IF(R2.GT.RZERO.AND.R2.LT.ROMN)ROMN=R2
          ENDIF
          IF(RINF(I).GT.RIMX)RIMX=RINF(I)
          IF(RTWO.LT.R2.AND.(I.GE.NCHPP1.OR.INTPQ.NE.0))RTWO=R2
          R2ST(I)=R2
!        WRITE(77,*)I,LLCH(I),FKNU(I),RINF(I),R2
        ENDIF
      ENDDO
      IF(RONE.GT.RTWO)RTWO=RONE
      IF(ROMN.LT.RZERO)ROMN=RZERO
      IF(RIMX.LT.RZERO)RIMX=RZERO
!      WRITE(77,*)RTWO
!
!  CASE OF IRAD.GT.0
!
      IF(IRAD.GT.0)THEN
!  CHECK VALUE OF RTWO
        IF(RTWO.GT.RTWOLD)THEN
          IF(WARN)WRITE(6,600)ETOT,RTWOLD
          WARN=.FALSE.
        ENDIF
!  RE-INSTATE OLD KP2,RTWO AND H
        KP2=KP2OLD
        RTWO=RTWOLD
        H=HOLD
        RTWOO=RTWOLD
        KP2O=KP2OLD
        RETURN
      ENDIF
!
! DETERMINE SAFE STARTING POINT FOR CORINT
!
      IF(IQDT.GT.0.AND.IPERT.NE.0.AND.INTPQ.EQ.0)THEN
        IF(ROMN.GE.RIMX)THEN
          RTWOC=ROMN
        ELSE
          RTWOC=RIMX
        ENDIF
!        IF(RONE.GT.2.0)RTWOC=RONE
        IF(RTWOC.GT.RTWO)RTWO=RTWOC
      ENDIF
!
!  FIND INTERVAL AND TABULAR POINTS BETWEEN RZERO AND RTWO
!
      IF(RTWO.LE.RZERO)THEN
        KP2=1
        H=TZERO
        RTWOO=RTWO
        RTWOC=RTWO
        KP2C=1
        RETURN
      ENDIF
!
      WM=TZERO
      DO 170 I=1,NCHOPT
        IF(IOMIT(I).GT.0)GO TO 170
        C=CCT(I)
        E=EPS(I)
        X=1.D0/RZERO
        W=ABS(E+X*(2.D0*TZED-C*X))
        IF(RINF(I).EQ.TZERO)W=W/16.D0
        IF(W.GT.WM)WM=W
        X=1.D0/RTWO
        WH=W
        W=ABS(E+X*(2.D0*TZED-C*X))
        IF(RINF(I).EQ.TZERO)W=W/16.D0
        IF(W.GT.WM)WM=W
        IF(W.GT.WH)WH=W
        IF(C.GT.RZERO.AND.C.LT.RTWO)THEN
          W=ABS(E+TZED/C)
          IF(RINF(I).EQ.TZERO)W=W/16.D0
          IF(W.GT.WM)WM=W
          IF(W.GT.WH)WH=W
        ENDIF
        H=ACNUM/SQRT(WH-TINY)
        IF(IPRINT.GT.2)WRITE(6,621)I,H
        TEMPH(I)=H
  170 CONTINUE
!
      H=ACNUM/SQRT(WM)
      IF(ABS(RTWO-RZERO).LT.(2.D0*H))THEN
        R2MAX=R2ST(1)
        IMAX=1
        IF(NCHPP.EQ.1) GO TO 190
        DO I=2,NCHPP
          IF(R2MAX.LT.R2ST(I)) THEN
            R2MAX=R2ST(I)
            IMAX=I
          END IF
        ENDDO
  190   RTWO=RZERO+2.01D0*H
        R2ST(IMAX)=RTWO
      ENDIF
!
      N=(RTWO-RZERO)/H
      N=4*((N-1)/4)+5
      KP2=N
      HO=H
      H=(RTWO-RZERO)/DBLE(KP2-1)
!
!  IF NOT ENOUGH OUTER-REGION POINTS, SEE IF WE CAN DROP DEEPLY CLOSED
!  CHANNELS (E.G. RMPS) AND THEREBY INCREASE H AND SO RETAIN ALL OPEN CHANNELS
!  LFM - NEVER? WITH ALLOCATIONS TO KP2 // KP2=4*((MZPTS-1)/4)+1
!        YES, SOMETIMES KP2 CAN BE REALLY HUGE, AND IT SHOULD BE RESIZED
!
      IF(KP2.GT.MZPTS)THEN
        IF(IPRINT.GT.0)WRITE(6,612)RTWO,KP2,H
  195   DO I=NCHOPT,NCHPP+1,-1
          IF(RINF(I).EQ.TZERO)THEN
            IF(TEMPH(I).LT.HO)THEN
              IF(IPRINT.GT.1)WRITE(6,613)I,TEMPH(I)
              TEMPH(I)=999.D0
              IOMIT(I)=1
              H=999.D0
              DO J=1,NCHF
                H=MIN(H,TEMPH(J))
              ENDDO
              N=(RTWO-RZERO)/H
              N=4*((N-1)/4)+5
              KP2=N
              HO=H-TINY
              H=(RTWO-RZERO)/DBLE(KP2-1)
              IF(KP2.GT.MZPTS)GO TO 195
              IF(IPRINT.GT.1)WRITE(6,614)H
              GO TO 196
            ENDIF
          ENDIF
        ENDDO
      ENDIF
!
!  CHECK DIMENSIONS FOR NUMBER OF OUTER-REGION POINTS
!
 196  KP2O=KP2
      RTWOO=RTWO
!
      IF(KP2.GT.MZPTS)THEN
        IPERTO=IPERT
        KP2=4*((MZPTS-1)/4)+1
        RTWO=(KP2-1)*H+RZERO
!
! MODIFICATION BY NRB 30/7/92 TO OVERCOME PROBLEM WITH SMALL NEGATIVE
! ENERGIES, NEGLECT C-C AND C-O CONTRIBUTION FROM RTWO TO INFINITY
! FOR AFFECTED (BOUND) CHANNEL RATHER THAN SWITCHING OFF PERTURBATION
! COMPLETELY. ALSO, STOP NOW IF DIMENSION PROBLEM ON OPEN CHANNEL.
! SEE ADDITIONAL CODING IN SR.ALPHA, SR.COUL, SR.NUMT.
! LFM  - NEVER, ARRAYS ARE ALLOCATED TO KP2
!        YES, SOMETIMES KP2 CAN BE REALLY HUGE, AND IT SHOULD BE RESIZED
!
        IF(NPERT.EQ.0)IPERT=0
!
        IF(IPERTO.NE.0)THEN
          IF(IPRINT.GT.0.AND.NPERT.EQ.1)WRITE(6,610)RTWOO,KP2O,RTWO,KP2
          IF(IPRINT.GT.0.AND.NPERT.EQ.0)WRITE(6,611)RTWOO,KP2O,RTWO,KP2
          IF(IQDT.LE.0)THEN
            DO I=1,NCHF
              IF(R2ST(I).GT.RTWO)THEN
                IF(RINF(I).GT.RZERO.AND.EPS(I).GT.0.1D0)
     X                         WRITE(6,619)I,EPS(I)
                IF(IPRINT.GT.0)WRITE(6,620)I,R2ST(I)
              ENDIF
            ENDDO
          ELSE
            DO I=1,NCHPP
              IF(R2ST(I).GT.RTWO)THEN
                IF(RINF(I).GT.RZERO.AND.EPS(I).GT.0.01D0)
     X                         WRITE(6,619)I,EPS(I)
                IF(IPRINT.GT.0)WRITE(6,620)I,R2ST(I)
              ENDIF
            ENDDO
            DO I=NCHPP+1,NCHF
              IF(RINF(I).GT.RTWO)THEN
                IF(IPRINT.GT.0)WRITE(6,620)I,R2ST(I)
              ENDIF
            ENDDO
          ENDIF
        ENDIF
      ENDIF
!
! SET-UP CORINT POINTS
!
      IF(IQDT.GT.0.AND.IPERT.NE.0.AND.INTPQ.EQ.0)THEN
!        RTWOC=RZERO
        N=(RTWOC-RZERO)/H+1.D-6
        IF(N.LT.2)THEN
          RTWOC=RZERO
          KP2C=1
        ELSE
          KP2C=4*((N-1)/4)+5
          IF(KP2C.GT.KP2)KP2C=KP2
          RTWOC=(KP2C-1)*H+RZERO
        ENDIF
!        WRITE(77,*)'KP2C=',KP2C,'  RTWOC=',RTWOC,' RTWO=',RTWO
      ELSE
        KP2C=KP2
      ENDIF
!
      DEALLOCATE(TEMPH)
!
!  WEIGHTS FOR BODE RULE INTEGRATION
!
      CALL BODE(H,KP2C,IPERT,RZERO)
!
!
  600 FORMAT(2X,' **WARNING** FOR ETOT = ',
     1 F10.5,' RTWO REDUCED TO MAXIMUM VALUE OF',F7.2
     + /10X,' NO MORE SIMILAR WARNINGS GIVEN'/)
  610 FORMAT(/10X,'USE OF PERTURBATION REQUIRES RTWO = ',
     +  F8.2,', KP2 = ',I5,/10X,'WHICH IS LARGER THAN MAXIMUM OF ',
     +  'MZPTS ALLOWED BY DIMENSIONS'/10X,'NEGLECT CLOSED-CLOSED'
     X,' CLOSED-OPEN AND OPEN-OPEN CONTRIBUTION FROM R .GT.'
     X,' RTWO = ',F8.2,'  KP2 = ',I5/10X,'(FROM R .GT. RZERO'
     X,' IF RINF GT. RZERO)')
  611 FORMAT(/10X,'USE OF PERTURBATION REQUIRES RTWO = ',
     +  F8.2,', KP2 = ',I5,/10X,'WHICH IS LARGER THAN MAXIMUM OF ',
     +  'MZPTS ALLOWED BY DIMENSIONS'/10X,'SET IPERT = 0'
     X,' RTWO = ',F8.2,'  KP2 = ',I5)
  612 FORMAT(/10X,'USE OF PERTURBATION REQUIRES RTWO = ',
     +  F8.2,', KP2 = ',I5,/10X,'WHICH IS LARGER THAN MAXIMUM OF ',
     +  'MZPTS ALLOWED BY DIMENSIONS'/10X,'ATTEMPTING TO DROP',
     X ' DEEPLY-CLOSED CHANNELS SO AS TO INCREASE'/10X,
     X 'THE STEP LENGTH, CURRENTLY H=',F8.5)
  613 FORMAT(/10X,'DROP CHANNEL I =',I5,3X,' STEP =',F8.5)
  614 FORMAT(/10X,'NEW STEP H =',F8.5/)
  619 FORMAT(10X,'CHANNEL ',I3,' EPS = ',1PE11.2,
     + ' OUTER REGION CONTRIBUTION DROPPED')
  620 FORMAT(10X,'CHANNEL ',I3,' REQUIRES RTWO = ',F8.2,
     + ' FOR USE OF PERTURBATION')
  621 FORMAT(10X,'CHANNEL ',I3,' REQUIRES H = ',F11.6,
     + ' FOR USE OF PERTURBATION')
  630 FORMAT(/1X,5('*'),' QDT USED'/7X,'NCHPP = ',I3/
     +  7X,'NQ    = ',I3/)
!
      RETURN
      END
!***************************************************************
!
      SUBROUTINE PQ(LFAIL)
      USE CHAN
      USE NRBRCT
      USE NRBWRK
!
! NRB:
!  CALCULATE MATRICES P AND Q,
!    TRANSMISSION MATRIX  IS -2*I*(P+I*Q), I=SQRT(-1)
!    NOTE: BLAS MATRIX MULTIPLICATIONS ARE SLOWER THAN NON-BLAS
!          BECAUSE THE LATTER ONLY COMPUTES HALF THE PRODUCT.
!          ALSO, MATRIX INVERSION IS FASTER THAN AX=B BECAUSE
!          Q = 1- INVERSE A, AND WE ONLY DO HALF THE PRODUCT FOR P.
!
      IMPLICIT REAL*8 (A-H,O-Y)
      IMPLICIT COMPLEX*16 (Z)
      LOGICAL,INTENT(OUT) :: LFAIL
!      PARAMETER (LWORK=MZCHF*MZCHF)
!      PARAMETER (MWORK=MZDEG*MZDEG)
      PARAMETER (ONE=1.0)
      PARAMETER (TZERO=0.0)
!
      INTEGER,ALLOCATABLE :: IPIV(:)
!
      LFAIL=.FALSE.
      ALLOCATE(IPIV(NCHF))
      P=>CSP
      Q=>CC

!  A=(1+RK**2)**(-1)
!
!STRTNBL
!NBL  DO J=1,NCHPP
!NBL    DO I=1,J
!NBL      A(I,J)=TZERO
!NBL    ENDDO
!NBL    DO K=1,NCHPP
!NBL      DO I=1,J
!NBL        A(I,J)=A(I,J)+RK(I,K)*RK(K,J)
!NBL      ENDDO
!NBL    ENDDO
!NBL  ENDDO
!ENDNBL
!STRTBL
      CALL DSYMM('L','U',NCHPP,NCHPP,ONE,RK,NCHF,RK,NCHF,
     X  TZERO,A,NCHF)
!ENDBL
!
      DO J=1,NCHPP
        DO I=1,J
          A(J,I)=A(I,J)
        ENDDO
      ENDDO
      DO I=1,NCHPP
        A(I,I)=A(I,I)+ONE
      ENDDO
!
!STRTNBL
!NBL  CALL VERTS(A,NCHF,NCHPP,WORK,IERR)
!NBL  IF (IERR.NE.0) THEN
!NBL    WRITE(6,100)
!NBL      LFAIL=.TRUE.
!NBL      RETURN
!!NBL     STOP 'STOP BECAUSE NO INVERSE FOUND IN SR.PQ'
!NBL  END IF
!ENDNBL
!
!STRTBL
      LWORK=NCHF*NCHF
      ALLOCATE(WORK(LWORK))
      CALL DSYTRF('L',NCHPP,A,NCHF,IPIV,WORK,LWORK,INFO)
      IF (INFO.NE.0) THEN
         WRITE(6,602) INFO
         LFAIL=.TRUE.
         DEALLOCATE(WORK,IPIV)
         RETURN
!        STOP 'FAILURE IN BLAS ROUTINE DSYTRF'
      ENDIF
      CALL DSYTRI('L',NCHPP,A,NCHF,IPIV,WORK,INFO)
      IF (INFO.NE.0) THEN
         WRITE(6,603) INFO
         LFAIL=.TRUE.
         DEALLOCATE(WORK,IPIV)
         RETURN
!        STOP 'FAILURE IN BLAS ROUTINE DSYTRI'
      ENDIF
!ENDBL
!
      DO J=1,NCHPP
        DO I=J,NCHPP
          A(J,I)=A(I,J)
        ENDDO
      ENDDO
!
!  P=RK*A
!
!STRTNBL
!NBL  DO J=1,NCHPP
!NBL    DO I=1,J
!NBL      P(I,J)=TZERO
!NBL    ENDDO
!NBL    DO K=1,NCHPP
!NBL      DO I=1,J
!NBL        P(I,J)=P(I,J)+RK(I,K)*A(K,J)
!NBL      ENDDO
!NBL    ENDDO
!NBL  ENDDO
!NBL  DO J=1,NCHPP
!NBL    DO I=1,J
!NBL      P(J,I)=P(I,J)
!NBL    ENDDO
!NBL  ENDDO
!ENDNBL
!STRTBL
      CALL DSYMM('L','U',NCHPP,NCHPP,ONE,RK,NCHF,A,NCHF,
     X  TZERO,P,NCHF)
!ENDBL
!
!  Q=1-A (=RK*P)
!
! NRB: REMOVED UNNECESSARY MATRIX MULTIPLICATION
!
      DO J=1,NCHPP
        DO I=1,NCHPP
          Q(I,J)=-A(I,J)
        ENDDO
      ENDDO
      DO I=1,NCHPP
        Q(I,I)=Q(I,I)+ONE
      ENDDO
!
!      NULLIFY(P,Q)
      DEALLOCATE(WORK,IPIV) 
!
      RETURN
!
  100   FORMAT(' THE MATRIX: 1 + K^2 HAS NO INVERSE IN SUBROUTINE '
     1  ,'PQ - MUST STOP')
  602 FORMAT(//10X,10('*'),' SR.PQ: DSYTRF RETURNED WITH INFO = ',I4)
  603 FORMAT(//10X,10('*'),' SR.PQ: DSYTRI RETURNED WITH INFO = ',I4)
!
      END
!***************************************************************
!
      SUBROUTINE PRUNE(ABVTHR,BELTHR)
      USE WMATALLOC
      USE CEN
      USE CINPUT
      USE CINPTX
      USE CNTRL
      USE CMESH
!
!NRB
! PRUNE UNDESIREABLE ENERGIES
!
      IMPLICIT REAL*8 (A-H,O-Z)
      REAL*8,ALLOCATABLE :: EMESH0(:)
!
      IF(ABVTHR.LT.0.OR.BELTHR.LT.0.)RETURN
!
! HOLD
      MXE0=MXE
      ALLOCATE(EMESH0(MXE0))
!
      IT0=1
      DO IE=1,MXE0
        IFLAGE=0
        EMESH0(IE)=EMESH(IE)
        DO 2 IT=IT0,NAST
           IF(EMESH(IE).LT.ENAT(IT)-BELTHR)GO TO 1
           IF(EMESH(IE).GT.ENAT(IT)+ABVTHR)GO TO 2
           IFLAGE=1
   2    CONTINUE
   1    IF(IFLAGE.NE.0)EMESH0(IE)=-EMESH(IE)
        IT0=IT-1
      ENDDO
!      
      MXE=0
      IF(IPRINT.GE.0)THEN
        WRITE(6,*)' '
        WRITE(6,*)'ORIGINAL MESH POINTS DROPPED:'
      ENDIF
      ID=0
      DO IE=1,MXE0
        IF(EMESH0(IE).GT.0.)THEN
          MXE=MXE+1
          EMESH(MXE)=EMESH0(IE)
        ELSE
          IF(IPRINT.GE.0)THEN
            ID=ID+1
            WRITE(6,100)ID,IE,-EMESH0(IE)
          ENDIF
        ENDIF
      ENDDO
!
! Reallocate EMESH, drop EMESH0
!
      EMESH0(1:MXE)=EMESH(1:MXE)
      DEALLOCATE(EMESH)
      ALLOCATE(EMESH(MXE))
      EMESH(1:MXE)=EMESH0(1:MXE)
      DEALLOCATE(EMESH0)
!
      RETURN
 100  FORMAT(2I6,1PE16.8)
      END
!
!***************************************************************
!
      SUBROUTINE RAD
      USE WMATALLOC
      USE COMEGA
      USE CINPUT
      USE CINPTX
      USE CHAN
      USE CDEC
      USE NRBSKP
      USE CNTRL
!
! NRB: EXTENDED TO JK- AND JJ-COUPLING.
!
      IMPLICIT REAL*8 (A-H,O-Z)
!
!      PARAMETER (MXTST=(MZTAR*MZTAR+MZTAR)/2)
      PARAMETER (TZERO=0.0)
      PARAMETER (ONE=1.0)
      PARAMETER (TWO=2.0)
      PARAMETER (THREE=3.0)
      PARAMETER (FOUR=4.0)
      PARAMETER (SIX=6.0)
      PARAMETER (EINF=1.0D6)
      PARAMETER (CON1=.007297353)
      PARAMETER (CON2=4.134D16)
      PARAMETER (TINY=1.D-6)
!
      AZ=MAX(NZED-NELC,1)
      IONE=1
      IF(ELAS.EQ.'YES')IONE=0
      PI=ACOS(-ONE)
!
!  RADIATIVE PROBABILITIES
!
      K=0
      IC0=0
      IF(IONE.EQ.1)IC0=NCONAT(1)
!
      DO I=1+IONE,NAST
        JC0=0
        DO 130 J=1,I-IONE
          K=K+1
          IF(NOMWRT.GE.0)KS=I+1-IONE+(NAST+1-IONE)*(J-1)-(J*(J+1))/2
          IF(NOMWRT.LT.0)KS=K
          IF(I.EQ.J)THEN
            ARAD(K)=TZERO
            SLIN(KS)=TZERO
            GO TO 130
          ENDIF
          IF(NSPN2.NE.0)THEN
            IF(IABS(ISAT(I)).NE.IABS(ISAT(J)).OR.
     X        LAT(I)+LAT(J).EQ.0.OR.ABS(LAT(I)-LAT(J)).GT.1)THEN
              ARAD(K)=TZERO
              SLIN(KS)=TZERO
            ENDIF
          ENDIF
          IF(ARAD(K).GE.TZERO)GOTO 30
          DO IC=IC0+1,IC0+NCONAT(I)
            DO JC=JC0+1,JC0+NCONAT(J)
              IF(ABS(CF(IC,JC,1)).LT.TINY)THEN
         	ARAD(K)=TZERO
         	SLIN(KS)=TZERO
              ELSE
               IF(NSPN2.NE.0)THEN    !LS
         	SLIN(KS)=DBLE(2*LRGL2+1)*(CF(IC,JC,1)**2)
     +   	/(WSQ(LAT(I),LAT(J),LLCH(IC),LLCH(JC),LRGL2,ISIGN)
     +   	*DBLE(2*LAT(I)+1)*MAX(LLCH(IC),LLCH(JC)))
         	WGT=DBLE(IABS(ISAT(I))*(2*LAT(I)+1))
         	ISLN=(-1)**(LAT(I)+LLCH(IC)+LRGL2+1)
         	IF(ISIGN.LT.0)ISLN=-ISLN
         	IF(CF(IC,JC,1).LT.TZERO)ISLN=-ISLN
         	SLIN(KS)=SLIN(KS)*ISLN
               ELSE				   !JK OR JJ
         	 IF(LAT(I)+LAT(J).EQ.0.OR.ABS(LAT(I)-LAT(J)).GT.2)THEN
         	   SLIN(KS)=TZERO
         	 ELSE
         	   IF(KFLAG.GE.0.AND.KJ(IC).EQ.KJ(JC))THEN !JK-NRB
         	     SLIN(KS)=DBLE(KJ(IC)+1)*(CF(IC,JC,1)**2)
     +   	    /(WSQ2(LAT(I),LAT(J),2*LLCH(IC),2*LLCH(JC),KJ(IC),
     +   	      ISIGN)*DBLE(LAT(I)+1)*MAX(LLCH(IC),LLCH(JC)))
         	     WGT=DBLE(LAT(I)+1)
         	   ELSEIF(KFLAG.LT.0)THEN      !JJ-NRB
         	     IC2=2*(L2P(IC)/2)+1       !2J-VALENCE
         	     JC2=2*(L2P(JC)/2)+1       !2J-VALENCE
                     IF(IC2.EQ.JC2)THEN
                       W3J=DBLE(IC2+1)/(DBLE((IC2+2)*IC2))
                     ELSE
                       IC3=MIN(IC2,JC2)
                       W3J=DBLE((IC3+3)*(IC3+1))/DBLE(2*(IC3+2))
                     ENDIF
!                     WRITE(6,*)LLCH(IC),LLCH(JC),IC2,JC2,W3J
         	     SLIN(KS)=DBLE(LRGL2+1)*(CF(IC,JC,1)**2)
     +   	    /(WSQ2(LAT(I),LAT(J),IC2,JC2,LRGL2,ISIGN)*
     +                W3J*DBLE(LAT(I)+1))
         	     WGT=DBLE(LAT(I)+1)
         	   ENDIF
         	 ENDIF
               ENDIF
!
! ENER   	 GIES ARE UNSCALED ATOMIC (NOT RYDBERGS) UNITS HERE.
!
               ARAD(K)=((CON1*(ENAT(I)-ENAT(J)))**3)*ABS(SLIN(KS))
               SLIN(KS)=SLIN(KS)*WGT*LOG(EINF*AZ**2)
               GOTO 30
              ENDIF
            ENDDO
          ENDDO
   30     JC0=JC0+NCONAT(J)
  130   ENDDO
        IC0=IC0+NCONAT(I)
      ENDDO
!
!  CHECK COMPLETENESS OF ARAD
!
      DO K=1,(NAST*(NAST-2*IONE+1))/2
        IF(ARAD(K).LT.TZERO)THEN
          IEND=0
          RETURN
        ENDIF
      ENDDO
      IEND=1
!
!  CALCULATE ARDEC= = 2*PI*ARAD/Z**2
!
      C=TWO*PI/AZ**2
      K=0
      DO I=1+IONE,NAST
        A=TZERO
        DO J=1,I-IONE
          K=K+1
          A=A+ARAD(K)
        ENDDO
        ARDEC(I)=A*C
      ENDDO
!
!  WRITES
!
      IF(IRDEC.GT.0)WRITE(6,600)
      WRITE(6,605)(I,ISAT(I),LAT(I),IPAR(I),I=1,NAST)
      WRITE(6,610)
!
      K=0
      DO I=1+IONE,NAST
        DO J=1,I-IONE
          K=K+1
          IF(NOMWRT.GE.0)KS=I+1-IONE+(NAST+1-IONE)*(J-1)-(J*(J+1))/2
          IF(NOMWRT.LT.0)KS=K
          IF(ARAD(K).GT.TZERO)THEN
            S=SLIN(KS)*THREE/(LOG(EINF*AZ**2)*FOUR)
            GF=TWO*S*(ENAT(I)-ENAT(J))/THREE
            WRITE(6,620)I,J,ARAD(K),ARAD(K)*CON2,S,GF
          ENDIF
        ENDDO
      ENDDO
!
      WRITE(6,630)(I,ARDEC(I),I=2,NAST)
      WRITE(6,640)
!
      RETURN
!
!  FORMATS
  600 FORMAT(//72('+')//2X,'*** OMEGA AND/OR OMEGDR'
     + ,' CALCULATED ALLOWING FOR RADIATIVE DECAYS ***')
  605 FORMAT(//72('+')//
     + 25X,'TARGET STATES'//12X,'INDEX',5X,'2*S+1',7X,
     + 'L',8X,'PARITY'/20X,'OR P      OR 2*J'
     +//(I15,3I10))
  610 FORMAT(//10X,'RADIATIVE PROBABILITIES (ATOMIC UNITS,'
     + ,' NOT Z-SCALED)'//14X,'I',9X,'J',13X,'A(I,J)',15X,'*SEC'
     X ,18X,'S',14X,'GF'/)
  620 FORMAT(I15,I10,1P2E21.4,0PF20.6,F15.6)
  630 FORMAT(///14X,'I',11X,'ARDEC(I)'//(I15,1PE20.4))
  640 FORMAT(//72('+')//)
!
      END
!***************************************************************
!
      SUBROUTINE REACT(IOPT1,QJUMP,PQRD,LFAIL)
      USE WMATALLOC
      USE NRBRCT
      USE NRBCBE
      USE NRBKHI
      USE CALP
      USE COULSC
      USE CPOT
      USE MEMORY
      USE CTOP
      USE NRBTOP
      USE NRBDR
      USE COMEGA
      USE CEN
      USE CINPUT
      USE CINPTX
      USE CHAN
      USE CDEC
      USE NRBHYB
      USE NRBQDT
      USE NRBSKP
      USE NRBOMT
      USE CACC
      USE CPOINT
      USE CNTRLS
      USE CNTRL
      USE CQDT
      USE CLOGB
!
! NRB: HEAVILY REWORKED
!  CALCULATION OF REACTANCE MATRIX.
!
      IMPLICIT REAL*8 (A-H,O-Y)
      IMPLICIT COMPLEX*16 (Z)
!
      LOGICAL QJUMP,PQRD
      LOGICAL,INTENT(OUT) :: LFAIL
!
!      PARAMETER (MZKIL=  0)
!      PARAMETER (MXTST=(MZTAR*MZTAR+MZTAR)/2)
!      PARAMETER (MXOM=MZMEG*1000000+MZKIL*1000)
!      PARAMETER (MXF=(MZLMX+1)/2)
!
      PARAMETER (ONE=1.0)
      PARAMETER (TZERO=0.0)
      PARAMETER (TWO=2.0)
      PARAMETER (FOUR=4.0)
      PARAMETER (QUART=0.25)
!
      INTEGER, ALLOCATABLE :: INOUT(:)
      REAL(8), ALLOCATABLE :: SIGPW(:)
!
       IF(ALLOCATED(RBE))DEALLOCATE(RBE)
       ALLOCATE(RBE(NCHF,NCHF))
!
       Y=>RMAT
       P=>CSP
       Q=>CC
       POLD=>DSP
       QOLD=>DC
!
      LFAIL=.FALSE.
      CONSIG=87.97351
      AZ=MAX(NZED-NELC,1)
      AZAZ=AZ*AZ
!
!
! IN IQDT MODE SEE IF WE CAN CUT TO THE CHASE
!
      IF(IQDT.GT.0)THEN
        IF(QJUMP)THEN
          IF(NCHPP.EQ.0)RETURN
          IPERTO=IPERT
          NCHOPO=NCHPP
          NCHPP=NCHF
          IF(IQDT.EQ.2)GO TO 2000
          GO TO 1000
        ELSE
!OLD          IEE(IE)=IE
        ENDIF
      ENDIF
!
      IF(LRGL2.GT.ISKP(IE).AND.IPRKM.GT.0)THEN
        IF(IQDT.GT.0)THEN
          NCHOPO=NCHPP
          NCHPP=NCHF
        ENDIF
        DO J=1,NCHPP
          DO I=1,NCHPP
            RK(I,J)=TZERO
          ENDDO
        ENDDO
        GO TO 3000
      ENDIF
!
!
! CALCULATE INITIAL R-MATRIX
!
!     DEALLOCATE AS MANY COULOMB FNS AS POSSIBLE
!     BEFORE RMATRIX FORMATION, EVERY TIME !!!
!
!****************************************************
!
      IF(ALLOCATED(ASS)) DEALLOCATE(ASS)
      IF(ALLOCATED(ASC)) DEALLOCATE(ASC)
      IF(ALLOCATED(ACS)) DEALLOCATE(ACS)
      IF(ALLOCATED(ACC)) DEALLOCATE(ACC)
!
      CALL DEALLOCATE_NRBRCT()   ! ensures previous deallocates
      IF(ALLOCATED(FS)) DEALLOCATE(FS,FC)
!
!****************************************************
!
       NULLIFY(Y,P,Q,POLD,QOLD)
!
!! START CUDA
!CU     IF (NCHAN.GE.512) THEN
!CU       CALL RINIT_CUDA
!CU     ELSE
!CU       CALL RINIT
!CU     END IF
!! END CUDA
! START BLAS
       IF(NCHF.LT.2000)THEN
         CALL RINIT            ! TOO SMALL TO CARE 1 DGEMM
!      ELSE ! IF(NCHF.LT.4000)THEN
!        CALL RINIT2           ! DISK THRASH *3
       ELSE ! IF(NCHF.GE.4000)THEN
         CALL RINIT4          ! POINTER *3 DGEMM
       ENDIF
! END BLAS
!
! COULOMB FUNCTIONS AT RZERO
!
!       S(I) IS REGULAR FUNCTION FOR OPEN CHANNELS, DECAYING
!       (REAL) FUNCTION THETA FOR CLOSED CHANNELS.
!       C(I) IS IRREGULAR FUNCTION FOR OPEN CHANNELS,
!       THETAD = ENERGY DERIVATIVE OF DECAYING FUNCTIONS FOR
!       CLOSED CHANNELS.
!       IN IQDT MODE ALL CHANNELS ARE TREATED AS OPEN.
!
       CALL ALLOCATE_NRBRCT(NCHF)  ! DANGER IMPLICIT DEALLOCATE
       ALLOCATE(FS(KP2,NCHF),FC(KP2,NCHF))

       NULLIFY(Y,P,Q,POLD,QOLD)  ! RE-ATTACH POINTERS 
       IF(ASSOCIATED(TEMP3)) NULLIFY(TEMP3)

       Y=>RMAT
       P=>CSP
       Q=>CC
       POLD=>DSP
       QOLD=>DC
       ALLOCATE(INOUT(NCHF))

!      WRITE(0,*)'BEFORE COUL',IPERT
       CALL COUL(INOUT)
!      WRITE(0,*)'AFTER COUL',IPERT
!
      DO I=NCHPP+1,NCHF
        IF(FKNU(I).LT.FNUMIN)IOMIT(I)=1
      ENDDO
!
      NCHOPO=NCHPP
!
! RESET IPERT AND NCHPP FOR QDT CASE

      IF(QDT.OR.IQDT.NE.0) THEN
        IPERTO=IPERT
        IF(IQDT.GT.0)THEN
          NCHPP=NCHF
        ELSE
! NRB: IPERT SHOULD ALREADY HAVE BEEN ZEROED BY POINTS, BUT CHECK
          IF(IPERT.NE.0)STOP 'REACT: IPERT???'
          IPERT=0
          NCHPP=NQ
        ENDIF
      ENDIF
      IF(NCHPP.EQ.0) THEN
        IF(IRAD.NE.0)THEN
          WRITE(9) QDT
          WRITE(9) NCHPP
        ENDIF
        ISTOT=NSPN2
        IF(NPTY2.EQ.1) ISTOT=-ISTOT
        ETRYD=AZAZ*ETOT
        IF(IPRKM.EQ.1)WRITE(20,660) NCHPP,ISTOT,LRGL2,ETRYD
        NULLIFY(P,Q,POLD,QOLD,Y)
        RETURN
      ENDIF
!
! CALCULATE REACTANCE MATRIX
!
      AR=ONE/RZERO
      DO I=1,NCHF
        IF(IOMIT(I).LE.0)THEN
          IF(IQDT.EQ.0.OR.INTPQ.EQ.0)THEN
            S(I)=FS(1,I)
            SP(I)=FSP(I)-BSTO*S(I)
          ELSE
            SP(I)=SP(I)-BSTO*S(I)
          ENDIF
          HAM=AR*(AR*CCT(I)-TWO)-EPS(I)
          SPP(I)=HAM*S(I)
!OLD          IF(I.LT.NCHPP1.OR.IPERT.GT.0) THEN
          IF(IQDT.EQ.0.OR.INTPQ.EQ.0)THEN
            C(I)=FC(1,I)
            CP(I)=FCP(I)-BSTO*C(I)
          ELSE
            CP(I)=CP(I)-BSTO*C(I)
          ENDIF
          CPP(I)=HAM*C(I)
        ELSE
          S(I)=TZERO
          SP(I)=TZERO
          SPP(I)=TZERO
          C(I)=TZERO
          CP(I)=TZERO
          CPP(I)=TZERO
          DO J=1,NCHF
            RMAT(I,J)=TZERO
          ENDDO
        ENDIF
      ENDDO
!
! EVALUATE PERTURBATION INTEGRALS (ALL IPERT)
!
      IF(IPERT.NE.0)CALL ALPHA(NCHOPO)
!
!..CASE OF IPERT.GT.0 (PERTURB S & C)
!
      IF(IPERT.GT.0)THEN
!
        CALL PETFSC
!
      ELSE
!
!..CASE OF IPERT.LE.0
!
! CALCULATE MATRICES A AND B
        DO J=1,NCHPP
          DO I=1,NCHF
            A(I,J)=-RMAT(I,J)*CP(J)
            B(I,J)=-RMAT(I,J)*SP(J)
          ENDDO
        ENDDO
        DO I=1,NCHPP
          A(I,I)=A(I,I)+C(I)
          B(I,I)=B(I,I)+S(I)
        ENDDO
        DO J=NCHPP1,NCHF
          DO I=1,NCHF
            A(I,J)=-RMAT(I,J)*SP(J)
          ENDDO
        ENDDO
        DO I=NCHPP1,NCHF
          A(I,I)=A(I,I)+S(I)
        ENDDO
        IF(NCHHYB.GT.0)THEN
          DO N=1,NCHHYB
            J=ICHHYB(N)
            DO I=1,NCHF
              A(I,J)=-RMAT(I,J)*SP(J)
              B(I,J)=TZERO
            ENDDO
            A(J,J)=A(J,J)+S(J)
          ENDDO
        ENDIF
!
      ENDIF
!.....
!
! COMPLETE CALCULATION OF REACTANCE MATRIX
!
      CALL AINVB(LFAIL)
      IF (LFAIL) THEN
        DEALLOCATE(INOUT)
        RETURN
      END IF
!
! FOR CASE OF IRAD.NE.0, EVALUATE PERTURBATIONS TO RADIATIVE
! DATA AND WRITE REACTANCE MATRIX AND FUNCTIONS TO UNIT 9
! NOTE: COULD CALL PETRAD AFTER CALL PETKMX BUT LITTLE TO BE
! GAINED. INSTEAD WE FORCE IPERT>0 (DEFAULT) WHEN IRAD>0.
! RAD.NE.0 NOT IMPLEMENTED
!
      IF(IRAD.GT.0.AND.IOPT1.LT.10) THEN
        CALL PETRAD(INOUT)
        IF (IRAD.EQ.2) THEN
          DEALLOCATE(INOUT)
          RETURN
        END IF
      ENDIF
      DEALLOCATE(INOUT)
!
!
! USE VARIATIONAL METHOD FOR IPERT<0 (PERTURB K-MX [-2] OR T-MX [-1])
! EITHER WAY WE NEED THE PERTURBATION (Y) TO THE K-MX FIRST, SO
!
!      WRITE(0,*)'BEFORE PETKMX',IPERT
      IF(IPERT.LT.0)CALL PETKMX
!
! PERTURB K-MATRIX.
!
      IF(IPERT.LE.-2)THEN
        DO J=1,NCHPP
          DO I=1,NCHPP
            RK(I,J)=RK(I,J)+Y(I,J)
          ENDDO
        ENDDO
      ENDIF
!
! OPTIONALLY GO VIA K-UNPHYS->K-PHYS
!     (FIRST PUT RK IN P AND Q, THEN VARIOUS WRITES....)
!
 3000 IF(ABS(IQDT).EQ.2)THEN
!        IF(ALLOCATED(PPP))DEALLOCATE(PPP)
!        IF(ALLOCATED(QQQ))DEALLOCATE(QQQ)
!        ALLOCATE(PPP(NCHPP,NCHPP))
!        ALLOCATE(QQQ(NCHPP,NCHPP))
!        P=>PPP
!        Q=>QQQ

        DO J=1,NCHPP
          DO I=1,NCHPP
            P(I,J)=RK(I,J)
            Q(I,J)=TZERO
          ENDDO
        ENDDO
        IF(IQDT.EQ.2.AND..NOT.PQRD.AND.IPRKM.EQ.0.AND.IJBIN.NE.0)THEN
          MPTY=IABS(NSPN2)*10000+100*LRGL2+NPTY2
          IF(NSPN2.LT.0)MPTY=-MPTY
          WRITE(21) IE,MPTY,NCHPP,ETOT
          WRITE(21) (IOMIT(I),I=1,NCHPP)
          WRITE(21) ((P(I,J),I=J,NCHPP),J=1,NCHPP),
     X              ((Q(I,J),I=J,NCHPP),J=1,NCHPP)
        ENDIF
! NEVER
!        IF(IPRKM.EQ.2)THEN
!          WRITE(36)ETOT,((P(I,J),J=1,I),I=1,NCHPP) !LOWER HALF
!        ENDIF
        IF(IPRKM.EQ.4)THEN
          WRITE(32)((P(I,J),I=1,J),J=1,NCHPP)  !UPPER HALF
	  IF(IE.EQ.1)WRITE(6,999)IABS(NSPN2),LRGL2,NPTY2
  999       FORMAT(/72('*')/3X,'WROTE DATA TO KMTLS.DAT FOR NSPN2 = ',I3
     1      ,3X,'LRGL2 = ',I3,3X,'NPTY2 = ',I3/72('*'))
        END IF
        IF(IMODE.LT.0.OR.LRGL2.GT.ISKP(IE).OR.NCHOPO.EQ.0
     X     .OR.NOMWRT.EQ.0)GO TO 666
      ENDIF
!
 2000 IF(ABS(IQDT).EQ.2)THEN
        NCHPP=NCHOPO
        IF(QDT)NCHPP=NQ
        NCHPP1=NCHPP+1
!
        IF(NCHPP.LT.NCHF)THEN
!
          CALL MQDTK(LFAIL)
          IF(LFAIL) RETURN
!
        ELSE
          DO J=1,NCHF
            DO I=1,NCHF
              RK(I,J)=P(I,J)
              POLD(I,J)=P(I,J)
              QOLD(I,J)=Q(I,J)
            ENDDO
          ENDDO
        ENDIF
!
      ENDIF
!
! OPTIONALLY GET EIGENPHASE SUM
!
      IF(IPRINT.GT.0)CALL EPHASE
!
! GET NEW P,Q MATRICES (ALL CASES OF IPERT/IQDT)
! SUCH THAT TRANSMISSION MATRIX  IS -2*I*(P+I*Q), I=SQRT(-1)
!
      CALL PQ(LFAIL)
      IF(LFAIL) RETURN
!
! NOW, PERTURB (IN EFFECT) T-MATRIX, ALTHOUGH I CAN'T SEE WHY-NRB.
!
      IF(IPERT.EQ.-1)CALL PETTMX
!
!
! WRITE REACTANCE MATRIX (SO IPERT=-1 IS THE UNPERTURBED MATRIX)
!
      IF(IPRINT.GT.0)THEN
        WRITE(6,707)ETOT,(J,J=1,NCHPP)
        WRITE(6,*)
        DO I=1,NCHPP
          WRITE(6,710)I,(RK(I,J),J=1,NCHPP)
        ENDDO
        IF(IQDT.NE.2.AND.(IRAD.GT.0.OR.IPERT.LT.0))THEN
          WRITE(6,*)
          DO I=NCHPP1,NCHF
            WRITE(6,710)I,(RK(I,J),J=1,NCHPP)
          ENDDO
        ENDIF
      ENDIF
!
! WRITE K-MATRICES TO DIFFERENTIAL CROSS SECTION CODE
!
      IF(IPRKM.NE.0) THEN
        ISTOT=NSPN2
        IF(NPTY2.EQ.1) ISTOT=-ISTOT
        ETRYD=AZAZ*ETOT
      END IF
      IF(IPRKM.EQ.1) THEN
        WRITE(20,660) NCHPP,ISTOT,LRGL2,ETRYD
        IF(NSPN2.NE.0) THEN
	  DO I=1,NCHPP
	    WRITE(20,670) ITARG(I),LLCH(I)
	  ENDDO
        ELSE
	  DO I=1,NCHPP
	    WRITE(20,671) ITARG(I),LLCH(I),KJ(I)
	  END DO
        END IF  
        DO I=1,NCHPP
          DO IP=1,NCHPP
            WRITE(20,680) RK(I,IP)
          ENDDO
        ENDDO
      END IF
!
!TBD: WRITE PARTIAL WAVE LIST TO A SEQUENTIAL FILE AND THE K-MATRICES
!     TO A DIRECT ACCESS FILE FOR USE BY THE PARTIAL CROSS SECTION CODE
      IF(IPRKM.EQ.3) STOP 'SR.REACT: IPRKM=3 NOT ACTIVE YET'
!
!
! NOW P AND Q SUCH THAT KHI=P+I*Q (=1-T)
!
        DO J=1,NCHPP
          DO I=1,NCHPP
            T=P(I,J)
            P(I,J)=-TWO*Q(I,J)
            Q(I,J)=TWO*T
            IF(I.LE.NCHOPO.AND.J.LE.NCHOPO)THEN
              TT=P(I,J)**2+Q(I,J)**2
              IF (I.NE.J .AND. TT.GT.ONE) THEN
                WRITE(6,695)ETOT,IPERT,I,J,TT
                LFAIL=.TRUE.
                RETURN
              END IF
              IF (I.EQ.J .AND. TT.GT.FOUR) THEN 
                WRITE(6,695)ETOT,IPERT,I,J,TT
                LFAIL=.TRUE.
                RETURN
              END IF
            ENDIF
          ENDDO
        ENDDO
        DO I=1,NCHPP
          P(I,I)=ONE+P(I,I)
        ENDDO
!
! OUTPUT KHI MATRICES FOR JAJOM (KAB,94)
!
      IF(IOPT1.GT.9.OR.(IQDT.GT.0.AND.IQDT.EQ.1.AND..NOT.PQRD)) THEN
        IF(NSPN2.EQ.0.AND.IOPT1.GT.9)THEN
          WRITE(6,*)'***ERROR: IOPT1>9 REQUESTS (LS) OUTPUT FOR JAJOM'
     X   , ' BUT NSPN2=0. I.E. B.P. DATA BEING PROCESSED!'
          WRITE(6,*)' DELETE RECUPH.DAT AND RE-RUN STG3'
          STOP
        ENDIF
        IF(IJBIN.NE.0) THEN        
          IF(IOPT1.GT.9)THEN
! FOR JAJOM CONSISTENCY
            MPTY=NSPN2*1000+LRGL2
            IF(NPTY2.EQ.1) MPTY=-MPTY
            KNUM=(NCHPP*(NCHPP+1))/2
            WRITE(21) IE,MPTY,KNUM,ETOT
            WRITE(21) ((P(I,J),I=J,NCHPP),J=1,NCHPP),
     X                ((Q(I,J),I=J,NCHPP),J=1,NCHPP)
            IF(IOPT1.EQ.11)RETURN
          ELSEIF(IPRKM.EQ.0)THEN
! LS, BP, NX ETC.
            MPTY=IABS(NSPN2)*10000+100*LRGL2+NPTY2
            IF(NSPN2.LT.0)MPTY=-MPTY
            WRITE(21) IE,MPTY,NCHPP,ETOT
            WRITE(21) (IOMIT(I),I=1,NCHPP)
            WRITE(21) ((P(I,J),I=J,NCHPP),J=1,NCHPP),
     X                ((Q(I,J),I=J,NCHPP),J=1,NCHPP)
          ENDIF
        ENDIF
        IF(IPRKM.EQ.4)THEN
!          IF(NCHPP.GT.MZDEG)THEN
!            WRITE(6,*)'***INCREASE MZDEG TO AT LEAST: ',NCHPP 
!            STOP '***INCREASE MZDEG'
!          ENDIF
          DO J=1,NCHPP
            DO I=1,J
              ZKHICC(I,J)=DCMPLX(P(I,J),Q(I,J))
            ENDDO
          ENDDO
          WRITE(32)((ZKHICC(I,J),I=1,J),J=1,NCHPP)  !UPPER HALF
        IF(IE.EQ.1)WRITE(6,998)IABS(NSPN2),LRGL2,NPTY2
  998       FORMAT(/72('*')/3X,'WROTE DATA TO SMTLS.DAT FOR NSPN2 = ',I3
     1      ,3X,'LRGL2 = ',I3,3X,'NPTY2 = ',I3/72('*'))
        ENDIF
      ENDIF
!
      IF(IQDT.NE.2.AND.IMODE.LT.0.OR.
     X   LRGL2.GT.ISKP(IE).OR.NCHOPO.EQ.0.OR.NOMWRT.EQ.0)GO TO 666
!
!
!....I/QDT CASE (MQDT ENTRY POINT FOR S-MX)
!
 1000 IF(QDT.OR.(IQDT.NE.0.AND.NCHPP.GT.NCHOPO))THEN
!
! REVERT TO ORIGINAL IPERT, NCHPP
!
        NCHPP=NCHOPO
        IPERT=IPERTO                         !SHOULD BE UNNECESSARY....
!
! TREATED CHANNELS ASSOCIATED WITH ALL CLOSED TARGET STATES AS OPEN, NOW
! CLOSE THEM OFF (IF QDT=.TRUE. LEAVE LOWEST TARGET UNTIL SQDT)
!
        IF(IQDT.GT.0.AND.IQDT.NE.2)THEN
          IF(QDT)NCHPP=NQ
!
          IF (NCHPP.LT.NCHF) THEN 
            CALL MQDTS(LFAIL)
            IF(LFAIL) RETURN
          END IF
!
        ENDIF
!
! TREATED CHANNELS ASSOCIATED WITH SINGLE (LOWEST) CLOSED TARGET AS OPEN,
! NOW CLOSE THOSE OFF.
!
        IF(IQDT.LE.0.OR.QDT)THEN
          IF(QDT)NCHPP=NCHOPO
!
          CALL SQDTS
!
        ENDIF
!
        NCHPP1=NCHPP+1
!....
      ELSE
!
! CALCULATE COLLISION STRENGTHS FOR NON-I/QDT CASE AND STORE IN RK
!
        T=NWT*QUART
        DO J=1,NCHPP
          P(J,J)=P(J,J)-ONE
          DO I=1,NCHPP
            RK(I,J)=(P(I,J)**2+Q(I,J)**2)*T
          ENDDO
          P(J,J)=P(J,J)+ONE
        ENDDO
        IF(NDRMET.GT.0)THEN
          DO J=1,NCHPP
            PDR(J)=T
            DO I=1,NCHPP
              PDR(J)=PDR(J)-(P(I,J)**2+Q(I,J)**2)*T
            ENDDO
          ENDDO
        ENDIF
!....
      ENDIF
!
! RESTORE P AND Q WHEN WORKING WITH UNPHYSICAL K-MATRIX.
!
      IF(ABS(IQDT).EQ.2)THEN

        DO J=1,NCHF
          DO I=1,NCHF
            P(I,J)=POLD(I,J)
            Q(I,J)=QOLD(I,J)
          ENDDO
        ENDDO
      ENDIF
!
! SKIP OMEGA PROCESSING IF NOT REQUIRED
!
      IF(NOMWRT.EQ.0)GO TO 666
!
! WRITE COLLISION STRENGTH
!
      IF(IPRINT.GT.0)THEN
        WRITE(6,640)
        DO J=1,NCHPP
          DO I=1,J
            WRITE(6,650)ETOT,NSPN2,LRGL2,NPTY2,ITARG(I),LLCH(I),
     X      ITARG(J),LLCH(J),RK(I,J)
          ENDDO
       ENDDO
      ENDIF
!....
! EVALUATE CBE COLLISION STRENGTH
!
      IF(LCBE.GE.0.AND.LRGL2.GE.LCBE)THEN
        DO J=1,NCHPP
          DO I=1,J
            IF(LAMP(I,J).EQ.2)THEN
              IFAIL=IPRINT
              F=FDIP(EPS(I),LLCH(I),EPS(J),LLCH(J),IFAIL)
              IF (IFAIL.NE.0.AND.IPRINT.GE.0) THEN
                WRITE(6,655)IFAIL,I,J,EPS(I),LLCH(I),EPS(J),LLCH(J)
                LFAIL=.TRUE.
                RETURN
              END IF
              RBE(I,J)=NWT*(BW(I,J)*F)**2
            ELSE
              RBE(I,J)=TZERO
            ENDIF
            RBE(J,I)=RBE(I,J)
          ENDDO
        ENDDO
!
! WRITE CBE AND CC OMEGA'S
!
      IF(IPRINT.GT.-2)THEN
        WRITE(6,720)
        DO J=1,NCHPP
          DO I=1,J
            IF(LAMP(I,J).EQ.2.AND.RK(I,J).NE.0.)WRITE(6,721)I,J,ITARG(I)
     X     ,ITARG(J),RK(I,J),RBE(I,J),RBE(I,J)/RK(I,J)
          ENDDO
        ENDDO
      ENDIF
!
! REPLACE OMEGA-CC BY OMEGA-CBE FOR TOP-UP
! 
      IF(LRGL2.GE.LRGLAM-1.AND.LRGLAM.GE.0)THEN
        DO J=1,NCHPP
          DO I=1,J
            IF(LAMP(I,J).EQ.2)THEN
              RK(I,J)=ABS(RBE(I,J))
              RK(J,I)=RK(I,J)
            ENDIF
          ENDDO
        ENDDO
      ENDIF
!....
      ENDIF
!
! FOR LRGL2.GE.LRGLMN (SEE TOP1), TAKE OUT ALLOWED TRANSITIONS
! WITH LLCH.GT.LITLAM. IN LS, LRGLMN=LRGLAM BUT IS SMALLER FOR BP.
!
      IF(LRGLAM.GE.0.AND.LRGL2.GE.LRGLMN)THEN
        DO 100  I=1,INDM
          IT1=NTOP(I,1)
          IT2=NTOP(I,2)
          IF(IT1.LT.0)GO TO 150
          I2=NTCHAN(IT2,2)
          IF(I2.GT.NCHPP)GOTO 100
          I1=NTCHAN(IT1,2)
          IF(I1*I2.EQ.0)GO TO 100
          IF(MAX(LLCH(I1),LLCH(I2)).GT.LITLAM(I))RK(I1,I2)=TZERO
 150      IF(NSPN2.EQ.0)THEN 
            IFTOP=0
            IF(IT1.GT.0)IFTOP=1
            IF(IT2.LT.0.AND.LRGL2.GT.LRGLAM)IFTOP=-1     !QUADRUPOLE ETC.
            IF(IFTOP.NE.0)THEN
              IT1=ABS(IT1)
              IT2=ABS(IT2)
              DO I1=NTCHAN(IT1,1),NTCHAN(IT1,2)
                DO I2=NTCHAN(IT2,1),NTCHAN(IT2,2)
                  IF(IFTOP.GT.0.AND.MAX(LLCH(I1),LLCH(I2)).GT.LITLAM(I)
     X              .OR.IFTOP.LT.0)RK(I1,I2)=TZERO
                ENDDO
              ENDDO
            ENDIF
          ENDIF
 100    CONTINUE
      ENDIF
!
!
! STORE OMEGA RESULTS IN COMPACT FORM
!
! MXTST=(MZTAR*(MZTAR+1))/2
!
! IT AND JT ARE TARGET STATES
! OMEGA(IT,JT) FOR JT.GT.IT STORED IN
! OMST(IJST)=OMEGA(ITST(IJST),JTST(IJST))
!
! NUMBER OF OPEN TARGET LEVELS
      IF(NCHPP.EQ.0)GO TO 666
!
      NTAROP=ITARG(NCHPP)
      IONE=1
      IF(ELAS.EQ.'YES')IONE=0
      NOMT=(NTAROP*(NTAROP-2*IONE+1))/2
      JTMAX=NTAROP
      IF(NOMWRT.LT.0.AND.-NOMWRT.LT.NOMT)THEN
        NOMT=-NOMWRT
        K=0
        DO J=1+IONE,NTAROP
          DO I=1,J-IONE
            K=K+1
          ENDDO
          IF(K.GE.NOMT)GO TO 11
        ENDDO
  11    JTMAX=MIN(J,NTAROP)
      ENDIF
      IF(NOMWRT.GT.0.AND.NOMWRT.LT.(NAST*(NAST-2*IONE+1))/2)THEN
        K=0
        DO J=1+IONE,NTAROP
          IP=MIN(ITMAX,J-IONE)
          DO I=1,IP
            K=K+1
          ENDDO
        ENDDO
        NOMT=K
      ENDIF  
      DO J=1,NAST
        DO I=1,J
          KTST(I,J)=0
        ENDDO
      ENDDO
!
      IJST=0
      J2=IONE*NCONAT(1)
      DO 130  JT=1+IONE,JTMAX
        IF(NCONAT(JT).EQ.0)GOTO 130
        J1=J2+1
        J2=J2+NCONAT(JT)
        I2=0
        DO 120  IT=1,JT-IONE
          IF(NCONAT(IT).EQ.0)GOTO 120
          I1=I2+1
          I2=I2+NCONAT(IT)
          IJST=IJST+1
          OMT=TZERO
          KTST(IT,JT)=IJST
          DO J=J1,J2
            DO I=I1,I2
              OMT=OMT+RK(I,J)
            ENDDO
          ENDDO
          OMST(IJST)=OMT
          ITST(IJST)=IT
          JTST(IJST)=JT
 120    CONTINUE
 130  CONTINUE
!
! STORE DR OMEGA(IT,IE) BY INITIAL METASTABLE STATE IT AND ENERGY
! I.E. SUMMED OVER ALL FINAL STATES. (ELAS='YES' HERE).
!
      IF(NDRMET.GT.0.AND.NCHPP.LT.NCHF)THEN
        IF(QDT.OR.IQDT.NE.0)THEN
          ITT=MIN(NDRMET,NTAROP)
          I2=0
          DO 135  IT=1,ITT
            IF(NCONAT(IT).EQ.0)GO TO 135
            I1=I2+1
            I2=I2+NCONAT(IT)
            DO I=I1,I2
              OMEGDR(IT,IE)=OMEGDR(IT,IE)+PDR(I)
            ENDDO
 135      CONTINUE
        ENDIF
      ENDIF
!
! TOP-UP NOW CONTROLLED BY LRGLAM
!
      IF(LRGLAM.GE.0)CALL TOP2
!
! WRITE TARGET STATES AND COLLISION STRENGTHS
!
      IF(ISGPT.NE.0)WRITE(14,776)
      IF(IPRINT.GT.-2)WRITE(6,777)
      IF(IPRINT.GT.-2)
     X WRITE(6,601)ETOT,QDT,IPERT,(ITST(K),JTST(K),OMST(K),K=1,IJST)
!
! ADD TO OMEGA
!
      IF(NOMT.GT.0)THEN
        IFLG=0
        IF(MPOS(IE).GT.0) THEN
          K=MPOS(IE-1)
          DO JT=1+IONE,JTMAX
            ITP=MIN(JT-IONE,ITMAX)
            DO IT=1,ITP
              K=K+1
              KK=KTST(IT,JT)
              IF(KK.GT.0)THEN
                OMEM(K)=OMEM(K)+OMST(KK)
                IF(OMST(KK).GT.1.0E-10)IFLG=1
              ENDIF
            ENDDO
          ENDDO
        ELSEIF(MPOS(IE).LT.0) THEN
          NREC=-MPOS(IE)
          CALL OMREAD(OMEGA,NOMT,NREC)
          K=0
          DO JT=1+IONE,JTMAX
            ITP=MIN(JT-IONE,ITMAX)
            DO IT=1,ITP
              K=K+1
              KK=KTST(IT,JT)
              IF(KK.GT.0)THEN
                OMEGA(K)=OMEGA(K)+OMST(KK)
                IF(OMST(KK).GT.1.0E-10)IFLG=1
              ENDIF
            ENDDO
          ENDDO
          CALL OMWRIT(OMEGA,NOMT,NREC)
        ENDIF
        IF(IFLG.EQ.1.AND.LRGL2.GT.ISKP0)ISKP(IE)=LRGL2+LINC
      ENDIF
!
! CALCULATE THE PARTIAL CROSS SECTIONS
!
      IF(ISGPT*ITRMN*ITRMX.NE.0)THEN
        ALLOCATE(SIGPW(ITRMN:ITRMX))
        IF(NOMWRT.GT.0)THEN
          SIGPWT=TZERO
          K=0
          DO IT=1,ITMAX
            DO JT=IT+IONE,NAST
              K=K+1
              IF(K.GE.ITRMN.AND.K.LE.ITRMX) THEN
                ITST(K)=IT
                JTST(K)=JT
                KK=KTST(IT,JT)
                IF(KK.GT.0)THEN
                  WWI=ISAT(IT)*(2*LAT(IT)+1)
                  IF(NSPN2.EQ.0) WWI=LAT(IT)+1
                  EKI=AZAZ*(ETOT-ENAT(IT))
                  SIGPW(K)=CONSIG*OMST(KK)/(WWI*EKI)
                  SIGPWT=SIGPWT+SIGPW(K)
                ELSE
                  SIGPW(K)=TZERO
                ENDIF
              ENDIF
            ENDDO
          ENDDO
        ENDIF
        IF(NOMWRT.LT.0)THEN
          SIGPWT=TZERO
          K=0
          DO JT=1+IONE,NAST
            DO IT=1,JT-IONE
              K=K+1
              IF(K.GE.ITRMN.AND.K.LE.ITRMX) THEN
                ITST(K)=IT
                JTST(K)=JT
                KK=KTST(IT,JT)
                IF(KK.GT.0)THEN
                  WWI=ISAT(IT)*(2*LAT(IT)+1)
                  IF(NSPN2.EQ.0) WWI=LAT(IT)+1
                  EKI=AZAZ*(ETOT-ENAT(IT))
                  SIGPW(K)=CONSIG*OMST(KK)/(WWI*EKI)
                  SIGPWT=SIGPWT+SIGPW(K)
                ELSE
                  SIGPW(K)=TZERO
                ENDIF
              ENDIF
            ENDDO
          ENDDO
        ENDIF
        WRITE(14,6011) ETOT,(ITST(K),JTST(K),SIGPW(K),K=ITRMN,ITRMX)
        WRITE(14,6012) ITRMN,ITRMX,SIGPWT
        DEALLOCATE(SIGPW)
      ENDIF
!
!
 666  IPERT=IABS(IPERT)
!
      NULLIFY(P,Q,POLD,QOLD,Y)
!
      RETURN
!
! FORMATS
!
  600 FORMAT(' SR.REACT: MATRIX HAS NO INVERSE IN VERT')
  601 FORMAT(F9.5,L3,I6,2X,3(I3,I3,1PE11.3)/(20X,3(I3,I3,E11.3)))
  640 FORMAT(//7X,'ENERGY',13X,'S',2X,'L',1X,'PI',6X,'TI',
     1 1X,'LI',4X,'TJ',1X,'LJ',7X,'OMEGA'/)
  650 FORMAT(5X,E14.6,5X,3I3,5X,2I3,3X,2I3,5X,E14.6)
  655 FORMAT(' FDIP FAILURE: IFAIL=',I2,' FOR I,J=',2I4,' E,L='
     X       ,2(1PE13.5,I3))
  660 FORMAT(3I5,1PE14.8)
  670 FORMAT(2I5)
  671 FORMAT(3I5)
  680 FORMAT(E15.6)
  695 FORMAT(' T-MATRIX VIOLATES UNITARITY FOR ETOT=',F12.8,3X,
     X'IPERT=',I2,4X,'I,J=',2I3,4X,'T**2=',F7.4)
  707 FORMAT(/'   REACTANCE MATRIX FOR ETOT = ',F10.6//
     1  (I9,6I11))
  710 FORMAT(I3,1P7E11.3/(3X,7E11.3))
  720 FORMAT(//3X,'I',3X,'J',4X,' IT',1X,' JT',7X
     X,'OMEGA-CC',2X,'OMEGA-CBE',6X,'CBE/CC')
  721 FORMAT(2I4,3X,2I4,4X,2(1PE11.3),0PF12.5)
  776 FORMAT(//'   ETOT',3X,
     1 'INITIAL AND FINAL TARGET LEVELS, AND CROSS SECTIONS (MB)'/)  
  777 FORMAT(//'   ETOT',3X,'QDT',2X,'IPERT',2X,
     1 'INITIAL AND FINAL TARGET LEVELS, AND COLLISION STRENGTHS'/)
 6011 FORMAT(F9.5,1X,3(I3,I3,1PE13.5)/(10X,3(I3,I3,E13.5)))
 6012 FORMAT('THE TOTAL CROSS SECTION FOR TRANSITIONS',I3,' TO ',
     X I3,' EQUALS ',1PE13.5,' MB') 
!
      END
!***************************************************************
!
      SUBROUTINE READ1
      USE WMATALLOC
      USE PART
      USE DBUT
      USE CINPUT
      USE CINPTX
      USE CNTRL
!
!  READS DATA INDEPENDENT OF SLPI FROM R-MATRIX FILE, FILA
!
!  THE FOLLOWING DATA ARE READ _
!       NZ = NUCLEAR CHARGE
!       NELC = NUMBER OF ELECTRONS IN TARGET
!       NAST = NUMBER OF TARGET STATES
!       LRANG2 = TOTAL NUMBER OF SMALL L/BIG K VALUES
!       LAMAX = MAXIMUM LAMBDA FOR MULTIPOLE POTENTIALS
!       RA = R-MATRIX RADIUS
!       BSTO = LOGARITHMIC DERIVATIVE
!       FOR I = 1,NAST -
!               ENAT(I) = TARGET ENERGIES
!               LAT(I) = TARGET ORBITAL ANGULAR MOMENTA
!               ISAT(I) = VALUES OF (2*S+1) FOR TARGET STATES
!       FOR I = 1,3 AND L = 1,LRANG2 -
!               COEFF(I,L) = BUTTLE CORRECTION FITS
!       FOR I=1,NBUTD(L) FOR L=1,LRANG2 DARC BUTTLE CORRECTION
!               EBUTD(I,L),CBUTD(I,L)
!
      IMPLICIT REAL*8 (A-H,O-Z)
      LOGICAL EX
!
!  READ AND DIMENSION CHECKS
!
      IF(IWORD.EQ.2)GO TO 50
      READ(10)NELC,NZ,LRANG2,LAMAX,NAST,RA,BSTO
      NZED=NZ
!
      IF(LRANG2.LT.0)THEN     !FROM DARC DSTG2 VIA DTO3.
        LRANG2=-LRANG2
        KFLAG=-1
      ENDIF
      IF(NAST.LT.0)THEN
        IPRCENT=0
        NAST=-NAST
      ELSE
        IPRCENT=100
      ENDIF
!
! ALLOCATABLE
!      IF (NAST.GT.MZTAR) THEN
!        WRITE(6,610) NAST
!        STOP '*** DIMENSION EXCEEDED: INCREASE MZTAR'
!      ENDIF
!
      IF(ALLOCATED(LAT))   DEALLOCATE(LAT)
      IF(ALLOCATED(ISAT))  DEALLOCATE(ISAT)
      IF(ALLOCATED(ENAT))  DEALLOCATE(ENAT)
      IF(ALLOCATED(ENATK)) DEALLOCATE(ENATK)
      ALLOCATE(LAT(ABS(NAST)))
      ALLOCATE(ISAT(ABS(NAST)))
      ALLOCATE(ENAT(ABS(NAST)))
      ALLOCATE(ENATK(ABS(NAST)))
!
      READ(10)(ENAT(I),I=1,NAST)
      READ(10)(LAT(I),I=1,NAST)
      READ(10)(ISAT(I),I=1,NAST)
!
!      IF(LRANG2.GT.MZLP1)THEN
!        WRITE(6,620)LRANG2,MZLP1
!        STOP '*** DIMENSION EXCEEDED: INCREASE MZLP1'
!      ENDIF
!
      IF(ALLOCATED(COEFF))DEALLOCATE(COEFF)
      ALLOCATE(COEFF(3,LRANG2))
      
      READ(10)((COEFF(I,L),I=1,3),L=1,LRANG2)
!
      IF (IPRCENT.NE.100) THEN              ! FOR PARTITIONED
!
!     99/100 NEVER USED 
!
        READ(10) IPRCENT,NRANG2
!        IF (NRANG2.GT.MZNRG) THEN
!          WRITE(6,625) NRANG2,MZNRG
!          STOP  '*** DIMENSION EXCEEDED: INCREASE MZNRG'
!        ENDIF
!
        IF(ALLOCATED(EIGENS)) DEALLOCATE(EIGENS)
        IF(ALLOCATED(ENDS))   DEALLOCATE(ENDS)
        IF(ALLOCATED(SI))     DEALLOCATE(SI)
        ALLOCATE(EIGENS(NRANG2,LRANG2))
        ALLOCATE(ENDS(NRANG2,LRANG2))
        ALLOCATE(SI(MZCHF))
!
        DO L = 1,LRANG2
          READ(10) (EIGENS(N,L),N=1,NRANG2)
          READ(10) (ENDS(N,L),N=1,NRANG2)
        ENDDO
      ENDIF
!
      INQUIRE(FILE='DBUT.DAT',EXIST=EX)
      IF(EX)THEN
        IWORD=-1
        IB=11
        OPEN(11,FILE='DBUT.DAT',FORM='UNFORMATTED')
        GO TO 51
      ENDIF
!
      RETURN
!
! DARC
   50 READ(10)
      READ(10)NELC,NZ,NRANG2,LRANG2,NAST,RA,BSTO
      LAMAX=0
      NZED=NZ
!
! ALLOCATABLE
!      IF(NAST.GT.MZTAR)THEN
!        WRITE(6,610)NAST
!        STOP '*** DIMENSION EXCEEDED: INCREASE MZTAR'
!      ENDIF
!
      READ(10)(ENAT(I),I=1,NAST)
      READ(10)(LAT(I),I=1,NAST)
!
      DO I=1,NAST
        LAT(I)=ABS(LAT(I))-1
        ISAT(I)=0
      ENDDO
!
!      IF(LRANG2.GT.MZLP1)THEN
!        WRITE(6,620)LRANG2,MZLP1
!        STOP '*** DIMENSION EXCEEDED: INCREASE MZLP1'
!      ENDIF
!
      IB=10
!
!    ONLY IF DBUT.DAT EXISTS , DO THE FOLLOWING GET ALLOCATED
! LFM - REPROGRAMMED THIS PART TO GET THE EXACT DIMENSIONS
!
   51 CONTINUE
      ALLOCATE(NBUTD(LRANG2))
      MZN=1
      DO L=1,LRANG2
        READ(IB) NBUTD(L)
        READ(IB) ! (EBUTD(I,L),I=1,NBUTD(L))
        READ(IB) ! (CBUTD(I,L),I=1,NBUTD(L))
        MZN=MAX(MZN,NBUTD(L))
      END DO
      DO L=1,LRANG2
        BACKSPACE(IB) ! NBUTD(L)
        BACKSPACE(IB) ! (EBUTD(I,L),I=1,NBUTD(L))
        BACKSPACE(IB) ! (CBUTD(I,L),I=1,NBUTD(L))
      END DO
      ALLOCATE(EBUTD(MZN,LRANG2))
      ALLOCATE(CBUTD(MZN,LRANG2))
!
!      ALLOCATE(NBUTD(MZNRG))
!      ALLOCATE(EBUTD(MZNRG,LRANG2))
!      ALLOCATE(CBUTD(MZNRG,LRANG2))
!
      DO L=1,LRANG2
        READ(IB) NBUTD(L)
!        IF(NBUTD(L).GT.MZNRG)THEN
!          WRITE (6,699) NBUTD(L),MZNRG
!          STOP '*** DIMENSION EXCEEDED: INCREASE MZNRG'
!        ENDIF
        READ(IB) (EBUTD(I,L),I=1,NBUTD(L))
        READ(IB) (CBUTD(I,L),I=1,NBUTD(L))
      ENDDO
!
      IF(IB.EQ.11) CLOSE(11)
!
      RETURN
!
!  610 FORMAT(///20X,'TOO MANY TARGET STATES'//
!     1 10X,'VALUE READ FOR NAST IS ',I3//
!     2 10X,'MAXIMUM ALLOWED BY DIMENSIONS IS MZTAR'//)
!  620 FORMAT(///20X,'TOO MANNY BUTTLE COEFFICIENTS'//
!     1 10X,'VALUE READ FOR LRANG2 IS ',I3//
!     2 10X,'MAXIMUM VALUE ALLOWED BY DIMENSIONS IS MZLP1 =',I3//)
!  625 FORMAT(//' ******* NRANG2 = ',I4,'  LARGER THAN ',
!     + 'MZNRG = ',I4//)
!  699 FORMAT(//' ******* NBUT = ',I4,'  LARGER THAN ',
!     + 'MZNRG = ',I4//)
!
      END
!***************************************************************
!
      SUBROUTINE READ2N(IOPT1,KASE)
      USE WMATALLOC
      USE TERMSYM
      USE PART
      USE DBUT
      USE CINPUT
      USE CINPTX
      USE CHAN
      USE CDEGEN
      USE CDEC
      USE NRBLMX
      USE CNTRL
!
!  READS R-MATRIX DATA FOR ONE SLPI CASE, FROM FILA
!
!  LFM - JANUARY 2018
!  TAKES THE ARRAY IWHD FROM THE MODULE TERMSYM
!  IWHD(KASE) IS THE LABEL OF THE H.DATXXX FILE IT HAS TO OPEN
!  TO FIND THE INFORMATION ABOUT THE MSLP(KASE) PARTIAL WAVE
!  IF IWHD(KASE).EQ.-1 THEN THERE IS ONLY ONE COMPLETE H.DAT FILE
!
!  THE FOLLOWING DATA ARE READ -
!       LRGL2 = TOTAL ORBITAL ANGULAR MOMENTUM/ 2J
!       NSPN2 = TOTAL (2*S+1)/0
!       NPTY2 = TOTAL PARITY
!       NCHAN = NUMBER OF CHANNELS
!       MNP2 = NUMBER OF R-MATRIX POLES
!       MORE2 = ZERO TO TERMINATE SLPI CASES
!       FOR I = 1,NAST -
!               NCONAT(I) = NUMBER OF CHANNELS FOR TARGET STATE I
!       FOR I = 1,NCHAN -
!               L2P(I), KJ(I) = SMALL L AND BIG JK FOR CHANNEL I
!                             = K-1, 0 FROM DSTG2/DARC
!       FOR I = 1,NCHAN AND N = 1,NCHAN AND M = 1,LAMAX -
!               CF(I,N,M) = COEFFICIENTS IN MULTIPOLE POTENTIALS
!       FOR I = 1,MNP2 -
!               VALUE(I) = R-MATRIX POLE ENERGIES
!       FOR K = 1,NCHAN AND I = 1,MNP2 -
!               WMAT(I,K) = R-MATRIX AMPLITUDES
!
      IMPLICIT REAL*8 (A-H,O-Z)
      INTEGER, INTENT(IN) :: IOPT1,KASE
      CHARACTER(LEN=10) FIL
      LOGICAL FOUND
      CHARACTER(LEN=1) :: 
     &  NUM(0:9)=["0","1","2","3","4","5","6","7","8","9"]
!
      IF (IWHD(KASE).EQ.-1) THEN
! ONE SINGLE H.DAT OR DSTGH.DAT FILE
! IT SHOULD BE OPEN AND IT SHOULD BE REPOSITIONED AFTER READ1
        IF (IWORD.EQ.1) THEN
          FIL="H.DAT"
          READ(10) ! NELC,NZ,LRANG2,LAMAX,NAST,RA,BSTO
          READ(10) ! (ENAT(I),I=1,NAST)
          READ(10) ! (LAT(I),I=1,NAST)
          READ(10) ! (ISAT(I),I=1,NAST)
          READ(10) ! ((COEFF(I,L),I=1,3),L=1,LRANG2)
          IF (IPRCENT.NE.100) THEN              ! FOR PARTITIONED
            READ(10) ! IPRCENT,NRANG2
            DO L = 1,LRANG2
              READ(10) ! (EIGENS(N,L),N=1,NRANG2)
              READ(10) ! (ENDS(N,L),N=1,NRANG2)
            ENDDO
          ENDIF
        ELSE IF (IWORD.EQ.2) THEN
          FIL="DSTGH.DAT"
          READ(10)
          READ(10) ! NELC,NZ,NRANG2,LRANG2,NAST,RA,BSTO
          READ(10) ! (ENAT(I),I=1,NAST)
          READ(10) ! (LAT(I),I=1,NAST)
          DO L=1,LRANG2
            READ(10) ! NBUTD(L)
            READ(10) ! (EBUTD(I,L),I=1,NBUTD(L))
            READ(10) ! (CBUTD(I,L),I=1,NBUTD(L))
          ENDDO
        END IF
!
      ELSE IF (IWHD(KASE).EQ.0) THEN
! H.DAT000 FILE
! IT SHOULD BE CLOSED AND IT SHOULD CONTAIN THE 
! SLPI INDEPENDENT INFORMATION AT STARTING POINT
        FIL="H.DAT000"
        OPEN(UNIT=10,FILE=FIL,FORM="UNFORMATTED",STATUS="OLD")
        READ(10) ! NELC,NZ,LRANG2,LAMAX,NAST,RA,BSTO
        READ(10) ! (ENAT(I),I=1,NAST)
        READ(10) ! (LAT(I),I=1,NAST)
        READ(10) ! (ISAT(I),I=1,NAST)
        READ(10) ! ((COEFF(I,L),I=1,3),L=1,LRANG2)
        IF (IPRCENT.NE.100) THEN              ! FOR PARTITIONED
          READ(10) ! IPRCENT,NRANG2
          DO L = 1,LRANG2
            READ(10) ! (EIGENS(N,L),N=1,NRANG2)
            READ(10) ! (ENDS(N,L),N=1,NRANG2)
          ENDDO
        ENDIF
!
      ELSE IF (IWHD(KASE).GE.1) THEN
! H.DATXXX FILE (XXX=IWDH(KASE))
! FILE SHOULD BE CLOSED AND INFORMATION OF READ1 MISSING
        I1=IWHD(KASE)/100
        I2=(IWHD(KASE)-100*I1)/10
        I3=IWHD(KASE)-(100*I1)-10*I2
        FIL="H.DAT"//NUM(I1)//NUM(I2)//NUM(I3)
        OPEN(UNIT=10,FILE=FIL,FORM="UNFORMATTED",STATUS="OLD")
      ELSE IF (IWHD(KASE).EQ.-10) THEN
! SYMMETRY IS NOT IN ANY FILE
        WRITE(6,503) MSLP(KASE)
        MORE2=-1
        RETURN
      END IF
 503  FORMAT (//10X,"WARNING: SYMMETRY ",I8," IS NOT PRESENT"/
     &          10X,"IN H.DAT FILES. SKIPING.")
!
!  READ AND DIMENSION CHECKS
!
      FOUND=.FALSE.
      DO WHILE (.NOT.FOUND)
        IF (IWORD.NE.2) THEN
! H.DAT OR H.DATXXX
          READ(10,IOSTAT=IER) LRGL2,NSPN2,NPTY2,NCHAN,MNP2,MORE2
          IF (IER.LT.0) THEN
!            more2=-777
            EXIT
          END IF
        ELSE                            ! DARC
          READ(10,IOSTAT=IER) LRGL2,NPTY2,NCHAN,MNP2,NCFGP,LAMAX
          IF (IER.LT.0) THEN
            EXIT
          END IF
          MORE2=1
          LRGL2=ABS(LRGL2)-1
          NSPN2=0
          IF(NPTY2.EQ.-1)THEN
            NPTY2=1
          ELSE
            NPTY2=0
          ENDIF
          KFLAG=-2
        ENDIF
!
! CHECK IF THIS IS THE SYMMETRY WE WERE LOOKING FOR
!
        IF (10000*IABS(NSPN2)+100*LRGL2+NPTY2.NE.MSLP(KASE)) THEN
! GO TO THE NEXT SYMMETRY STORED, SKIP READING
          READ(10) ! (NCONAT(I),I=1,NASTR)  
          READ(10) ! (L2P(I),I=1,NCHAN) OR (L2P(I),I=1,NCHAN), (KJ(I),I=1,NCHAN)
          IF (LAMAX.GT.0) READ(10) ! (((CF(I,N,M),I=1,NCHAN),N=1,NCHAN),M=1,LAMAX)
          IF (IPRCENT.NE.100) READ(10) ! TRACE
          IF (MNP2.GE.0) THEN
            READ(10) ! (VALUE(I),I=1,MNP2)
            READ(10) ! ((WMAT(I,K),K=1,NCHAN),I=1,MNP2)
          ELSE    
            READ(10) ! (VALUE(I),I=1,-MNP2)
            ILOW=999
            DO WHILE (ILOW.NE.1)
              READ(10) ILOW,IUPPER,NDIV
              READ(10) ! ((WMAT(I-ILOW+1+KK,K),K=1,NCHAN),I=ILOW,IUPPER)
            END DO
          END IF
!
        ELSE
! READ THE INFORMATION 
          FOUND=.TRUE.
!
! CHECK DIMENSIONS
!          IF (NCHAN.GE.MZCHF) THEN            ! .GE. FOR SR.LU
!            WRITE(6,600) NSPN2,LRGL2,NPTY2,NCHAN
!            STOP  "*** DIMENSION EXCEEDED: INCREASE MZCHF"
!          END IF
!          IF (MNP2.GT.MZMNP) THEN
!            WRITE(6,610)NSPN2,LRGL2,NPTY2,MNP2
!            STOP "*** DIMENSION EXCEEDED: INCREASE MZMNP"
!          ENDIF
!          IF (LAMAX.GT.MZLMX) THEN
!            write(6,*) 'set mzlmx to lamax, see stage 3 rout3r'
!            stop
!          ENDIF
!
          IF (ALLOCATED(NCONAT)) DEALLOCATE(NCONAT)
          ALLOCATE(NCONAT(ABS(NAST)))
          READ(10) (NCONAT(I),I=1,NASTR)    ! NAST? NASTR=NAST OR NASTR=0

          IF (ALLOCATED(L2P)) DEALLOCATE(L2P)
          ALLOCATE(L2P(NCHAN))
          IF(ALLOCATED(KJ)) DEALLOCATE(KJ)
          ALLOCATE(KJ(NCHAN))
          IF (NSPN2.EQ.0 .AND. KFLAG.EQ.1) THEN
            READ(10) (L2P(I),I=1,NCHAN), (KJ(I),I=1,NCHAN)
          ELSE
            READ(10) (L2P(I),I=1,NCHAN)
          END IF

          MAXMUL=MAX(MAXMUL,LAMAX)     ! CPB 2016
          IF (ALLOCATED(CF)) DEALLOCATE(CF)
          IF (ALLOCATED(WMAT)) DEALLOCATE(WMAT)
          IF (ALLOCATED(VALUE)) DEALLOCATE(VALUE)
          ALLOCATE(CF(NCHAN,NCHAN,MAXMUL))
          IF (LAMAX.GT.0)
     &      READ(10) (((CF(I,N,M),I=1,NCHAN),N=1,NCHAN),M=1,LAMAX)
          MAXMUL=MIN(LAMAX,MAXMUL)
! REDIMENSION CF ARRAY, USING AUXILIAR ARRAY TTT
          ALLOCATE(TTT(NCHAN,NCHAN,MAXMUL))
          TTT(1:NCHAN,1:NCHAN,1:MAXMUL)=CF(1:NCHAN,1:NCHAN,1:MAXMUL)
          DEALLOCATE(CF)
          ALLOCATE(CF(1:NCHAN,1:NCHAN,1:MAXMUL))
          CF(1:NCHAN,1:NCHAN,1:MAXMUL)=TTT(1:NCHAN,1:NCHAN,1:MAXMUL)
          DEALLOCATE(TTT) 

          IF (IPRCENT.NE.100) READ(10) TRACE

          IF (ALLOCATED(WMAT)) DEALLOCATE(WMAT)
          IF (ALLOCATED(VALUE)) DEALLOCATE(VALUE)
          ALLOCATE(WMAT(ABS(MNP2),NCHAN))
          ALLOCATE(VALUE(ABS(MNP2)))
          IF (MNP2.GE.0) THEN
            READ(10) (VALUE(I),I=1,MNP2)
            READ(10) ((WMAT(I,K),K=1,NCHAN),I=1,MNP2)
          ELSE                                   !DIVIDED WMAT - CPB
            MNP2=-MNP2
!           WRITE(0,*) 'GET READ2 DIV',NCHAN,MNP2
            READ(10) (VALUE(I),I=1,MNP2)
            ILOW=999
            KK=0
            DO WHILE (ILOW.NE.1)
              READ(10) ILOW,IUPPER,NDIV
              READ(10) ((WMAT(I-ILOW+1+KK,K),K=1,NCHAN),I=ILOW,IUPPER)
              KK=KK+IUPPER-ILOW+1
            END DO
          END IF
!
! END OF READ FROM UNIT 10
! ALLOCATIONS
!
          IF(ALLOCATED(ECH))   DEALLOCATE(ECH)
          IF(ALLOCATED(EPS))   DEALLOCATE(EPS)
          IF(ALLOCATED(FKNU))  DEALLOCATE(FKNU)
          IF(ALLOCATED(CCT))   DEALLOCATE(CCT)
          IF(ALLOCATED(RINF))  DEALLOCATE(RINF)
          IF(ALLOCATED(ITARG)) DEALLOCATE(ITARG)
          IF(ALLOCATED(LLCH))  DEALLOCATE(LLCH)
          ALLOCATE(ITARG(NCHAN),LLCH(NCHAN))
          ALLOCATE(ECH(NCHAN),EPS(NCHAN),FKNU(NCHAN),CCT(NCHAN),
     &             RINF(NCHAN))
!
          IF (IWORD.EQ.2) THEN                   ! DARC
            DO I=1,NCHAN
              L2P(I)=2*L2P(I)                    ! 2*KAPPA
              IF (L2P(I).LT.0) L2P(I)=-L2P(I)-1  ! K, ANALOGUE OF L
              L2P(I)=L2P(I)-1                    ! K-1
            ENDDO
          ENDIF
!
          IF (KFLAG.LT.0) THEN                   ! L2P(I)=K-1
!
!     ARDEC/IDIP/RAD forces earlier read2 so allocate K2P
!
            IF (ALLOCATED(K2P)) DEALLOCATE(K2P)
            ALLOCATE(K2P(NCHAN))
            DO I=1,NCHAN
              LLCH(I)=(L2P(I)+1)/2               ! CHANNEL ORB ANG 
              K2P(I)=LLCH(I)
              IF (LLCH(I)*2.NE.L2P(I)+1) K2P(I)=-K2P(I)-1   ! KAPPA
              KJ(I)=0                            ! AS JJ COUPLING
            ENDDO
          ELSE
            DO I=1,NCHAN
              LLCH(I)=L2P(I)                     ! CHANNEL ORB ANG
            ENDDO
          ENDIF
!
!  GROUP TOGETHER CHANNELS BELONGING TO DEGENERATE LEVELS
!  AND PRESERVE THE READ VALUES OF NCONAT IN /CDEGEN/
          IF (IOPT1.GT.9) NASTR=NAST
          IF (NASTD.GT.0) THEN
            DO I=1,NASTR
              NCNATR(I)=NCONAT(I)
            ENDDO
            IF (IOPT1.LT.10) THEN
              N1=1
              DO I=1,NASTD
                NCON=0
                N2=NLEV(I)+N1-1
                DO IN=N1,N2
                  NCON=NCON+NCNATR(IN)
                ENDDO
                NCONAT(I)=NCON
                N1=N2+1
              ENDDO
            ENDIF
          ENDIF
!
!  TARGET PARITIES
!
          K=1
          DO I=1,NAST
            IF (NCONAT(I).NE.0) THEN
              IF (IPAR(I).LT.0) IPAR(I)=ABS(NPTY2-LLCH(K)+2*(LLCH(K)/2))
              K=K+NCONAT(I)
            END IF
          END DO
!
!      RECOVER THE K CHANNEL NUMBERS (BASED ON SR.NJCHAN) IF
!      NOT READ FROM H.DAT
!
          IF (NSPN2.EQ.0.AND.KFLAG.EQ.0) THEN
            NJCHA = 0
!
!      LOOP OVER THE TARGET STATES.
!
            DO I=1,NAST
!
!      LOOP OVER THE ONLY TWO POSSIBLE 2K VALUES, THAT IS LRGL2-1,LRGL2+1
              IFIN=3
              IF (LRGL2.EQ.0) IFIN=1
              DO K=1,IFIN,2
                JK=ABS(LRGL2-2+K)
!
!      RECOVER THE RANGE OF L(INCIDENT) VALUES.
                LMIN=ABS(LAT(I)-JK)
                LMAX=LAT(I) + JK
!
!      CHECK IF LMIN IS AN INTEGER.
                IF (MOD(LMIN,2).NE.0) CYCLE
!
!      CHECK THE PARITY OF LMIN.
                LP = LMIN/2 + IPAR(I)
                IF (MOD(LP,2).NE.NPTY2) LMIN=LMIN+2
                LP = (LMIN+LMAX)/2
                IF (MOD(LP,2).EQ.1) LMAX = LMAX - 2
                IF (LMIN.GT.LMAX) CYCLE
!
!      STORE THE 2K-VALUES IN KJ.
!
                LMIN = LMIN + 1
                LMAX = LMAX + 1
                DO L=LMIN,LMAX,4
                  NJCHA = NJCHA + 1
                  KJ(NJCHA) = JK
                END DO
              END DO
            END DO
!
! END RECOVERY OF K QUANTUM NUMBER
          ENDIF
!
! END OF THE READING OF THE SLPI DEPENDENT DATA
! CAN EXIT FROM SUBROUTINE
!
        END IF
      END DO
!
! CLOSE OR REWIND INPUT FILES
!
      IF (IWHD(KASE).EQ.-1) THEN
! REWIND THE FILE H.DAT OR DSTGH.DAT AND KEEP IT OPEN 
        REWIND(10)
      ELSE IF (IWHD(KASE).GE.0) THEN
! CLOSE THE FILE
        REWIND(10)
        CLOSE(10,STATUS="KEEP")
      END IF
!
      IF (FOUND) THEN
! SUCCESSFUL READ
!        WRITE(6,501) MSLP(KASE),FIL
!        WRITE(0,501) MSLP(KASE),FIL
        if (kase.eq.kslp) then
          more2=0
        else
          more2=1
        end if
!
      ELSE
! SYMMETRY NOT FOUND
        WRITE(6,502) MSLP(KASE),FIL
        MORE2=-1
      END IF
!
 501  FORMAT (/10X,"CALCULATING SYMMETRY ",I8," ,"/
     &         10X,"READ FROM FILE ",A10)
 502  FORMAT (//10X,"WARNING: UNEXPECTED END OF FILE FOUND"/
     &          10X,"WHILE READING ISLP= ",I8/
     &          10X,"IN FILE ",A10/
     &          10X,"SYMMETRY NOT FOUND. SKIPING.")
!  600 FORMAT(///20X,"TOO MANY CHANNELS FOR (IS, IL, IP) = (",
!     &  3I3,")"//10X,"VALUE READ FOR NCHAN IS ",I4//
!     &  10X,"MAXIMUM ALLOWED BY DIMENSIONS IS MZCHF"//)
!  610 FORMAT(///20X,"TOO MANY R-MATRIX STATES FOR (IS, IL, IP) = (",
!     & 3I3,")"//10X,"VALUE READ FOR MNP2 IS ",I5//    
!     & 10X,"MAXIMUM ALLOWED BY DIMENSIONS IS MZMNP"//)
!  620 FORMAT(///20X,"WARNING TOO MANY MULTIPOLES REQUESTED VIZ.",I3//
!     & 20X,"MAXIMUM ALLOWED BY DIMENSIONS IS MZLMX"//)
      
      RETURN
      END SUBROUTINE READ2N
!
!***************************************************************
!
      SUBROUTINE READPQ(IE,QETEST,ISLP,IOPT1,QJUMP,PQRD)
      USE NRBRCT
      USE CEN
      USE CHAN
      USE NRBQDT
      USE NRBOMT
      USE CMESH
      USE CPBSAVE
!
! NRB:
!  CALCULATE, READ AND/OR INTERPOLATE MATRICES P AND Q,
!  WHICH MAY BE UNPHYSICAL K OR S.
!
      IMPLICIT REAL*8 (A-H,O-Y)
      IMPLICIT COMPLEX*16 (Z)
!
      LOGICAL PQRD,QJUMP
      LOGICAL LFAIL
      PARAMETER (TZERO=0.0)
!
       SAVE IE1,IE2,MPTY1,MPTY2,E1,E2
!
       IF(.NOT.ALLOCATED(P1))THEN
       ALLOCATE(P1(NCHF,NCHF))
       ALLOCATE(Q1(NCHF,NCHF))
       ALLOCATE(IOMIT1(NCHF))
       ALLOCATE(IOMIT2(NCHF))
       ENDIF
!
       P=>CSP
       Q=>CC
       P2=>CCPP
       Q2=>DS
!
!
!
!
      IF(IE.LE.0)THEN
!
        IF(IMODE.LT.0)THEN
!
! CALCULATE UNPHYSICAL P,Q
!
!      WRITE(IAM+70,*)IE,ETOT
          CALL POINTS(IOPT1,.FALSE.)
          CALL REACT(IOPT1,.FALSE.,.TRUE.,LFAIL)
!
          IF(NCHPP.NE.NCHF)STOP'READPQ: NCHPP.NE.NCHF'
          E2=ETOT
          IE2=1-IE                               !PARALLEL
          MPTY2=0   !NOT NEEDED?
          DO J=1,NCHF
            IOMIT2(J)=IOMIT(J)
            DO I=J,NCHF
              P2(I,J)=P(I,J)
              Q2(I,J)=Q(I,J)
            ENDDO
          ENDDO
!
        ELSE
!
! FIRST ENERGY READ FOR THIS SYMMETRY, AND POSSIBLY FIRST READ AT ALL.
! IF IOPT1=2 WE FIRST NEED TO FIND THE CORRECT SYMMETRY.
! NCHF SHOULD BE EQUAL TO NCHF THROUGHOUT THE IQDT=1 PROBLEM.
!
          IF(IOPT1.EQ.2)REWIND(21)
   1      READ(21,END=100)IE2,MPTY2,NCHOPP,E2
          READ(21) (IOMIT2(I),I=1,NCHOPP)
          READ(21)((P2(I,J),I=J,NCHOPP),J=1,NCHOPP),
     X          ((Q2(I,J),I=J,NCHOPP),J=1,NCHOPP)
          IF(IE2.NE.1)THEN
! HIGH ENERGY OF OLD SLP
            IF(MPTY2.NE.ISLP)GO TO 1
          ELSE
! NO SEARCH ON SLP IF IOPT1.NE.2 SO STOP IF MIS-MATCH
            IF(MPTY2.NE.ISLP)THEN
              IF(IOPT1.EQ.2)GO TO 1
            WRITE(6,*)' CHECK JBIN? MIS-MATCH ON SYMMETRIES IN PQ DATA:'
     X                ,MPTY1,ISLP
              STOP 'READPQ'
            ENDIF
          ENDIF
        ENDIF
!
        QJUMP=.TRUE.
        PQRD=.TRUE.
        E1=999999.
        NULLIFY(P,Q,P2,Q2)
        RETURN
!
      ENDIF
!
! SEE IF WE NEED TO READ NEW DATA OR JUST INTERPOLATE EXISTING PQ.
!
      IF(ETOT.GT.E1-QETEST .AND. ETOT.LT.E2+QETEST)GO TO 3
!
! MOVE P2,Q2 TO P1,Q1 ETC.
!
   2  E1=E2
      IE1=IE2
      MPTY1=MPTY2
      DO J=1,NCHF
        IOMIT1(J)=IOMIT2(J)
        DO I=J,NCHF
          P1(I,J)=P2(I,J)
          Q1(I,J)=Q2(I,J)
        ENDDO
      ENDDO
!
      IF(IMODE.LT.0)THEN
!
! CALCULATE NEW P,Q BUT FIRST FIND APPROPRIATE ENERGY
!
        DO I=IE1+1,MXE
           IF(IEE(I).NE.0)GO TO 20
        ENDDO
        STOP ' READPQ CANNOT FIND NEXT ENERGY'
  20    IE2=I
        E2=EMESH(I)
        MPTY2=0   !NOT NEEDED?
        ETT=ETOT
        ETOT=E2
!
!      WRITE(IAM+70,*)IE,IE2
        CALL POINTS(IOPT1,.FALSE.)
        CALL REACT(IOPT1,.FALSE.,.TRUE.,LFAIL)
!
        IF(NCHPP.NE.NCHF)STOP'READPQ: NCHPP.NE.NCHF'
        ETOT=ETT
          DO J=1,NCHF
            IOMIT2(J)=IOMIT(J)
            DO I=J,NCHF
              P2(I,J)=P(I,J)
              Q2(I,J)=Q(I,J)
            ENDDO
          ENDDO
        GO TO 3
      ENDIF
!
! READ A NEW P,Q SET
!
      READ(21,END=100)IE2,MPTY2,NCHOPP,E2
      IF(IE2.LT.IE1)THEN      !WE HAVE MOVED ONTO THE NEXT SYMMETRY SO
        BACKSPACE(21)
        E2=999999.
        GO TO 100
      ENDIF
      IF(MPTY2.NE.ISLP)THEN   !NO SEARCH ON SLP HERE SO STOP IF MIS-MATCH
        WRITE(6,*)' CHECK JBIN? MIS-MATCH ON SYMMETRIES IN PQ DATA:'
     X            ,MPTY1,ISLP
        STOP 'READPQ'
      ENDIF
      IF(NCHOPP.NE.NCHF)STOP'READPQ: NCHPP.NE.NCHF'
      READ(21) (IOMIT2(I),I=1,NCHF)
      READ(21)((P2(I,J),I=J,NCHF),J=1,NCHF),
     X        ((Q2(I,J),I=J,NCHF),J=1,NCHF)    
!                                              !SEE IF THIS ENERGY IS ENOUGH,
      IF(ETOT.GT.E2+QETEST)GO TO 2             ! ELSE MOVE AND RE-READ
!
! NOW FORM NEW P,Q (SIMPLE LINEAR INTERPOLATION), CHECK ENERGIES O.K.
!
   3    IF(ETOT.LT.E1-2.*QETEST.OR.ETOT.GT.E2+2.*QETEST)THEN
          WRITE(6,*)' CHECK JBIN? ENERGY MIS-MATCH, E1,E2,ETOT'
     X              ,E1,E2,ETOT
          STOP 'ENERGY MIS-MATCH'
        ENDIF
!
! WATCH-OUT FOR MIS-MATCH IN OMITTED CHANNELS
!
        IFOM=0
        DO J=1,NCHF
          IOMIT(J)=IOMIT1(J)+IOMIT2(J)
          IF(IOMIT(J).EQ.1)THEN
            IF(IFOM.NE.0)THEN  !IF THERE'S MORE THAN MORE CASE......
!              WRITE(75,*)ETOT,IFOM,J
            ELSE
              IFOM=J           !USE ONE WITH LOWEST CHANNEL OMITTED
            ENDIF
          ENDIF
        ENDDO
        IF(IFOM.NE.0)THEN                      !DON'T INTERPOLATE, INSTEAD
!          IF(ABS(E2-ETOT).LT.ABS(ETOT-E1))THEN ! USE NEAREST
          IF(IOMIT2(IFOM).EQ.1)THEN            ! USE OMITTED, MORE STABLE
            T2=-1.0
            T1=0.0
            DO J=1,NCHF
              IOMIT(J)=IOMIT2(J)
            ENDDO
          ELSE
            T1=1.0
            T2=0.0
            DO J=1,NCHF
              IOMIT(J)=IOMIT1(J)
            ENDDO
          ENDIF
        ELSE                  !INDEED, SAFE TO INTERPOLATE
          T1=(E2-ETOT)/(E2-E1)
          T2=(E1-ETOT)/(E2-E1)
        ENDIF
!
! NOW FORM NEW P,Q
!
        DO J=1,NCHF
          DO I=J,NCHF
            P(I,J)=T1*P1(I,J)-T2*P2(I,J)
            Q(I,J)=T1*Q1(I,J)-T2*Q2(I,J)
            P(J,I)=P(I,J)
            Q(J,I)=Q(I,J)
          ENDDO
        ENDDO
!
! ZERO-OUT OMITTED CHANNELS (IF ANY IOMIT(ICHAN)=1). 
!  
        IF(IQDT.GT.0)THEN
          DO J=1,NCHF
            IF(IOMIT(J).EQ.1)THEN
              DO I=1,NCHF
                P(I,J)=TZERO
                Q(I,J)=TZERO
                P(J,I)=TZERO
                Q(J,I)=TZERO
              ENDDO
            ENDIF
          ENDDO
        ELSE
          DO I=1,NCHF
            IOMIT(I)=0
          ENDDO
        ENDIF
!
      NULLIFY(P,Q,P2,Q2)
      RETURN
!
!
 100  IF(IE.LE.0)THEN    !PQ FILE DOES NOT EXIST
        PQRD=.FALSE.
        REWIND(21)
      ELSE
! NO MORE DATA AVAILABLE, PRINT WARNING, AND CONTINUE WITH OLD P,Q.
        WRITE(6,*)' ****NO MORE PQ DATA ON FILE FOR THIS SYMMETRY'
     X            ,' USING LAST ENERGY'
      ENDIF
!
      NULLIFY(P,Q,P2,Q2)
      RETURN
      END
!***************************************************************
!
      SUBROUTINE RINIT
      USE WMATALLOC, ONLY : MXEONE,WMAT,VALUE
      USE NRBRCT, ONLY : RMAT
      USE PART
      USE DBUT
      USE CEN
      USE CINPUT
      USE CINPTX
      USE CHAN
      USE CPOINT
      USE CNTRL
      USE CLOGB
!
! NRB: USES WMAT WITH INDEXES INTERCHANGED COMPARED TO CPC.
!
!  INITIALIZE CALCULATION OF R-MATRIX.
!
      IMPLICIT REAL*8 (A-H,O-Y)
      IMPLICIT COMPLEX*16 (Z)
!
      PARAMETER (ONE=1.D0)
      PARAMETER (TZERO=0.D0)
!
      ALLOCATABLE :: TEMP1A(:),TEMP2A(:,:),RMATA(:,:)
!      
!    ALLOCATE
!
      ALLOCATE (TEMP1A(MNP2))
      ALLOCATE (TEMP2A(NCHF,MNP2),STAT=IERR)
      IF(IERR.NE.0)STOP 'FAILURE TO ALLOCATE TEMP2A'
!
!    INITIALISE 
!
      ALLOCATE (RMATA(NCHF,NCHF),STAT=IERR)
      RMATA(1:NCHF,1:NCHF)=TZERO
!
! PRELIMINARY SET UP
!
      DO K=1,MNP2
        TEMP1A(K)=ONE/(VALUE(K)-ETOT)-EZERO
      ENDDO
      DO I=1,NCHF
        DO K=1,MNP2
          TEMP2A(I,K)=TEMP1A(K)*WMAT(K,I)
        ENDDO
      ENDDO
!
!    FORM R-MATRIX (TIME CONSUMING)
!
      DEALLOCATE(TEMP1A,STAT=IERR)
      IF(IERR.NE.0)STOP 'FAILURE TO DEALLOCATE TEMP1A IN RINIT' 
!
!STRT BLAS
      CALL DGEMM('N','N',NCHF,NCHF,MNP2,ONE,TEMP2A,NCHF,WMAT,MNP2,
     X           TZERO,RMATA,NCHF)
!END BLAS 
!
      IF(ALLOCATED(RMAT))DEALLOCATE(RMAT)
      ALLOCATE(RMAT(NCHF,NCHF))
      RMAT(1:NCHF,1:NCHF)=RMATA(1:NCHF,1:NCHF)
!
!    OR DDOT TO GENERATE LOWER HALF RMAT (HARD ON CACHE)
!BL   DO I=1,NCHF
!BL     DO J=I,NCHF
!BL       RMAT(J,I)=RMAT(J,I)+DDOT(MNP2,TEMP2A(I,1),1,WMAT(1,J),1)
!BL     ENDDO
!BL   ENDDO
!
!STRTNBL
!NBL  DO I=1,NCHF
!NBL    DO J=I,NCHF
!NBL      DO K=1,MNP2
!NBL        RMAT(J,I)=RMAT(J,I)+TEMP2A(I,K)*WMAT(K,J)
!NBL      ENDDO
!NBL    ENDDO
!NBL  ENDDO
!ENDNBL
!
!    ADD BUTTLE CORRECTION TO DIAGONAL ELEMENTS 
!
      IF(IWORD.EQ.1)THEN
        IF(NEWBUT)THEN
          RA2=RZERO*RZERO
          DO I=1,NCHF
            L=L2P(I)+1
            NBUT=COEFF(3,L)
            RMAT(I,I)=RMAT(I,I)+COEFF(1,L)*BUT0(NBUT,COEFF(2,L)+
     &                RA2*EPS(I))
          ENDDO
        ELSE
          DO I=1,NCHF
            L=L2P(I)+1
            E=EPS(I)
            RMAT(I,I)=RMAT(I,I)+COEFF(1,L)+E*(COEFF(2,L)+E*COEFF(3,L))
          ENDDO
        ENDIF
      ELSE                     !DARC
        DO I=1,NCHF
          L=L2P(I)+1
          E=EPS(I)
          CALL INTBUT(L,E,BUTT)
          RMAT(I,I)=RMAT(I,I)+BUTT
        ENDDO
      ENDIF
!
!    NOW ADD-IN ANY PARTITIONED CORRECTION
!
      EZERO=TZERO
      IF(IPRCENT.NE.100)THEN
        EZERO=ONE/(TRACE-ETOT)
        DO I=1,NCHF
          RMAT(I,I)=RMAT(I,I)+SI(I)*EZERO
          L=L2P(I)+1
          DO N=NRANG1(L),NRANG2
            V=ONE/(EIGENS(N,L)-ETOT)
            RMAT(I,I)=RMAT(I,I)+ENDS(N,L)*(V-EZERO)
          ENDDO
        ENDDO
      ENDIF
!
!    SYMMETRISE (UNNECESSARY IF DGEMM USED)
!
!DG   DO I=1,NCHF
!DG   DO J=I+1,NCHF
!DG    RMAT(I,J)=RMAT(J,I)
!DG   ENDDO
!DG   ENDDO
!
!    DEALLOCATE
!
      DEALLOCATE(TEMP2A,STAT=IERR)
      IF(IERR.NE.0)STOP 'FAILURE TEMP2A TO DEALLOCATE IN RINIT'
!
      DEALLOCATE(RMATA,STAT=IERR)
      IF(IERR.NE.0)STOP 'FAILURE  RMATA TO DEALLOCATE IN RINIT'
!     **************** CPB 2008 **************************
!
      IF(MXEONE.EQ.1)THEN
      DEALLOCATE(WMAT,STAT=IERR)
!      WRITE(6,*)'ONLY 1 ENERGY PT PER PARTIAL PER PROC'
      IF(IERR.NE.0)STOP 'FAILURE TO DEALLOCATE IN WMAT IN RINIT'
      ENDIF
!
!      ****************************************************      
!
      RETURN
      END
!***************************************************************
!!
!      SUBROUTINE RINIT_CUDA
!      USE WMATALLOC, ONLY : MXEONE,WMAT,VALUE
!      USE NRBRCT, ONLY : RMAT
!      USE mmul_mod
!      USE PART
!      USE DBUT
!      USE CEN
!      USE CINPUT
!      USE CINPTX
!      USE CHAN
!      USE CPOINT
!      USE CNTRL
!      USE CLOGB
!!
!! NRB: USES WMAT WITH INDEXES INTERCHANGED COMPARED TO CPC.
!!
!!  INITIALIZE CALCULATION OF R-MATRIX.
!!
!      IMPLICIT REAL*8 (A-H,O-Y)
!      IMPLICIT COMPLEX*16 (Z)
!
!      integer idevice, istat
!!
!      PARAMETER (ONE=1.D0)
!      PARAMETER (TZERO=0.D0)
!!
!!      ALLOCATABLE :: TEMP2A(:,:),RMATA(:,:),WMATA(:,:)
!      ALLOCATABLE :: TEMP1A(:),TEMP2AA(:,:),RMATAA(:,:),WMATAA(:,:)
!!      
!!    ALLOCATE
!!
!      EZERO=0.0d0
!      if(allocated(RMAT)) deallocate(RMAT)
!      allocate(RMAT(nchf,nchf))  ! allocate main rmat
!!
!      NCHF4=NCHF/4
!      IA=MOD(NCHF4-1,igpusplt)+1
!      NCHFS=NCHF4+(igpusplt-IA)
!
!      IB=MOD(MNP2-1,igpusplt)+1    !just maintain mmul black size
!      IMATF=MNP2+(igpusplt-IB)
!
!      ALLOCATE (TEMP1A(IMATF))
!      TEMP1A=0.0d0
!!
!!    INITIALISE 
!!
!      ALLOCATE (RMATAA(NCHFS,NCHFS),STAT=IERR)
!      RMATAA=0.0d0
!      ALLOCATE (WMATAA(IMATF,NCHFS),STAT=IERR)
!      WMATAA=0.0d0
!!
!      DO K=1,MNP2
!        TEMP1A(K)=ONE/(VALUE(K)-ETOT)-EZERO
!      ENDDO 
!!
!      ALLOCATE (TEMP2AA(NCHFS,IMATF),STAT=IERR)
!      IF(IERR.NE.0)STOP 'FAILURE TO ALLOCATE TEMP2A'
!      TEMP2AA=0.0d0
!!
!      idevice=0
!      istat = cudaSetDevice(idevice)
!!
!!     1st block: RMAT(0:1/4,0:1) 
!!
!      DO I=1,NCHFS
!        DO K=1,MNP2
!          TEMP2AA(I,K)=TEMP1A(K)*WMAT(K,I)
!        ENDDO
!      ENDDO
!!
!!     1) RMAT(0:1/4,0:1/4)
!      WMATAA(1:MNP2,1:NCHFS)=WMAT(1:MNP2,1:NCHFS)
!      call cpu_time(tii)
!      call mmul(temp2aa,wmataa,rmataa)
!!      write(0,*)'after 1'
!      RMAT(1:nchfs,1:nchfs)=rmataa(1:nchfs,1:nchfs)
!!
!!     2) RMAT(0:1/4,1/4:2/4) 
!      rmataa=0.0d0   ! Not necessary, it is INTENT(out)
!      WMATAA(1:MNP2,1:NCHFS)=WMAT(1:MNP2,NCHFS+1:2*NCHFS)
!      call mmul(temp2aa,wmataa,rmataa)
!!      write(0,*)'after 2'
!      RMAT(1:nchfs,nchfs+1:2*nchfs)=rmataa(1:nchfs,1:nchfs)
!!
!!     3) RMAT(0:1/4,2/4:3/4) 
!      rmataa=0.0d0   ! Not necessary, it is INTENT(out)
!      WMATAA(1:MNP2,1:NCHFS)=WMAT(1:MNP2,2*NCHFS+1:3*NCHFS)
!      call mmul(temp2aa,wmataa,rmataa)
!!      write(0,*)'after 3'
!      RMAT(1:nchfs,2*nchfs+1:3*nchfs)=rmataa(1:nchfs,1:nchfs)
!!
!!     4) RMAT(0:1/4,3/4:4/4) 
!      rmataa=0.0d0   ! Not necessary, it is INTENT(out)
!      WMATAA(1:MNP2,1:NCHFS)=WMAT(1:MNP2,NCHF-NCHFS+1:NCHF)
!      call mmul(temp2aa,wmataa,rmataa)
!!      write(0,*)'after 4'
!      RMAT(1:nchfs,nchf-nchfs+1:nchf)=rmataa(1:nchfs,1:nchfs)
!!      
!!     2nd block  RMAT(1/4:2/4,1/4:1) 
!!
!      DO I=1,NCHFS
!        DO K=1,MNP2
!          TEMP2AA(I,K)=TEMP1A(K)*WMAT(K,NCHFS+I)
!        ENDDO
!      ENDDO
!!
!!     7) RMAT(1/4:2/4,3/4:4/4) 
!      rmataa=0.0d0   ! Not necessary, it is INTENT(out)
!!      WMATAA(1:MNP2,1:NCHFS)=WMAT(1:MNP2,NCHF-NCHFS+1:NCHF) already done
!      call mmul(temp2aa,wmataa,rmataa)
!!      write(0,*)'after 7'
!      RMAT(nchfs+1:2*nchfs,nchf-nchfs+1:nchf)=rmataa(1:nchfs,1:nchfs)
!!
!!     6) RMAT(1/4:2/4,2/4:3/4) 
!      rmataa=0.0d0   ! Not necessary, it is INTENT(out)
!      WMATAA(1:MNP2,1:NCHFS)=WMAT(1:MNP2,2*NCHFS+1:3*NCHFS)
!      call mmul(temp2aa,wmataa,rmataa)
!!      write(0,*)'after 6'
!      RMAT(nchfs+1:2*nchfs,2*nchfs+1:3*nchfs)=rmataa(1:nchfs,1:nchfs)
!!
!!     5) RMAT(1/4:2/4,1/4:2/4) 
!      WMATAA(1:MNP2,1:NCHFS)=WMAT(1:MNP2,NCHFS+1:2*NCHFS)
!      call mmul(temp2aa,wmataa,rmataa)
!!      write(0,*)'after 5'
!      RMAT(nchfs+1:2*nchfs,nchfs+1:2*nchfs)=rmataa(1:nchfs,1:nchfs)
!!
!!      3rd block  RMAT(2/4:3/4,2/4:1) 
!!
!      DO I=1,NCHFS
!        DO K=1,MNP2
!          TEMP2AA(I,K)=TEMP1A(K)*WMAT(K,2*NCHFS+I)
!        ENDDO
!      ENDDO
!!     8) RMAT(2/4:3/4,2/4:3/4) 
!      rmataa=0.0d0   ! Not necessary, it is INTENT(out)
!      WMATAA(1:MNP2,1:NCHFS)=WMAT(1:MNP2,2*NCHFS+1:3*NCHFS)
!      call mmul(temp2aa,wmataa,rmataa)
!!      write(0,*)'after 8'
!      RMAT(2*nchfs+1:3*nchfs,2*nchfs+1:3*nchfs)=rmataa(1:nchfs,1:nchfs)
!!
!!     9) RMAT(2/4:3/4,3/4:4/4) 
!      rmataa=0.0d0   ! Not necessary, it is INTENT(out)
!      WMATAA(1:MNP2,1:NCHFS)=WMAT(1:MNP2,NCHF-NCHFS+1:NCHF)
!      call mmul(temp2aa,wmataa,rmataa)
!!      write(0,*)'after 9'
!      RMAT(2*nchfs+1:3*nchfs,nchf-nchfs+1:nchf)=rmataa(1:nchfs,1:nchfs)
!!
!!     4th block  RMAT(3/4:4/4,3/4:4/4) 
!!
!      DO I=1,NCHFS
!        DO K=1,MNP2
!          TEMP2AA(I,K)=TEMP1A(K)*WMAT(K,NCHF-NCHFS+I)
!        ENDDO
!      ENDDO
!!     10) RMAT(3/4:4/4,3/4:4/4) 
!!      WMATAA(1:MNP2,1:NCHFS)=WMAT(1:MNP2,NCHF-NCHFS+1:NCHF) already done
!      call mmul(temp2aa,wmataa,rmataa)
!      call cpu_time(ttt)
!!      write(0,*)'after 10'
!      write(0,*)'time = ',ttt-tii,' secs'
!      RMAT(nchf-nchfs+1:nchf,nchf-nchfs+1:nchf)=rmataa(1:nchfs,1:nchfs)
!!
!      deallocate(temp2aa,wmataa,rmataa,temp1a)
!!
!! Add the Buttle correction
!!
!      IF(IWORD.EQ.1)THEN
!        IF(NEWBUT)THEN
!          RA2=RZERO*RZERO
!          DO I=1,NCHF
!            L=L2P(I)+1
!            NBUT=COEFF(3,L)
!            RMAT(I,I)=RMAT(I,I)+
!     &              COEFF(1,L)*BUT0(NBUT,COEFF(2,L)+RA2*EPS(I))
!          ENDDO
!        ELSE
!          DO I=1,NCHF
!            L=L2P(I)+1
!            E=EPS(I)
!        RMAT(I,I)=RMAT(I,I)+COEFF(1,L)+E*(COEFF(2,L)+E*COEFF(3,L))
!          ENDDO
!        ENDIF
!      ELSE                     !DARC
!        DO I=1,NCHF
!          L=L2P(I)+1
!          E=EPS(I)
!          CALL INTBUT(L,E,BUTT)
!          RMAT(I,I)=BUTT+RMAT(I,I)
!        ENDDO
!      ENDIF
!!
!!    NOW ADD-IN ANY PARTITIONED CORRECTION
!!
!      EZERO=TZERO
!      IF(IPRCENT.NE.100)THEN
!        EZERO=ONE/(TRACE-ETOT)
!        DO I=1,NCHF
!          RMAT(I,I)=RMAT(I,I)+SI(I)*EZERO
!          L=L2P(I)+1
!          DO N=NRANG1(L),NRANG2
!            V=ONE/(EIGENS(N,L)-ETOT)
!            RMAT(I,I)=RMAT(I,I)+ENDS(N,L)*(V-EZERO)
!          ENDDO
!        ENDDO
!      ENDIF
!!
!!    PRELIMINARY SET-UP 
!!
!!
!!    OR DDOT TO GENERATE LOWER HALF RMAT (HARD ON CACHE)
!!
!!BL   DO I=1,NCHF
!!BL     DO J=I,NCHF
!!BL       RMAT(J,I)=RMAT(J,I)+DDOT(MNP2,TEMP2A(1,I),1,WMAT(1,J),1)
!!BL     ENDDO
!!BL   ENDDO
!!ENDBL
!!
!!STRTNBL
!!NBL  DO I=1,NCHF
!!NBL    DO J=I,NCHF
!!NBL      DO K=1,MNP2
!!NBL        RMAT(J,I)=RMAT(J,I)+TEMP2A(K,I)*WMAT(K,J)
!!NBL      ENDDO
!!NBL    ENDDO
!!NBL  ENDDO
!!ENDNBL
!!
!!    SYMMETRISE (UNNECESSARY IF DGEMM USED)
!!
!      DO I=1,NCHF
!       DO J=I+1,NCHF
!        RMAT(J,I)=RMAT(I,J)
!       ENDDO
!      ENDDO
!!
!!    DEALLOCATE
!!
!!      DEALLOCATE(TEMP2A,STAT=IERR)
!!      IF(IERR.NE.0)STOP 'FAILURE TEMP2A TO DEALLOCATE IN RINIT'
!!
!!      DEALLOCATE(RMATA,STAT=IERR)
!!      IF(IERR.NE.0)STOP 'FAILURE  RMATA TO DEALLOCATE IN RINIT'
!!     **************** CPB 2008 **************************
!!
!!
!!      ****************************************************      
!!
!      RETURN
!      END
!***************************************************************
!
      SUBROUTINE RINIT2
      USE WMATALLOC, ONLY : MXEONE,VALUE,WMAT
      USE NRBRCT, ONLY : RMAT
      USE PART
      USE DBUT
      USE CEN
      USE CINPUT
      USE CINPTX
      USE CHAN
      USE CPOINT
      USE CNTRL
      USE CLOGB
!
! LFM: USES SCRATCH FILES, VERY SLOW BUT EFFICIENT FOR VERY LARGE MATRIXES
! NRB: USES WMAT WITH INDEXES INTERCHANGED COMPARED TO CPC.
!
!  INITIALIZE CALCULATION OF R-MATRIX.
!
      IMPLICIT REAL*8 (A-H,O-Y)
      IMPLICIT COMPLEX*16 (Z)
!
      PARAMETER (ONE=1.D0)
      PARAMETER (TZERO=0.D0)
!
      ALLOCATABLE :: TEMP1A(:),TEMP2A(:,:)
      ALLOCATABLE :: RMATB(:,:),WMATA(:,:)
!
      LOGICAL :: ISOPEN 
!     
!
      ALLOCATE (TEMP1A(MNP2),STAT=IERR)
      IF(IERR.NE.0)STOP ' FAILURE TO ALLOCATE TEMP1A IN RINIT'
!
!    INITIALISE 
!
!
       IF(ALLOCATED(RMAT))DEALLOCATE(RMAT)
!
!    PRELIMINARY SET-UP 
!
      DO K=1,MNP2
        TEMP1A(K)=ONE/(VALUE(K)-ETOT)-EZERO
      ENDDO
!
!      3 DGEMM MULTIPLICATIONS INSTEAD OF ONE, BUT 
!      A 50 % REDUCTION IN MEMORY OVERHEAD       CPB 2008
!
      OPEN(5000,STATUS='SCRATCH',
     X         FORM='UNFORMATTED')
!
      IF(MOD(NCHF,2).EQ.0)THEN
      ISPLIT=NCHF/2
      IODD=0
      ELSE
      ISPLIT=(NCHF/2)+1
      IODD=1
      ENDIF
!
!
!
      ALLOCATE(RMATB(ISPLIT,ISPLIT),STAT=IERR)

      IF(IERR.NE.0)STOP ' FAILURE TO ALLOCATE RMATB IN RINIT'
!
      DO I=1,ISPLIT
       DO J=1,ISPLIT
        RMATB(J,I)=0.0D0
       ENDDO
      ENDDO
!
!
      IF(.NOT.ALLOCATED(WMAT))THEN

        ALLOCATE(WMATA(MNP2,1:ISPLIT),STAT=IERR)
      IF(IERR.NE.0)STOP ' FAILURE TO ALLOCATE WMATA IN RINIT'

       REWIND(5000)
       READ(5000)((WMATA(I,J),I=1,MNP2),J=1,ISPLIT)

      ELSE

      INQUIRE(5000,OPENED=ISOPEN)

      IF(ISOPEN)CLOSE(5000)

      OPEN(5000,STATUS='SCRATCH',
     X         FORM='UNFORMATTED')
!
      WRITE(5000)((WMAT(I,J),I=1,MNP2),J=1,ISPLIT)
      WRITE(5000)((WMAT(I,J),I=1,MNP2),J=NCHF-ISPLIT+1,NCHF)
!
      DEALLOCATE(WMAT,STAT=IERR)
      IF(IERR.NE.0)STOP ' FAILURE TO DEALLOCATE WMAT IN RINIT'
!      
        ALLOCATE(WMATA(MNP2,1:ISPLIT),STAT=IERR)
      IF(IERR.NE.0)STOP ' FAILURE TO ALLOCATE WMATA IN RINIT'
!
!
      REWIND(5000)
      READ(5000)((WMATA(I,J),I=1,MNP2),J=1,ISPLIT)

      ENDIF
!
        ALLOCATE(TEMP2A(MNP2,1:ISPLIT),STAT=IERR)
      IF(IERR.NE.0)STOP ' FAILURE TO ALLOCATE TEMP2A IN RINIT'
!
	DO JJ=1,ISPLIT
	 DO K=1,MNP2
	  TEMP2A(K,JJ)=WMATA(K,JJ)*TEMP1A(K)	  	  	  
	 ENDDO
	ENDDO
!
!
!       R_LEFTUPPER  :
!

!	
      CALL DGEMM('T','N',ISPLIT,ISPLIT,MNP2,ONE,TEMP2A,
     X           MNP2,WMATA,MNP2,
     X           ONE,RMATB,ISPLIT)                        
!
!
      DEALLOCATE(WMATA,STAT=IERR)
      IF(IERR.NE.0)STOP ' FAILURE TO DEALLOCATE WMATA IN RINIT'
!
!
      ALLOCATE(RMAT(NCHF,NCHF))
!
      DO I=1,NCHF
        DO J=1,NCHF
          RMAT(J,I)=TZERO
        ENDDO
      ENDDO
!
!
      DO I=1,ISPLIT
       DO J=I,ISPLIT
        RMAT(J,I)=RMATB(J,I)
       ENDDO
      ENDDO
!
      DO I=1,ISPLIT
       DO J=1,ISPLIT
        RMATB(J,I)=0.0D0
       ENDDO
      ENDDO
!
!
!     R_RIGHTUPPER   :
!
!
      ALLOCATE(WMATA(MNP2,NCHF-ISPLIT+1:NCHF),STAT=IERR)

      IF(IERR.NE.0)STOP ' FAILURE TO ALLOCATE WMATA IN RINIT'

      READ(5000)((WMATA(I,J),I=1,MNP2),J=NCHF-ISPLIT+1,NCHF)
!
      CALL DGEMM('T','N',ISPLIT,ISPLIT,MNP2,ONE,TEMP2A,
     X           MNP2,WMATA(1,NCHF-ISPLIT+1),MNP2,
     X           ONE,RMATB,ISPLIT)
!
!
!
      DEALLOCATE(TEMP2A,STAT=IERR)
      IF(IERR.NE.0)STOP 'FAILURE TO DEALLOCATE TEMP2A IN RINIT'
!
!
       DO I=1,ISPLIT
        DO J=1,ISPLIT
         RMAT(J+ISPLIT-IODD,I)=RMATB(I,J)
        ENDDO
       ENDDO
!
!      INITIALISE
!
       DO I=1,ISPLIT
        DO J=1,ISPLIT
        RMATB(J,I)=0.0D0
        ENDDO
       ENDDO
!
!
       ALLOCATE(TEMP2A(MNP2,NCHF-ISPLIT+1:NCHF),STAT=IERR)
       IF(IERR.NE.0)STOP ' FAILURE TO ALLOCATE TEMP2A IN RINIT'
!
        DO JJ=NCHF-ISPLIT+1,NCHF
         DO K=1,MNP2
          TEMP2A(K,JJ)=WMATA(K,JJ)*TEMP1A(K)
         ENDDO
        ENDDO
!
      DEALLOCATE(TEMP1A,STAT=IERR)
      IF(IERR.NE.0)STOP 'FAILURE TO DEALLOCATE TEMP1A IN RINIT'
!
!
!      R_RIGHTLOWER
!
!
      CALL DGEMM('T','N',ISPLIT,ISPLIT,MNP2,ONE,TEMP2A,
     X           MNP2,WMATA(1,NCHF-ISPLIT+1),MNP2,
     X           ONE,RMATB,ISPLIT)
!
!
      DEALLOCATE(TEMP2A,STAT=IERR)
      IF(IERR.NE.0)STOP 'FAILURE TEMP2A TO DEALLOCATE IN RINIT'
!
      DEALLOCATE(WMATA,STAT=IERR)
      IF(IERR.NE.0)STOP 'FAILURE WMATA TO DEALLOCATE IN RINIT'
!
       DO I=1,ISPLIT
        DO J=I,ISPLIT
         RMAT(J+ISPLIT-IODD,I+ISPLIT-IODD)=RMATB(I,J)
        ENDDO
       ENDDO 
!
      DEALLOCATE(RMATB,STAT=IERR)
      IF(IERR.NE.0)STOP 'FAILURE RMATB TO DEALLOCATE IN RINIT'       
!
!
!    SYMMETRISE 
!
      DO I=2,NCHF
       DO J=1,I-1
        RMAT(J,I)=RMAT(I,J)
        ENDDO
      ENDDO
!
!      NOW ADD IN THE DIAGONAL BUTTLE CORRECTION
!
!
      IF(IWORD.EQ.1)THEN
   	IF(NEWBUT)THEN
   	  RA2=RZERO*RZERO
   	  DO I=1,NCHF
            L=L2P(I)+1
   	    NBUT=COEFF(3,L)
   	 RMAT(I,I)=RMAT(I,I)+COEFF(1,L)*BUT0(NBUT,COEFF(2,L)+RA2*EPS(I))
   	  ENDDO
   	ELSE
   	  DO I=1,NCHF
   	    L=L2P(I)+1
   	    E=EPS(I)
   	    RMAT(I,I)=RMAT(I,I)+COEFF(1,L)+E*(COEFF(2,L)+E*COEFF(3,L))
   	  ENDDO
   	ENDIF
      ELSE                     !DARC
        DO I=1,NCHF
   	  L=L2P(I)+1
          E=EPS(I)
          CALL INTBUT(L,E,BUTT)
          RMAT(I,I)=RMAT(I,I)+BUTT
        ENDDO
      ENDIF
!
!    NOW ADD-IN ANY PARTITIONED CORRECTION
!
      EZERO=TZERO
      IF(IPRCENT.NE.100)THEN
        EZERO=ONE/(TRACE-ETOT)
        DO I=1,NCHF
          RMAT(I,I)=RMAT(I,I)+SI(I)*EZERO
          L=L2P(I)+1
          DO N=NRANG1(L),NRANG2
            V=ONE/(EIGENS(N,L)-ETOT)
            RMAT(I,I)=RMAT(I,I)+ENDS(N,L)*(V-EZERO)
          ENDDO
        ENDDO
      ENDIF
!
!
!
!     **************** CPB 2008 **************************
!
      IF(MXEONE.EQ.1)THEN
      WRITE(6,*)'ONLY 1 ENERGY PT PER PARTIAL PER PROC'
       ELSE
!      WRITE(6,*)' > 1 ENERGY PT PER PARTIAL PER PROC'
!
!      BETTER PUT THOSE SURFACE AMPLITUDES BACK INTO WMAT
!      BEFORE LEAVING IF MORE THAN ONE ENERGY PER PROC
!
!       ALLOCATE(WMAT(MNP2,NCHF))
       REWIND(5000)
!       READ(5000)((WMAT(I,J),I=1,MNP2),J=1,ISPLIT)
!       READ(5000)((WMAT(I,J),I=1,MNP2),J=NCHF-ISPLIT+1,NCHF)      
!       
       ENDIF
!
!       CLOSE(5000)
!
!      ****************************************************      
!
      RETURN
      END
!***************************************************************
!
      SUBROUTINE RINIT4
      USE WMATALLOC, ONLY : MXEONE,VALUE,WMAT,WM1,WM2
      USE NRBRCT, ONLY : RMAT
      USE PART
      USE DBUT
      USE CEN
      USE CINPUT
      USE CINPTX
      USE CHAN
      USE CPOINT
      USE CNTRL
      USE CLOGB
!
! LFM - FEBRUARY 2018
! SPLITS THE MATRIXES WMAT IN BLOCKS TO IMPROVE THE EFFICIENCY
! OF LAPACK ROUTINE DGEMM
!
! NRB: USES WMAT WITH INDEXES INTERCHANGED COMPARED TO CPC.
!
!  INITIALIZE CALCULATION OF R-MATRIX.
!
      IMPLICIT REAL*8 (A-H,O-Y)
      IMPLICIT COMPLEX*16 (Z)
!
      PARAMETER (ONE=1.D0)
      PARAMETER (TZERO=0.D0)
!
      REAL*8, ALLOCATABLE :: TEMP1A(:),TEMP2A(:,:)
      REAL*8, ALLOCATABLE :: RMATB(:,:)
      INTEGER, ALLOCATABLE :: ISTART(:),IEND(:) 
      REAL*8, POINTER :: RMT(:)
!
!    INITIALISE 
!
      IF(ALLOCATED(RMAT)) DEALLOCATE(RMAT)
      ALLOCATE(RMAT(NCHF,NCHF))
!      RMAT(1:NCHF,1:NCHF)=TZERO
!
!    PRELIMINARY SET-UP 
!
      ALLOCATE (TEMP1A(MNP2),STAT=IERR)
      IF(IERR.NE.0) STOP ' FAILURE TO ALLOCATE TEMP1A IN RINIT'
      TEMP1A(1:MNP2)=ONE/(VALUE(1:MNP2)-ETOT)-EZERO
!
! GET THE OPTIMUM NUMBER OF DIVISION
! JUST DOING ISPLIT < 1500
!
      ISPLIT=1500
      ISPLITUP=(NCHF/ISPLIT)+1
      ALLOCATE(ISTART(ISPLITUP),IEND(ISPLITUP))
!
! FIX THE NUMBER OF DIVISIONS
      IODD=MOD(NCHF,ISPLITUP)
      ISPLIT=NCHF/ISPLITUP
      IF (IODD.EQ.0) THEN 
        ILEN=ISPLIT
      ELSE
        ILEN=ISPLIT+1
      END IF
!
! SET THE DIVISIONS
! THERE IS AN OVERLAP OF IODD COLUMNS BETWEEN ISTART(2) AND IEND(1)
      ISTART(1)=1
      IEND(1)=ILEN
      IEND(ISPLITUP)=NCHF
      ISTART(ISPLITUP)=NCHF-ILEN+1
      DO I=ISPLITUP-1,2,-1
        IEND(I)=ISTART(I+1)-1
        ISTART(I)=IEND(I)-ILEN+1
      ENDDO  
!      do jj=1,isplitup
!        write(0,'(" I, START, END",3I5)') JJ,ISTART(JJ),IEND(JJ)
!      end do
!
! ALLOCATIONS OF ALL THE MATRIXES
!
      ALLOCATE(RMATB(ILEN,ILEN),STAT=IERR)
      IF(IERR.NE.0) STOP ' FAILURE TO ALLOCATE RMATB IN RINIT'
      ALLOCATE(TEMP2A(MNP2,ILEN),STAT=IERR)
      IF(IERR.NE.0) STOP ' FAILURE TO ALLOCATE TEMP2A IN RINIT'
!
! START THE BY BLOCK PRODUCTS
!
!       RMATB(ILEN,ILEN) = TEMP2A^T(ILEN,MNP2) * WM1(MNP2,ILEN) 
! WITH  TEMP2A(MNP2,ILEN) = WM1(MNP2,ILEN) * 1/(E-E(J))
!  EQUIVALENT TO:
!       DO I=1,ILEN
!        DO J=1,ILEN
!          RMATB(I,J)=TZERO
!          DO K=1,MNP2
!            RMATB(I,J)=RMATB(I,J)+ ONE*TEMP2A(K,I)*WM1(K,J)
!          END DO
!        END DO
!       END DO
!
      DO I=1,ISPLITUP
! MATRIX TEMP2, EQUAL FOR ALL THE ROW-BLOCKS
        NULLIFY(WM2)
        WM2=>WMAT(1:MNP2,ISTART(I):IEND(I))
        DO II=1,ILEN
          TEMP2A(1:MNP2,II)=WM2(1:MNP2,II)*TEMP1A(1:MNP2)
        END DO
        DO J=I,ISPLITUP
! MATRIX WM1
          NULLIFY(WM1)
          WM1=>WMAT(1:MNP2,ISTART(J):IEND(J))
!
! DO THE PRODUCT, CALL TO DGEMM
!
          CALL DGEMM('T','N',ILEN,ILEN,MNP2,
     &               ONE,TEMP2A,MNP2,
     &               WM1,MNP2,
     &               TZERO,RMATB,ILEN)      
!
! STORE THE BLOCK OF RMAT
!
          RMAT(ISTART(I):IEND(I),ISTART(J):IEND(J)) =
     &    RMATB(1:ILEN,1:ILEN)
!
! SYMMETRIZE IF THE BLOCK IS NON-DIAGONAL
!          IF (I.NE.J) THEN
!            RMAT(ISTART(J):IEND(J),ISTART(I):IEND(I))=
!     &      TRANSPOSE(RMATB(1:ILEN,1:ILEN))
!          END IF
        END DO
      END DO
!
! SYMMETRIZE THE NON-DIAGONAL BLOCKS
!
      DO I=1,ISPLITUP-1
! THE ROW TO TRANSPOSE COVERS ALL THE ROW-BLOCKS (I,I+1:ISPLITUP)
!       JINI=I+1
!       JFIN=ISPLITUP
        DO II=1,ILEN
          NULLIFY(RMT)
          RMT=>RMAT(ISTART(I)-1+II,ISTART(I+1):IEND(ISPLITUP))
          RMAT(ISTART(I+1):IEND(ISPLITUP),ISTART(I)-1+II)=RMT
        END DO
      END DO
!
! DEALLOCATIONS
      DEALLOCATE(ISTART,IEND)
      DEALLOCATE(TEMP2A,STAT=IERR)
      IF(IERR.NE.0) STOP 'FAILURE TEMP2A TO DEALLOCATE IN RINIT'
      DEALLOCATE(TEMP1A,STAT=IERR)
      IF(IERR.NE.0) STOP 'FAILURE TO DEALLOCATE TEMP1A IN RINIT'
      DEALLOCATE(RMATB,STAT=IERR)
      IF(IERR.NE.0) STOP 'FAILURE RMATB TO DEALLOCATE IN RINIT'       
!
!     NOW ADD IN THE DIAGONAL BUTTLE CORRECTION
!
      IF (IWORD.EQ.1) THEN
        IF (NEWBUT) THEN
          RA2=RZERO*RZERO
          DO I=1,NCHF
            L=L2P(I)+1
            NBUT=COEFF(3,L)
            RMAT(I,I)=RMAT(I,I)+
     &                COEFF(1,L)*BUT0(NBUT,COEFF(2,L)+RA2*EPS(I))
          ENDDO
        ELSE
          DO I=1,NCHF
            L=L2P(I)+1
            E=EPS(I)
            RMAT(I,I)=RMAT(I,I)+
     &                COEFF(1,L)+E*(COEFF(2,L)+E*COEFF(3,L))
          ENDDO
        ENDIF
      ELSE                     !DARC
        DO I=1,NCHF
          L=L2P(I)+1
          E=EPS(I)
          CALL INTBUT(L,E,BUTT)
          RMAT(I,I)=RMAT(I,I)+BUTT
        ENDDO
      ENDIF
!
!    NOW ADD-IN ANY PARTITIONED CORRECTION
!
      EZERO=TZERO
      IF (IPRCENT.NE.100) THEN
        EZERO=ONE/(TRACE-ETOT)
        DO I=1,NCHF
          RMAT(I,I)=RMAT(I,I)+SI(I)*EZERO
          L=L2P(I)+1
          DO N=NRANG1(L),NRANG2
            V=ONE/(EIGENS(N,L)-ETOT)
            RMAT(I,I)=RMAT(I,I)+ENDS(N,L)*(V-EZERO)
          ENDDO
        ENDDO
      ENDIF
!
!     **************** CPB 2008 **************************
!
      IF (MXEONE.EQ.1) THEN
        WRITE(6,*) 'ONLY 1 ENERGY PT PER PARTIAL PER PROC'
        DEALLOCATE(WMAT)
      ENDIF
!
!      ****************************************************      
!
       RETURN
       END SUBROUTINE RINIT4
!***************************************************************
!
      SUBROUTINE SC(E,L,R,AC,S,SP,C,CP)
      USE NRBOMT
      USE NRBZED
!
!  COULOMB FUNCTIONS S AND C FROM
!  POWER SERIES EXPANSIONS.
! NRB:
! MODIFIED TO HANDLE DEEPLY BOUND CHANNELS.
! NEUTRAL CASE TO BE ADDED.
!
      IMPLICIT REAL*8 (A-H,O-Y)
      IMPLICIT COMPLEX*16 (Z)
      PARAMETER (XEPS=1.E-1)
      PARAMETER (TZERO=0.0)
      PARAMETER (ONE=1.0)
      PARAMETER (TWO=2.0)
!
      PI=ACOS(-ONE)
      TPI=TWO*PI
!
!  NEUTRAL CASE
!
      IF(TZED.EQ.0)THEN
!
        IF(E.LT.0)THEN
          WRITE(6,*)'USE STGFDAMP FOR NEUTRAL MQDT'
          STOP 'USE STGFDAMP FOR NEUTRAL MQDT'
        ENDIF
!
        CALL SC0(E,L,R,AC,ZS,S,SP,ZC,C,CP,IERR)
!
        RETURN
      ENDIF
!
!  POWER SERIES FOR F AND G (POSITIVE IONS ONLY)
!
      CALL COULFG(L,E,R,AC,F,FP,G,GP,K,IERR,ACT)
!
      FNU=E
      IF(E.LT.0.0)FNU=ONE/SQRT(-E)
!
      IF(IERR.GT.0)THEN
        IF(IFLEG.LE.0)THEN
          IF(IERR.EQ.1)WRITE(6,610)ICHAN,L,FNU,K,ACT
          IF(IERR.EQ.2)WRITE(6,600)ICHAN,L,FNU,ACT
          IF(IERR.EQ.3)WRITE(6,602)ICHAN,L,FNU
          IFLEG=-1
        ENDIF
        IF(IERR.GT.1)THEN
          IF(IFLEG.LE.0)THEN
            WRITE(6,*)' ***OMITTING THIS CHANNEL***',ICHAN
            WRITE(6,*)'***NO MORE SIMILAR WARNINGS FOR THIS SYMMETRY***'
            IFLEG=-1
          ENDIF
          IOMIT(ICHAN)=1
! ALLOW EXECUTION TO PROCEED AND OMIT LATER
          S=0.5
          SP=-0.05
          C=100.
          CP=-10.
          RETURN
        ELSE
          TEST=10.*AC
          IF(ABS(ACT).GT.TEST)THEN
            IF(IFLEG.LE.0)THEN
            WRITE(6,*)' ***OMITTING THIS CHANNEL***',ICHAN
            WRITE(6,*)'***NO MORE SIMILAR WARNINGS FOR THIS SYMMETRY***'
            IFLEG=-1
            ENDIF
            IOMIT(ICHAN)=1
            RETURN
          ENDIF
        ENDIF
      ENDIF
!
!  CASE OF E.GE.0
!
      IF(E.GE.0)THEN
!  CALCULATE CAP. A
      A=ONE
      IF(L.GT.0)THEN
      A1=ONE
      A2=-E
      A3=E+E
      DO I=1,L
      A2=A2+A3
      A1=A1+A2
      A=A*A1
      ENDDO
      ENDIF
!  CALCULATE SCRIPT G AND COULOMB FUNCTION H AND DERIVATIVE
      SG=A*FKHI(E,L,AC)/PI
      H=-G-SG*F
      HP=-GP-SG*FP
!  CALCULATE CAP B
      IF(E.LT.0.01)THEN
      B=A
      ELSE
      B=A/(ONE-EXP(-TPI/SQRT(E)))
      ENDIF
!
      ELSE
!
!  CASE OF E.LT.0
!
      CALL ABG(E,L,AC,A,BG)
!
      IF(A.LT.0.0.OR.(FNU-XEPS).LT.L)THEN
      IF(IFLAG.LE.0)THEN
        WRITE(6,*)' WARNING, N.LT.L+0.1 IN SUBROUTINE SC',
     X  ' ICHAN,E,N,L = ',ICHAN,E,FNU,L
        IF(IOMSW.GT.0)WRITE(6,*)' ***OMITTING THIS CHANNEL***',ICHAN
        WRITE(6,*)'***NO MORE SIMILAR WARNINGS FOR THIS SYMMETRY***'
        IFLAG=-1
      ENDIF
      IF(A.LT.0.0)THEN
        A=-A
        IF(IOMSW.LE.0)IOMIT(ICHAN)=-1
      ENDIF
      IF(IOMSW.GT.0)IOMIT(ICHAN)=1
      ENDIF
!
      H=-(G+BG*F)
      HP=-(GP+BG*FP)
      B=A
      ENDIF
!
!  COMPLETE CALCULATION OF S AND C
!
      C1=SQRT(B*PI/TWO)
      S=C1*F
      SP=C1*FP
      C1=C1/B
      C=H*C1
      CP=HP*C1
!
      RETURN
!
  600 FORMAT(///10X,'SERIES IN COULFG NOT CONVERGED'/5X,'ICHAN= ',I3,
     +', SMALL L = ',I3,', EPS/NU = ',1PE12.4,', ACTACC = ',1PE12.4//)
  602 FORMAT(///10X,'SERIES IN COULFG NOT CONVERGED'/5X,'ICHAN= ',I3
     +,', SMALL L = ',I3,', EPS/NU = ',1PE12.4//)
  610 FORMAT(//5X,'*** FUNCTIONS FROM COULFG INACCURATE ***'/
     +5X,'ICHAN= ',I3,', SMALL L =',I3,', EPS/NU = ',1PE12.4,', K = ',
     +I3,', ACTACC = ',E12.4//)
!
      END
!**********************************************************
!
      SUBROUTINE SC0(EPS,LL,RHO,AC,ZS,S,SP,ZC,C,CP,IERR)
      USE NRBOMT
      USE CNTRL
      USE NRBZED
!
!NRB
!  CALCULATES REGULAR AND IRREGULAR SPHERICAL BESSEL FUNCTIONS
!  S AND C AND THEIR DERIVATIVES SP, CP FROM POWER-SERIES EXPANSION.
!  ZS AND ZC ARE THE COMLPEX CONSTANTS TO BE MULTIPLED BY, CASE EPS.LT.0
!
      IMPLICIT REAL*8 (A-H,O-Y)
      IMPLICIT COMPLEX*16 (Z)
      PARAMETER (TZERO=0.0)
      PARAMETER (ONE=1.0)
      PARAMETER (TWO=2.0)
!
! INITIALISATIONS
!
      IF(TZED.NE.0)THEN
        WRITE(6,*)'ERROR: SC0 IS FOR NEUTRALS ONLY'
        STOP 'ERROR: SC0 IS FOR NEUTRALS ONLY'
      ENDIF
      IF(EPS.EQ.TZERO)THEN
        S=TZERO
        SP=TZERO
        C=TZERO
        CP=TZERO
        IOMIT(ICHAN)=1
        RETURN
      ENDIF
!
      IERR=0
      ACC10=.1*AC
      DEPS=ABS(EPS)
      T0=SQRT(SQRT(DEPS))
      LAM=-3*LL
!
! LOOP OVER C AND S GENERATION
!
      DO LLL=-1,0
!
      LAM=LAM+2*LL
      FLP1=LAM+1+LLL
!
! POWER-SERIES EXPANSION
! **********************
!
! VALUES FOR N=0
      FNPLP1=FLP1
      C1=RHO**(LAM+LLL)
      U0=C1
      D0=FNPLP1*U0
      DM=ABS(D0)
      FP=D0
      U0=U0*RHO
      UM=ABS(U0)
      F=U0
!
! INITIALIZE FOR COEFFICIENTS IN RECURSION FORMULAE
      P1=-TWO*FLP1
      P2=P1
      REPS=RHO*EPS
!
! LOOP FOR N=2 TO 400, 2
      DO N=2,400,2
! COMPUTE COEFFICIENTS IN RECURSION FORMULAE
        P1=P1-TWO
        P2=P2+P1
! NOW HAVE P2=-N*(N+LAM)
! COMPUTE U2 AND INCREMENT FP
        FNPLP1=FNPLP1+TWO
        U2=REPS*U0/P2
        D2=FNPLP1*U2
        DM=MAX(ABS(D2),DM)
        FP=FP+D2
! MODIFY U2 AND INCREMENT F
        U2=U2*RHO
        UM=MAX(ABS(U2),UM)
        F=F+U2
! TEST CONVERGENCE
        IF(ABS(U2).LT.ABS(F)*ACC10.AND.ABS(D2).LT.ABS(FP)*ACC10)THEN
          IF(IPRINT.GT.1)THEN
            UM=UM/ABS(F)
            DM=DM/ABS(FP)
            IF(EPS.GT.TZERO)THEN
              WRITE(6,610)ICHAN,LL,EPS,UM
              WRITE(6,620)ICHAN,LL,EPS,DM
            ELSE
              FNU=ONE/SQRT(-EPS)
              WRITE(6,611)ICHAN,LL,FNU,UM
              WRITE(6,621)ICHAN,LL,FNU,DM
            ENDIF
          ENDIF
          GO TO 50
        ELSE
!  NEW U0
          U0=U2
          P1=P1-TWO
          P2=P2+P1
        ENDIF
      ENDDO
!
!  SERIES NOT CONVERGED
      WRITE(6,600)LL,EPS,RHO
      IERR=3
      RETURN
!      STOP 'SC0: SERIES NOT CONVERGED'
!
   50 IF(LLL.EQ.-1)THEN
        C=F/T0
        CP=FP/T0
      ELSE
        S=F*T0
        SP=FP*T0
      ENDIF
!
! END LOOP OVER C AND S
      ENDDO
!
! NORMALIZE FOR FUNCTIONS .5*F AND .5*FP
!
      CON=ONE
      DO K=1,LL
        CON=CON*DBLE(K*(2*K+1))
      ENDDO
      C=C*CON
      CP=CP*CON
      S=S/CON
      SP=SP/CON
!
! CALCULATE CAP A 
      A=ONE
      IF(LL.GT.0)THEN
        CON=2*LL+1
        C=C/CON
        CP=CP/CON
        A1=TZED
        A2=-DEPS
        A3=DEPS+DEPS
        DO I=1,LL
          A2=A2+A3
          A1=A1+A2
          A=A*A1
        ENDDO
      ENDIF
      CON=SQRT(A)
      S=S*CON
      SP=SP*CON
      C=C/CON
      CP=CP/CON
!
      W=C*SP-S*CP
      IF(ABS(W-ONE).GT.100*AC)THEN
        IF(EPS.GT.TZERO)THEN
          WRITE(6,630)ICHAN,LL,EPS,RHO,W
        ELSE
          FNU=ONE/SQRT(-EPS)
          WRITE(6,631)ICHAN,LL,FNU,RHO,W
        ENDIF
        IERR=1
      ENDIF
!
! COMPLEX COEFFICIENT
      IF(EPS.LT.0.)THEN
        ZZ=CMPLX(TZERO,ONE)
        ZS=ZZ**(LL+1)
        ZC=ZZ/ZS
        ZSQ=ONE+ZZ
        ZSQ=-ZSQ/SQRT(TWO)
        ZS=ZS/ZSQ
        ZC=ZC/ZSQ
      ELSE
        ZS=ONE
        ZC=ONE
      ENDIF
!
      RETURN
!
  600 FORMAT(//10X,60('*')//10X,'SERIES IN SC0 NOT CONVERGED'
     + /10X,' LL =',I3,',  EPS =',1PE15.5,',  RHO =',
     +  E15.5//10X,60('*')//)
  610 FORMAT(/5X,'SUBROUTINE SC0, ICHAN=',I3,'  LL = ',I2,',  EPS = ',
     +  1PE12.5,',  UM = ',E10.2)
  611 FORMAT(/5X,'SUBROUTINE SC0, ICHAN=',I3,'  LL = ',I2,',  FNU = ',
     +  0PF10.2,',  UM = ',E10.2)
  620 FORMAT(/5X,'SUBROUTINE SC0, ICHAN=',I3,'  LL = ',I2,',  EPS = ',
     +  1PE12.5,',  DM = ',E10.2)
  621 FORMAT(/5X,'SUBROUTINE SC0, ICHAN=',I3,'  LL = ',I2,',  FNU = ',
     +  0PF10.2,',  DM = ',E10.2)
  630 FORMAT(/5X,'SUBROUTINE SC0, ICHAN=',I3,'  LL = ',I2,',  EPS = ',
     +  1PE12.5,',  RHO = ',E10.3,',  W = ',0PF10.6)
  631 FORMAT(/5X,'SUBROUTINE SC0, ICHAN=',I3,'  LL = ',I2,',  FNU = ',
     +  0PF10.2,',  RHO = ',E10.3,',  W = ',0PF10.6)
      END
!
!***************************************************************
!
      SUBROUTINE SCALE1(IOPT1)
      USE CPOT
      USE PART
      USE DBUT
      USE CEN
      USE CINPUT
      USE CINPTX
      USE CHAN
      USE CDEGEN
      USE CDEC
      USE CACC
      USE CPOINT
      USE CNTRL
      USE CBUT
      USE CLOGB
      USE CENAT1
!
!   CONVERTS R-MATRIX TARGET DATA TO Z-SCALED FORM
!
      IMPLICIT REAL*8 (A-H,O-Z)
!      PARAMETER (MXTST=(MZTAR*MZTAR+MZTAR)/2)
      PARAMETER (TZERO=0.0)
      PARAMETER (ONE=1.0)
      PARAMETER (TWO=2.0)
!
      INTEGER, ALLOCATABLE :: INDEX(:)
      REAL(8), ALLOCATABLE :: AVENGY(:)
!
      ALLOCATE(INDEX(NAST))
      ALLOCATE(AVENGY(NAST))
!
!  Z-SCALING FACTORS
!
      AZ=MAX(NZED-NELC,1)
      AZ1=ONE/AZ
      AZ2=AZ1*AZ1
      TAZ2=2*AZ2
      AZAZ=AZ*AZ
!
!  Z-SCALED PARTITIONED DATA
!
      IF(IPRCENT.NE.100)THEN
        WRITE(6,3085)IPRCENT
        DO L=1,LRANG2
          DO N=1,NRANG2
            ENDS(N,L)=AZ1*ENDS(N,L)**2
            EIGENS(N,L)=AZ2*EIGENS(N,L)
          ENDDO
        ENDDO
      ENDIF
!
!  Z-SCALED TARGET ENERGIES, RELATIVE TO ZERO FOR TARGET GROUND STATE
!
      ENAT1=ENAT(1)
!OLD  IF(NAST.EQ.1)GOTO 20
      DO I=1,NAST
        ENAT(I)=TAZ2*(ENAT(I)-ENAT1)
      ENDDO
!
!  TEST FOR DEGENERACY AND FORM AVERAGED TARGET ENERGY LEVELS
!
      IF (NASTD.GT.0) THEN
        NASTR=0
        DO N=1,NASTD
          NASTR=NASTR+NLEV(N)
        ENDDO
        IF(NASTR.NE.NAST)THEN
          WRITE(6,640)NASTR,NAST,(NLEV(N),N=1,NASTD)
          STOP 'LEVEL INCONSISTANCY ON GROUPING...'
        END IF
!
!OLD  AVENGY(1)=0.0
!OLD  N1=NLEV(1)+1
!OLD  INDEX(1)=NLEV(1)
        N1=1
        DO J=1,NASTD
          N=NLEV(J)
          N2=N+N1-1
          INDEX(J)=N2
          ENSUM=TZERO
          ARSUM=TZERO
          WTSUM=TZERO
          DO IN=N1,N2
            IF(IWT.LT.0)THEN
              WT=LAT(IN)+1
            ELSEIF(IWT.EQ.0)THEN
              WT=1
            ELSE
              WT=ISAT(IN)*(2*LAT(IN)+1)
            ENDIF
            ENSUM=ENAT(IN)*WT+ENSUM
            WTSUM=WTSUM+WT
            IF(IRDEC.GT.0) ARSUM=ARSUM+ARDEC(IN)
          ENDDO
          AVENGY(J)=ENSUM/WTSUM
          IWD(J)=WTSUM-0.5
          IF(IRDEC.GT.0) ARDEC(J)=ARSUM
          IF (IOPT1.GT.9) THEN
! REPLACE ENERGY BY AVERAGE FOR BUNCHED TERMS BUT KEEP ALL ID'S
            DO IN=N1,N2
              ENATR(IN)=ENAT(IN)
              ENAT(IN)=AVENGY(J)
            ENDDO
          ENDIF
          N1=N2+1
        ENDDO
      ENDIF
!
!OL20 ENAT(1)=0.
      NASTR=NAST
!
!  RA AND BSTO
      RZERO=RA*AZ
      BSTO=BSTO/RZERO
!
!  BUTTLE CORRECTION
!
      IF (IWORD.EQ.1) THEN
     	IF (COEFF(3,1).GT.-10000) THEN
     	  NEWBUT=.FALSE.
     	  AA=RZERO*AZ2
     	  DO M=1,3
     	    AA=AA*AZAZ
     	    DO L=1,LRANG2
     	      COEFF(M,L)=COEFF(M,L)*AA
     	    ENDDO
     	  ENDDO
     	ELSE
     	  NEWBUT=.TRUE.
     	  NBMX=0
     	  DO L=1,LRANG2
     	    NBUT=-INT(COEFF(3,L))/10000
!            IF (NBUT.GT.MZNRG) THEN
!              WRITE(6,699) NBUT,MZNRG
!              STOP '*** DIMENSION EXCEEDED: INCREASE MZNRG'
!            ENDIF
     	    COEFF(3,L)=NBUT
     	    COEFF(1,L)=RZERO*COEFF(1,L)
     	    NBMX=MAX(NBMX,NBUT)
     	  ENDDO
!          IF (NBMX.GT.MZNRG) THEN
!            WRITE(6,699) NBMX,MZNRG
!            STOP '*** DIMENSION EXCEEDED: INCREASE MZNRG'
!          ENDIF
!
!    	  INITIALISE FKN AND UKN
     	  PI=ACOS(-ONE)
     	  G=-PI/TWO
          IF(ALLOCATED(FKN)) DEALLOCATE(FKN)
          IF(ALLOCATED(UKN)) DEALLOCATE(UKN)
!         ALLOCATE(FKN(0:MZNRG),UKN(0:MZNRG))
          ALLOCATE(FKN(0:NBMX),UKN(0:NBMX))
!         DO I=0,MZNRG
          DO I=0,NBMX
     	    G=G+PI
     	    FKN(I)=G
     	    UKN(I)=G*G
     	  ENDDO
     	ENDIF
      ELSE                   !DARC
        DO L=1,LRANG2
          DO N=1,NBUTD(L)
            EBUTD(N,L)=EBUTD(N,L)*TAZ2
            CBUTD(N,L)=CBUTD(N,L)*RZERO
          ENDDO
        ENDDO
      ENDIF
!
!  WRITE TARGET PROPERTIES
!
      WRITE(6,650)NZED,NELC
      WRITE(6,655)
      DO J=1,NAST
        WRITE(6,660)J,ISAT(J),LAT(J),ENAT(J)
      ENDDO
!
!  IF THERE ARE DEGENERATE LEVELS PRINT THE AVERAGED ENERGIES
!  PUT THEM IN /CINPUT/ AND PRESERVE THE ENERGIES READ FROM
!  THE H FILE IN /CDEGEN/
!
      IF(NASTD.NE.0)THEN
!
!     PRESERVE THE ENERGIES READ FROM THE H-FILE IN /CDEGEN/
     	IF(IOPT1.GT.9) THEN
!
!    	  MORE CODING FROM VALF (HES)
!    	  IF THERE ARE BUNCHED TERMSPUT ENERGIES IN CINPUT
     	  WRITE(6,665)
     	  DO J=1,NAST
!    	     ENAT(J)=AVENGY(J)
     	    WRITE(6,'(3X,I14,F14.8,9X,F12.6)') J,ENAT(J),ENATR(J)
     	  ENDDO
!
     	ELSE
!
     	  DO I=1,NAST
     	    ENATR(I)=ENAT(I)
     	  ENDDO
     	  INR1=1
     	  DO IND=1,NASTD
     	    N=NLEV(IND)
     	    INR2=INR1+N-1
     	    IF(N.GT.1)WRITE(6,664)INR1,INR2
     	    INR1=INR2+1
     	  ENDDO
     	  WRITE(6,665)IWT
     	  AV1=AVENGY(1)
     	  DO J=1,NASTD
     	    ENAT(J)=AVENGY(J)-AV1
     	    WRITE(6,666)INDEX(J),J,ENAT(J)
     	  ENDDO
     	ENDIF
!
        NAST=NASTD
      END IF
!
      WRITE(6,670)RZERO,BSTO
      WRITE(6,680)AC
!
      DEALLOCATE(INDEX)
      DEALLOCATE(AVENGY)

!
  640 FORMAT(///5X,'*****INCORRECT DATA   ',/
     1          5X,'EXPECT ',I3, 'LEVELS FROM NLEV DATA BUT NAST IS',
     2          I3,/5X,'NLEV=',20I3)
  650   FORMAT(//10X,'NUCLEAR CHARGE =',I3,
     1           ', NUMBER OF TARGET ELECTRONS =',I3/10X,53('*'))
  655 FORMAT(//20X,'TARGET STATES -'/20X,15('*')
     1       //10X,'INDEX',5X,'2*S+1',5X,'TOTAL L',5X,'SCALED ENERGY'
     2        /20X,'OR P      OR 2*J'/)
  660 FORMAT(3X,3I10,7X,F12.6)
  664 FORMAT(/' LEVELS ',I3,' TO ',I3,' ARE COMBINED')
  665 FORMAT(//' IWT=',I2,8X,'EQUIVALENT TARGET STATES -'/15X,26('*')//
     1       12X,'OLD INDEX',5X,'NEW INDEX',7X,'SCALED ENERGY'//)
  666 FORMAT(3X,2I14,9X,F12.6)
  670 FORMAT(//5X,'RZERO = ',F10.4,3X,'BSTO = ',F10.4/)
  680 FORMAT(5X,'AC = ',E12.4)
!  699 FORMAT(//' ******* NBUT = ',I4,'  LARGER THAN ',
!     + 'MZNRG = ',I4//)
 3085 FORMAT (/'***** PARTITIONED R-MATRIX IN USE: ',I2,
     X'% OF E-SOLUTIONS IN USE.'//)
!
      RETURN
      END
!***************************************************************
!
      SUBROUTINE SCALE2
      USE WMATALLOC
      USE CPOT
      USE CTOP
      USE PART
      USE DBUT
      USE CEN
      USE CINPUT
      USE CINPTX
      USE CHAN
      USE NRBKUT
      USE NRBLMX
      USE NRBSKP
      USE CPOINT
      USE CNTRLS
      USE CNTRL
      USE CENAT1
!
!   CONVERTS R-MATRIX DATA TO Z-SCALED FORM
!
      IMPLICIT REAL*8 (A-H,O-Z)
!      PARAMETER (MXTST=(MZTAR*MZTAR+MZTAR)/2)
!      PARAMETER (MXF=(MZLMX+1)/2)
!
      PARAMETER (TZERO=0.0)
      PARAMETER (ONE=1.0)
!
!  STATISTICAL WEIGHT * 2 (OR * 4 FOR NON-EXCHANGE CASE)
!  LRGL2=2*J FOR B.P.
!
      NWT=(2*LRGL2+1)
      IF(NSPN2.EQ.0)THEN
        NWT=NWT+1
        IF(LINC.EQ.1)LINC=2
      ELSE
        IF(NSPN2.LT.0)NWT=-2*NWT
        NWT=NWT*NSPN2*2
      ENDIF
!
!  Z-SCALING FACTORS
!
      AZ=MAX(NZED-NELC,1)
      AZ1=ONE/AZ
      AZ2=AZ1*AZ1
      TAZ2=2*AZ2
      AZHR=ONE/SQRT(AZ)
      AZAZ=AZ*AZ
!
!  CHANNELS
!
      NCHF=NCHAN
      DO I=1,NCHF
        LL=LLCH(I)
        CCT(I)=DBLE(LL*(LL+1))
      ENDDO
      I=0
      NCHOPT=0
      DO 50 J=1,NAST
        K=NCONAT(J)
        IF(K.EQ.0)GOTO 50
        IF(J.LE.KUTPS)NCHOPT=NCHOPT+K
        I0=I
        M=0
        JK=-1
        DO L=1,K
          I=I+1
          ITARG(I)=J
          ECH(I)=ENAT(J)
          IF(NSPN2.NE.0)THEN
            IF(LLCH(I).LT.M)THEN
              WRITE(6,645)J,(LLCH(I0+N),N=1,K)
              STOP '***INPUT ERROR IN SR.SCALE2***'
            ENDIF
          ELSE
            IF(KJ(I).EQ.JK.AND.LLCH(I).LT.M)THEN
              WRITE(6,646)J,(LLCH(I0+N),N=1,K)
              STOP '***INPUT ERROR IN SR.SCALE2***'
            ENDIF
          ENDIF
          M=LLCH(I)
          JK=KJ(I)
        ENDDO
   50 CONTINUE
!
!  R-MATRIX
!  VALUE AND WMAT
!
      DO I=1,NCHF
        DO N=1,MNP2
          WMAT(N,I)=WMAT(N,I)*AZHR
        ENDDO
      ENDDO
      IF(IPRINT.GT.3)WRITE(6,635)
      DO N=1,MNP2
        VALUE(N)=TAZ2*(VALUE(N)-ENAT1)
        IF(IPRINT.GT.3)WRITE(6,640)N,VALUE(N),(WMAT(N,I),I=1,NCHF)
      ENDDO
!
!  PARTITIONED
!
      IF(IPRCENT.NE.100)THEN
        TRACE=TAZ2*(TRACE-ENAT1)
        IF(IPRINT.GT.-2)WRITE(6,641)TRACE                  !EZERO
        DO L=1,LRANG2
          NRANG1(L)=NRANG2+1
          DO N=1,NRANG2
            IF(EIGENS(N,L).GT.VALUE(1))THEN
              NRANG1(L)=N
              GO TO 55
            ENDIF
          ENDDO
   55   ENDDO
        DO I=1,NCHF
          L=L2P(I)+1
          SI(I)=TZERO
          DO N=1,NRANG2
            SI(I)=SI(I)+ENDS(N,L)
          ENDDO
        ENDDO
      ENDIF
!
!  COEFFICIENTS IN POTENTIAL
!
!  LAMP(I,J) AND BW(I,J)
!
      IF(ALLOCATED(BW))DEALLOCATE(BW)
      IF(ALLOCATED(LAMP))DEALLOCATE(LAMP)
!
      ALLOCATE(BW(NCHF,NCHF))
      ALLOCATE(LAMP(NCHF,NCHF))
!
      IF(IPERT.EQ.0.AND.LRGLAM.LT.0)GOTO 160
      IF(IPRINT.GT.2.AND.IPERT.NE.0)WRITE(6,655)
!
!
!
      DO J=1,NCHF
        DO I=1,NCHF
          LAMP(I,J)=1
        ENDDO
      ENDDO

      IF(LAMAX.EQ.0)GOTO 160
!
!
! LOOK FOR DIPOLE
!
      IABW=IABS(IWORD)
      A1=ONE/RZERO
      A2=A1/2
      AZZ=ONE
      IF(NCHF.GT.1)THEN
      DO J=2,NCHF
        J1=J-1
        DO 130 I=1,J1
          CF(I,J,1)=CF(I,J,1)*IABW
          BU1=-CF(I,J,1)
          IF(ABS(BU1).LT.1.D-6)GOTO 130
          LAMP(I,J)=2
          BW(I,J)=BU1
          P=A2*BU1
          IF(IPRINT.GT.2.AND.IPERT.NE.0)
     X       WRITE(6,660)ITARG(I),I,ITARG(J),J,P
  130   CONTINUE
      ENDDO
      ENDIF
      IF(LAMAX.EQ.1)GOTO 160
!
! LOOK FOR QUADRUPOLE
!
      IF(IPRINT.GT.2.AND.IPERT.NE.0)WRITE(6,665)
      A2=A2*A1
      AZZ=AZZ*AZ
      DO J=1,NCHF
        DO 150 I=1,J
          CF(I,J,2)=CF(I,J,2)*IABW
          BU2=-CF(I,J,2)
          IF(ABS(BU2).LT.1.D-6)GOTO 150
          LAMP(I,J)=3
          BU2=BU2*AZZ
          BW(I,J)=BU2
          CF(I,J,2)=AZZ*CF(I,J,2)
          P=BU2*A2
          IF(IPRINT.GT.2.AND.IPERT.NE.0)
     X       WRITE(6,660)ITARG(I),I,ITARG(J),J,P
  150   CONTINUE
      ENDDO
      IF(LAMAX.EQ.2)GOTO 160
!
! LOOK FOR OCTUPOLE ETC.
!
      IF(IPRINT.GT.2.AND.IPERT.NE.0)WRITE(6,666)
      DO L=3,MAXMUL    ! USED TO BE LAMAX CPB 2015
        A2=A2*A1
        AZZ=AZZ*AZ
        DO J=1,NCHF
          DO 155 I=1,J
            CF(I,J,L)=CF(I,J,L)*IABW
            BUL=-CF(I,J,L)
            IF(ABS(BUL).LT.1.D-6)GOTO 155
! SKIP IF LOWER MULTIPOLE EXISTS
            IF(LAMP(I,J).NE.1)GO TO 155
            LAMP(I,J)=L+1
            P=TZERO
            IF(L.LE.LMX)THEN
              BUL=BUL*AZZ
              BW(I,J)=BUL
              CF(I,J,L)=AZZ*CF(I,J,L)
              P=BUL*A2
              IF(IPRINT.GT.2.AND.IPERT.NE.0)
     X           WRITE(6,660)ITARG(I),I,ITARG(J),J,P
            ENDIF
  155     CONTINUE
        ENDDO
      ENDDO
!
  160 CONTINUE
!
! SYMMETRIZE
!
      DO I=1,NCHF
        DO J=I,NCHF
          LAMP(J,I)=LAMP(I,J)
          BW(J,I)=BW(I,J)
        ENDDO
      ENDDO
!
!  WRITE SLPI AND CHANNEL DATA
!
      WRITE(6,600)NSPN2,LRGL2,NPTY2
      IF(ISGPT.NE.0) WRITE(14,600) NSPN2,LRGL2,NPTY2
      IF(NSPN2.NE.0)THEN
        WRITE(6,669)
        WRITE(6,670)(I,ITARG(I),LLCH(I),I=1,NCHF)
      ELSEIF(KFLAG.GE.0)THEN
        WRITE(6,671)
        WRITE(6,672)(I,ITARG(I),LLCH(I),KJ(I),I=1,NCHF)
      ELSE
        WRITE(6,673)
        WRITE(6,672)(I,ITARG(I),LLCH(I),K2P(I),I=1,NCHF)
      ENDIF
!
  600 FORMAT(///80('+')//12X,'(2S+1) L/2J P =',3I3/12X,24('*')//)
  635 FORMAT(//' N, VALUE(N) AND WMAT(N,I)'/)
  640 FORMAT(I5,E12.3,5X,9E12.3,(/5X,9E12.3))
  641 FORMAT(/'PARTITIONED R-MATRIX, EZERO=',F10.6,' Z**2 RYD'/)
  645 FORMAT(//'CHANNEL L MUST BE ASCENDING WITHIN A TERM:',I5/
     X(20I3))
  646 FORMAT(//'CHANNEL L MUST BE ASCENDING WITHIN A LEVEL (FIXED-K):'
     X,I5/(20I3))
  655 FORMAT(/' PERTURBATION P FOR MULTIPOLE POTENTIAL'//
     1 '  - DIPOLE PART'/)
  660 FORMAT(2(I5,I3),E12.4)
  665 FORMAT(/'  - QUADRUPOLE PART'/)
  666 FORMAT(/'  - OCTUPOLE ETC. PART'/)
  669 FORMAT(12X,'CHANNEL',2X,'TARGET',4X,'SMALL',
     1          /12X,'INDEX ',3X,'INDEX  ',5X,'L'/)
  670 FORMAT(7X,I8,I9,I10)
  671 FORMAT(12X,'CHANNEL',2X,'TARGET',4X,'SMALL',
     1          /12X,'INDEX ',3X,'INDEX  ',5X,'L',5X,'2K'/)
  672 FORMAT(7X,I8,I9,I10,I7)
  673 FORMAT(12X,'CHANNEL',2X,'TARGET',4X,'SMALL',
     1          /12X,'INDEX ',3X,'INDEX  ',5X,'L',4X,'KAPPA'/)
!
      RETURN
      END
!***************************************************************
!
      SUBROUTINE SQDTS
      USE WMATALLOC
      USE NRBRCT
      USE NRBWRK
      USE NRBKHI
      USE NRBDR
      USE COMEGA
      USE CEN
      USE CINPUT
      USE CINPTX
      USE CHAN
      USE CDEC
      USE NRBQDT
      USE NRBSKP
      USE CACC
      USE CQDT
!
! NRB:
!  CALCULATION OF OMEGA FROM KHI-MX IN QDT, OPTIONAL GAILITIS AVERAGE,
!  OPTIONAL (TYPE-I) RADIATION DAMPING.
!
      IMPLICIT REAL*8 (A-H,O-Y)
      IMPLICIT COMPLEX*16 (Z)
!
!      PARAMETER (MXTST=(MZTAR*MZTAR+MZTAR)/2)
!      PARAMETER (LWORK=MZCHF*MZCHF)
!      PARAMETER (MWORK=MZDEG*MZDEG)
      PARAMETER (ZERO=(0.0,0.0))
      PARAMETER (TZERO=0.0)
      PARAMETER (ONE=1.0)
      PARAMETER (TWO=2.0)
      PARAMETER (QUART=0.25)
      PARAMETER (BIG=150.0)
!
       REAL(8), ALLOCATABLE :: RWORK(:)      ! DIM=(2*NCHF)
       COMPLEX(8), ALLOCATABLE :: ZVL(:,:),ZVR(:,:)   ! DIM=(NCHF,NCHF)
!
       P=>CSP
       Q=>CC
       POLD=>DSP
       QOLD=>DC
!
      NCC=NQ-NCHPP
!      IF(NCC.GT.MZDEG)THEN
!        WRITE(6,601)NCC
!        STOP
!      ENDIF
!
      PI=ACOS(-ONE)
      TPI=TWO*PI
      CONST=TPI/DBLE((NZED-NELC)**2)
      IONE=1
      IF(ELAS.EQ.'YES')IONE=0
!
!  DIAGONALISE KHICC
!
        DO N2=1,NCC
          DO N1=1,NCC
            ZKHICC(N1,N2)=DCMPLX(P(NCHPP+N1,NCHPP+N2)
     X            ,Q(NCHPP+N1,NCHPP+N2))
          ENDDO
        ENDDO
!
!STRTNBL
!        CALL ZEIGEN(ZKHICC(1:NCC,1:NCC),ZVAL(1:NCC),NCC,AC)
!ENDNBL
!STRTBL
        MWORK=NCHF*NCHF
        ALLOCATE(ZWORK(MWORK))
        ALLOCATE(RWORK(2*NCHF))
        ALLOCATE(ZVL(NCHF,NCHF),ZVR(NCHF,NCHF))
        CALL ZGEEV('N','V',NCC,ZCHICC,NCHF,ZVAL,ZVL,NCHF,ZVR,NCHF,
     X       ZWORK,MWORK,RWORK,INFO)
        IF (INFO.NE.0) THEN
           WRITE(6,'("ZGEEV CALLED, INFO= ",I5)') INFO
           STOP
        ENDIF			
        LWOPT=INT(ZWORK(1))
        IF (LWOPT.GT.MWORK) THEN
           WRITE(6,'("ZGEEV: OPTIMAL WORK SPACE LENGTH = ",I5)') LWOPT
        ENDIF
        DO N1=1,NCC
          DO N2=1,NCC
            ZKHICC(N2,N1)=ZVR(N2,N1)
          ENDDO
        ENDDO
        DEALLOCATE(ZWORK,RWORK)
        DEALLOCATE(ZVL,ZVR)
!ENDBL
!
!  CALCULATE ZKHIOC
!
        DO N2=1,NCC
          DO N1=1,NCHPP
            ZKHIOC(N1,N2)=ZERO
          ENDDO
          DO K=1,NCC
            DO N1=1,NCHPP
              ZKHIOC(N1,N2)=ZKHIOC(N1,N2)+
     X        DCMPLX(P(N1,NCHPP+K),Q(N1,NCHPP+K))*ZKHICC(K,N2)
            ENDDO
          ENDDO
        ENDDO
!
!  RADIATIVE DECAYS; RECALCULATE ARDEC (DEPENDS ON ETOT) IF NECESS.
!
      IF(IRDEC.GT.0)THEN
        NNN=NCHPP+1
        IF(NEWAR.GT.0)THEN
          KVEC=((ITARG(NNN)-IONE)*(ITARG(NNN)-1-IONE))/2
          ARDEC(ITARG(NNN))=TZERO
          DO JLOOP=1,ITARG(NNN)-1
            KVEC=KVEC+1
            IF(ETOT-(ENAT(ITARG(NNN))-ENAT(JLOOP)).LE.ENAT(1))THEN
              ARDEC(ITARG(NNN))=ARDEC(ITARG(NNN))+ARAD(KVEC)
            ENDIF
          ENDDO
          ARDEC(ITARG(NNN))=ARDEC(ITARG(NNN))*CONST
        ENDIF
        T=ARDEC(ITARG(NNN))*(FKNU(NNN)**3)
        T=MIN(T,BIG)
        FDEC=EXP(T)
      ELSE
        FDEC=ONE
      ENDIF
!
! INITIALIZE DR PROBABILITY
!
      IF(ALLOCATED(PDR))THEN
       CONTINUE
      ELSE
       ALLOCATE(PDR(NCHF))
      ENDIF

      DO I=1,NCHPP
        PDR(I)=ONE
      ENDDO
!
!
      IF(QDT)THEN
!
!  CALCULATE AVERAGE COLLISION STRENGTH
!
        DO J=1,NCHPP
          P(J,J)=P(J,J)-ONE
          DO I=1,J
            RK(I,J)=P(I,J)**2+Q(I,J)**2
            PDR(I)=PDR(I)-RK(I,J)
            PDR(J)=PDR(J)-RK(I,J)
          ENDDO
          PDR(J)=PDR(J)+RK(J,J)-TWO*P(J,J)-ONE
          P(J,J)=P(J,J)+ONE
        ENDDO
        DO K=1,NCC
          VV=ABS(ZVAL(K))
          IF((ONE-VV).LT.AC) THEN
            VV=FDEC-ONE
            DO M=1,NCHPP
              VV=VV+ABS(ZKHIOC(M,K))**2
            ENDDO
          ELSE
            VV=FDEC-VV**2
          ENDIF
          DO J=1,NCHPP
            DO I=1,J
              T=ABS(ZKHIOC(I,K)*ZKHIOC(J,K))**2/VV
              RK(I,J)=RK(I,J)+T
              PDR(I)=PDR(I)-T
              PDR(J)=PDR(J)-T
            ENDDO
            PDR(J)=PDR(J)+T
          ENDDO
        ENDDO
        DO K1=1,NCC-1
          DO K2=K1+1,NCC
            IF((ONE-ABS(ZVAL(K1))).LT.AC.AND.
     X             (ONE-ABS(ZVAL(K2))).LT.AC) THEN
              ZVV=ZVAL(K1)*CONJG(ZVAL(K1)-ZVAL(K2))+FDEC-ONE
              DO M=1,NCHPP
                ZVV=ZVV+ABS(ZKHIOC(M,K1))**2
              ENDDO
            ELSE
              ZVV=FDEC-ZVAL(K1)*CONJG(ZVAL(K2))
            ENDIF
            DO J=1,NCHPP
              DO I=1,J
                T=TWO*DBLE(ZKHIOC(I,K1)*ZKHIOC(J,K1)*
     X          CONJG(ZKHIOC(I,K2)*ZKHIOC(J,K2))/ZVV)
                RK(I,J)=RK(I,J)+T
                PDR(I)=PDR(I)-T
                PDR(J)=PDR(J)-T
              ENDDO
              PDR(J)=PDR(J)+T
            ENDDO
          ENDDO
        ENDDO
!
      ELSE
!
! DETAILED OMEGA
!
        TPINU=FKNU(NCHPP+1)*TPI
        ZFDEC=SQRT(FDEC)*EXP(DCMPLX(TZERO,-TPINU))
        DO J=1,NCHPP
          DO I=1,J
            ZKHI=DCMPLX(P(I,J),Q(I,J))
            DO K=1,NCC
              ZKHI=ZKHI-ZKHIOC(I,K)*ZKHIOC(J,K)/(ZVAL(K)-ZFDEC)
            ENDDO
            PP=ZKHI*CONJG(ZKHI)
            PDR(J)=PDR(J)-PP
            PDR(I)=PDR(I)-PP
            IF(I.EQ.J)THEN
              PDR(I)=PDR(I)+PP
              ZKHI=ZKHI-DCMPLX(ONE,TZERO)
            ENDIF
            RK(I,J)=ZKHI*CONJG(ZKHI)
          ENDDO
        ENDDO
!
      ENDIF
!
! SYMMETRIZE AND ADJUST WEIGHTING
!
      T=QUART*NWT
      DO J=1,NCHPP
        PDR(J)=PDR(J)*T
        DO I=1,J
          RK(I,J)=T*RK(I,J)
          RK(J,I)=RK(I,J)
        ENDDO
      ENDDO
!
! IF REALLY MQDT RESTORE ORIGINAL P,Q
!
      IF(IQDT.GT.0.AND.NQ.LT.NCHF)THEN
        DO J=1,NCHF
          DO I=1,NCHF
            P(I,J)=POLD(I,J)
            Q(I,J)=QOLD(I,J)
          ENDDO
        ENDDO
      ENDIF
!
      NULLIFY(P,Q,POLD,QOLD)
      RETURN
!  601 FORMAT(//10X,10('*'),' NUMBER OF DEGENERATE CLOSED',
!     X ' CHANNELS, NCC = ',I2/20X,' LARGER THAN DIMENSION',
!     X ' VALUE OF DEG = MZDEG'//)
      END
!**********************************************************
!
      SUBROUTINE TANDTDN(FK,N1,N2,I,TA,TDA,TP)
      USE CTHET
      USE CPOINT
      USE NRBZED
      USE CBLK
!
!
!  CALCULATES THETA AND THETAD =THETA DOT FOR  R REAL
!     THETA = TA*EXP(TP)
!     THETAD = TDA*EXP(TP)
!     TP = FNU*LOG(R) - R/FNU
! NRB
!     NEUTRAL CASE ADDED
!
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION TP(30),TA(30),TDA(30)
!
      MI=MSUM(I)
      E=BG(I,1)
      F2=BG(I,2)
      FNU=BB(I,1)
!
      DO N=N1,N2
        U=XLAG(N)
        R=RTWO+U/FK
        X=2.*R/FNU
        Y=1./X
        AS=1.
        S=BB(I,2)
        CX=0.
        DO L=3,MI
          AS=AS*Y
          S=S+BB(I,L)*AS
          CX=CX+BG(I,L)*AS
        ENDDO
!
        DLR=LOG(R)*TZED
        TP(N)=-.5*X+FNU*DLR
        TA(N)=S
        TDA(N)=E*((DLR+R*F2)*S+CX)
      ENDDO
!
      RETURN
      END
!**********************************************************
!
      SUBROUTINE THETA(R,I,T,TP,TD,TDP,ICONV)
      USE CEN
      USE CTHET
      USE CACC
      USE CHAN
      USE CPOINT
      USE CNTRL
      USE NRBZED
!
      IMPLICIT REAL*8 (A-H,O-Z)
      PARAMETER (TZERO=0.0)
      PARAMETER (ONE=1.0)
      PARAMETER (TWO=2.0)
!
!NRB
!  NEUTRAL CASE ADDED
!
      LOGICAL FLAG
!
      PI=ACOS(-ONE)
      FNU=FKNU(I)
      LL=LLCH(I)
      M=FNU+LL+12
!
!      IF(M.LE.MZTET)GOTO 10
!      IF(TZED.EQ.0)THEN
!       M=MZTET       !SHOULD CONVERGE AT M=LL
!       GO TO 10
!      ENDIF
!      WRITE(6,700)I,M
!      GOTO 102
!
   10 F1=ONE/FNU
      F2=F1*F1
      X=TWO*R*F1
      Y=ONE/X
      FL=DBLE(LL)
!
      R1=ONE/R
      A=TZED*FNU*R1-F1
      B=TZED*LOG(R)+R*F2
      C=TZED*R1+F2
      D=-R1
      E=FNU**3/TWO
!
      BB(I,1)=FNU
      BB(I,2)=ONE
      BG(I,1)=E
      BG(I,2)=F2
!
      YN=ONE
      A1=FL-FNU*TZED
      A2=FL+FNU*TZED+ONE
      BN=-TWO*FNU-ONE
!
      BET=ONE
      GAM=TZERO
      W0=C
!
      S=ONE
      U=TZERO
      CX=TZERO
      CY=TZERO
!
!
      N=0
      FLAG=.FALSE.
      DO 20 MN=3,M
       N=N+1
       A1=A1+ONE
       A2=A2-ONE
       BN=BN+TWO
       CN=A1*A2
       DN=BN*TZED+F1*CN
       GAM=CN*GAM+DN*BET
       BET=CN*BET
       YN=YN*Y
       U=U+BET*YN
       CY=CY+GAM*YN
       AN=ONE/DBLE(N)
       GAM=GAM*AN
       BET=BET*AN
       S=S+BET*YN
       CX=CX+GAM*YN
       W1=C*S*S+D*(S*CY-U*CX)
       BB(I,MN)=BET
       BG(I,MN)=GAM
       IF(W1.EQ.TZERO) GO TO 30
       AC0=(W1-W0)/W1
!       WRITE(6,*)MN,AC0
       IF(ABS(AC0).LT.10*AC)THEN
         IF(FLAG)THEN
           GO TO 30
         ELSE
           FLAG=.TRUE.
         END IF
       ELSE
         FLAG=.FALSE.
       END IF
   20  W0=W1
!-NRB
      IF(R*F1.GT.150.)THEN
       T=TZERO
       TP=TZERO
       TD=TZERO
       TDP=TZERO
       ICONV=0
       MSUM(I)=M
       RETURN
      ENDIF
!-NRB
!
!  NOT CONVERGED
      GOTO 100
!
!  SUMMATIONS CONVERGED
!  30 P=EXP(-R*F1)*R**FNU
!
   30 P=EXP(-R*F1/TWO)
      IF(TZED.GT.0) P=P*R**(FNU/TWO)
      IF(ABS(P).GT.1.D-100)THEN
       CFACT=ONE/P
      ELSE
       CFACT=TZERO
      ENDIF
      BB(I,2)=BB(I,2)*CFACT
      DO 40 J=3,N+2
       BB(I,J)=BB(I,J)*CFACT
       BG(I,J)=BG(I,J)*CFACT
   40 CONTINUE
!
      T=P*S
      TP=P*(A*S+D*U)
      TD=P*E*(B*S+CX)
      TDP=P*E*((A*B+C)*S+B*D*U+A*CX+D*CY)
      N2=N+2
      MSUM(I)=N2
      ICONV=0
      RETURN
!
! RELAX TEST FOR ENERGIES CASE A POSSIBLY NEGATIVE IN SC 
  100 IF(FNU.GE.DBLE(LL+1))GO TO 101
!
! PROCEED BUT PRINT WARNING
      IF(ABS(AC0).GT.1.E2*AC.AND.NCHPP.GT.0)
     XWRITE(6,650)M,I,EPS(I),LL,AC0,AC
      GO TO 30
!
!  USE SUBROUTINE SC AND SET IPERT = 0
  101 IF(TZED.GT.0)WRITE(6,610)N
      MSUM(I)=M
      IF(TZED.EQ.0)THEN
       WRITE(6,611)
       STOP 'FAILURE FOR NEUTRAL THETA - TOO CLOSE TO THRESHOLD?'
      ENDIF
  102 IF(NPERT.GT.0)THEN
       WRITE(6,750)IABS(IPERT)+2,IABS(IPERT)
       STOP 750
      ENDIF
      CALL SC(EPS(I),LLCH(I),RZERO,AC,FSA,FSPA,FCA,FCPA)
      SINF=SIN(PI*FKNU(I))
      COSF=COS(PI*FKNU(I))
      T=FCA*SINF-FSA*COSF
      TP=FCPA*SINF-FSPA*COSF
      IPERT=0
      ICONV=1
      RETURN
!
!  700 FORMAT(//10X,30('*')/10X,'SUBROUTINE THETA'/
!     1 10X,'FOR I=',I3,' REQUIRE M=',I4/
!     2 10X,'WHICH IS LARGER THAN MAXIMUM VALUE OF MZTET ALLOWED BY '
!     3 ,'DIMENSIONS'/10X,'USING SUBROUTINE SC WITH IPERT = 0'/10X,
!     4 30('*'))
  750 FORMAT(//' EXECUATION HALTED BECAUSE IPERT=',I3,' RESET IPERT=',I3
     X,' OR FIX INDICATED PROBLEM')
  650 FORMAT(//10X,30('*')
     X //10X,'WARNING SUBROUTINE THETA'
     X //'M=',I3,' SUMMATIONS NOT CONVERGED FOR CHANNEL',I3,
     X   '  E .LT. -(L+1)**(-2): E,L= ',F10.6,I5
     X //' CHECK ACCURACY AC0 WITH REQUIRED AC: AC0,AC= ',2F10.6
     X //10X,30('*'))
  610 FORMAT(//10X,30('*')//10X,'WARNING SUBROUTINE THETA'
     2 //' SUMMATIONS NOT CONVERGED WITH ',I4,' TERMS'
     3 //10X,'USING SUBROUTINE SC WITH IPERT = 0'
     4 /10X,30('*'))
  611 FORMAT(//10X,30('*')//10X,'WARNING SUBROUTINE THETA'
     2 //'FAILURE FOR NEUTRAL CASE - TOO CLOSE TO THRESHOLD?'
     3 /10X,30('*'))
      END
!**********************************************************
!
      SUBROUTINE TOP1(LLL)
      USE WMATALLOC
      USE CPOT
      USE CTOP
      USE CEN
      USE CINPUT 
      USE CINPTX
      USE CHAN
!
! NRB:
! DETERMINE MULTIPOLE TOP-UP (LS AND BP)
!
      IMPLICIT REAL*8 (A-H,O-Z)
!      PARAMETER (MXTST=(MZTAR*MZTAR+MZTAR)/2)
!      PARAMETER (MXF=(MZLMX+1)/2)
      PARAMETER (TZERO=0.0)
      PARAMETER (ONE=1.0)
      PARAMETER (SIX=6.0)
      PARAMETER (TOLE=1.E-10)
!
      MXF=(MAXMUL+1)/2
      LRGLAM=LLL
      LRGLMN=LRGLAM  !LOWEST LRGL2 THAT MAY CONTAIN OMEGAS TO BE STRUCK OUT
!
!
!
!
!  FIND ALLOWED TRANSITIONS
!  ************************
!  (THIS IS DONE AND NEEDED FOR EVERY LRGL2 IF LRGLAM.GE.0)
!
      DO 20 I1=1,NCHF
      DO 20 I2=I1,NCHF
!
! DIPOLE
!
        IF(LAMP(I1,I2).EQ.2)THEN
          IT1=ITARG(I1)
          IT2=ITARG(I2)
          DO I=1,INDM
            IF(IABS(NTOP(I,1)).EQ.IT1.AND.IABS(NTOP(I,2)).EQ.IT2)THEN
              NTOP(I,1)=IT1
              NTOP(I,2)=IT2
              LLM=MIN(LAT(IT1),LAT(IT2))
              IF(NSPN2.EQ.0)THEN
                LRGLMN=MIN(LRGLMN,LRGLAM-2*LLM)
                LLM=-LLM+1                          !NO K/J TOP-UP
              ENDIF
              LITLAM(I)=LRGLAM+LLM
              IF(NSPN2.EQ.0)LITLAM(I)=LITLAM(I)/2
              GO TO 20
            ENDIF
          ENDDO
          INDM=INDM+1
          NTOP(INDM,1)=IT1
          NTOP(INDM,2)=IT2
          LLM=MIN(LAT(IT1),LAT(IT2))
          IF(NSPN2.EQ.0)THEN
            LRGLMN=MIN(LRGLMN,LRGLAM-2*LLM)
            LLM=-LLM+1                          !NO K/J TOP-UP
          ENDIF
          LITLAM(INDM)=LRGLAM+LLM
          IF(NSPN2.EQ.0)LITLAM(INDM)=LITLAM(INDM)/2
!
! QUADRUPOLE, OCTUPOLE
!
        ELSEIF(LAMP(I1,I2).GE.3)THEN
          IT1=-ITARG(I1)
          IT2=-ITARG(I2)
          DO I=1,INDM
            IF(IABS(NTOP(I,1)).EQ.-IT1.AND.IABS(NTOP(I,2)).EQ.-IT2)THEN
              IF(NTOP(I,1).LT.0)LITLAM(I)=MIN(LITLAM(I),LAMP(I1,I2)-1)
              GO TO 20
            ENDIF
          ENDDO
          INDM=INDM+1
          NTOP(INDM,1)=IT1
          NTOP(INDM,2)=IT2
          LITLAM(INDM)=LAMP(I1,I2)-1
        ENDIF
   20 CONTINUE
!
!  CALCULATE INVERSE CHANNEL LIST
!
      NC=0
      DO IT=1,NAST
        IF(NCONAT(IT).EQ.0)THEN
          NTCHAN(IT,1)=0
          NTCHAN(IT,2)=0
        ELSE
          NC=NC+1
          NTCHAN(IT,1)=NC
          NC=NC+NCONAT(IT)-1
          NTCHAN(IT,2)=NC
        ENDIF
      ENDDO
!
      IF(LRGL2.LT.(LRGLAM-1).OR.LRGL2.GT.LRGLAM)RETURN
!
!  INITIALISATIONS FOR LRGL2.GE.(LRGLAM-1)
!  ***************************************
!
!  INITIALISE TOPA, TOPB
!
      DO I=1,INDM
        TOPA(I)=TZERO
        TOPB(I)=TZERO
      ENDDO
!
!  PRINT CHANNEL LIST
!
      WRITE(6,600)LRGLAM
      DO I=1,INDM
        WRITE(6,610)I,NTOP(I,1),NTOP(I,2),LITLAM(I)
      ENDDO
!
!  CASE OF LRGL2.EQ.(LRGLAM-1)
!  ***************************
!
      IF(LRGL2.EQ.(LRGLAM-1))THEN
        WRITE(6,620)
!
!  TOP-UP IN SMALL L (BYPASSED BY BP - NOT NEEDED)
!
        DO 70 I=1,INDM
          IF(LITLAM(I).NE.LRGLAM)GOTO 70
          IT1=NTOP(I,1)
          IF(IT1.LT.0)GO TO 70
          IT2=NTOP(I,2)
          IF(NCONAT(IT1)*NCONAT(IT2).EQ.0)GO TO 70
          DELE=ENAT(IT2)-ENAT(IT1)
          IF(DELE.LT.TOLE)GO TO 70
          IF(LLCH(NTCHAN(IT1,2)).EQ.LRGLAM)THEN
            TOPA(I)=ONE/(DBLE(LRGLAM**2)*DELE)
            NTOPA(I,1)=NTCHAN(IT1,2)
            NTOPA(I,2)=NTCHAN(IT2,1)
            WRITE(6,630)I,NTOPA(I,1),NTOPA(I,2),TOPA(I)
          ENDIF
          DELE=-DELE
          IF(LLCH(NTCHAN(IT2,2)).EQ.LRGLAM)THEN
            TOPB(I)=ONE/(DBLE(LRGLAM**2)*DELE)
            NTOPB(I,1)=NTCHAN(IT1,1)
            NTOPB(I,2)=NTCHAN(IT2,2)
            WRITE(6,631)I,NTOPB(I,1),NTOPB(I,2),TOPB(I)
          ENDIF
   70   CONTINUE
        RETURN
      ENDIF
!
!  CASE OF LRGL2.EQ.LRGLAM
!  ***********************
!
      IF(LRGL2.EQ.LRGLAM)THEN
      WRITE(6,620)
!
!  TOP-UP IN SMALL L (LS AND BP)
!
      DO 100 I=1,INDM
        LAM=LITLAM(I)
        IT1=NTOP(I,1)
        IF(IT1.LT.0)GO TO 100
        IT2=NTOP(I,2)
        IF(NCONAT(IT1).EQ.0.OR.NCONAT(IT2).EQ.0)GOTO 100
        DO 152 I1=NTCHAN(IT1,2),NTCHAN(IT1,1),-1
        DO 151 I2=NTCHAN(IT2,2),NTCHAN(IT2,1),-1
          IF(NSPN2.EQ.0.AND.KFLAG.GE.0.AND.KJ(I1).NE.KJ(I2))GO TO 151
          LL1=LLCH(I1)
          LL2=LLCH(I2)
          DELE=MAX(ENAT(IT2)-ENAT(IT1),TOLE)
        IF(LL1.EQ.LAM.AND.LL2.EQ.LL1-1)THEN
          NTOPA(I,1)=I1
          NTOPA(I,2)=I2
          IF(NSPN2.NE.0)THEN          !LS-NRB
            TOPA(I)=ONE/(DBLE(LAM*LAM)*DELE*
     +              WSQ(LAT(IT1),LAT(IT2),LL1,LL2,LRGLAM,ISIGN))
          ELSEIF(KFLAG.GE.0)THEN      !JK-NRB
            W=DBLE(LRGL2+1)/DBLE(2*KJ(I1)+2)
            TOPA(I)=ONE/(DBLE(LAM*LAM)*DELE*
     +              W*WSQ2(LAT(IT1),LAT(IT2),2*LL1,2*LL2,KJ(I1),ISIGN))
          ELSE                        !JJ-NRB
            JV1=2*(L2P(I1)/2)+1       !2J-VALENCE
            JV2=2*(L2P(I2)/2)+1       !2J-VALENCE
            IF(JV1.EQ.JV2)THEN
              W=DBLE(JV2+1)/(DBLE((JV2+2)*JV2))
            ELSE
              JV3=MIN(JV1,JV2)
              W=DBLE((JV3+3)*(JV3+1))/DBLE(2*(JV3+2))
            ENDIF
            W=W/DBLE(2*MAX(LL1,LL2))
!            WRITE(6,*)LL1,LL2,JV1,JV2,W
            TOPA(I)=ONE/(DBLE(LAM*LAM)*DELE*
     +              W*WSQ2(LAT(IT1),LAT(IT2),JV1,JV2,LRGL2,ISIGN))
          ENDIF
          WRITE(6,630)I,NTOPA(I,1),NTOPA(I,2),TOPA(I)
          GO TO 100
        ENDIF
        DELE=-DELE
        IF(LL2.EQ.LAM.AND.LL1.EQ.LL2-1)THEN
          NTOPB(I,2)=I2
          NTOPB(I,1)=I1
          IF(NSPN2.NE.0)THEN          !LS-NRB
            TOPB(I)=ONE/(DBLE(LAM*LAM)*DELE*
     +              WSQ(LAT(IT1),LAT(IT2),LL1,LL2,LRGLAM,ISIGN))
          ELSEIF(KFLAG.GE.0)THEN      !JK-NRB
            W=DBLE(LRGL2+1)/DBLE(2*KJ(I1)+2)
            TOPB(I)=ONE/(DBLE(LAM*LAM)*DELE*
     +              W*WSQ2(LAT(IT1),LAT(IT2),2*LL1,2*LL2,KJ(I1),ISIGN))
          ELSE                        !JJ-NRB
            JV1=2*(L2P(I1)/2)+1       !2J-VALENCE
            JV2=2*(L2P(I2)/2)+1       !2J-VALENCE
            IF(JV1.EQ.JV2)THEN
              W=DBLE(JV2+1)/(DBLE((JV2+2)*JV2))
            ELSE
              JV3=MIN(JV1,JV2)
              W=DBLE((JV3+3)*(JV3+1))/DBLE(2*(JV3+2))
            ENDIF
            W=W/DBLE(2*MAX(LL1,LL2))
!            WRITE(6,*)LL1,LL2,JV1,JV2,W
            TOPB(I)=ONE/(DBLE(LAM*LAM)*DELE*
     +              W*WSQ2(LAT(IT1),LAT(IT2),JV1,JV2,LRGL2,ISIGN))
          ENDIF
          WRITE(6,631)I,NTOPB(I,1),NTOPB(I,2),TOPB(I)
          GO TO 100
        ENDIF
  151   CONTINUE
  152   CONTINUE
  100 CONTINUE
!
!  TOP-UP IN LARGE L (LS ONLY)
!
! NRB: THIS IS NEGLIGIBLE, SO WON'T BOTHER WITH BP VERSION .
! (IF BOTHERED THEN TOP-UP AT LRGLAM .EQ. LRGL2-2 SO THE LAST
!  LRGL2 IS THEN CYCLED THRU TO TOP-UP REMAINING K/J.)
!
      IF(NSPN2.EQ.0)RETURN
!
      WRITE(6,640)
      DO 200 I=1,INDM
      IT1=NTOP(I,1)
      IF(IT1.LT.0)GO TO 200
      IT2=NTOP(I,2)
!
      IF(NTCHAN(IT1,1).EQ.0.OR.NTCHAN(IT2,1).EQ.0)THEN
        MTOPA(I,1)=0
        MTOPA(I,2)=-1
        MTOPB(I,1)=0
        MTOPB(I,2)=-1
        GOTO 200
      ENDIF
!
      L=LLCH(NTCHAN(IT2,1))+1
      DO 110 N=NTCHAN(IT1,1),NTCHAN(IT1,2)
      IF(LLCH(N).EQ.L)THEN
        MTOPA(I,1)=N
        GOTO 120
      ENDIF
  110 CONTINUE
  120 IF(LLCH(NTCHAN(IT1,2)).GT.LITLAM(I))THEN
        MTOPA(I,2)=NTCHAN(IT1,2)-1
      ELSE
        MTOPA(I,2)=NTCHAN(IT1,2)
      ENDIF
      IF(MTOPA(I,2).LT.MTOPA(I,1))THEN
        MTOPA(I,1)=0
        MTOPA(I,2)=-1
      ENDIF
!
      L=LLCH(NTCHAN(IT1,1))+1
      DO 130 N=NTCHAN(IT2,1),NTCHAN(IT2,2)
      IF(LLCH(N).EQ.L)THEN
        MTOPB(I,1)=N
        GOTO 140
      ENDIF
  130 CONTINUE
  140 IF(LLCH(NTCHAN(IT2,2)).GT.LITLAM(I))THEN
        MTOPB(I,2)=NTCHAN(IT2,2)-1
      ELSE
        MTOPB(I,2)=NTCHAN(IT2,2)
      ENDIF
      IF(MTOPB(I,2).LT.MTOPB(I,1))THEN
        MTOPB(I,1)=0
        MTOPB(I,2)=-1
      ENDIF
!
      LAT1=LAT(IT1)
      LAT2=LAT(IT2)
!
      KTOPA(I)=NTCHAN(IT2,1)
      LL1=-2
      IF(MTOPA(I,1).GT.0)LL1=LL1+LLCH(MTOPA(I,1))!ARRAY OUT OF BOUNDS
! IF(MTOPA(I,1).EQ.0)THEN NN.EQ.0 AND THE LOOP IS SKIPPED
      LL2=LLCH(KTOPA(I))-2
      M=0
      NN=MTOPA(I,2)-MTOPA(I,1)+1
      DO 160 N=1,NN
       LL1=LL1+2
       LL2=LL2+2
       MAXL=LAT2+LL2
       IF(MAXL.GT.LRGL2)THEN
         F=0.
         W=1./WSQ(LAT1,LAT2,LL1,LL2,LRGL2,ISIGN)
         DO 150 L=LRGL2+1,MAXL
  150     F=F+W*WSQ(LAT1,LAT2,LL1,LL2,L,ISIGN)
         M=M+1
         IF(M.GT.MXF)THEN
           WRITE(6,661)
           STOP '***ERROR: DIMENSION EXCEEDED IN TOP1'
         ENDIF
         FTOPA(I,M)=F
         I1=MTOPA(I,1)+M-1
         I2=KTOPA(I)+M-1
         WRITE(6,650)I,I1,I2,FTOPA(I,M)
       ELSE
         KTOPA(I)=KTOPA(I)+1
         MTOPA(I,1)=MTOPA(I,1)+1
       ENDIF
  160 CONTINUE
!
      KTOPB(I)=NTCHAN(IT1,1)
      LL1=LLCH(KTOPB(I))-2
      LL2=-2
      IF(MTOPB(I,1).GT.0)LL2=LL2+LLCH(MTOPB(I,1))  !ARRAY OUT OF BOUNDS
! IF(MTOPB(I,1).EQ.0)THEN NN.EQ.0 AND THE LOOP IS SKIPPED
      NN=MTOPB(I,2)-MTOPB(I,1)+1
      M=0
      DO 180 N=1,NN
       LL1=LL1+2
       LL2=LL2+2
       MAXL=LAT1+LL1
       IF(MAXL.GT.LRGL2)THEN
         F=0.
         W=1./WSQ(LAT1,LAT2,LL1,LL2,LRGL2,ISIGN)
         DO 170 L=LRGL2+1,MAXL
  170     F=F+W*WSQ(LAT1,LAT2,LL1,LL2,L,ISIGN)
         M=M+1
         IF(M.GT.MXF)THEN
           WRITE(6,661)
           STOP '***ERROR: DIMENSION EXCEEDED IN TOP1'
         ENDIF
         FTOPB(I,M)=F
         I1=KTOPB(I)+M-1
         I2=MTOPB(I,1)+M-1
         WRITE(6,651)I,I1,I2,FTOPB(I,M)
       ELSE
         KTOPB(I)=KTOPB(I)+1
         MTOPB(I,1)=MTOPB(I,1)+1
       ENDIF
  180 CONTINUE
  200 CONTINUE
      ENDIF
!
      RETURN
!
  600 FORMAT(//10X,'TOP-UP FOR (LARGE L).GT.',I2,
     +  ', (SMALL L).GT.LITLAM'//
     +  15X,'ALLOWED TRANSITIONS ARE'//
     +  15X,'INDEX  TARGET STATES     LITLAM')
  610 FORMAT(I18,I9,I5,I12)
  620 FORMAT(/15X,'TOP-UP IN SMALL L FOR'/
     + 15X,'INDEX     CHANNELS')
  630 FORMAT(I18,I9,I5,10X,'TOPA = ',1PE12.4)
  631 FORMAT(I18,I9,I5,10X,'TOPB = ',1PE12.4)
  640 FORMAT(/15X,'TOP-UP IN LARGE L FOR'/15X,'INDEX     CHANNELS')
  650 FORMAT(I18,I9,I5,9X,'FTOPA = ',1PE12.4)
  651 FORMAT(I18,I9,I5,9X,'FTOPB = ',1PE12.4)
  661 FORMAT(/'***INCREASE INTERNAL PARAMETER MXF***')
!
      END
!**********************************************************
!
      SUBROUTINE TOP2
      USE WMATALLOC
      USE NRBRCT
      USE CTOP
      USE NRBTOP
      USE CEN
      USE CINPUT
      USE CINPTX
      USE CHAN
      USE CNTRL
!
! NRB:
! APPLY MULIPOLE TOP-UP (LS AND BP)
!
      IMPLICIT REAL*8 (A-H,O-Y)
      IMPLICIT COMPLEX*16 (Z)
!      PARAMETER (MXTST=(MZTAR*MZTAR+MZTAR)/2)
!      PARAMETER (MXF=(MZLMX+1)/2)
      PARAMETER (TZERO=0.0)
      PARAMETER (ONE=1.0)
      PARAMETER (TWO=2.0)
!
      NTAROP=ITARG(NCHPP)
!
! DIPOLE TOP-UP IN SMALL L (LS AND BP)
!
        IF(LRGL2.EQ.(LRGLAM-1).OR.LRGL2.EQ.LRGLAM)THEN
!
          DO 3000 I=1,INDM
          IT2=NTOP(I,2)
          IF(IT2.LT.0)GO TO 3000
          IF(IT2.GT.NTAROP)GOTO 3000
          IT1=NTOP(I,1)
        IF(NCONAT(IT1)*NCONAT(IT2).EQ.0)GO TO 3000
          IF(TOPA(I).NE.TZERO)THEN
            K=KTST(IT1,IT2)
            I1=NTOPA(I,1)
            I2=NTOPA(I,2)
            OMST(K)=OMST(K)+RK(I1,I2)*(1+(LITLAM(I)**2)*
     X      EPS(I2))*TOPA(I)
          ENDIF
          IF(TOPB(I).NE.TZERO)THEN
            K=KTST(IT1,IT2)
            I1=NTOPB(I,1)
            I2=NTOPB(I,2)
            OMST(K)=OMST(K)+RK(I1,I2)*(1+(LITLAM(I)**2)*
     X      EPS(I1))*TOPB(I)
          ENDIF
 3000     CONTINUE
        ENDIF
!
! DIPOLE TOP-UP IN LARGE L (LS ONLY, SEE TOP1)
!
        IF(LRGL2.EQ.LRGLAM.AND.NSPN2.NE.0)THEN
!
          DO 4000 I=1,INDM
          IT2=NTOP(I,2)
          IF(IT2.LT.0)GO TO 4000
          IF(IT2.GT.NTAROP)GOTO 4000
          IT1=NTOP(I,1)
        IF(NCONAT(IT1)*NCONAT(IT2).EQ.0)GO TO 4000
          K=KTST(IT1,IT2)
          I1=MTOPA(I,1)-1
          I2=KTOPA(I)-1
          DO 4100 N=1,MTOPA(I,2)-MTOPA(I,1)+1
          I1=I1+1
          I2=I2+1
 4100     OMST(K)=OMST(K)+RK(I1,I2)*FTOPA(I,N)
          I1=KTOPB(I)-1
          I2=MTOPB(I,1)-1
          DO 4200 N=1,MTOPB(I,2)-MTOPB(I,1)+1
          I1=I1+1
          I2=I2+1
 4200     OMST(K)=OMST(K)+RK(I1,I2)*FTOPB(I,N)
 4000     CONTINUE
        ENDIF
!
! NRB:
! QUADRUPOLE, OCTUPOLE TOP-UP (LS AND BP)
!
        IF(LRGL2.EQ.LRGLAM)THEN
          TXTRP=DBLE(LRGL2)
          IF(NSPN2.EQ.0)TXTRP=TXTRP/TWO
          IF(IPRINT.GE.0)WRITE(6,801)ETOT
!
          DO 5000 I=1,INDM
            IT2=NTOP(I,2)
            IF(IT2.GT.0)GO TO 5000
            IT2=-IT2
            IF(IT2.GT.NTAROP)GO TO 5000
            IT1=NTOP(I,1)
            IT1=-IT1
            IF(NCONAT(IT1)*NCONAT(IT2).EQ.0)GO TO 5000
            K=KTST(IT1,IT2)
            IF(K.EQ.0)GO TO 5000
            TK2MIN=ETOT-ENAT(IT2)
            TK2MAX=ETOT-ENAT(IT1)
!
! INTERPOLATE BETWEEN DEGENERATE AND NON-DEGENERATE LIMITS WHEN
!           L.LT.2*TK2MIN/(TK2MAX-TK2MIN)
!
            IF(ITOP.EQ.-1)THEN
              AQ=TK2MIN/TK2MAX
              O1=ONE+TXTRP/DBLE(LITLAM(I)-1)
              O1=O1/TWO
              IF(AQ.GT.0.99)THEN
                OMST(K)=OMST(K)*O1
                IF(IPRINT.GE.0)WRITE(6,803)IT1,IT2,AQ,O1
                GO TO 5000
              ENDIF
              O2=ONE/(ONE-AQ)
              BQ=AQ*O2
              IF(TXTRP.GT.TWO*BQ)THEN
                OMST(K)=OMST(K)*O2
                IF(IPRINT.GE.0)WRITE(6,804)IT1,IT2,BQ,O2
                GO TO 5000
              ENDIF
              T=TXTRP/(BQ*TWO)
              O3=O2*T+O1*(ONE-T)
              OMST(K)=OMST(K)*O3
              IF(IPRINT.GE.0)WRITE(6,802)IT1,IT2,AQ,BQ,O1,O2,O3
            ELSE
!
! INTERPOLATE BETWEEN DEGENERATE AND NON-DEGENERATE LIMITS WHEN
!        ENERGY-RATIO EXCEEDS J-RATIO 
!
              AQ=TK2MIN/TK2MAX
              BQ=TXTRP/(TXTRP+1)
              BQ=BQ**(2*LITLAM(I)-1)
              O1=ONE/(ONE-AQ)
              IF(AQ.LT.BQ)THEN
                OMST(K)=OMST(K)*O1
                IF(IPRINT.GE.0)WRITE(6,803)IT1,IT2,AQ,O1
              ELSE
                O2=ONE+TXTRP/DBLE(LITLAM(I)-1)
                O2=O2/TWO
                O3=O1*((ONE-AQ)/(ONE-BQ))**2
     X                       +O2*(AQ-BQ)*(TWO-AQ-BQ)/(ONE-BQ)**2
                OMST(K)=OMST(K)*O3
                IF(IPRINT.GE.0)WRITE(6,802)IT1,IT2,AQ,BQ,O1,O2,O3
              ENDIF
            ENDIF
 5000     CONTINUE
        ENDIF
!
        RETURN
!
  801 FORMAT(//'2**LAM-POLE TOP-UP',3X,'I1',3X,'I2',6X,'AQ',8X,'BQ'
     X,8X,'O1',8X,'O2',8X,'O3'/F9.5)
  802 FORMAT(18X,2I5,5F10.3)
  803 FORMAT(18X,2I5,F10.3,10X,F10.3)
  804 FORMAT(18X,2I5,10X,F10.3,10X,F10.3)
!
        END
!
!******************************************************************
!
      REAL*8 FUNCTION WSQ(A,B,C,D,F,ISIGN)
!
!  CALCULATES 3*(2*F+1)*W(A,B,C,D,1,F)**2 WHERE W IS RACAH COEFFICIENT.
! NRB:
!  ISIGN IS THE SIGN OF W BEFORE SQUARING.
!
      IMPLICIT INTEGER(A-N)
      IMPLICIT REAL*8 (W)
!
!
      ISIGN=(-1)**(F-A-C)
!
      IF(B-A)10,20,30
!
   10 IF(D-C)11,12,13
   20 IF(D-C)21,22,23
   30 IF(D-C)31,32,33
!
   11 WSQ=.75*DBLE((2*F+1)*(B+D+F+2)*(B+D+F+3)*(B+D-F+1)*(B+D-F+2))/
     +    DBLE((B+1)*(2*B+1)*(2*B+3)*(D+1)*(2*D+1)*(2*D+3))
      RETURN
   12 WSQ=.75*DBLE((2*F+1)*(B+D+F+2)*(-B+D+F)*(B-D+F+1)*(B+D-F+1))/
     +    DBLE((B+1)*(2*B+1)*(2*B+3)*D*(D+1)*(2*D+1))
      ISIGN=-ISIGN
      RETURN
   13 WSQ=.75*DBLE((2*F+1)*(-B+D+F-1)*(-B+D+F)*(B-D+F+1)*(B-D+F+2))/
     +    DBLE((B+1)*(2*B+1)*(2*B+3)*D*(2*D-1)*(2*D+1))
      RETURN
!
   21 WSQ=.75*DBLE((2*F+1)*(B+D+F+2)*(-B+D+F+1)*(B-D+F)*(B+D-F+1))/
     +    DBLE(B*(B+1)*(2*B+1)*(D+1)*(2*D+1)*(2*D+3))
      ISIGN=-ISIGN
      RETURN
   22 G=-(B*(B+1)+D*(D+1)-F*(F+1))
      IF(G.LT.0)ISIGN=-ISIGN
      WSQ=.75*DBLE((2*F+1)*G*G)/                !(2*F+1) IS NOT SQUARED-NRB
     +    DBLE(B*(B+1)*(2*B+1)*D*(D+1)*(2*D+1))
      RETURN
   23 WSQ=.75*DBLE((2*F+1)*(B+D+F+1)*(-B+D+F)*(B-D+F+1)*(B+D-F))/
     +    DBLE(B*(B+1)*(2*B+1)*D*(2*D-1)*(2*D+1))
      RETURN
!
   31 WSQ=.75*DBLE((2*F+1)*(-B+D+F+1)*(-B+D+F+2)*(B-D+F-1)*(B-D+F))/
     +    DBLE(B*(2*B-1)*(2*B+1)*(D+1)*(2*D+1)*(2*D+3))
      RETURN
   32 WSQ=.75*DBLE((2*F+1)*(B+D+F+1)*(-B+D+F+1)*(B-D+F)*(B+D-F))/
     +    DBLE(B*(2*B-1)*(2*B+1)*D*(D+1)*(2*D+1))
      RETURN
   33 WSQ=.75*DBLE((2*F+1)*(B+D+F)*(B+D+F+1)*(B+D-F-1)*(B+D-F))/
     +    DBLE(B*(2*B-1)*(2*B+1)*D*(2*D-1)*(2*D+1))
      RETURN
!
      END
!**********************************************************
!
      REAL*8 FUNCTION WSQ2(A,B,C,D,F,ISIGN)
!
!  CALCULATES 3*(2F+1)*W(A,B,C,D,1,F)**2 WHERE W IS RACAH COEFFICIENT.
! NRB:
!  ISIGN IS THE SIGN OF W BEFORE SQUARING,
!*** AND *** A,B,C,D,E,F ARE INPUT TWICE THEIR ACTUAL VALUE.
!
      IMPLICIT INTEGER(A-N)
      IMPLICIT REAL*8 (W)
!
!
      ISIGN=(-1)**((F-A-C)/2)
!
      IF(B-A)10,20,30
!
   10 IF(D-C)11,12,13
   20 IF(D-C)21,22,23
   30 IF(D-C)31,32,33
!
   11 WSQ2=.75*DBLE((F+1)*(B+D+F+4)*(B+D+F+6)*(B+D-F+2)*(B+D-F+4))/
     +    DBLE((B+2)*(B+1)*(B+3)*(D+2)*(D+1)*(D+3))/4.
      RETURN
   12 WSQ2=.75*DBLE((F+1)*(B+D+F+4)*(-B+D+F)*(B-D+F+2)*(B+D-F+2))/
     +    DBLE((B+2)*(B+1)*(B+3)*D*(D+2)*(D+1))/2.
      ISIGN=-ISIGN
      RETURN
   13 WSQ2=.75*DBLE((F+1)*(-B+D+F-2)*(-B+D+F)*(B-D+F+2)*(B-D+F+4))/
     +    DBLE((B+2)*(B+1)*(B+3)*D*(D-1)*(D+1))/4.
      RETURN
!
   21 WSQ2=.75*DBLE((F+1)*(B+D+F+4)*(-B+D+F+2)*(B-D+F)*(B+D-F+2))/
     +    DBLE(B*(B+2)*(B+1)*(D+2)*(D+1)*(D+3))/2.
      ISIGN=-ISIGN
      RETURN
   22 G=-(B*(B+2)+D*(D+2)-F*(F+2))
      IF(G.LT.0)ISIGN=-ISIGN
      WSQ2=.75*DBLE((F+1)*G*G)/                !(F+1) IS NOT SQUARED-NRB
     +    DBLE(B*(B+2)*(B+1)*D*(D+2)*(D+1))
      RETURN
   23 WSQ2=.75*DBLE((F+1)*(B+D+F+2)*(-B+D+F)*(B-D+F+2)*(B+D-F))/
     +    DBLE(B*(B+2)*(B+1)*D*(D-1)*(D+1))/2.
      RETURN
!
   31 WSQ2=.75*DBLE((F+1)*(-B+D+F+2)*(-B+D+F+4)*(B-D+F-2)*(B-D+F))/
     +    DBLE(B*(B-1)*(B+1)*(D+2)*(D+1)*(D+3))/4
      RETURN
   32 WSQ2=.75*DBLE((F+1)*(B+D+F+2)*(-B+D+F+2)*(B-D+F)*(B+D-F))/
     +    DBLE(B*(B-1)*(B+1)*D*(D+2)*(D+1))/2.
      RETURN
   33 WSQ2=.75*DBLE((F+1)*(B+D+F)*(B+D+F+2)*(B+D-F-2)*(B+D-F))/
     +    DBLE(B*(B-1)*(B+1)*D*(D-1)*(D+1))/4.
      RETURN
!
      END
!******************************************************************
!
      SUBROUTINE ZPHIN(I,ZR,N1,N2,ZAI,ZPI)
      USE CACC
      USE CJWBK
      USE NRBZED
!
!  COMPUTES AMPLITUDE ZAI AND PHASE ZPI OF COULOMB FUNCTION ZPHI
!  FOR COMPLEX-RADIAL CO-ORDINATE ZR.
!  USES DATA IN ARRAY D WHICH IS HELD IN MODULE CJWBK
!  AND SHOULD HAVE BEEN COMPUTED IN SUBROUTINE INJWBK.
!  THE STRUCTURE OF ZPHI IS SIMILAR TO THAT OF SUBROUTINE JWBK.
!NRB
!  NEUTRAL CASE ADDED
!
      IMPLICIT REAL*8 (A-H,O-Y)
      IMPLICIT COMPLEX*16 (Z)
!      PARAMETER (MX15N=15*MZCHF)
      DIMENSION ZAI(30),ZPI(30),ZR(30)
!
!
      NP = 0
!
      JKB=(I-1)*15
      E=DDDD(JKB+1)
      C=DDDD(JKB+4)
!
      IF(TZED.EQ.0)THEN
        FK=DDDD(JKB+2)
        IF(C.EQ.0)THEN
          DO N=N1,N2
            NP=NP+1
            ZAI(NP)=1.0/SQRT(FK)
            ZPI(NP)=FK*ZR(NP)
          ENDDO
        ELSE
          DO N=N1,N2
            NP=NP+1
            ZX=1./ZR(NP)
            ZXSQ=ZX*ZX
            ZW=E-C*ZXSQ
            ZA1=0.0625*(ZX/ZW)**3
            ZCC=ZA1*(DDDD(JKB+14)*ZXSQ+DDDD(JKB+12))*ZX
            ZWH=SQRT(ZW)
            Z=ZR(NP)*ZWH
            ZSQ=Z*Z
            ZP=(ZSQ-0.125-.2083333*C/ZSQ)/Z+DDDD(JKB+15)
            ZA1=1./(ZR(NP)*FK)
            ZS=DDDD(JKB+5)*ZA1
            ZG=Z*ZA1
            ZP=ZP+DDDD(JKB+6)*(0.,-1.)*LOG(ZG+(0.,1.)*ZS)
            ZPI(NP)=ZP
            ZAI(NP)=(1.-ZCC)/SQRT(ZWH)
          ENDDO
        ENDIF
        RETURN
      ENDIF
!
      IF(E.GT.0) THEN
        IF(C.GT.0) THEN
          DO N=N1,N2
            NP = NP+1
            ZX=1./ZR(NP)
            ZW=E+ZX*(2.-C*ZX)
            ZWH=SQRT(ZW)
            Z=ZR(NP)*ZWH
            FK=DDDD(JKB+2)
            ZRK=ZR(NP)*FK
            ZRMC=ZR(NP)-C
            ZALP=Z+ZRK
            CK=DDDD(JKB+10)
!  COMPUTE PHASE
            ZP=Z+DDDD(JKB+15)
!  LOG TERM
            ZB=FK*ZALP
            IF(ABS(ZB).GT.ACJWBK) THEN
              ZP=ZP+DDDD(JKB+3)*LOG(1.+ZB)
            ELSE
              ZB=-ZB
              ZP=ZP+ZALP*((((.2*ZB+.25)*ZB+.33333333)*ZB+.5)*ZB+1.)
            END IF
!  ARCTAN TERM
            ZA1=1./(ZR(NP)*DDDD(JKB+7))
            ZS=DDDD(JKB+5)*(Z-FK*ZRMC)*ZA1
            ZG=(CK*Z+ZRMC)*ZA1
            ZP=ZP+DDDD(JKB+6)*(0.,-1.)*LOG(ZG+(0.,1.)*ZS)
!  CAP PHI TERM
            ZP=ZP+((5.*ZRMC/(Z*Z))-(Z*DDDD(JKB+9)+ZRK*DDDD(JKB+8)+CK)/
     X        (ZALP*DDDD(JKB+7)))/(24.*Z)
!  COMPLETE CALCULATION OF ZPHI
            ZA1=.0625*(ZX/ZW)**3
      ZCC=ZA1*
     X (((DDDD(JKB+14)*ZX+DDDD(JKB+13))*
     X ZX+DDDD(JKB+12))*ZX+DDDD(JKB+11))
            ZPI(NP)=ZP
            ZAI(NP)=(1.-ZCC)/SQRT(ZWH)
          ENDDO
        ELSE
!  CASE OF C.EQ.0 AND E.GT.0
!
          DO N=N1,N2
            NP = NP+1
            ZX=1./ZR(NP)
            ZW=2.*ZX+E
            ZWH=SQRT(ZW)
            Z=ZR(NP)*ZWH
            FK=DDDD(JKB+2)
            ZRK=ZR(NP)*FK
            ZALP=Z+ZRK
!  COMPUTE PHASE
            ZP=Z+DDDD(JKB+15)
            ZB=FK*ZALP
            IF(ABS(ZB).GT.ACJWBK) THEN
              ZP=ZP+DDDD(JKB+3)*LOG(1.+ZB)
            ELSE
              ZB=-ZB
              ZP=ZP+ZALP*((((.2*ZB+.25)*ZB+.33333333)*ZB+.5)*ZB+1.)
            END IF
            ZP=ZP+1/(4.*ZALP)+(5.*ZR(NP)/(Z*Z)-2.*(Z+ZALP)/ZALP)/(24.*Z)
!  COMPLETE CALCULATION OF ZPHI
            ZA1=.0625*(ZX/ZW)**3
            ZCC=ZA1*(-4.*E-3.*ZX)
            ZPI(NP)=ZP
            ZAI(NP)=(1.-ZCC)/SQRT(ZWH)
          ENDDO
        ENDIF
      ELSE
        IF(C.EQ.0) THEN
          DO N=N1,N2
            NP = NP+1
            ZX=1./ZR(NP)
            ZW=2.*ZX
            ZWH=SQRT(ZW)
            Z=ZR(NP)*ZWH
            ZP=2.*Z*(1.+.046875*ZX)+DDDD(JKB+15)
            ZWMQ=1./SQRT(ZWH)
            ZET=(1.+.0234375*ZX)*ZWMQ
            ZAI(NP)=ZET
            ZPI(NP)=ZP
          ENDDO
        ELSE
!
!  CASE OF E.EQ.0 AND C.GT.0
!
          DO N=N1,N2
            NP = NP+1
            ZX=1./ZR(NP)
            ZW=ZX*(2.-C*ZX)
            ZWH=SQRT(ZW)
            Z=ZR(NP)*ZWH
            ZRMC=ZR(NP)-C
!  COMPUTE PHASE
            ZP=2.*Z+DDDD(JKB+15)
            ZA1=1./ZR(NP)
            ZS=DDDD(JKB+5)*Z*ZA1
            ZG=ZRMC*ZA1
            ZP=ZP+DDDD(JKB+6)*(0.,-1.)*LOG(ZG+(0.,1.)*ZS)
            ZP=ZP-(3.*ZR(NP)+C)/(24.*(ZRMC+ZR(NP))*Z)
!  COMPLETE CALCULATION OF ZPHI
            ZA1=.0625*(ZX/ZW)**3
            ZCC=((DDDD(JKB+14)*ZX+DDDD(JKB+13))*ZX-3.)*ZX*ZA1
            ZAI(NP)=(1.-ZCC)/SQRT(ZWH)
            ZPI(NP)=ZP
          ENDDO
        ENDIF
      ENDIF
      RETURN
      END
!***************************************************************
!
      SUBROUTINE ZTHETAS(I,ZA1,ZA2,ZA3,ZB,ZG,N1,N2,ZTA,ZTDA,ZTP)
      USE CTHET 
      USE CPOINT
      USE NRBZED
      USE CBLK
!
!  CALCULATES THETA AND THETAD FOR CHANNEL I AND COMPLEX-ZR
!     THETA = ZTA*CEXP(ZTP)
!     THETAD = ZTDA**CEXP(ZTP)
!     ZTP = FNUI*LOG(ZR) - ZR/FNUI
!NRB
!  NEUTRAL CASE ADDED
!
      IMPLICIT REAL*8 (A-H,O-Y)
      IMPLICIT COMPLEX*16 (Z)
      DIMENSION ZTP(30),ZTA(30),ZTDA(30)
!
!
      X=RTWO
      MI=MSUM(I)
      FNUI=BB(I,1)
      E=BG(I,1)
      F2=BG(I,2)
      NP = 0
      DO N=N1,N2
        NP = NP + 1
        U=XLAG(N)
        ZET=ZA1*(SQRT(ZA2+ZA3*U)-1.)
        ZMUM=-.5*ZET/(ZB*(1.+ZET*ZG))
        ZET=ZET*ZET
        ZR=ZET*X
        Z=2.*ZR/FNUI
        ZY=1./Z
        ZAS=1.
        ZS=BB(I,2)
        ZX=0.
        DO M=3,MI
          ZAS=ZAS*ZY
          ZX=ZX+BG(I,M)*ZAS
          ZS=ZS+BB(I,M)*ZAS
        ENDDO
!
        ZDLR=LOG(ZR)*TZED
        ZTP(NP)=(FNUI*ZDLR-.5*Z)
        ZTA(NP)=ZS
        ZTDA(NP)=E*((ZDLR+ZR*F2)*ZS+ZX)
      ENDDO
!
      RETURN
      END
!
!***************************************************************
      SUBROUTINE ALLOCATE_NRBRCT(NCHAN1)
      USE NRBRCT
      USE COULSC
      USE NRBHYB
      USE NRBDD2
      USE CJWBK
      INTEGER :: NCHAN1
!
!      WRITE(0,*)'INSIDE ALLOCATE_NRBRCT'
      ALLOCATE(S(NCHAN1))
      ALLOCATE(SP(NCHAN1))
      ALLOCATE(C(NCHAN1))
      ALLOCATE(CP(NCHAN1))
!
      ALLOCATE(CS(NCHAN1,NCHAN1))
      ALLOCATE(SPP(NCHAN1))
      ALLOCATE(A(NCHAN1,NCHAN1))
      ALLOCATE(B(NCHAN1,NCHAN1))
      ALLOCATE(RK(NCHAN1,NCHAN1))
      ALLOCATE(CSPP(NCHAN1,NCHAN1))
!
      ALLOCATE(CPP(NCHAN1))
      ALLOCATE(CSP(NCHAN1,NCHAN1))
      ALLOCATE(CC(NCHAN1,NCHAN1))
      ALLOCATE(CCP(NCHAN1,NCHAN1))
      ALLOCATE(DSP(NCHAN1,NCHAN1))
!      ALLOCATE(RMAT(NCHAN1,NCHAN1))      
      ALLOCATE(DC(NCHAN1,NCHAN1))
      ALLOCATE(DCP(NCHAN1,NCHAN1))
      ALLOCATE(CCPP(NCHAN1,NCHAN1))
      ALLOCATE(DS(NCHAN1,NCHAN1))
      ALLOCATE(DFPP(NCHAN1,NCHAN1))
!
!     COULSC MODULE
!
      ALLOCATE(FSP(NCHAN1))
      ALLOCATE(FCP(NCHAN1))
!
      ALLOCATE(ICHCL(NCHAN1))
      ALLOCATE(ICHHYB(NCHAN1))
!
      ALLOCATE(FSP2(NCHAN1))
      ALLOCATE(FCP2(NCHAN1))
!
      ALLOCATE(DDDD(NCHAN1*15))
      RETURN
      END
!
!***************************************************************
      SUBROUTINE DEALLOCATE_NRBRCT()
      USE NRBRCT
      USE COULSC
      USE NRBHYB
      USE NRBDD2
      USE CJWBK

!
      IF(ALLOCATED(S))  DEALLOCATE(S)
      IF(ALLOCATED(SP)) DEALLOCATE(SP)
      IF(ALLOCATED(C))  DEALLOCATE(C)
      IF(ALLOCATED(CP)) DEALLOCATE(CP)
!
      IF(ALLOCATED(CS))   DEALLOCATE(CS)
      IF(ALLOCATED(SPP))  DEALLOCATE(SPP)
      IF(ALLOCATED(A))    DEALLOCATE(A)
      IF(ALLOCATED(B))    DEALLOCATE(B)
      IF(ALLOCATED(RK))   DEALLOCATE(RK)
      IF(ALLOCATED(CSPP)) DEALLOCATE(CSPP)
!
      IF(ALLOCATED(CPP))  DEALLOCATE(CPP)
      IF(ALLOCATED(CSP))  DEALLOCATE(CSP)
      IF(ALLOCATED(CC))   DEALLOCATE(CC)
      IF(ALLOCATED(CCP))  DEALLOCATE(CCP)
      IF(ALLOCATED(DSP))  DEALLOCATE(DSP)
!      IF(ALLOCATED())     DEALLOCATE(RMAT)
      IF(ALLOCATED(DC))   DEALLOCATE(DC)
      IF(ALLOCATED(DCP))  DEALLOCATE(DCP)
      IF(ALLOCATED(CCPP)) DEALLOCATE(CCPP)
      IF(ALLOCATED(DS))   DEALLOCATE(DS)
      IF(ALLOCATED(DFPP)) DEALLOCATE(DFPP)
!
!      NOW THE COULSC VARIABLES
!
      IF(ALLOCATED(FSP)) DEALLOCATE(FSP)
      IF(ALLOCATED(FCP)) DEALLOCATE(FCP)
!
      IF(ALLOCATED(ICHCL))  DEALLOCATE(ICHCL)
      IF(ALLOCATED(ICHHYB)) DEALLOCATE(ICHHYB)
!
      IF(ALLOCATED(FSP2)) DEALLOCATE(FSP2)
      IF(ALLOCATED(FCP2)) DEALLOCATE(FCP2)
!
      IF(ALLOCATED(DDDD)) DEALLOCATE(DDDD)
!      
      RETURN
      END
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
      SUBROUTINE CTOP_ALLOCATE(NAST)
      USE CTOP
      USE WMATALLOC, ONLY : MAXMUL
!      PARAMETER (MXF=(MZLMX+1)/2)
      INTEGER :: NAST,MXTST
!
      MXTST=(NAST*NAST+NAST)/2
!
      IF(ALLOCATED(LITLAM)) DEALLOCATE(LITLAM)
      ALLOCATE(LITLAM(MXTST))
      IF(ALLOCATED(NTOP))   DEALLOCATE(NTOP)
      ALLOCATE(NTOP(MXTST,2))
      IF(ALLOCATED(NTOPA))  DEALLOCATE(NTOPA)
      ALLOCATE(NTOPA(MXTST,2))
      IF(ALLOCATED(NTOPB))  DEALLOCATE(NTOPB)
      ALLOCATE(NTOPB(MXTST,2))
      IF(ALLOCATED(NTCHAN)) DEALLOCATE(NTCHAN)
      ALLOCATE(NTCHAN(NAST,2))
      IF(ALLOCATED(TOPA))   DEALLOCATE(TOPA)
      ALLOCATE(TOPA(MXTST))
      IF(ALLOCATED(TOPB))   DEALLOCATE(TOPB)
      ALLOCATE(TOPB(MXTST))
      IF(ALLOCATED(MTOPA))  DEALLOCATE(MTOPA)
      ALLOCATE(MTOPA(MXTST,2))
      IF(ALLOCATED(MTOPB))  DEALLOCATE(MTOPB)
      ALLOCATE(MTOPB(MXTST,2))
      IF(ALLOCATED(KTOPA))  DEALLOCATE(KTOPA)
      ALLOCATE(KTOPA(MXTST))
      IF(ALLOCATED(KTOPB))  DEALLOCATE(KTOPB)
      ALLOCATE(KTOPB(MXTST))
      MXF=(MAXMUL+1)/2
      IF(ALLOCATED(FTOPA))  DEALLOCATE(FTOPA)
      ALLOCATE(FTOPA(MXTST,MXF))
      IF(ALLOCATED(FTOPB))  DEALLOCATE(FTOPB)
      ALLOCATE(FTOPB(MXTST,MXF))
!
      RETURN
      END
!
!***************************************************************
!
! No longer used parts, moved at the end of the code
! Old subroutines, old common blocks, old PARAM file
!
!---------------------------------------------------------------------
!
!        MODULE OLDPARAM
!
! Module with the old PARAM file.
! Now change the INCLUDE "PARAM" sentence for USE OLDPARAM
!
!        INCLUDE 'PARAM'
! - or -
!        PARAMETER (MZCHF=  2580)    !CHANNELS
!        PARAMETER (MZTAR=   300)    !TARGET STATES
!        PARAMETER (MZLMX=    14)    !MULTIPOLES
!        PARAMETER (MZLP1=    87)    !L VALUES FOR BUTTLE
!        PARAMETER (MZPTS=  3201)    !OUTER-REGION POINTS
!        PARAMETER (MZMSH=100001)    !FREE-STATE ENERGIES
!        PARAMETER (MZMNP=  9000)    !R-MATRIX EIGENVALUES
!        PARAMETER (MZSLP=   125)    !SLPI VALUES
!        PARAMETER (MZTET=   250)    !COEFFICIENTS FOR THETA EXPANSION
!        PARAMETER (MZDEG= MZCHF)    !CLOSED CHANNELS FOR MQDT
!        PARAMETER (MZNRG=    90)    !PARAMETER FOR NEW BUTTLE FIT (APPR.=NRANG2)
!        PARAMETER (MZREC=     8)    !RECORD LENGTH FOR ONE WORD
!        PARAMETER (MZMEG=     0)    !MEGA-WORDS OF INTERNAL MEMORY TO REDUCE I/O
!        PARAMETER (MZMEG=(HUGE(1)/1E6)-1)    !MEGA-WORDS OF INTERNAL MEMORY TO REDUCE I/O
!
! STGFDAMP/STGBF0DAMP ONLY
! DR part to remove
!        PARAMETER (MZMET=     1)    !INITIAL STATES/SYMMETRIES FOR PR/PI
!
!        END MODULE OLDPARAM
!
!***************************************************************
!C
!      SUBROUTINE OMEG(IE,EMESH,MXE,ENAT,NAST,OMEGA,NOMWRT,IONE)
!      USE MEMORY
!      USE NRBTOP
!      USE OLDPARAM
!C
!C   USE MEMORY FOR SUMMING OMEGA, WITH OVERFLOW ONTO DISK
!C   (ADAPTED BY NRB, FROM KAB, FOR ELASTIC TRANSITIONS
!C    AND USE OF NOMT=ABS(NOMWRT) TO REDUCE I/O)
!C   IE = 0 TO INITIALIZE
!C   IE .GT.0 TO RETRIEVE OMEGA FOR ENERGY IE
!C   IF NOMWRT .LT. 0 THEN RECOVER -NOMWRT NON-ZERO OMEGAS FOR 
!C                    FINAL WRITE RUNNING DOWN THE COLUMNS
!C   IF NOMWRT .GT. 0 THEN RECOVER NOMWRT OMEGAS (INC ZEROES) FOR
!C                    FINAL WRITE RUNNING ALONG THE ROWS
!C          (EXC ZEROES CAN BE IMPLEMENTED BY SWITCHING JJJJ BELOW)
!C   EMESH(MXE)  = ENERGY MESH
!C   ENAT(NAST)  = EXCITATION THRESHOLDS: NEEDED TO MINIMISE STORAGE
!C   OMEGA(NOMT)= RETURNS OMEGA MATRIX AS AN ARRAY
!C   /MEMORY/ STORAGE IN OMEM(MXOM)
!C   MPOS(IE)    = LAST POSITION OCCUPIED IN OMEM AT ENERGY IE, OR
!C   IF MPOS.LT.0 = -RECORD NUMBER ON DA SCRATCH FILE
!C
!      IMPLICIT REAL*8 (A-H,O-Z)
!C
!!      INCLUDE 'PARAM'
!C
!      PARAMETER (TZERO=0.0)
!C
!      PARAMETER (MZKIL=  0)
!!      PARAMETER (MXTST=(MZTAR*MZTAR+MZTAR)/2)
!      PARAMETER (MXOM=MZMEG*1000000+MZKIL*1000)
!C
!C      COMMON/MEMORY/OMEM(MXOM+1),MPOS(0:MZMSH),ITMAX,JTMAX
!C      COMMON/NRBTOP/ITST(MXTST),JTST(MXTST),KTST(MZTAR,MZTAR)
!C     X             ,OMST(MXTST),ITOP
!C
!      DIMENSION EMESH(MXE),ENAT(NAST),OMEGA(IABS(NOMWRT))
!C
!      NOMT=ABS(NOMWRT)
!      N=0
!      IF(NOMWRT.GT.0)THEN
!        DO I=1,NAST
!          DO J=I+IONE,NAST
!            N=N+1
!            OMEGA(N)=TZERO
!          ENDDO
!          IF(N.GE.NOMT)GO TO 20
!        ENDDO
!        I=NAST
!      ENDIF
!      IF(NOMWRT.LT.0)THEN
!        DO J=1+IONE,NAST
!          DO I=1,J-IONE
!            N=N+1
!            OMEGA(N)=TZERO
!          ENDDO
!          IF(N.GE.NOMT)GO TO 20
!        ENDDO
!        J=NAST
!      ENDIF
!C
!C   INITIALISE
!C
!C
!C
!   20 IF(IE.EQ.0) THEN
!
!        ALLOCATE(MPOS(0:MZMSH))
!
!        IF(NOMWRT.GT.0)THEN
!          ITMAX=I
!          JTMAX=NAST
!        ELSE
!          ITMAX=NAST
!          JTMAX=J
!        ENDIF
!        MPOS(0)=0
!        NPOS=0
!        NREC=0
!        NST=1
!        DO I=1,MXE
!    4     IF(NST.LE.NAST)THEN
!            IF(EMESH(I).GE.ENAT(NST)) THEN
!              NP=MIN(NST,ITMAX,JTMAX)
!              NTRAN=(NP*(NP+1-2*IONE))/2
!              IF(NST.GT.ITMAX)NTRAN=NTRAN+(NST-ITMAX)*ITMAX
!              NST=NST+1
!              GO TO 4
!            ENDIF
!          ENDIF
!          NPOS=NPOS+NTRAN
!          IF(NPOS.LE.MXOM) THEN
!            MPOS(I)=NPOS
!          ELSE
!            NREC=NREC+1
!            MPOS(I)=-NREC
!          ENDIF
!        ENDDO
!C
!        MMM=MIN(NPOS,MXOM)
!
!        ALLOCATE(OMEM(MMM))
!
!        DO N=1,MMM
!          OMEM(N)=TZERO
!        ENDDO
!C
!        IF(NREC.GT.0) THEN
!          MMEG=NPOS/1000000
!          MMEG=MMEG+1
!          NPOS=MXOM
!          WRITE(6,100)MMEG
!          NLEN=MIN(NOMT,(NAST*(NAST+1-2*IONE))/2)
!          OPEN(1,STATUS='SCRATCH',ACCESS='DIRECT',RECL=MZREC*NLEN,
!     X         FORM='UNFORMATTED')
!          DO MREC=1,NREC
!            CALL OMWRIT(OMEGA,NLEN,MREC)
!          ENDDO
!        ENDIF
!C
!        WRITE(6,101)MXOM,NPOS
!C
!      ELSE
!C
!C   FINALLY, RETRIEVE AND RETURN OMEGA
!C
!        IF(MPOS(IE).EQ.0) THEN
!          DO N=1,NOMT
!            OMEGA(N)=TZERO
!          ENDDO
!          GO TO 25
!        ENDIF
!C
!        JJJJ=NAST                !INC ZEROES NOMWRT.GT.0
!C        JJJJ=JTMAX               !EXC ZEROES NOMWRT.GT.0
!C
!        IF(MPOS(IE).GT.0) THEN
!          M=MPOS(IE-1)
!          K=0
!          DO JT=1+IONE,JJJJ
!            IP=MIN(JT-IONE,ITMAX)
!            DO IT=1,IP
!              M=M+1
!              IF(NOMWRT.GT.0)THEN
!                K=JT+1-IONE+(JJJJ+1-IONE)*(IT-1)-(IT*(IT+1))/2
!              ELSE
!                K=K+1
!              ENDIF
!              OMEGA(K)=OMEM(M)
!            ENDDO
!            IF(M.GE.MPOS(IE)) GO TO 25
!          ENDDO
!        ELSE
!          MREC=-MPOS(IE)
!          CALL OMREAD(OMEGA,NOMT,MREC)
!          IF(NOMWRT.GT.0)THEN
!            DO I=1,NOMT
!              OMST(I)=OMEGA(I)
!            ENDDO
!            M=0
!            DO JT=1+IONE,JJJJ
!              IP=MIN(JT-IONE,ITMAX)
!              DO IT=1,IP
!                M=M+1
!                K=JT+1-IONE+(JJJJ+1-IONE)*(IT-1)-(IT*(IT+1))/2
!                OMEGA(K)=OMST(M)
!              ENDDO
!              IF(M.GE.NOMT) GO TO 25
!            ENDDO
!          ENDIF
!        ENDIF
!      ENDIF
!C
!C
!   25 RETURN
!C
!  100     FORMAT(//' ****OPENING SCRATCH FILE, COULD AVOID'
!     X    ,' BY INCREASING MZMEG TO: ',I4//)
!  101   FORMAT(//' MXOM = ', I9,'  USED = ',I9//)
!      END
!!
!************************************************************************
!!
!      SUBROUTINE READ2(IOPT1)
!      Use comm_interface
!      USE WMATALLOC
!      USE TERMSYM
!      USE PART
!      USE DBUT
!      USE CINPUT
!      USE CINPTX
!      USE CHAN
!      USE CDEGEN
!      USE CDEC
!      USE NRBLMX
!      USE CNTRL
!!      USE OLDPARAM
!!
!!  READS R-MATRIX DATA FOR ONE SLPI CASE, FROM FILA
!!
!!  THE FOLLOWING DATA ARE READ -
!!       LRGL2 = TOTAL ORBITAL ANGULAR MOMENTUM/ 2J
!!       NSPN2 = TOTAL (2*S+1)/0
!!       NPTY2 = TOTAL PARITY
!!       NCHAN = NUMBER OF CHANNELS
!!       MNP2 = NUMBER OF R-MATRIX POLES
!!       MORE2 = ZERO TO TERMINATE SLPI CASES
!!       FOR I = 1,NAST -
!!               NCONAT(I) = NUMBER OF CHANNELS FOR TARGET STATE I
!!       FOR I = 1,NCHAN -
!!               L2P(I), KJ(I) = SMALL L AND BIG JK FOR CHANNEL I
!!                             = K-1, 0 FROM DSTG2/DARC
!!       FOR I = 1,NCHAN AND N = 1,NCHAN AND M = 1,LAMAX -
!!               CF(I,N,M) = COEFFICIENTS IN MULTIPOLE POTENTIALS
!!       FOR I = 1,MNP2 -
!!               VALUE(I) = R-MATRIX POLE ENERGIES
!!       FOR K = 1,NCHAN AND I = 1,MNP2 -
!!               WMAT(I,K) = R-MATRIX AMPLITUDES
!!
!      IMPLICIT REAL*8 (A-H,O-Z)
!!
!!      INCLUDE 'PARAM'
!!      PARAMETER (MXTST=(MZTAR*MZTAR+MZTAR)/2)
!!
!!      COMMON/CHAN/ECH(MZCHF),EPS(MZCHF),FKNU(MZCHF),CCT(MZCHF)
!!     1  ,RINF(MZCHF),ITARG(MZCHF),LLCH(MZCHF),NCHF,NCHPP,NCHPP1
!!      COMMON/CINPUT/
!!     1 LAMAX,LRANG2,LRGL2,MNP2,NAST,NCHAN,NELC,NPTY2,NSPN2,NZ,MORE2,
!!     2 ISAT(MZTAR),LAT(MZTAR),L2P(MZCHF),NCONAT(MZTAR),KJ(MZCHF),KFLAG
!!      COMMON/CINPTX/BSTO,RA,
!!     4 COEFF(3,MZLP1),ENAT(MZTAR)
!!     5 ,VALUE(MZMNP)
!!      COMMON/CDEC/ARAD(MXTST),ARDEC(MZTAR),SLIN(MXTST),IRDEC,IEND
!!     X,IPAR(MZTAR),NEWAR
!!      COMMON/CDEGEN/ENATR(MZTAR),NASTD,NASTR,NLEV(MZTAR),NCNATR(MZTAR)
!!     X,IWD(MZTAR),IWT
!!      COMMON/CNTRL/IPRINT,IPRKM,IWORD,IRAD,IPERT,NPERT,IPRTSW
!!      COMMON/DBUT/EBUTD(MZNRG,MZLP1),CBUTD(MZNRG,MZLP1),NBUTD(MZNRG)
!!     X           ,K2P(MZCHF)
!!      COMMON/NRBLMX/LMX
!!      COMMON/PART/EIGENS(MZNRG,MZLP1),ENDS(MZNRG,MZLP1),SI(MZCHF),
!!     X            TRACE,NRANG1(MZLP1),NRANG2,IPRCENT
!!      COMMON/PARAINFO/IAM,NPROC
!!
!      LOGICAL BFORM,LASTSYM
!      CHARACTER(LEN=1) :: NUM(0:9)=
!     &   [ "0","1","2","3","4","5","6","7","8","9" ]
!
!      DATA KCOUNT/0/,JCOUNT/0/
!!
!      SAVE KCOUNT,JCOUNT
!!
!!
!!  READ AND DIMENSION CHECKS
!!
!      IF(IWORD.NE.2)THEN
!        READ(10,END=999)LRGL2,NSPN2,NPTY2,NCHAN,MNP2,MORE2
!      ELSE                               !DARC
!        READ(10,END=999)LRGL2,NPTY2,NCHAN,MNP2,NCFGP,LAMAX
!        MORE2=1
!        LRGL2=ABS(LRGL2)-1
!        NSPN2=0
!        IF(NPTY2.EQ.-1)THEN
!          NPTY2=1
!        ELSE
!          NPTY2=0
!        ENDIF
!        KFLAG=-2
!      ENDIF
!!
!!
!       INQUIRE(FILE='H.DAT',EXIST=BFORM)
!       IF(BFORM)GOTO 1000
!!
!!      WRITE(0,*)'INSIDE READ2',KCOUNT,MSLP(KCOUNT+1),IONEONE
!!
!      IF(MSLP(JCOUNT+1).NE.(10000*IABS(NSPN2)+100*LRGL2+NPTY2)
!     X   .AND.(IOPT1.EQ.2)
!     X   .AND.(IONEONE.EQ.1))THEN
!
!      CLOSE(10)
!
!      IF(IAM.EQ.0)WRITE(0,*)'SKIP ',10000*IABS(NSPN2)+100*LRGL2+NPTY2
!
!!      WRITE(0,*)'GET HERE',10000*IABS(NSPN2)+100*LRGL2+NPTY2,
!!     X          MSLP(JCOUNT+1),KCOUNT
!
!      KCOUNT=KCOUNT+1
!
!      I1=KCOUNT/100
!      I2=(KCOUNT-100*(KCOUNT/100))/10
!      I3=KCOUNT-(100*(KCOUNT/100))-I2*10
!
!      OPEN(10,FILE='H.DAT'//NUM(I1)//NUM(I2)//NUM(I3),
!     X FORM='UNFORMATTED',STATUS='OLD')
!
!      RETURN
!
!      ELSE
!
!      IF(IAM.EQ.0)WRITE(0,*)'CALC ',10000*IABS(NSPN2)+100*LRGL2+NPTY2
!
!      KCOUNT=KCOUNT+1
!      JCOUNT=JCOUNT+1
!
!        IF(JCOUNT.EQ.KSLP)THEN
!        IF(IAM.EQ.0)WRITE(0,*)'LAST SPECIFIED SYMMETRY FOUND'
!        MORE2=0
!        ENDIF
!      ENDIF
!!
! 1000 CONTINUE
!!
!      IF(ALLOCATED(NCONAT))DEALLOCATE(NCONAT)
!      ALLOCATE(NCONAT(ABS(NAST)))
!!
!      READ(10)(NCONAT(I),I=1,NASTR)     ! NAST?
!
!!      IF(NCHAN.GE.MZCHF)THEN            !.GE. FOR SR.LU
!!        WRITE(6,600)NSPN2,LRGL2,NPTY2,NCHAN
!!        STOP  '*** DIMENSION EXCEEDED: INCREASE MZCHF'
!!      ENDIF
!!
!       IF(ALLOCATED(L2P))DEALLOCATE(L2P)
!       ALLOCATE(L2P(NCHAN))
!!
!       IF(ALLOCATED(KJ))DEALLOCATE(KJ)
!       ALLOCATE(KJ(NCHAN))
!!
!       IF(NSPN2.NE.0)READ(10)(L2P(I),I=1,NCHAN)
!
!       IF(NSPN2.EQ.0)THEN
!
!        IF(KFLAG.EQ.1)READ(10,ERR=997)(L2P(I),I=1,NCHAN),
!     X                                      (KJ(I),I=1,NCHAN)
!        IF(KFLAG.NE.1)READ(10)(L2P(I),I=1,NCHAN)
!      ENDIF
!!
!!      IF(LAMAX.GT.MZLMX)THEN
!!        MLAMRD=MZLMX
!!        IF(LMX.GT.MLAMRD)WRITE(6,620)LMX
!!      ELSE
!!        MLAMRD=LAMAX
!!      ENDIF
!!      IF(LAMAX.GT.MZLMX)THEN
!!      write(0,*)'set mzlmx to lamax, see stage 3 rout3r'
!!      stop
!!      ENDIF
!!
!!     ****************** CPB 2016 *****************
!
!      MAXMUL=MAX(MAXMUL,LAMAX)
!
!      IF(ALLOCATED(CF))DEALLOCATE(CF)
!      IF(ALLOCATED(WMAT))DEALLOCATE(WMAT)
!      IF(ALLOCATED(VALUE))DEALLOCATE(VALUE)
!      ALLOCATE(CF(NCHAN,NCHAN,MAXMUL))
!!
!!     ****************** CPB 2016 *****************
!!
!!
!!  
!      IF(LAMAX.GT.0)
!     XREAD(10)(((CF(I,N,M),I=1,NCHAN),N=1,NCHAN),M=1,LAMAX)
!!      IF(LAMAX.GT.MLAMRD)
!!     XREAD(10)(((DUM,I=1,NCHAN),N=1,NCHAN),M=1,LAMAX-MLAMRD)
!!
!!
!      MAXMUL=MIN(LAMAX,MAXMUL)
!
!      ALLOCATE(TTT(NCHAN,NCHAN,MAXMUL))
!!
!      TTT(1:NCHAN,1:NCHAN,1:MAXMUL)=CF(1:NCHAN,1:NCHAN,1:MAXMUL)
!!
!      DEALLOCATE(CF)
!!
!      ALLOCATE(CF(1:NCHAN,1:NCHAN,1:MAXMUL))
!!
!      CF(1:NCHAN,1:NCHAN,1:MAXMUL)=TTT(1:NCHAN,1:NCHAN,1:MAXMUL)
!!
!      DEALLOCATE(TTT) 
!!
!!      IF(MNP2.GT.MZMNP)THEN
!!        WRITE(6,610)NSPN2,LRGL2,NPTY2,MNP2
!!        STOP '*** DIMENSION EXCEEDED: INCREASE MZMNP'
!!      ENDIF
!!
!      IF(IPRCENT.NE.100)READ(10)TRACE
!!
!!
!!    ************************************************
!!
!       IF(ALLOCATED(WMAT))THEN
!       DEALLOCATE(WMAT)
!       ENDIF
!!
!       IF(ALLOCATED(VALUE))THEN
!       DEALLOCATE(VALUE)
!       ENDIF
!!
!       ALLOCATE(WMAT(ABS(MNP2),NCHAN)) 
!       ALLOCATE(VALUE(ABS(MNP2)))
!!     ************************************************         
!!
!      IF(MNP2.GE.0)THEN
!        READ(10)(VALUE(I),I=1,MNP2)
!        READ(10)((WMAT(I,K),K=1,NCHAN),I=1,MNP2)
!      ELSE                                   !DIVIDED WMAT - CPB
!        MNP2=-MNP2
!!      WRITE(0,*)'GET READ2 DIV',NCHAN,MNP2
!        READ(10)(VALUE(I),I=1,MNP2)
!        KK=0
!  13    READ(10)ILOW,IUPPER,NDIV
!        READ(10)((WMAT(I-ILOW+1+KK,K),K=1,NCHAN),I=ILOW,IUPPER)
!        KK=KK+IUPPER-ILOW+1
!        IF(ILOW.NE.1) GO TO 13
!      ENDIF
!!
!      IF(IWORD.EQ.2)THEN                     !DARC
!        DO I=1,NCHAN
!          L2P(I)=2*L2P(I)                    !2*KAPPA
!          IF(L2P(I).LT.0)L2P(I)=-L2P(I)-1    !K, ANALOGUE OF L
!          L2P(I)=L2P(I)-1                    !K-1
!        ENDDO
!      ENDIF
!      IF(KFLAG.LT.0)THEN                     !L2P(I)=K-1
!!
!!     ARDEC/IDIP/RAD forces earlier read2 so allocate K2P
!!
!       if(allocated(K2P)) DEALLOCATE(K2P)
!       allocate(k2p(nchan))
!!
!        DO I=1,NCHAN
!          LLCH(I)=(L2P(I)+1)/2               !CHANNEL ORB ANG 
!          K2P(I)=LLCH(I)
!          IF(LLCH(I)*2.NE.L2P(I)+1)K2P(I)=-K2P(I)-1   !KAPPA
!          KJ(I)=0                            !AS JJ COUPLING
!        ENDDO
!      ELSE
!        DO I=1,NCHAN
!          LLCH(I)=L2P(I)                     !CHANNEL ORB ANG
!        ENDDO
!      ENDIF
!!
!!  GROUP TOGETHER CHANNELS BELONGING TO DEGENERATE LEVELS
!!  AND PRESERVE THE READ VALUES OF NCONAT IN /CDEGEN/
!      IF(IOPT1.GT.9) NASTR=NAST
!      IF(NASTD.GT.0)THEN
!        DO I=1,NASTR
!          NCNATR(I)=NCONAT(I)
!        ENDDO
!        IF(IOPT1.LT.10)THEN
!          N1=1
!          DO I=1,NASTD
!            NCON=0
!            N2=NLEV(I)+N1-1
!            DO IN=N1,N2
!              NCON=NCON+NCNATR(IN)
!            ENDDO
!            NCONAT(I)=NCON
!            N1=N2+1
!          ENDDO
!        ENDIF
!      ENDIF
!!
!!  TARGET PARITIES
!!
!      K=1
!      DO I=1,NAST
!        IF(NCONAT(I).NE.0)THEN
!          IF(IPAR(I).LT.0)IPAR(I)=ABS(NPTY2-LLCH(K)+2*(LLCH(K)/2))
!          K=K+NCONAT(I)
!        ENDIF
!      ENDDO
!!
!      IF (NSPN2.EQ.0.AND.KFLAG.EQ.0) THEN
!!
!!      RECOVER THE K CHANNEL NUMBERS (BASED ON SR.NJCHAN) IF
!!      NOT READ FROM H.DAT
!!
!      NJCHA = 0
!!
!!      LOOP OVER THE TARGET STATES.
!!
!      DO 4 I = 1,NAST
!!
!!      LOOP OVER THE ONLY TWO POSSIBLE 2K VALUES, THAT IS LRGL2-1,LRGL2+1
!!
!        IFIN = 3
!        IF (LRGL2.EQ.0) IFIN = 1
!        DO 3 K = 1,IFIN,2
!          JK = ABS(LRGL2-2+K)
!!
!!      RECOVER THE RANGE OF L(INCIDENT) VALUES.
!!
!          LMIN = ABS(LAT(I)-JK)
!          LMAX = LAT(I) + JK
!!
!!      CHECK IF LMIN IS AN INTEGER.
!!
!          IF (MOD(LMIN,2).NE.0) GOTO 3
!!
!!      CHECK THE PARITY OF LMIN.
!!
!          LP = LMIN/2 + IPAR(I)
!          IF (MOD(LP,2).NE.NPTY2) LMIN = LMIN + 2
!!
!          LP = (LMIN+LMAX)/2
!          IF (MOD(LP,2).EQ.1) LMAX = LMAX - 2
!          IF (LMIN.GT.LMAX) GOTO 3
!!
!!      STORE THE 2K-VALUES IN KJ.
!!
!          LMIN = LMIN + 1
!          LMAX = LMAX + 1
!          DO 2 L = LMIN,LMAX,4
!            NJCHA = NJCHA + 1
!            KJ(NJCHA) = JK
!    2     CONTINUE
!    3   CONTINUE
!    4 CONTINUE
!!
!! END RECOVERY OF K QUANTUM NUMBER
!!
!      ENDIF
!!
!!      IF((JCOUNT.EQ.KSLP).AND.(IONEONE.EQ.1))THEN
!!      MORE2=-777
!!      ENDIF
!!
!      IF((IONEONE.EQ.1).AND.(JCOUNT.NE.KSLP))THEN
!
!      CLOSE(10)
!
!      I1=KCOUNT/100
!      I2=(KCOUNT-100*(KCOUNT/100))/10
!      I3=KCOUNT-(100*(KCOUNT/100))-I2*10
!
!      INQUIRE(FILE='H.DAT'//NUM(I1)//NUM(I2)//NUM(I3),EXIST=LASTSYM)
!  
!      IF(.NOT.LASTSYM)THEN
!      MORE2=-777
!      RETURN
!      ENDIF
!
!!      IF(IAM.EQ.0)WRITE(0,*),'OPEN ','H.DAT'//NUM(I1)//NUM(I2)//NUM(I3)
!
!
!
!      OPEN(10,FILE='H.DAT'//NUM(I1)//NUM(I2)//NUM(I3),
!     X FORM='UNFORMATTED',STATUS='OLD')
!
!      ENDIF
!!
!!      IF((JCOUNT.EQ.KSLP).AND.(IONEONE.EQ.1))THEN
!!      MORE2=-777
!!      ENDIF
!      RETURN
!!
!  999 MORE2=-777
!      RETURN
!  997 WRITE(6,*)'***SR.READ2 ERROR, TRY SETTING KLAG=0'
!      STOP '***SR.READ2 ERROR, TRY SETTING KLAG=0'
!!  600 FORMAT(///20X,'TOO MANY CHANNELS FOR (IS, IL, IP) = (',
!!     1  3I3,')'//10X,'VALUE READ FOR NCHAN IS ',I4//
!!     2  10X,'MAXIMUM ALLOWED BY DIMENSIONS IS MZCHF'//)
!!  610 FORMAT(///20X,'TOO MANY R-MATRIX STATES FOR (IS, IL, IP) = (',
!!     * 3I3,')'//10X,'VALUE READ FOR MNP2 IS ',I5//    
!!     3 10X,'MAXIMUM ALLOWED BY DIMENSIONS IS MZMNP'//)
!!  620 FORMAT(///20X,'WARNING TOO MANY MULTIPOLES REQUESTED VIZ.',I3//
!!     2 20X,'MAXIMUM ALLOWED BY DIMENSIONS IS MZLMX'//)
!!
!      END
!!
!***************************************************************
!C
!      SUBROUTINE RINIT3
!      USE WMATALLOC, ONLY : MXEONE,VALUE,WMAT,WM1,WM2
!      USE NRBRCT, ONLY : RMAT
!      USE PART
!      USE DBUT
!      USE CEN
!      USE CINPUT
!      USE CINPTX
!      USE CHAN
!      USE CPOINT
!      USE CNTRL
!      USE CLOGB
!C
!C NRB: USES WMAT WITH INDEXES INTERCHANGED COMPARED TO CPC.
!C
!C  INITIALIZE CALCULATION OF R-MATRIX.
!C
!      IMPLICIT REAL*8 (A-H,O-Y)
!      IMPLICIT COMPLEX*16 (Z)
!C
!C      LOGICAL NEWBUT
!C
!      INCLUDE 'PARAM'
!C
!      PARAMETER (ONE=1.D0)
!      PARAMETER (TZERO=0.D0)
!C
!C      COMMON/CEN/ETOT,MXE,NWT,NZ
!C  ***  NOTE CHANGE OF CC TO CCT IN /CHAN/ ***
!C      COMMON/CHAN/ECH(MZCHF),EPS(MZCHF),FKNU(MZCHF),CCT(MZCHF)
!C     1  ,RINF(MZCHF),ITARG(MZCHF),LLCH(MZCHF),NCHF,NCHPP,NCHPP1
!c      COMMON/CINPUT/
!c     1 LAMAX,LRANG2,LRGL2,MNP2,NAST,NCHAN,NELC,NPTY2,NSPN2,NZED,MORE2,
!c     2 ISAT(MZTAR),LAT(MZTAR),L2P(MZCHF),NCONAT(MZTAR),KJ(MZCHF),KFLAG
!c      COMMON/CINPTX/BSTO,RA,
!c     4 COEFF(3,MZLP1),ENAT(MZTAR)
!C     5 ,VALUE(MZMNP)
!C      COMMON/CLOGB/NEWBUT
!C      COMMON/CPOINT/RZERO,RONE,RTWO,H,KP0,KP1,KP2
!C      COMMON/CNTRL/IPRINT,IPRKM,IWORD,IRAD,IPERT,NPERT,IPRTSW
!C      COMMON/DBUT/EBUTD(MZNRG,MZLP1),CBUTD(MZNRG,MZLP1),NBUTD(MZNRG)
!C     X           ,K2P(MZCHF)
!C      COMMON/PART/EIGENS(MZNRG,MZLP1),ENDS(MZNRG,MZLP1),SI(MZCHF),
!C     X            TRACE,NRANG1(MZLP1),NRANG2,IPRCENT
!C
!C      COMMON/NRBRCT/
!C     X S(MZCHF),SP(MZCHF),C(MZCHF),CP(MZCHF),SPP(MZCHF),CPP(MZCHF)
!C     X,A(MZCHF,MZCHF),B(MZCHF,MZCHF),RK(MZCHF,MZCHF)
!C     X,CS(MZCHF,MZCHF),CSP(MZCHF,MZCHF),CC(MZCHF,MZCHF)
!C     X,CCP(MZCHF,MZCHF),CSPP(MZCHF,MZCHF),CCPP(MZCHF,MZCHF)
!C     X,DS(MZCHF,MZCHF),DSP(MZCHF,MZCHF),DC(MZCHF,MZCHF)
!C     X,DCP(MZCHF,MZCHF),DFPP(MZCHF,MZCHF)
!C     X,RMAT(MZCHF,MZCHF)
!C
!      ALLOCATABLE :: TEMP1A(:),TEMP2A(:,:)
!      ALLOCATABLE :: RMATB(:,:)
!C
!C     
!C
!      ALLOCATE (TEMP1A(MNP2),STAT=IERR)
!      IF(IERR.NE.0)STOP ' FAILURE TO ALLOCATE TEMP1A IN RINIT'
!C
!C    INITIALISE 
!C
!C
!       IF(ALLOCATED(RMAT))DEALLOCATE(RMAT)
!C
!C    PRELIMINARY SET-UP 
!C
!      DO K=1,MNP2
!        TEMP1A(K)=ONE/(VALUE(K)-ETOT)-EZERO
!      ENDDO
!C
!C      3 DGEMM MULTIPLICATIONS INSTEAD OF ONE, BUT 
!C      A 50 % REDUCTION IN MEMORY OVERHEAD       CPB 2008
!C
!C      OPEN(5000,STATUS='SCRATCH',
!C     X         FORM='UNFORMATTED')
!C
!      IF(MOD(NCHF,2).EQ.0)THEN
!      ISPLIT=NCHF/2
!      IODD=0
!      ELSE
!      ISPLIT=(NCHF/2)+1
!      IODD=1
!      ENDIF
!C
!C
!C
!      ALLOCATE(RMATB(ISPLIT,ISPLIT),STAT=IERR)
!
!      IF(IERR.NE.0)STOP ' FAILURE TO ALLOCATE RMATB IN RINIT'
!C
!      DO I=1,ISPLIT
!       DO J=1,ISPLIT
!        RMATB(J,I)=0.0D0
!       ENDDO
!      ENDDO
!C
!       WM1=>WMAT(1:MNP2,1:ISPLIT)
!C
!C
!        ALLOCATE(TEMP2A(MNP2,1:ISPLIT),STAT=IERR)
!      IF(IERR.NE.0)STOP ' FAILURE TO ALLOCATE TEMP2A IN RINIT'
!C
!	DO JJ=1,ISPLIT
!	 DO K=1,MNP2
!	  TEMP2A(K,JJ)=WM1(K,JJ)*TEMP1A(K)	  	  	  
!	 ENDDO
!	ENDDO
!C
!C
!C       R_LEFTUPPER  :
!C
!C	
!      CALL DGEMM('T','N',ISPLIT,ISPLIT,MNP2,ONE,TEMP2A,
!     X           MNP2,WM1,MNP2,
!     X           ONE,RMATB,ISPLIT)                        
!C
!C
!C
!C
!      ALLOCATE(RMAT(NCHF,NCHF))
!C
!      DO I=1,NCHF
!        DO J=1,NCHF
!          RMAT(J,I)=TZERO
!        ENDDO
!      ENDDO
!C
!C
!      DO I=1,ISPLIT
!       DO J=I,ISPLIT
!        RMAT(J,I)=RMATB(J,I)
!       ENDDO
!      ENDDO
!C
!      DO I=1,ISPLIT
!       DO J=1,ISPLIT
!        RMATB(J,I)=0.0D0
!       ENDDO
!      ENDDO
!C
!C
!C     R_RIGHTUPPER   :
!C
!       NULLIFY(WM1)
!C
!       WM2=>WMAT(1:MNP2,NCHF-ISPLIT+1:NCHF)
!C
!C
!      CALL DGEMM('T','N',ISPLIT,ISPLIT,MNP2,ONE,TEMP2A,
!     X           MNP2,WM2,MNP2,
!     X           ONE,RMATB,ISPLIT)
!C
!C
!C
!      DEALLOCATE(TEMP2A,STAT=IERR)
!      IF(IERR.NE.0)STOP 'FAILURE TO DEALLOCATE TEMP2A IN RINIT'
!C
!C
!       DO I=1,ISPLIT
!        DO J=1,ISPLIT
!         RMAT(J+ISPLIT-IODD,I)=RMATB(I,J)
!        ENDDO
!       ENDDO
!C
!C      INITIALISE
!C
!       DO I=1,ISPLIT
!        DO J=1,ISPLIT
!        RMATB(J,I)=0.0D0
!        ENDDO
!       ENDDO
!C
!C
!       ALLOCATE(TEMP2A(MNP2,NCHF-ISPLIT+1:NCHF),STAT=IERR)
!       IF(IERR.NE.0)STOP ' FAILURE TO ALLOCATE TEMP2A IN RINIT'
!C
!        II=0
!        DO JJ=NCHF-ISPLIT+1,NCHF
!         II=II+1
!         DO K=1,MNP2
!          TEMP2A(K,JJ)=WM2(K,II)*TEMP1A(K)
!         ENDDO
!        ENDDO
!C
!      DEALLOCATE(TEMP1A,STAT=IERR)
!      IF(IERR.NE.0)STOP 'FAILURE TO DEALLOCATE TEMP1A IN RINIT'
!C
!C
!C      R_RIGHTLOWER
!C
!C
!      CALL DGEMM('T','N',ISPLIT,ISPLIT,MNP2,ONE,TEMP2A,
!     X           MNP2,WM2,MNP2,
!     X           ONE,RMATB,ISPLIT)
!C
!C
!      DEALLOCATE(TEMP2A,STAT=IERR)
!      IF(IERR.NE.0)STOP 'FAILURE TEMP2A TO DEALLOCATE IN RINIT'
!C
!C
!       DO I=1,ISPLIT
!        DO J=I,ISPLIT
!         RMAT(J+ISPLIT-IODD,I+ISPLIT-IODD)=RMATB(I,J)
!        ENDDO
!       ENDDO 
!C
!      DEALLOCATE(RMATB,STAT=IERR)
!      IF(IERR.NE.0)STOP 'FAILURE RMATB TO DEALLOCATE IN RINIT'       
!C
!C
!C    SYMMETRISE 
!C
!      DO I=2,NCHF
!       DO J=1,I-1
!        RMAT(J,I)=RMAT(I,J)
!        ENDDO
!      ENDDO
!C
!C      NOW ADD IN THE DIAGONAL BUTTLE CORRECTION
!C
!C
!      IF(IWORD.EQ.1)THEN
!   	IF(NEWBUT)THEN
!   	  RA2=RZERO*RZERO
!   	  DO I=1,NCHF
!            L=L2P(I)+1
!   	    NBUT=COEFF(3,L)
!   	 RMAT(I,I)=RMAT(I,I)+COEFF(1,L)*BUT0(NBUT,COEFF(2,L)+RA2*EPS(I))
!   	  ENDDO
!   	ELSE
!   	  DO I=1,NCHF
!   	    L=L2P(I)+1
!   	    E=EPS(I)
!   	    RMAT(I,I)=RMAT(I,I)+COEFF(1,L)+E*(COEFF(2,L)+E*COEFF(3,L))
!   	  ENDDO
!   	ENDIF
!      ELSE                     !DARC
!        DO I=1,NCHF
!   	  L=L2P(I)+1
!          E=EPS(I)
!          CALL INTBUT(L,E,BUTT)
!          RMAT(I,I)=RMAT(I,I)+BUTT
!        ENDDO
!      ENDIF
!C
!C    NOW ADD-IN ANY PARTITIONED CORRECTION
!C
!      EZERO=TZERO
!      IF(IPRCENT.NE.100)THEN
!        EZERO=ONE/(TRACE-ETOT)
!        DO I=1,NCHF
!          RMAT(I,I)=RMAT(I,I)+SI(I)*EZERO
!          L=L2P(I)+1
!          DO N=NRANG1(L),NRANG2
!            V=ONE/(EIGENS(N,L)-ETOT)
!            RMAT(I,I)=RMAT(I,I)+ENDS(N,L)*(V-EZERO)
!          ENDDO
!        ENDDO
!      ENDIF
!C
!C
!C
!C     **************** CPB 2008 **************************
!C
!        IF(MXEONE.EQ.1)THEN
!        WRITE(6,*)'ONLY 1 ENERGY PT PER PARTIAL PER PROC'
!        DEALLOCATE(WMAT)
!        ENDIF
!C
!        NULLIFY(WM2)
!C
!C      ****************************************************      
!C
!       RETURN
!       END
!!***************************************************************
!!
!      SUBROUTINE LU(A,LA,N,IERR)
!!
!!      ________________________________________________________
!!     |                                                        |
!!     |    LU FACTOR A GENERAL MATRIX WITH PARTIAL PIVOTING    |
!!     |                                                        |
!!     |    INPUT:                                              |
!!     |                                                        |
!!     |         A     --ARRAY CONTAINING MATRIX                |
!!     |                 (LENGTH AT LEAST 3 + N(N+1))           |
!!     |                                                        |
!!     |         LA    --LEADING (ROW) DIMENSION OF ARRAY A     |
!!     |                                                        |
!!     |         N     --DIMENSION OF MATRIX STORED IN A        |
!!     |                                                        |
!!     |    OUTPUT:                                             |
!!     |                                                        |
!!     |         A     --LU FACTORED MATRIX                     |
!!     |________________________________________________________|
!!
!      REAL*8 A(*),R,S,T,TZERO
!      INTEGER E,F,G,H,I,J,K,L,LA,M,N,O,P,IERR
!!
!      PARAMETER (TZERO=0.0)
!!
!      IERR=0
!!
!!     ----------------
!!     |*** PACK A ***|
!!     ----------------
!      H = LA - N
!      IF ( H .EQ. 0 ) THEN  ! CASE DIM A = N*N
!        IERR=1
!        RETURN
!      ENDIF
!      IF ( H .LT. 0 ) THEN
!        IERR=2
!        RETURN
!      ENDIF
!      I = 0
!      K = 1
!      L = N
!      O = N*N
!2     IF ( L .EQ. O ) GO TO 4
!      I = I + H
!      K = K + N
!      L = L + N
!      DO 3  J = K,L
!3          A(J) = A(I+J)
!      GOTO 2 
!!
!4     R = TZERO
!      O = N + 1
!      P = O + 1
!      L = 5 + N*P
!      I = -N - 3
!!     ---------------------------------------------
!!     |*** INSERT PIVOT ROW AND COMPUTE 1-NORM ***|
!!     ---------------------------------------------
!10    L = L - O
!      IF ( L .EQ. 4 ) GOTO 30
!      S = TZERO
!      DO 20 K = 1,N
!           J = L - K
!           T = A(I+J)
!           A(J) = T
!20         S = S + ABS(T)
!      IF ( R .LT. S ) R = S
!      I = I + 1
!      GOTO 10
!30    A(1) = 1230
!      A(2) = N
!      A(3) = R
!      I = 5 - P
!      K = 1
!40    I = I + P
!      IF ( K .EQ. N ) GOTO 110
!      E = N - K
!      M = I + 1
!      H = I
!      L = I + E
!!     ---------------------------------------
!!     |*** FIND PIVOT AND START ROW SWAP ***|
!!     ---------------------------------------
!      DO 50 J = M,L
!50         IF ( ABS(A(J)) .GT. ABS(A(H)) ) H = J
!      G = H - I
!      J = I - K
!      A(J) = G + K
!      T = A(H)
!      A(H) = A(I)
!      A(I) = T
!      K = K + 1
!      IF ( T .EQ. TZERO ) GOTO 100
!!     -----------------------------
!!     |*** COMPUTE MULTIPLIERS ***|
!!     -----------------------------
!      DO 60 J = M,L
!60         A(J) = A(J)/T
!      F = I + E*O
!70    J = K + L
!      H = J + G
!      T = A(H)
!      A(H) = A(J)
!      A(J) = T
!      L = E + J
!      IF ( T .EQ. TZERO ) GOTO 90
!      H = I - J
!!     ------------------------------
!!     |*** ELIMINATE BY COLUMNS ***|
!!     ------------------------------
!      M = J + 1
!      DO 80 J = M,L
!80         A(J) = A(J) - T*A(J+H)
!90    IF ( L .LT. F ) GOTO 70
!      GOTO 40
!100   A(1) = -1230
!      GOTO 40
!110   IF ( A(I) .EQ. TZERO ) A(1) = -1230
!      RETURN
!      END
!******************************************************************
!
      SUBROUTINE LUB(A,B,LB,NB,IERR)
!      ________________________________________________________
!     |                                                        |
!     |            SOLVE A GENERAL LU FACTORED SYSTEM          |
!     |                                                        |
!     |    INPUT:                                              |
!     |                                                        |
!     |         A     --LU'S OUTPUT                            |
!     |                                                        |
!     |         B     --RIGHT SIDE (DESTROYED)                 |
!     |                                                        |
!     |         LB    --LEADING (ROW) DIMENSION OF ARRAY B     |
!     |                                                        |
!     |         NB    --DIMENSION OF MATRIX STORED IN B        |
!     |                                                        |
!     |    OUTPUT:                                             |
!     |                                                        |
!     |         B     --SOLUTION                               |
!     |________________________________________________________|
!
      REAL*8 A(*),B(LB,*),T,TZERO,ONE
      INTEGER I,J,K,L,M,N,LB,NB,IB,I1,IERR
!
      PARAMETER (TZERO=0.0)
      PARAMETER (ONE=1.0)
!
      IERR=0
!
      I1 = NINT(A(1))
      IF ( ABS(I1) .NE. 1230 ) THEN
        IERR=1         !ERROR, MUST FACTOR BEFORE SOLVING
        RETURN
      ENDIF
!     -----------------------------
!     |*** FORWARD ELIMINATION ***|
!     -----------------------------
      DO 20 IB = 1,NB
      N = NINT(A(2))
      M = N + 1
      J = 4 - M
      IF ( I1 .LT. 0 ) GOTO 80
      K = 1
30    J = J + M
      IF ( A(J+K) .EQ. TZERO ) GOTO 80
      IF ( K .EQ. N ) GOTO 50
      L = NINT(A(J))
      T = B(L,IB)
      B(L,IB) = B(K,IB)
      B(K,IB) = T
      K = K + 1
      IF ( T .EQ. TZERO ) GOTO 30
      DO 40 I = K,N
40         B(I,IB) = B(I,IB) - T*A(I+J)
      GOTO 30
!     --------------------------------------
!     |*** BACK SUBSTITUTION BY COLUMNS ***|
!     --------------------------------------
50    T = B(K,IB)/A(J+K)
60    B(K,IB) = T
      IF ( K .EQ. 1 ) GO TO 20
      K = K - 1
      DO 70 I = 1,K
70         B(I,IB) = B(I,IB) - T*A(I+J)
      J = J - M
      GOTO 50
!     -----------------------------
!     |*** COMPUTE NULL VECTOR ***|
!     -----------------------------
80    K = 0
90    K = K + 1
      J = J + M
      IF ( A(J+K) .NE. TZERO ) GOTO 90
      DO 100 I = 1,N
100        B(I,IB) = TZERO
      T = ONE
      GOTO 60
 20   CONTINUE
      RETURN
      END
!******************************************************************
!!
!      SUBROUTINE LUBS(A,B,LB,NB,IERR)
!!      ________________________________________________________
!!     |                                                        |
!!     | SOLVE AN LU FACTORED SYMMETRIC SYSTEM WITHOUT PIVOTING |
!!     |                                                        |
!!     |    INPUT:                                              |
!!     |                                                        |
!!     |         A     --LUS'S OUTPUT                           |
!!     |                                                        |
!!     |         B     --RIGHT SIDE (DESTROYED)                 |
!!     |                                                        |
!!     |         LB    --LEADING (ROW) DIMENSION OF ARRAY B     |
!!     |                                                        |
!!     |         NB    --DIMENSION OF MATRIX STORED IN B        |
!!     |                                                        |
!!     |    OUTPUT:                                             |
!!     |                                                        |
!!     |         B     --SOLUTION                               |
!!     |________________________________________________________|
!!
!      REAL*8 A(*),B(LB,*),T,TZERO,ONE
!      INTEGER I,J,K,L,N,LB,NB,IB,I1,IERR,KB,NB0
!!
!      PARAMETER (TZERO=0.0)
!      PARAMETER (ONE=1.0)
!!
!      IERR=0
!      NB0=ABS(NB)
!!
!      I1 = NINT(A(1))
!      IF ( ABS(I1) .NE. 1233 ) THEN
!        IERR=1         !ERROR, MUST FACTOR BEFORE SOLVING
!        RETURN
!      ENDIF
!!     -----------------------------
!!     |*** FORWARD ELIMINATION ***|
!!     -----------------------------
!      DO 20 IB = 1,NB0
!      KB=1
!      IF(NB.LT.0)KB=IB
!      N = NINT(A(2))
!      L = 3
!      K=1
!      IF ( I1 .LT. 0 ) GOTO 80
!30    IF ( K .EQ. N ) GOTO 50
!      T = B(K,IB)/A(K+L)
!      J = L
!      L = L + N - K
!      K = K + 1
!      IF ( T .EQ. TZERO ) GOTO 30
!      DO 40 I = K,N
!40         B(I,IB) = B(I,IB) - T*A(I+J)
!      GOTO 30
!!     -----------------------------------
!!     |*** BACK SUBSTITUTION BY ROWS ***|
!!     -----------------------------------
!50    B(N,IB) = B(N,IB)/A(K+L)
!60    IF ( K .EQ. KB ) GO TO 20
!      J = K
!      K = K - 1
!      L = L + K - N
!      T = B(K,IB)
!      DO 70 I = J,N
!70         T = T - B(I,IB)*A(I+L)
!      B(K,IB) = T/A(K+L)
!      GOTO 60
!!     -----------------------------
!!     |*** COMPUTE NULL VECTOR ***|
!!     -----------------------------
!80    IF ( A(K+L) .EQ. TZERO ) GOTO 90
!      L = L + N - K
!      K = K + 1
!      GOTO 80
!90    DO 100 I = 1,N
!100        B(I,IB) = TZERO
!      B(K,IB) = ONE
!      GOTO 60
! 20   CONTINUE
!      IF(NB.LT.0)THEN
!        DO J=1,IB
!          DO I=J,IB
!            B(J,I)=B(I,J)
!          ENDDO
!        ENDDO
!      ENDIF
!      RETURN
!      END
!!******************************************************************
!!
!      SUBROUTINE LUBT(A,B,LB,NB,IERR)
!!      ________________________________________________________
!!     |                                                        |
!!     |    SOLVE THE TRANSPOSE OF A GENERAL LU FACTORED SYSTEM |
!!     |                                                        |
!!     |    INPUT:                                              |
!!     |                                                        |
!!     |         A     --LU'S OUTPUT                            |
!!     |                                                        |
!!     |         B     --RIGHT SIDE (DESTROYED)                 |
!!     |                                                        |
!!     |         LB    --LEADING (ROW) DIMENSION OF ARRAY B     |
!!     |                                                        |
!!     |         NB    --DIMENSION OF MATRIX STORED IN B        |
!!     |                                                        |
!!     |    OUTPUT:                                             |
!!     |                                                        |
!!     |         B     --SOLUTION                               |
!!     |________________________________________________________|
!!
!      REAL*8 A(*),B(LB,*),T,TZERO,ONE
!      INTEGER I,J,K,L,M,N,LB,NB,IB,I1,IERR
!!
!      PARAMETER (TZERO=0.0)
!      PARAMETER (ONE=1.0)
!!
!      IERR=0
!!
!      I1 = NINT(A(1))
!      IF ( ABS(I1) .NE. 1230 ) THEN
!        IERR=1         !ERROR, MUST FACTOR BEFORE SOLVING
!        RETURN
!      ENDIF
!!
!      DO 10 IB = 1,NB
!      N = NINT(A(2))
!      M = N + 1
!      IF ( I1 .LT. 0 ) GOTO 80
!      T = TZERO
!      J = 4
!      K = 1
!!     -------------------------
!!     |*** SKIP OVER ZEROS ***|
!!     -------------------------
!20    IF ( B(K,IB) .NE. TZERO ) GOTO 30
!      K = K + 1
!      IF ( K .LE. N ) GOTO 20
!      GO TO 10
!!     ---------------------------
!!     |*** FORE SUBSTITUTION ***|
!!     ---------------------------
!30    J = J - M + M*K
!40    B(K,IB) = (B(K,IB)-T)/A(J+K)
!      IF ( K .EQ. N ) GOTO 60
!      T = TZERO
!      J = J + M
!      DO 50 I = 1,K
!50         T = T + A(I+J)*B(I,IB)
!      K = K + 1
!      GOTO 40
!!     ---------------------------
!!     |*** BACK SUBSTITUTION ***|
!!     ---------------------------
!60    IF ( K .EQ. 1 ) GO TO 10
!      J = J - M
!      T = B(K-1,IB)
!      DO 70 I = K,N
!70         T = T - B(I,IB)*A(I+J)
!      K = K - 1
!      I = A(J)
!      B(K,IB) = B(I,IB)
!      B(I,IB) = T
!      GOTO 60
!!     -----------------------------
!!     |*** COMPUTE NULL VECTOR ***|
!!     -----------------------------
!80    I = 5 + N + M*N
!      L = M
!90    I = I - M - 1
!      L = L - 1
!      IF ( A(I) .NE. TZERO ) GOTO 90
!      K = L
!      J = I - K
!      DO 100 I = 1,N
!100        B(I,IB) = TZERO
!      B(K,IB) = ONE
!110   IF ( K .EQ. N ) GOTO 60
!      T = TZERO
!      J = J + M
!      DO 120 I = L,K
!120        T = T - A(I+J)*B(I,IB)
!      K = K + 1
!      B(K,IB) = T/A(J+K)
!      GOTO 110
!10    CONTINUE
!      RETURN
!      END
!!******************************************************************
!!
!      SUBROUTINE LUS(A,LA,N,W,IERR)
!!
!!      ________________________________________________________
!!     |                                                        |
!!     |       LU FACTOR A SYMMETRIC MATRIX WITHOUT PIVOTING    |
!!     |                                                        |
!!     |    INPUT:                                              |
!!     |                                                        |
!!     |        A     --ARRAY CONTAINING MATRIX                 |
!!     |                (ONLY THE LOWER HALF NEED BE DEFINED)   |
!!     |                                                        |
!!     |        LA    --LEADING (ROW) DIMENSION OF ARRAY A      |
!!     |                                                        |
!!     |        N     --MATRIX DIMENSION                        |
!!     |                                                        |
!!     |        W     --WORK ARRAY WITH LENGTH AT LEAST N       |
!!     |                                                        |
!!     |    OUTPUT:                                             |
!!     |                                                        |
!!     |        A     --INVERSE (IN LOWER HALF ONLY)            |
!!     |________________________________________________________|
!!
!      REAL*8 A(*),W(*),R,S,T
!      INTEGER G,H,I,J,K,L,M,N
!      REAL*8 TZERO
!!
!      PARAMETER (TZERO=0.0)
!!
!      IERR=0
!!
!!     ----------------
!!NRB |*** PACK A ***|
!!     ----------------
!      H=LA-N
!      I=0
!      M=0
!      L=N
!      G=(N*(N+1))/2
!2     IF(L.EQ.G)GO TO 4
!      K=L+1
!      M=M+1
!      L=L+N-M
!      I=I+H+M
!      DO J=K,L
!        A(J)=A(I+J)
!      ENDDO
!      GO TO 2
!!
!!     ------------------------
!!     |*** COMPUTE 1-NORM ***|
!!     ------------------------
!4     DO 10 I = 1,N
!10         W(I) = TZERO
!      I = -N
!      K = 0
!      R = TZERO
!      S = TZERO
!20    I = I + N - K
!      K = K + 1
!      J = K
!      S = ABS(A(I+J))
!30    IF ( J .EQ. N ) GOTO 40
!      J = J + 1
!      T = ABS(A(I+J))
!      S = S + T
!      W(J) = W(J) + T
!      GOTO 30
!40    S = S + W(K)
!      IF ( R .LT. S ) R = S
!      IF ( K .LT. N ) GOTO 20
!      J = 3 + (N+N*N)/2
!!     -----------------------------------
!!     |*** SHIFT MATRIX DOWN 3 SLOTS ***|
!!     -----------------------------------
!50    A(J) = A(J-3)
!      J = J - 1
!      IF ( J .GT. 3 ) GOTO 50
!      A(1) = 1233
!      A(2) = N
!      A(3) = R
!      H = N
!      K = 4
!60    IF ( H .EQ. 1 ) GOTO 90
!!     --------------------------
!!     |*** SAVE PIVOT ENTRY ***|
!!     --------------------------
!      S = A(K)
!      K = K + H
!      G = K
!      H = H - 1
!      M = H
!      IF ( S .EQ. TZERO ) GOTO 100
!      J = 0
!70    J = J - M
!      M = M - 1
!      L = G + M
!      T = A(G+J)/S
!!     ---------------------------
!!     |*** ELIMINATE BY ROWS ***|
!!     ---------------------------
!      DO 80 I = G,L
!80         A(I) = A(I) - T*A(I+J)
!      G = L + 1
!      IF ( M .GT. 0 ) GOTO 70
!      GOTO 60
!90    IF ( A(K) .NE. TZERO ) RETURN
!      A(1) = -1233
!      RETURN
!100   A(1) = -1233
!      GOTO 60
!      END
!**********************************************************
!
      SUBROUTINE VERT(V,LV,N,W,IERR)
!
!      ________________________________________________________
!     |                                                        |
!     |                INVERT A GENERAL MATRIX                 |
!     |                                                        |
!     |    INPUT:                                              |
!     |                                                        |
!     |         V     --ARRAY CONTAINING MATRIX                |
!     |                                                        |
!     |         LV    --LEADING (ROW) DIMENSION OF ARRAY V     |
!     |                                                        |
!     |         N     --DIMENSION OF MATRIX STORED IN ARRAY V  |
!     |                                                        |
!     |         W     --INTEGER WORK ARRAY WITH AT LEAST N-1   |
!     |                      ELEMENTS                          |
!     |                                                        |
!     |    OUTPUT:                                             |
!     |                                                        |
!     |         V     --INVERSE                                |
!     |                                                        |
!     |    BUILTIN FUNCTIONS: ABS                              |
!     |________________________________________________________|
!
      REAL*8 V(LV,*),S,T
      REAL*8 TZERO,ONE
      INTEGER W(*),I,J,K,L,M,N,P,IERR
!
      PARAMETER (TZERO=0.0)
      PARAMETER (ONE=1.0)
!
      IERR=0
!
      IF ( N .EQ. 1 ) GOTO 110
      L = 0
      M = 1
10    IF ( L .EQ. N ) GOTO 90
      K = L
      L = M
      M = M + 1
!     ---------------------------------------
!     |*** FIND PIVOT AND START ROW SWAP ***|
!     ---------------------------------------
      P = L
      IF ( M .GT. N ) GOTO 30
      S = ABS(V(L,L))
      DO 20 I = M,N
           T = ABS(V(I,L))
           IF ( T .LE. S ) GOTO 20
           P = I
           S = T
20    CONTINUE
      W(L) = P
30    S = V(P,L)
      V(P,L) = V(L,L)
      IF ( S .EQ. TZERO ) GOTO 120
!     -----------------------------
!     |*** COMPUTE MULTIPLIERS ***|
!     -----------------------------
      V(L,L) = -ONE
      S = ONE/S
      DO 40 I = 1,N
40         V(I,L) = -S*V(I,L)
      J = L
50    J = J + 1
      IF ( J .GT. N ) J = 1
      IF ( J .EQ. L ) GOTO 10
      T = V(P,J)
      V(P,J) = V(L,J)
      V(L,J) = T
      IF ( T .EQ. TZERO ) GOTO 50
!     ------------------------------
!     |*** ELIMINATE BY COLUMNS ***|
!     ------------------------------
      IF ( K .EQ. 0 ) GOTO 70
      DO 60 I = 1,K
60         V(I,J) = V(I,J) + T*V(I,L)
70    V(L,J) = S*T
      IF ( M .GT. N ) GOTO 50
      DO 80 I = M,N
80         V(I,J) = V(I,J) + T*V(I,L)
      GOTO 50
!     -----------------------
!     |*** PIVOT COLUMNS ***|
!     -----------------------
90    L = W(K)
      DO 100 I = 1,N
           T = V(I,L)
           V(I,L) = V(I,K)
100        V(I,K) = T
      K = K - 1
      IF ( K .GT. 0 ) GOTO 90
      RETURN
110   IF ( V(1,1) .EQ. TZERO ) GOTO 120
      V(1,1) = ONE/V(1,1)
      RETURN
120   IERR=1
      RETURN
      END
!!     ******************************************************************
!!
!      SUBROUTINE VERTS(V,LV,N,W,IERR)
!!
!!      ________________________________________________________
!!     |                                                        |
!!     |       INVERT A SYMMETRIC MATRIX WITHOUT PIVOTING       |
!!     |                                                        |
!!     |    INPUT:                                              |
!!     |                                                        |
!!     |        V     --ARRAY CONTAINING MATRIX                 |
!!     |                (ONLY THE LOWER HALF NEED BE DEFINED)   |
!!     |                                                        |
!!     |        LV    --LEADING (ROW) DIMENSION OF ARRAY V      |
!!     |                                                        |
!!     |        N     --MATRIX DIMENSION                        |
!!     |                                                        |
!!     |        W     --WORK ARRAY WITH LENGTH AT LEAST N       |
!!     |                                                        |
!!     |    OUTPUT:                                             |
!!     |                                                        |
!!     |        V     --INVERSE (IN LOWER HALF ONLY)            |
!!     |________________________________________________________|
!!
!      REAL*8 V(*),W(*),S,T
!      INTEGER G,H,I,J,K,L,M,N,LV,IERR
!      REAL*8 TZERO,ONE
!!
!      PARAMETER (TZERO=0.0)
!      PARAMETER (ONE=1.0)
!!
!      IERR=0
!!
!!     ----------------
!!NRB |*** PACK V ***|
!!     ----------------
!      H=LV-N
!      I=0
!      M=0
!      L=N
!      G=(N*(N+1))/2
!2     IF(L.EQ.G)GO TO 4
!      K=L+1
!      M=M+1
!      L=L+N-M
!      I=I+H+M
!      DO J=K,L
!        V(J)=V(I+J)
!      ENDDO
!      GO TO 2
!!
!4     H = N
!      K = 1
!10    IF ( H .EQ. 1 ) GOTO 40
!!     --------------------------
!!     |*** SAVE PIVOT ENTRY ***|
!!     --------------------------
!      S = V(K)
!      K = K + H
!      G = K
!      H = H - 1
!      M = H
!      IF ( S .EQ. TZERO ) GOTO 50
!      J = 0
!20    J = J - M
!      M = M - 1
!      L = G + M
!      T = V(G+J)/S
!!     ---------------------------
!!     |*** ELIMINATE BY ROWS ***|
!!     ---------------------------
!      DO 30 I = G,L
!30         V(I) = V(I) - T*V(I+J)
!      G = L + 1
!      IF ( M .GT. 0 ) GOTO 20
!      GOTO 10
!40    IF ( V(K) .NE. TZERO ) GOTO 60
!      IERR=2
!      RETURN
!50    IERR=1
!      RETURN
!!     ------------------------------------------
!!     |*** SOLVE FOR ROWS OF INVERSE MATRIX ***|
!!     ------------------------------------------
!60    G = N + N
!      DO 150 M = 1,N
!           L = ((G-M)*(M-1))/2
!           H = L
!           K = M
!           DO 70 I = M,N
!70              W(I) = TZERO
!           W(M) = ONE
!80         IF ( K .EQ. N ) GOTO 100
!           T = W(K)/V(K+L)
!           J = L
!           L = L + N - K
!           K = K + 1
!           IF ( T .EQ. TZERO ) GOTO 80
!           DO 90 I = K,N
!90              W(I) = W(I) - T*V(I+J)
!           GOTO 80
!!     -----------------------------------
!!     |*** BACK SUBSTITUTION BY ROWS ***|
!!     -----------------------------------
!100        W(N) = W(N)/V(K+L)
!110        IF ( K .EQ. M ) GOTO 130
!           J = K
!           K = K - 1
!           L = L + K - N
!           T = W(K)
!           DO 120 I = J,N
!120             T = T - W(I)*V(I+L)
!           W(K) = T/V(K+L)
!           GOTO 110
!130        DO 140 I = M,N
!140             V(I+H) = W(I)
!150   CONTINUE
!!     ------------------
!!NRB  |*** UNPACK V ***|
!!     ------------------
!      H=LV-N
!      I=(N-1)*H+(N*(N-1))/2
!      M=N-1
!      L=(N*(N+1))/2
!      K=L
!200   IF(I.EQ.0)RETURN
!      DO J=L,K,-1
!        V(I+J)=V(J)
!      ENDDO
!      I=I-H-M
!      L=K-1
!      M=M-1
!      K=K-N+M
!      GO TO 200
!!
!      END
!!
!!**********************************************************
!!
!      SUBROUTINE ZEIGEN(A,VAL,N,DELTA)
!!
!!  DIAGONALISATION OF COMPLEX SYMMETRIC MATRIX
!!  USING METHOD DESCRIBED BY M.J.SEATON, COMPUTER JOURNAL, VOL.12,
!!  PAGE 156, 1969.
!!
!      IMPLICIT REAL*8 (A-H,O-Z)
!!
!      COMPLEX*16 A,X,VAL,C,S,H,P,Q,CIM,ZERO,ZONE
!      PARAMETER (ZERO=(0.0,0.0))
!      PARAMETER (ZONE=(1.0,0.0))
!      PARAMETER (CIM=(0.0,1.0))
!      PARAMETER (TZERO=0.0)
!      PARAMETER (QUART=0.25)
!      PARAMETER (HALF=0.5)
!      PARAMETER (ONE=1.0)
!      PARAMETER (TWO=2.0)
!!
!      DIMENSION A(N,N),X(N,N),VAL(N)
!!
!      DATA IROTM/20/
!!
!      SQ(H)=H* CONJG(H)
!      NM1=N-1
!!
!      IF(N.LE.0)RETURN
!!
!      IF(N.EQ.1)THEN
!        VAL(1)=A(1,1)
!        A(1,1)=ONE
!        RETURN
!      ENDIF
!!
!      IF(N.EQ.2)THEN
!        Q=A(1,2)
!        P = (A(1,1)-A(2,2))*HALF
!        FL= ABS(P*P+Q*Q)
!        V=-QUART*LOG(SQ(P-CIM*Q)/FL)
!        T= CONJG(P)*Q+CONJG(Q)*P
!        U=-QUART * ATAN2(T/FL,(SQ(P)-SQ(Q))/FL)
!        S= SIN(U)*COSH(V) +CIM*COS(U)*SINH(V)
!        C= COS(U)*COSH(V) -CIM*SIN(U)*SINH(V)
!        H=(S*P+C*Q)*S*TWO
!        VAL(1)=A(1,1)-H
!        VAL(2)=A(2,2)+H
!        A(1,1)=C
!        A(1,2)=S
!        A(2,1)=-S
!        A(2,2)=C
!        RETURN
!      ENDIF
!!
!! INITIAL D1,D2 AND X
!!
!      D1=TZERO
!      H=ZERO
!      DO I=1,N
!        X(I,I)=ZONE
!        D1=D1+SQ(A(I,I))
!        H=H+A(I,I)
!      ENDDO
!      D1=D1-SQ(H)/DBLE(N)
!
!      D2=TZERO
!      DO I=1,NM1
!        IP1=I+1
!        DO J=IP1,N
!          X(I,J)=ZERO
!          X(J,I)=ZERO
!          D2=D2+SQ(A(I,J))
!        ENDDO
!      ENDDO
!      D1 = (N-1)*(D1*HALF+D2)*DELTA*DELTA/DBLE((N+1))
!!
!! BEGIN ROTATIONS
!!
!      DO 1010 IROT=1,IROTM
!      DO 1000 IP=1,NM1
!      IPP1=IP+1
!      DO 1000 IQ=IPP1,N
!!
!! ROTATION CONSTANTS
!!
!      Q=A(IP,IQ)
!      P = (A(IP,IP)-A(IQ,IQ))*HALF
!      FL = ABS(P*P+Q*Q)
!      BETA = LOG(SQ(P-CIM*Q)/FL)*HALF
!      T=( CONJG(P)*Q+P* CONJG(Q))/FL
!      D=(SQ(P)-SQ(Q))/FL
!      U=-QUART*ATAN2(T,D)
!!
!      T=TZERO
!      D=TZERO
!      DO I=1,N
!        IF(I.NE.IP.AND.I.NE.IQ)THEN
!          D=D+SQ(A(IP,I))+SQ(A(IQ,I))
!          T=T+SQ(A(IP,I)+CIM*A(IQ,I))
!        ENDIF
!      ENDDO
!      T=D-T
!      FN= SQRT(D*D-T*T)
!      GAMMA=LOG((D+T)/FN)
!!
!! ITERATION FOR V
!!
!      V0=-HALF*(BETA+GAMMA)
!      DO ITERV=1,100
!        V=V0-(FL*SINH((V0+BETA)*TWO)+FN*SINH(V0+GAMMA))/
!     *    (TWO*FL*COSH((V0+BETA)*TWO)+FN*COSH(V0+GAMMA))
!        IF(ABS(V-V0).LT.DELTA*0.1) GO TO 580
!        V0=V
!      ENDDO
!      WRITE(6,5010) DELTA
!!
!! NEW A,X AND D2
!!
!  580 V=HALF*V
!      S= SIN(U)*COSH(V) +CIM*COS(U)*SINH(V)
!      C= COS(U)*COSH(V) -CIM*SIN(U)*SINH(V)
!      H = (S*P+C*Q)*S*TWO
!      A(IP,IP)=A(IP,IP)-H
!      A(IQ,IQ)=A(IQ,IQ)+H
!      A(IP,IQ)=(C*C-S*S)*Q+TWO*C*S*P
!      D2=D2-SQ(A(IQ,IP))+SQ(A(IP,IQ))
!      A(IQ,IP)=A(IP,IQ)
!!
!      DO I=1,N
!        H=X(I,IP)
!        X(I,IP)=H*C-X(I,IQ)*S
!        X(I,IQ)=H*S+X(I,IQ)*C
!        IF(I.NE.IP.AND.I.NE.IQ)THEN
!          H=A(IP,I)
!          A(IP,I)=C*H-A(IQ,I)*S
!          A(IQ,I)=S*H+A(IQ,I)*C
!          D2=D2-SQ(A(I,IP))-SQ(A(I,IQ))+SQ(A(IP,I))+SQ(A(IQ,I))
!          A(I,IP)=A(IP,I)
!          A(I,IQ)=A(IQ,I)
!        ENDIF
!      ENDDO
!!
!! TEST CONVERGENCE
!      IF(D2.LT.D1) GO TO 610
!!
!! END ROTATIONS
!!
! 1000 CONTINUE
!!
!!  RECALCULATE D2
!!
!      D2=TZERO
!      DO I=1,NM1
!        IP1=I+1
!        DO J=IP1,N
!          D2=D2+SQ(A(I,J))
!        ENDDO
!      ENDDO
!!
! 1010 CONTINUE
!      WRITE(6,5020)IROTM,DELTA
!!
!! EIGENVALUES AND EIGENVECTORS
!!
!  610 DO I=1,N
!        VAL(I)=A(I,I)
!        DO J=1,N
!          A(I,J)=X(I,J)
!        ENDDO
!      ENDDO
!!
!      RETURN
!!
! 5010 FORMAT(//10X,'*** SUBROUTINE ZEIGEN ***'//
!     + ' WARNING - NO CONVERGENCE IN ITERATIONS FOR V'/
!     + ' ACCURACY PARAMETER DELTA = ',E9.2/
!     + ' NEXT VALUE OF AVERAGED OMEGA MAY BE INCORRECT'//)
! 5020 FORMAT(//11X,'*** SUBROUTINE ZEIGEN ***'
!     + /'0NO CONVERGENCE FOR IROTM =',I3/' DELTA =',E11.2/
!     +  ' NEXT VALUE OF AVERAGED OMEGA MAY BE INCORRECT'//)
!!
!      END
!!******************************************************************
!!
!      SUBROUTINE ZLUBS(A,B,LB,NB,IERR)
!!      ________________________________________________________
!!     |                                                        |
!!     |    SOLVE A COMPLEX LU FACTORED SYMMETRIC SYSTEM        |
!!     |                 WITHOUT PIVOTING                       |
!!     |                                                        |
!!     |    INPUT:                                              |
!!     |                                                        |
!!     |         A     --ZLUS'S OUTPUT                          |
!!     |                                                        |
!!     |         B     --RIGHT SIDE (DESTROYED)                 |
!!     |                                                        |
!!     |         LB    --LEADING (ROW) DIMENSION OF ARRAY B     |
!!     |                                                        |
!!     |         NB    --DIMENSION OF MATRIX STORED IN B        |
!!     |                                                        |
!!     |    OUTPUT:                                             |
!!     |                                                        |
!!     |         B     --SOLUTION                               |
!!     |________________________________________________________|
!!
!      COMPLEX*16 A(*),B(LB,*),T
!      REAL*8 TZERO,ONE
!      INTEGER I,J,K,L,N,LB,NB,IB,I1,IERR,KB,NB0
!!
!      PARAMETER (TZERO=0.0)
!      PARAMETER (ONE=1.0)
!!
!      IERR=0
!      NB0=ABS(NB)
!!
!      I1 = NINT(DBLE(A(1)))
!      IF ( ABS(I1) .NE. 1237 ) THEN
!        IERR=1         !ERROR, MUST FACTOR BEFORE SOLVING
!        RETURN
!      ENDIF
!!     -----------------------------
!!     |*** FORWARD ELIMINATION ***|
!!     -----------------------------
!      DO 20 IB = 1,NB0
!      KB=1
!      IF(NB.LT.0)KB=IB
!      N = NINT(DBLE(A(2)))
!      L = 3
!      K = 1
!      IF ( I1 .LT. 0 ) GOTO 80
!30    IF ( K .EQ. N ) GOTO 50
!      T = B(K,IB)/A(K+L)
!      J = L
!      L = L + N - K
!      K = K + 1
!      IF ( ABS(T) .EQ. TZERO ) GOTO 30
!      DO 40 I = K,N
!40         B(I,IB) = B(I,IB) - T*A(I+J)
!      GOTO 30
!!     -----------------------------------
!!     |*** BACK SUBSTITUTION BY ROWS ***|
!!     -----------------------------------
!50    B(N,IB) = B(N,IB)/A(K+L)
!60    IF ( K .EQ. KB ) GO TO 20
!      J = K
!      K = K - 1
!      L = L + K - N
!      T = B(K,IB)
!      DO 70 I = J,N
!70         T = T - B(I,IB)*A(I+L)
!      B(K,IB) = T/A(K+L)
!      GOTO 60
!!     -----------------------------
!!     |*** COMPUTE NULL VECTOR ***|
!!     -----------------------------
!80    IF ( ABS(A(K+L)) .EQ. TZERO ) GOTO 90
!      L = L + N - K
!      K = K + 1
!      GOTO 80
!90    DO 100 I = 1,N
!100        B(I,IB) = TZERO
!      B(K,IB) = ONE
!      GOTO 60
! 20   CONTINUE
!      IF(NB.LT.0)THEN
!        DO J=1,IB
!          DO I=J,IB
!            B(J,I)=B(I,J)
!          ENDDO
!        ENDDO
!      ENDIF
!      RETURN
!      END
!!******************************************************************
!!
!      SUBROUTINE ZLUS(A,LA,N,W,IERR)
!!
!!      ________________________________________________________
!!     |                                                        |
!!     |  LU FACTOR A SYMMETRIC COMPLEX MATRIX WITHOUT PIVOTING |
!!     |                                                        |
!!     |    INPUT:                                              |
!!     |                                                        |
!!     |        A     --COMPLEX ARRAY CONTAINING MATRIX         |
!!     |                (ONLY THE LOWER HALF NEED BE DEFINED)   |
!!     |                                                        |
!!     |        LA    --LEADING (ROW) DIMENSION OF ARRAY A      |
!!     |                                                        |
!!     |        N     --MATRIX DIMENSION                        |
!!     |                                                        |
!!     |        W     --REAL WORK ARRAY WITH LENGTH AT LEAST N  |
!!     |                                                        |
!!     |    OUTPUT:                                             |
!!     |                                                        |
!!     |        A     --INVERSE (IN LOWER HALF ONLY)            |
!!     |________________________________________________________|
!!
!      COMPLEX*16 A(*),Y,Z
!      INTEGER G,H,I,J,K,L,M,N
!      REAL*8 R,S,T,W(*),TZERO
!!
!      PARAMETER (TZERO=0.0)
!!
!      IERR=0
!!
!!     ----------------
!!NRB |*** PACK A ***|
!!     ----------------
!      H=LA-N
!      I=0
!      M=0
!      L=N
!      G=(N*(N+1))/2
!2     IF(L.EQ.G)GO TO 4
!      K=L+1
!      M=M+1
!      L=L+N-M
!      I=I+H+M
!      DO J=K,L
!        A(J)=A(I+J)
!      ENDDO
!      GO TO 2
!!
!!     ------------------------
!!     |*** COMPUTE 1-NORM ***|
!!     ------------------------
!4     DO 10 I = 1,N
!10         W(I) = TZERO
!      I = -N
!      K = 0
!      R = TZERO
!      S = TZERO
!20    I = I + N - K
!      K = K + 1
!      J = K
!      S = ABS(A(I+J))
!30    IF ( J .EQ. N ) GOTO 40
!      J = J + 1
!      T = ABS(A(I+J))
!      S = S + T
!      W(J) = W(J) + T
!      GOTO 30
!40    S = S + W(K)
!      IF ( R .LT. S ) R = S
!      IF ( K .LT. N ) GOTO 20
!      J = 3 + (N+N*N)/2
!!     -----------------------------------
!!     |*** SHIFT MATRIX DOWN 3 SLOTS ***|
!!     -----------------------------------
!50    A(J) = A(J-3)
!      J = J - 1
!      IF ( J .GT. 3 ) GOTO 50
!      A(1) = 1237
!      A(2) = N
!      A(3) = R
!      H = N
!      K = 4
!60    IF ( H .EQ. 1 ) GOTO 90
!!     --------------------------
!!     |*** SAVE PIVOT ENTRY ***|
!!     --------------------------
!      Z = A(K)
!      K = K + H
!      G = K
!      H = H - 1
!      M = H
!      IF ( ABS(Z) .EQ. TZERO ) GOTO 100
!      J = 0
!70    J = J - M
!      M = M - 1
!      L = G + M
!      Y = A(G+J)/Z
!!     ---------------------------
!!     |*** ELIMINATE BY ROWS ***|
!!     ---------------------------
!      DO 80 I = G,L
!80         A(I) = A(I) - Y*A(I+J)
!      G = L + 1
!      IF ( M .GT. 0 ) GOTO 70
!      GOTO 60
!90    IF ( ABS(A(K)) .NE. TZERO ) RETURN
!      A(1) = -1237
!      RETURN
!100   A(1) = -1237
!      GOTO 60
!      END
!!***************************************************************
!!
!      SUBROUTINE ZVERTS(V,LV,N,W,IERR)
!!
!!      ________________________________________________________
!!     |                                                        |
!!     |   INVERT A SYMMETRIC COMPLEX MATRIX WITHOUT PIVOTING   |
!!     |                                                        |
!!     |    INPUT:                                              |
!!     |                                                        |
!!     |        V     --ARRAY CONTAINING MATRIX                 |
!!     |                (ONLY THE LOWER HALF NEED BE DEFINED)   |
!!     |                                                        |
!!     |        LV    --LEADING (ROW) DIMENSION OF ARRAY V      |
!!     |                                                        |
!!     |        N     --MATRIX DIMENSION                        |
!!     |                                                        |
!!     |        W     --WORK ARRAY WITH LENGTH AT LEAST N       |
!!     |                                                        |
!!     |    OUTPUT:                                             |
!!     |                                                        |
!!     |        V     --INVERSE (IN LOWER HALF ONLY)            |
!!     |________________________________________________________|
!!
!      COMPLEX*16 V(*),W(*),S,T
!      INTEGER G,H,I,J,K,L,M,N,LV,IERR
!      REAL*8 TZERO,ONE
!!
!      PARAMETER (TZERO=0.0)
!      PARAMETER (ONE=1.0)
!!
!      IERR=0
!!
!!     ----------------
!!NRB |*** PACK V ***|
!!     ----------------
!      H=LV-N
!      I=0
!      M=0
!      L=N
!      G=(N*(N+1))/2
!2     IF(L.EQ.G)GO TO 4
!      K=L+1
!      M=M+1
!      L=L+N-M
!      I=I+H+M
!      DO J=K,L
!        V(J)=V(I+J)
!      ENDDO
!      GO TO 2
!!
!4     H = N
!      K = 1
!10    IF ( H .EQ. 1 ) GOTO 40
!!     --------------------------
!!     |*** SAVE PIVOT ENTRY ***|
!!     --------------------------
!      S = V(K)
!      K = K + H
!      G = K
!      H = H - 1
!      M = H
!      IF ( ABS(S) .EQ. TZERO ) GOTO 50
!      J = 0
!20    J = J - M
!      M = M - 1
!      L = G + M
!      T = V(G+J)/S
!!     ---------------------------
!!     |*** ELIMINATE BY ROWS ***|
!!     ---------------------------
!      DO 30 I = G,L
!30         V(I) = V(I) - T*V(I+J)
!      G = L + 1
!      IF ( M .GT. 0 ) GOTO 20
!      GOTO 10
!40    IF ( ABS(V(K)) .NE. TZERO ) GOTO 60
!      IERR=2
!      RETURN
!50    IERR=1
!      RETURN
!!     ------------------------------------------
!!     |*** SOLVE FOR ROWS OF INVERSE MATRIX ***|
!!     ------------------------------------------
!60    G = N + N
!      DO 150 M = 1,N
!           L = ((G-M)*(M-1))/2
!           H = L
!           K = M
!           DO 70 I = M,N
!70              W(I) = TZERO
!           W(M) = ONE
!80         IF ( K .EQ. N ) GOTO 100
!           T = W(K)/V(K+L)
!           J = L
!           L = L + N - K
!           K = K + 1
!           IF ( ABS(T) .EQ. TZERO ) GOTO 80
!           DO 90 I = K,N
!90              W(I) = W(I) - T*V(I+J)
!           GOTO 80
!!     -----------------------------------
!!     |*** BACK SUBSTITUTION BY ROWS ***|
!!     -----------------------------------
!100        W(N) = W(N)/V(K+L)
!110        IF ( K .EQ. M ) GOTO 130
!           J = K
!           K = K - 1
!           L = L + K - N
!           T = W(K)
!           DO 120 I = J,N
!120             T = T - W(I)*V(I+L)
!           W(K) = T/V(K+L)
!           GOTO 110
!130        DO 140 I = M,N
!140             V(I+H) = W(I)
!150   CONTINUE
!!     ------------------
!!NRB  |*** UNPACK V ***|
!!     ------------------
!      H=LV-N
!      I=(N-1)*H+(N*(N-1))/2
!      M=N-1
!      L=(N*(N+1))/2
!      K=L
!200   IF(I.EQ.0)RETURN
!      DO J=L,K,-1
!        V(I+J)=V(J)
!      ENDDO
!      I=I-H-M
!      L=K-1
!      M=M-1
!      K=K-N+M
!      GO TO 200
!!
!      END
!!
!!***************************************************************
